/*############################################################################
                  << ASK_MAUP_v1.c >>      1994
      (Analysis and Simulation Kit for Modifiable Areal Unit Problem)

	Acknowledgments: Two subroutines (Repartition and CalcSAC) are based on a 
	Fortran program originally written by Dr. Ye Qi, which laid the ground 
	for developing ASK_MAUP.  
############################################################################*/
#include    <stdio.h>
#include    <stdlib.h>
#include    <math.h>
#include    "ASK_MAUP.h"
#include    "ASK_MAUP_Ctrl.h"

/*############################## Main() ####################################*/
main()
{

   /*======================================================================= 
   The main program opens the output file and two list files that contain
   the names of data files and scales over which the analysis is performed.
   =======================================================================*/

   int      datSetCuntr, scaleCuntr;
   char   *FilHdg_ptr, *DatSetNam_ptr;   
   void   Pop_Menu ();                     
   
/*    Pop_Menu();   */

   choice_NUM = 1;
   AutoCorr_Switch = 1;
   ScaleVar_Switch = 0;
/*   ScaleVar_Switch = 1;  */
   see_DAT = 'n';
   see_ACT = 'n';

                                /*========== Open Files ==========*/
                                
      if ( ( rfPtr_1 = fopen(INPUT_FILE_1, "r") ) == NULL )
         Err_Msg ("Can't open INPUT_FILE_1!");
         
      if ( ( rfPtr_2 = fopen(INPUT_FILE_2, "r") ) == NULL )
         Err_Msg ("Can't open INPUT_FILE_2!");
   
      if ( ( wfPtr_1 = fopen(OUTPUT_FILE_1, "w") ) == NULL )
         Err_Msg ("Can't open OUTPUT_FILE_1!");
   
      if ( ( wfPtr_2 = fopen(OUTPUT_FILE_2, "w") ) == NULL )
         Err_Msg ("Can't open OUTPUT_FILE_2!");

            /** Read the number of data sets to be processed and **/ 
            /** the numbers of rows and columns in each data set **/   

   if ( (FilHdg_ptr = (char *) malloc (100 * sizeof (char))) == NULL )
         Err_Msg ("Memory problem when allocating FilHdg_ptr!");
         
                  /*** Remove the file heading in DatFile.Lst ***/
      fscanf (rfPtr_1, "%[^\n]", FilHdg_ptr);   
      
      fscanf (rfPtr_1, "%d", &Num_DatSets);
   

               /* ################################# */
               /* Main Loop for Different Data Sets */
               /* ################################# */
                     
   for (datSetCuntr=0; datSetCuntr<Num_DatSets; datSetCuntr++)  {
   
                     /* Read the name of dataset to be processed */
      if ((DatSetNam_ptr = (char *) malloc (50 * sizeof (char))) == NULL)
         Err_Msg ("Memory problem when allocating DatSetNam_ptr!");
   
         fscanf (rfPtr_1, "%s", DatSetNam_ptr);
         fscanf (rfPtr_1, "%d %d", &orig_NUM_ROW, &orig_NUM_COL);
      
                           /* Write it out in the output file */
      fprintf (wfPtr_1, "%s\n", DatSetNam_ptr);
      fprintf (wfPtr_2, "%s\n", DatSetNam_ptr);

if (see_ACT=='y' || see_ACT=='Y')  {
         printf ("\n===========================\n");
         printf (  "Data Set Name: %s\n", DatSetNam_ptr);
         printf (  "===========================\n");
}

                     /* Remove the file heading in Scales.list */
      fscanf (rfPtr_2, "%[^\n]", FilHdg_ptr);   

                        /*** The total #scales for analysis ***/
      fscanf (rfPtr_2, "%d", &Num_Scale);
      
if (see_ACT=='y' || see_ACT=='Y')  {
      printf ("The #scales/zoning alternatives for analysis = %3d\n\n", 
            Num_Scale);
}
      
      fprintf (wfPtr_1, "The number of scales for analysis = %3d\n", 
             Num_Scale);

      fprintf (wfPtr_2, "The number of scales for analysis = %3d\n", 
             Num_Scale);

                           /*** Names of variables for output files ***/
      if (AutoCorr_Switch)  {                     
         fprintf (wfPtr_1, "BSU#/R/C/Grn   Mean       ");
         fprintf (wfPtr_1, "MC      expect_MC    adj_MC      CO         GR\n");
      
         fprintf (wfPtr_2, "BSU_ROW_Grain BSU_COL_Grain   grid_Mean    ");
         fprintf (wfPtr_2, "variance    pct_Var\n");
      }
      

               /* ================================= */
               /* Loop for Different Spatial Scales */
               /* ================================= */

      for (scaleCuntr=0; scaleCuntr<Num_Scale; scaleCuntr++)  {
      
                  /** Read in information on scale specification **/
                  
         fscanf (rfPtr_2, "%d %d %d %d", &BSU_ROW_Grain, &BSU_COL_Grain, 
               &startROW_0, &startCOL_0);


         Repartition ( DatSetNam_ptr ); 

         if (AutoCorr_Switch) 
            CalcSAC ();  
                                   
         if (ScaleVar_Switch)     
            CalcSVAR (Num_Scale);
                                 
      }
      
      rewind(rfPtr_2);  
      
      free (DatSetNam_ptr);
      fprintf (wfPtr_1, "\n\n\n");
      fprintf (wfPtr_2, "\n\n\n");
   }
      
      fclose ( rfPtr_1 );
      fclose ( rfPtr_2 );
      fclose ( wfPtr_1 );

  if (see_ACT=='y' || see_ACT=='Y') {
   printf ("\n");
   printf ("############################################################\n");
   printf ("####### Computation has been completed successfully. #######\n");
   printf ("############################################################\n");
  }
}

/*############################ End of Main() ###############################*/


/*============================ Repartition() =================================
   Repartition() reads in spatial data indicated by the input file name 
   and repartition the data set according to the specified scale.  To save 
   memory, the original data is read in, used, and deleted from the memory 
   each time this func is called for each scale or zoning alternative.
============================================================================*/
void  Repartition (ptr_DatSetNam) 
char *ptr_DatSetNam;
{
   int         row, col, startROW, startCOL, endROW, endCOL; 
   int         New_ROW, New_COL, cur_COL, cur_ROW;
   float      **origData, blk_Mean, grid_Mean, *skipper_ptr;
   static int   num_Call=0;
   FILE       *rfp_DatFile;
   

      if ( BSU_ROW_Grain > orig_NUM_ROW || BSU_COL_Grain > orig_NUM_COL)  {
         printf ("\n\t!!!!!!!!!!!! WARNING !!!!!!!!!!!!");
         printf ("\n\t##### Grain size > extent?! #####\n");
         printf (  "\t!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n");
      }
                                       /* Do it anyway, pal. */
      if ( BSU_ROW_Grain > orig_NUM_ROW )   BSU_ROW_Grain = orig_NUM_ROW;
      if ( BSU_COL_Grain > orig_NUM_COL )   BSU_COL_Grain = orig_NUM_COL;
      
      
               /*** Allocate memory for origData and repartData ***/
      
   origData    = (float **) Matrix_2D_flt(orig_NUM_ROW, orig_NUM_COL);
   repartData   = (float **) Matrix_2D_flt(orig_NUM_ROW, orig_NUM_COL);
                        
                     /*##########################################*/
                     /*** Open and read in the input data file ***/
                     /*##########################################*/
                     
   if ((skipper_ptr = (float *) malloc (skip_LENGTH*sizeof (float)))==NULL)
         Err_Msg ("Memory problem when allocating skipper_ptr!");   

   if ( (rfp_DatFile = fopen( ptr_DatSetNam, "r")) == NULL )  
      Err_Msg ("From within Repartition(): Can't open data file!");
         
   for(row=0; row<orig_NUM_ROW; row++) {
      for(col=0; col<orig_NUM_COL; col++)  {
         fscanf(rfp_DatFile, "%f", &origData[row][col]);
      }
      fscanf (rfp_DatFile, "%[^\n]", skipper_ptr);  
   }                           /* Get rid of the rest of the line */
   
   fclose (rfp_DatFile);
   
   free (skipper_ptr);

            /* If BSU_ROW_Grain and BSU_COL_Grain are both equal to */
            /* 1 (no aggregation), the data set remains unchanged.  */

   if ( (BSU_ROW_Grain==1) && (BSU_COL_Grain==1) )  {
   
               /* Adjust the #s of row and col against starting pos */
               
      cur_NUM_ROW = orig_NUM_ROW - startROW_0;
      cur_NUM_COL = orig_NUM_COL - startCOL_0;
      
               /* Adjust the BSU values accordingly */
      for (row=0; row<orig_NUM_ROW; row++)  {
         for (col=0; col<orig_NUM_COL; col++)  {
            New_ROW = row + startROW_0;
            New_COL = col + startCOL_0;
            repartData[row][col] = origData[New_ROW][New_COL];
         }   
      }
   }
   
            /*** Print out the re-partitioned data for checking; the ***/
            /***  format for BSU_ROW_Grain ==1 & BSU_COL_Grain ==1   ***/ 
      
   if (see_DAT == 'y' || see_DAT == 'Y')  {
      if ( BSU_ROW_Grain ==1 && BSU_COL_Grain ==1 )  {
         printf ("Rescaled Data from Repartition():\n");
         printf ("BSU_ROW_Grain ==1 && BSU_COL_Grain ==1:\n");
         for (row=0; row<orig_NUM_ROW; row++)  {
            for (col=0; col<orig_NUM_COL; col++)  {
               printf ("%6.2f  ", repartData[row][col]);
            }   printf ("\n");
         }
      }
      printf ("\n");
   }


         /** If BSU_ROW_Grain and/or BSU_COL_Grain are larger than 1, **/
         /** assign values to starting row and column, and ending row **/
         /** and column of the first block, and initialize the #'s of **/
         /** rows and cols at the new scale taking a "snow-balling"   **/
         /** strategy, with which data is aggregated block by block.  **/ 

   if ( (BSU_ROW_Grain !=1) || (BSU_COL_Grain !=1) )  {
   
      startROW = startROW_0;
      startCOL = startCOL_0;
      endROW = startROW + BSU_ROW_Grain - 1;
      endCOL = startCOL + BSU_COL_Grain - 1;
      
                     /*** Set cur_ROW and cur_COL to point to the   **/
                     /*** first cell of the data set to be analyzed **/
      cur_ROW = 0;
      cur_COL = 0;
         
      
                  /*##################################################*/
                  /** Repartition data block by block and row by row **/
                  
      while (endROW < orig_NUM_ROW)  {
         while (endCOL < orig_NUM_COL)  {    
         
                              /*#################################*/
                              /*##### RESAMPLING PROCEDURES #####*/
                              /*#################################*/

            /** SIMPLE ABERAGE: Compute the mean for all the cells, and */
            /** then assign it to the newly formed areal unit (block).  */
     
            blk_Mean = 0.;
            
            for (row=startROW; row<=endROW; row++)  {
               for (col=startCOL; col<=endCOL; col++)  {
                  blk_Mean = blk_Mean + origData[row][col];
               }
            }
            
            blk_Mean = blk_Mean / (float) (BSU_ROW_Grain * BSU_COL_Grain);
               
            repartData[cur_ROW][cur_COL] = blk_Mean;
         
                                 /** Increment the #Columns **/
                           
            startCOL = endCOL + 1;
            endCOL = startCOL + BSU_COL_Grain - 1;
            cur_COL = cur_COL + 1;         
         }      
         
               /** Up to the end of the row? Increment the #rows **/
                  
         startROW = endROW + 1;
         endROW = startROW + BSU_ROW_Grain - 1;
         cur_ROW = cur_ROW + 1;
   
               /** Restart aggregating from the 1st col in a new row. */
         
         if (endROW < orig_NUM_ROW)  {
            startCOL = startCOL_0;
            endCOL = startCOL + BSU_COL_Grain - 1;
            cur_COL = 0;
         }
         
         if (see_ACT == 'y' || see_ACT == 'Y')  {
           if ( (endROW % 5) == 0 )  {
            printf ("\n      +++++++++++++++++++++++++++++\n");
            printf (  "      * Repartitioning ROW[%d]...\n", endROW);
            printf (  "      +++++++++++++++++++++++++++++\n\n");
           }
         }
      }
      
                           /*** The number of rows and cols ***/
                           /*** in the rescaled data set.   ***/
      cur_NUM_ROW = cur_ROW;
      cur_NUM_COL = cur_COL;
      
      
                  /*** Check out the repartitioned data ***/
               /*** when BSU_ROW_Grain and/or BSU_COL_Grain !=1 ***/ 
               
      if (see_DAT == 'y' || see_DAT == 'Y')  {
         printf ("\n");
         printf ("Repartitioned Data from Repartition():\n");
         for (row=0; row<cur_NUM_ROW; row++)  {
            for (col=0; col<cur_NUM_COL; col++)  {
               printf ("%6.2f  ", repartData[row][col]);
            }   printf ("\n");
         }
         printf ("\n");
      }
   }
   
   Free_2D_Mat_flt (origData, orig_NUM_ROW);
}


/*============================== CalcSAC() ===================================
   CalcSAC() generates the connectivity matrices C's (or W's) for any given 
   raster data sets, and computes the three spatial autocorrelation indices.
============================================================================*/
void 
CalcSAC ()  
{

   /*=======================================================================
   connect_M[][]:   Records the revised connectivity matrix
   order_NZ[][]:   Rrecords the order of non-void valued grid cells
   Kcuntr:         Counter used for ordinally counting non-void areal units  
   R[k]:          Stores the row coordinate of the kth non-void areal units
   C[k]:          Stores the col coordinate of the kth non-void areal units
   grid_Mean:       The mean over the entire grids
   Sum_Cij:      Summation of all Cij values in the connectivity matrix
   SumXiXj_bar:    Sigma { c(i,j) [x(i)-x_bar] [x(j)-x_bar] }
   SumXi_XjSq:    Sigma { [x(i) - x(j)] ^2 }
   SumXi_barSq:    Sigma { [X(i) - X_bar]^2 } 
   SumC_j:         Sigma { c(.j) }
   MC:          Moran coefficient (Moran's I)
   GR:          Geary ratio (Geary's c)
   CO:          Cliff-Ord statistic
   adj_MC:         Adjusted Moran coefficient (i.e., MC - expect_MC)
   expect_MC:      Expected value of Moran coefficient
   =======================================================================*/
   
   int       row, col, k, p, v, Kcuntr, Sum_Cij;
   int         **order_NZ, **connect_M, *SumC_j, *R, *C; 
   float      grid_Mean, SumXiXj_bar, Sum_XiXj_CO, SumXi_XjSq, SumXi_barSq;
   float      MC, adj_MC=0, expect_MC=0, GR, CO, variance, pct_Var;
   static int   num_Call=0;
   static float   var_0;
   

               /*** Number of times this function has been called ***/
         
   if (++num_Call == Num_Scale)   num_Call = 0; 

         
                        /*** Allocate memory for order_NZ ***/
                     
   if ((order_NZ = (int **) Matrix_2D_int(cur_NUM_ROW,cur_NUM_COL)) == NULL)
      Err_Msg ( "Memory problem when allocating order_NZ[][]!" );


      if (see_ACT == 'y' || see_ACT == 'Y')  
         printf ("\n>>>>> Computation is in progress ... ");
   
                     /** Record the order and coordinates of  **/
                     /** non-void areal units in the data set **/
                     
   Kcuntr = 0;   
   for (row=0; row<cur_NUM_ROW; row++)  {
      for (col=0; col<cur_NUM_COL; col++)  {
      
         order_NZ[row][col] = 0;

          if ( repartData[row][col] != void_Value )  { 
                                              
            Kcuntr = Kcuntr + 1;
            order_NZ[row][col] = Kcuntr;
         }
      }
   }
   
                        /*** Allocate and initiate connect_M ***/
                     
   if ( (connect_M = (int **) Matrix_2D_int((Kcuntr+1), 4)) == NULL )  {
      Err_Msg ( "Memory problem when allocating connect_M[][]!" );
   }
   
   if ( (R = (int *) malloc ((Kcuntr+1) * sizeof (int))) == NULL ||
       (C = (int *) malloc ((Kcuntr+1) * sizeof (int))) == NULL )  {
      Err_Msg ( "Memory problem when allocating R[] or C[]!" );
   }

                        /*** Find the coordinates for Kcuntr ***/
      
   for (k=1; k<=Kcuntr; k++)  {
      for (row=0; row<cur_NUM_ROW; row++)  {
         for (col=0; col<cur_NUM_COL; col++)  {
            if ( order_NZ[row][col] == k )  {
               R[k] = row;
               C[k] = col;
            }
         }
      }
   }
               /*** Determine the values of Cij in the C matrix ***/
               /*#################################################*/
               
   for (k=0; k<=Kcuntr; k++)  {
      for (p=0; p<4; p++)  {
         connect_M[k][p] = 0;
      }
   }

   grid_Mean = 0.;
    
   for (row=0; row<cur_NUM_ROW; row++)  {
      for (col=0; col<cur_NUM_COL; col++)  {
 
          if ( repartData[row][col] != void_Value )  {

                  /*** BTW, compute the grid_Mean for repartData ***/
            
            grid_Mean = grid_Mean + repartData[row][col]/(float) Kcuntr; 
            

                              /*** Check the upper cell ***/
            
            if ( row > 0 )  {
               if ( repartData[row-1][col] != void_Value )  {

                       connect_M[ order_NZ[row][col] ][0] 
                             = order_NZ[row-1][col];
               }
            }
                              /*** Check the lower cell ***/
               
            if ( (row+1) < cur_NUM_ROW )  {
               if ( repartData[row+1][col] != void_Value )  {
                  
                  connect_M[ order_NZ[row][col] ][1] 
                        = order_NZ[row+1][col];
               }
            }
                              /*** Check the left cell ***/
               
            if ( col > 0 )  {
               if ( repartData[row][col-1] != void_Value )  {
                  
                  connect_M[ order_NZ[row][col] ][2] 
                        = order_NZ[row][col-1];
               }
            }
                              /*** Check the right cell ***/
               
            if ( (col+1) < cur_NUM_COL)  {
               if ( repartData[row][col+1] != void_Value )  {
                  
                  connect_M[ order_NZ[row][col] ][3] 
                        = order_NZ[row][col+1];
               }
            }
         }
      }
   }
                              /* Check out the mean */
      if (see_ACT == 'y' || see_ACT == 'Y')  {   
         printf ("\n============================\n");
         printf (  "I am done with the C matrix!\n");
         printf (  "Grid_Mean = %.4f\n", grid_Mean);  
         printf ("============================\n\n");
      }
      
   Free_2D_Mat_int(order_NZ, cur_NUM_ROW);
   
   
            /*#########################################*/
            /*** Sums for computing autocorr indices ***/
            /*#########################################*/
                        
   if ( (SumC_j = (int *) malloc ((Kcuntr+1) * sizeof (int))) == NULL )
      Err_Msg ("Memory problem when allocating SumC_j.");

    Sum_Cij = 0;
   SumXiXj_bar = 0.;
    SumXi_XjSq = 0.;
   SumXi_barSq = 0.;
                           /*########################*/
                           /*** Compute the SigmaS ***/
                           /*########################*/
   for (k=1; k<=Kcuntr; k++)  {
      SumC_j[k] = 0.;
      
      if (see_ACT == 'y' || see_ACT == 'Y')  {
         if ( (k % 50) == 0 )  {
          printf ("-------------------------------------------\n");
          printf ("Computing the Sigmas - status: %d of %d\n",k,Kcuntr);
          printf ("-------------------------------------------\n\n");
        }
      }
      
       for (p=0; p<4; p++)  {
       
        if ( (connect_M[k][p] < (-MINFLOAT)) || 
             (connect_M[k][p] > MINFLOAT) )  {
             
         Sum_Cij = Sum_Cij + 1;
         SumC_j[k] = SumC_j[k] + 1;
            
         for (v=1; v<=Kcuntr; v++)  {
           if (connect_M[k][p] == v)  {
               
            SumXiXj_bar = SumXiXj_bar + (repartData[R[k]][C[k]] - 
              grid_Mean) * (repartData[R[v]][C[v]] - grid_Mean);   
                  
            SumXi_XjSq = SumXi_XjSq + ( repartData[R[k]] [C[k]] - 
              repartData[R[v]] [C[v]] ) * ( repartData[R[k]][C[k]]
              - repartData[R[v]] [C[v]] );
           }
         }
        }
      }
   }
   
   for (row=0; row<cur_NUM_ROW; row++)  {
      for (col=0; col<cur_NUM_COL; col++)  {
      
         if ( repartData[row][col] != void_Value )  {
            
            SumXi_barSq = SumXi_barSq + (repartData[row][col]-grid_Mean)
               * (repartData[row][col]-grid_Mean);
         }
      }
   }
                  /*** For Cliff-Ord Statistic only ***/
                  
   if ( choice_NUM == 2 )  {
      Sum_XiXj_CO = 0.;
      for (k=1; k<=Kcuntr; k++)  {
         for (p=0; p<4; p++)  {
         
           if ( connect_M[k][p] < (-MINFLOAT) ||
                connect_M[k][p] > MINFLOAT )  {
                
            for (v=1; v<=Kcuntr; v++)  {
            
              if (connect_M[k][p] == v)  {
              
               Sum_XiXj_CO = Sum_XiXj_CO + 1. / (float) SumC_j[k] *
                  (repartData[R[k]][C[k]] - grid_Mean) * 
                  (repartData[R[v]][C[v]] - grid_Mean);
              }
            }
           }
         }
      }
   }
                  /***=========================***/
                  /***  Variance at each scale ***/
                  /***=========================***/
         
   if (cur_NUM_ROW*cur_NUM_COL > 30)       
      variance = SumXi_barSq / (cur_NUM_ROW*cur_NUM_COL - 1);
   else
      variance = SumXi_barSq / (cur_NUM_ROW*cur_NUM_COL);
   
   if (num_Call == 1)  
      var_0 = variance;
   
   if (var_0 != 0)   
      pct_Var = variance / var_0;
   else
      pct_Var = 1.;
         
         /*=====================================================*/
         /* Moran coefficient, Geary ratio, Cliff-Ord statistic */
         /*=====================================================*/
         
   if ( (SumXiXj_bar == 0) || (SumXi_barSq == 0) )  
   {
      MC = 1;   CO = 1; GR = 0;
   }
   else 
   {
      MC = (float) Kcuntr / (float) Sum_Cij * SumXiXj_bar / SumXi_barSq;
      
      GR = ((float) (Kcuntr-1)/(float) (2*Sum_Cij)) * SumXi_XjSq 
            / SumXi_barSq;
            
      CO = Sum_XiXj_CO / SumXi_barSq;
      
      expect_MC = -1.0 / (float) (cur_NUM_ROW * cur_NUM_COL - 1);
   
      adj_MC = MC - expect_MC;         /* Adjusted against the mean */
   
   }
      
   if ( choice_NUM == 1 )  {
     fprintf (wfPtr_1,"%4d  %4d  %10.4f %10.4f  %10.4f  %9.4f    ---  %9.4f\n", 
         BSU_ROW_Grain, BSU_COL_Grain, grid_Mean, MC, expect_MC, adj_MC, GR);
   }
   else if ( choice_NUM == 2)  {
     fprintf (wfPtr_1,"%4d  %4d  %10.4f %10.4f  %10.4f  %9.4f  %9.4f  %9.4f\n", 
         BSU_ROW_Grain, BSU_COL_Grain, grid_Mean, MC, expect_MC, adj_MC, 
         CO, GR);
   }
   
   fprintf (wfPtr_2, "%12d  %12d  %10.4f  %10.4f  %10.4f\n",
         BSU_ROW_Grain, BSU_COL_Grain, grid_Mean, variance, pct_Var);

      
                     /*** Check the results out on the screen ***/

if (see_ACT=='y' || see_ACT=='Y')  {
      printf ("<><><><><><><><> Results from ASK_MAUP <><><><><><><><>\n");
      printf ("=======================================================\n\n");
      printf ("BSUs/Grain\n");
      printf (" Row     Col     MC      adj_MC    CO      GR         ");
      printf (" Mean      Var\n");
      if ( choice_NUM == 1 )  {
        printf (" %3d    %3d    %6.4f   %6.4f   ---  %6.4f     %.4f    %.4f\n",
         BSU_ROW_Grain, BSU_COL_Grain, MC, adj_MC, GR, grid_Mean, variance);
      } else if ( choice_NUM == 2 )  {
      printf (" %3d    %3d    %6.4f   %6.4f    %6.4f  %6.4f     %.4f    %.4f\n",
         BSU_ROW_Grain, BSU_COL_Grain, MC, adj_MC, CO, GR, grid_Mean, variance);
      }
      printf ("\n");
}
                                   /*** Free up the memory ***/
   Free_2D_Mat_int(connect_M, Kcuntr);
   free (SumC_j);
   free (R);
   free (C);
}


/*========================= CalcSVAR() =============================
==================================================================*/
void CalcSVAR ( Num_Level )  
int Num_Level;
{
   
   int          row, col, R, C, tot_DF;
   int            startROW, startCOL, endCOL, endROW;
   float         sum_Temp, sumXi_barSq, tot_pct_TSS; 
   static int      *DF, prev_BSU_ROW_Grain, prev_BSU_COL_Grain;
   static int      ctr_Alloc=0, num_Calls=0, prev_NUM_ROW, prev_NUM_COL;
   static float   **data_Prev, SS_total, *sum_Sqs, *pct_SumSqs, *scale_Var;
   static float   *mean_Sq, grand_Mean, *grid_mean, *variance, *pct_Var;


                                   /*** Allocate memory for vars ***/
   if (ctr_Alloc == 0)  
   {
      data_Prev  = (float **) Matrix_2D_flt(cur_NUM_ROW,cur_NUM_COL);
      sum_Sqs    = (float *) malloc ((Num_Level) * sizeof (float));
      scale_Var  = (float *) malloc ((Num_Level) * sizeof (float));
      mean_Sq    = (float *) malloc ((Num_Level) * sizeof (float));
      grid_mean  = (float *) malloc ((Num_Level) * sizeof (float));
      pct_SumSqs = (float *) malloc ((Num_Level) * sizeof (float)); 
      variance   = (float *) malloc ((Num_Level) * sizeof (float));
      pct_Var    = (float *) malloc ((Num_Level) * sizeof (float));
      DF         = (int *) malloc ((Num_Level) * sizeof (int));
   }
   
               /*** Count the number of calls made to the function ***/
   ctr_Alloc++;
   num_Calls++;   

                                    /*** Check data read in ***/
   if (see_DAT == 'y' || see_DAT == 'Y')  {
      printf (">>>>>>>>>>>>>> num_Calls = %d\n\n", num_Calls);
      printf ("=====> From CalcSVAR():\n");
      printf ("Check these variables out in the beginning of CalcSVAR():\n");
      printf ("--------------------------------------------------------\n");
      printf ("cur_NUM_ROW  cur_NUM_COL  BSU_ROW_Grain  BSU_COL_Grain\n");
      printf ("%d           %d           %d             %d\n\n",
            cur_NUM_ROW, cur_NUM_COL, BSU_ROW_Grain, BSU_COL_Grain);
      
      printf ("\n");
      printf ("Repartitioned Data from CalcSVAR():\n");
      for (row=0; row<cur_NUM_ROW; row++)  
      {
         for (col=0; col<cur_NUM_COL; col++)  
         {
            printf ("%6.2f  ", repartData[row][col]);
         }   printf ("\n");
      }
      printf ("\n");
   }

   if (num_Calls == 1)  { 
      for (R=0; R<Num_Level; R++)  {
         grid_mean[R]          = 0.;      
         variance[R]      = 0.;
         pct_SumSqs[R]    = 0.;      
         scale_Var[R]     = 0.;
         sum_Sqs[R]       = 0.;      
         mean_Sq[R]       = 0.;
         DF[R]            = 0;
      }
      
      for (row=0; row<cur_NUM_ROW; row++)  {
         for (col=0; col<cur_NUM_COL; col++)  {
            data_Prev[row][col] = 0.;
         }
      }
   }

      /*##############################################################*/
      /*** Compute the grand mean and SS_total at Level 1, and mean ***/
      /*** and variance at each level/scale (m & v not required for ***/
      /*** computing SV. NOTE: Scale.lst must start with 1 by 1 BSU.***/
      /*##############################################################*/
         
   sum_Temp = 0.;
   
   for (row=0; row<cur_NUM_ROW; row++)           /* Mean at each level */
   {
      for (col=0; col<cur_NUM_COL; col++)  
      {   
         sum_Temp = sum_Temp + repartData[row][col]; 
         }
   }

   grid_mean[num_Calls-1] = sum_Temp / (cur_NUM_ROW * cur_NUM_COL);
   
   sumXi_barSq = 0.;
                                 /* Sum of sq for ANOVA variance */
   for (row=0; row<cur_NUM_ROW; row++)  {   
      for (col=0; col<cur_NUM_COL; col++)  { 
         sumXi_barSq = sumXi_barSq
                  + (repartData[row][col] - grid_mean[num_Calls-1])
                  * (repartData[row][col] - grid_mean[num_Calls-1]); 
                  
                                 /** Store current data in data_Prev **/
            if (num_Calls == 1)                    
               data_Prev[row][col] = repartData[row][col];
         }
      }
                                 /** Store current row & col info **/
         if (num_Calls == 1)  {   
            prev_NUM_ROW = cur_NUM_ROW;
            prev_NUM_COL = cur_NUM_COL;
         prev_BSU_ROW_Grain    = BSU_ROW_Grain;
         prev_BSU_COL_Grain    = BSU_COL_Grain;
         }               
                                    /* Variance at each level */
   if ( (cur_NUM_ROW*cur_NUM_COL) > 30)       
         variance[num_Calls-1] = sumXi_barSq / (cur_NUM_ROW * cur_NUM_COL - 1);
   else
         variance[num_Calls-1] = sumXi_barSq / (cur_NUM_ROW * cur_NUM_COL);
   
                              /* Percent variance at each level */
   if (variance[0] != 0)   
      pct_Var[num_Calls-1] = variance[num_Calls-1] / variance[0] * 100.;
   else
      pct_Var[num_Calls-1] = 100.;
   
   if (num_Calls == 1)  {
      grand_Mean    = grid_mean[0];
      SS_total    = sumXi_barSq;

         return;
   }
                                 /*###########################*/
                                 /*## INFORMATION ON SCREEN ##*/
                                 /*###########################*/
if (see_ACT=='y' || see_ACT=='Y')  {
      printf ("\n");                        
      printf ("## Current Level = num_Calls = %d \n", num_Calls);
      printf ("cur_NUM_ROW   = %d;   cur_NUM_COL   = %d\n", 
             cur_NUM_ROW, cur_NUM_COL);
      printf ("BSU_ROW_Grain = %d;   BSU_COL_Grain = %d\n", 
            BSU_ROW_Grain, BSU_COL_Grain);
}


         /*#####################################################*/
         /***  Sum of squares for calculating scale variance  ***/
         /*#####################################################*/
   
   startROW    = 0;
   startCOL    = 0;
   endCOL       = prev_NUM_COL/cur_NUM_COL - 1;
   endROW       = prev_NUM_ROW/cur_NUM_ROW - 1;
      
   for (R=0; R<cur_NUM_ROW; R++)  {
      for (C=0; C<cur_NUM_COL; C++)  {
      
         for (row=startROW; row<=endROW; row++)  {
              for (col=startCOL; col<=endCOL; col++)  {
      
               sum_Sqs[num_Calls-2] = sum_Sqs[num_Calls-2]
                        + (data_Prev[row][col] - repartData[R][C])
                        * (data_Prev[row][col] - repartData[R][C]); 
                }
           }                        
                                /** Move startCOL along the row **/
           if (endCOL < prev_NUM_COL-1)  {
              startCOL = endCOL + 1; 
              endCOL    = startCOL + (prev_NUM_COL/cur_NUM_COL) - 1;
           }
                                /** When finishing up last col,  **/
                                /** set startCOL back to 1st col **/
           else  {
              startCOL = 0; 
            endCOL = prev_NUM_COL/cur_NUM_COL - 1;
            
                                /** When finishing up last col,  **/
                                /** set startROW to the next row **/
            startROW = endROW + 1;
            endROW    = startROW + (prev_NUM_ROW/cur_NUM_ROW) - 1;
           }
      }        
   }
                            /*** Degree of freedom at each level ***/
                            
   if ( BSU_ROW_Grain*BSU_COL_Grain == prev_BSU_ROW_Grain*prev_BSU_COL_Grain )
      DF[num_Calls-2] = DF[num_Calls-3];
   else                
      DF[num_Calls-2] = cur_NUM_ROW*cur_NUM_COL*((BSU_ROW_Grain*BSU_COL_Grain
                / (prev_BSU_ROW_Grain * prev_BSU_COL_Grain) - 1));

                            /** Adjust sum of squares for mean sq **/

   scale_Var[num_Calls-2] = sum_Sqs[num_Calls-2] / DF[num_Calls-2];
   
                            /** Adjust sum of squares for mean sq **/
   if ( (num_Calls-2) > 0)  {
      sum_Sqs[num_Calls-2] = (prev_BSU_ROW_Grain * prev_BSU_COL_Grain) 
                     * sum_Sqs[num_Calls-2];
   }
                            /*** Percent of total sum of squares ***/
   
   pct_SumSqs[num_Calls-2] = sum_Sqs[num_Calls-2] / SS_total * 100.;
   
                            /*** Mean square at each level ***/

   mean_Sq[num_Calls-2] = sum_Sqs[num_Calls-2] / DF[num_Calls-2]; 
   
                              /** Store current data in data_Prev **/
   for (row=0; row<cur_NUM_ROW; row++)  {   
      for (col=0; col<cur_NUM_COL; col++)  { 
            data_Prev[row][col] = repartData[row][col];
         }
      }
                                 /** Store current row &col info **/
         prev_NUM_ROW = cur_NUM_ROW;
         prev_NUM_COL = cur_NUM_COL;
      prev_BSU_ROW_Grain    = BSU_ROW_Grain;
      prev_BSU_COL_Grain    = BSU_COL_Grain;
   
   
               /*################################*/
               /*#####        OUTPUT       ######*/
               /*################################*/

   if (num_Calls == Num_Level)  
   {
      tot_pct_TSS = 0.;
      tot_DF = 0;
      for (R=0; R<Num_Level; R++)  
      {
         tot_pct_TSS = tot_pct_TSS + pct_SumSqs[R];
         tot_DF = tot_DF + DF[R];
      }
      fprintf (wfPtr_1, "Level  DF     mean_Sq     sum_Sqs      ");
      fprintf (wfPtr_1, "pct_TSS      scale_Var \n");
      
      fprintf (wfPtr_2, "Level     mean      variance    pct_Var\n");
      
      for (R=0; R<Num_Level-1; R++)  
      {
         fprintf (wfPtr_1,"%3d    %4d  %10.4f  %10.4f  %10.2f   %8.4f\n", (R+1), 
            DF[R], mean_Sq[R], sum_Sqs[R], pct_SumSqs[R], scale_Var[R]);
      }
      
      fprintf (wfPtr_1, "Total  %4d        ---   %10.4f  %10.4f       ---\n", 
            tot_DF, SS_total, tot_pct_TSS);
            
      for (R=0; R<Num_Level; R++)  
      {
         fprintf (wfPtr_2,"%3d  %10.4f  %10.4f  %10.2f\n", (R+1), grid_mean[R], 
            variance[R], pct_Var[R]);
      }

               /*** Set these counters back to 0 for a new data set ***/

      num_Calls = 0;       
      ctr_Alloc = 0;
   }
      
}


/*============================= Pop_Menu() ================================
   Pop_Menu() gives a pop-up menu for input and output information.
=========================================================================*/
void  
Pop_Menu ()
{

      printf ("\t$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n");
      printf ("\t$$$$$                                               $$$$$$\n");
      printf ("\t$$$$$              WELCOME TO ASK_MAUP              $$$$$$\n");
      printf ("\t$$$$$                                               $$$$$$\n");
      printf ("\t$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n");
      printf ("\n\n\n");
   
   while   (choice_NUM < 1 || choice_NUM > 4 )  {
      printf ("\t##########################################################\n");
      printf ("\t##########################################################\n");
      printf ("\t#####           Make your choice, please:            #####\n");
      printf ("\t#####           =========================            #####\n");
      printf ("\t##### 1  --- Autocorrelation Analysis (MC & GR)      #####\n");
      printf ("\t##### 2  --- Autocorrelation Analysis (MC, GR & CO)  #####\n");
      printf ("\t##### 3  --- Scale Variance Calculation              #####\n");
      printf ("\t##### 4  --- Quit                                    #####\n");
      printf ("\t##########################################################\n");
      printf ("\t##########################################################\n");
      printf ("\n");
      scanf  ("%d", &choice_NUM);
      Flush();
   }

      if ( choice_NUM == 1 || choice_NUM == 2 )   
            AutoCorr_Switch = 1;
      else if ( choice_NUM == 3 )               
            ScaleVar_Switch = 1;
      else if ( choice_NUM == 4 )    {            
            printf("\n****************  BYE :-)  ****************\n"); 
            exit(1);
      }
         
   while (see_DAT != 'y' && see_DAT != 'Y' && see_DAT != 'n' && see_DAT != 'N') 
   {
      printf ("\tOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO\n");
      printf ("\tOOOOO                                                OOOOO\n");
      printf ("\tOOOOO        Want to see repartioned data sets       OOOOO\n");
      printf ("\tOOOOO           at each scale on scrn?               OOOOO\n");
      printf ("\tOOOOO                                                OOOOO\n");
      printf ("\tOOOOO                y/Y  --- Yes                    OOOOO\n");
      printf ("\tOOOOO                n/N  --- No                     OOOOO\n");
      printf ("\tOOOOO                                                OOOOO\n");
      printf ("\tOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO\n");
      printf ("\n");
      scanf  ("%c", &see_DAT);
      Flush();
   }
      printf ("\n");
      
   while (see_ACT != 'y' && see_ACT != 'Y' && see_ACT != 'n' && see_ACT != 'N') 
   {
      printf ("\t**********************************************************\n");
      printf ("\t*****                                                *****\n");
      printf ("\t*****     Want to keep track of what's going on      *****\n");
      printf ("\t*****          during the computation?               *****\n");
      printf ("\t*****                                                *****\n");
      printf ("\t*****                y/Y  --- Yes                    *****\n");
      printf ("\t*****                n/N  --- No                     *****\n");
      printf ("\t*****                                                *****\n");
      printf ("\t**********************************************************\n");
      printf ("\n");
      scanf  ("%c", &see_ACT);
   }
}


/*============================= Err_Msg() =================================
   Err_Msg() writes out a message whch is an input string of .
=========================================================================*/
void Err_Msg (msg)
char *msg;
{
   puts (msg);
   exit(1);
}

/*============================ Matrix_2D_int() ============================
   Matrix_2D_int() dynamically allocates a 2-dimensional INTEGER array.
=========================================================================*/
int **Matrix_2D_int (mRow, nCol)
int mRow, nCol;
{
   int    i;
   int      **ptr_Mat;
   
   if ((ptr_Mat = (int **) calloc(sizeof(int *), mRow)) == NULL)
      Err_Msg ("Error within Matrix_2D_int(): 1st calloc.");
      
   for (i=0; i<mRow; i++)
      if ((ptr_Mat[i] = (int *) calloc(sizeof(int), nCol)) == NULL)
         Err_Msg ("Error within Matrix_2D_flt(): 2nd calloc.");
   
   return (ptr_Mat);
}

/*============================ Matrix_2D_flt() ============================
      Matrix_2D() dynamically allocates a 2-dimensional FLOAT array.
=========================================================================*/
float **Matrix_2D_flt (mRow, nCol)
int mRow, nCol;
{
   int    i;
   float   **ptr_Mat;
   
   if ((ptr_Mat = (float **) calloc(sizeof(float *), mRow)) == NULL)
      Err_Msg ("Error within Matrix_2D_flt(): 1st calloc.");
      
   for (i=0; i<mRow; i++)
      if ((ptr_Mat[i] = (float *) calloc(sizeof(float), nCol)) == NULL)
         Err_Msg ("Error within Matrix_2D_flt(): 2nd calloc.");
   
   return (ptr_Mat);
}

/*=========================== Free_2D_Mat_int() ===========================
   Free_2D_Mat_int() frees up the memory space occupied by a 2_D int array.
=========================================================================*/
void  Free_2D_Mat_int (ptr_Mat, mRow)
int **ptr_Mat, mRow;
{
   int i;
   
   for (i=0; i<mRow; i++)
      free (ptr_Mat[i]);
      
   free (ptr_Mat);   
}

/*=========================== Free_2D_Mat_flt() ===========================
   Free_2D_Mat() frees up the memory space taken up by a 2_D float array.
=========================================================================*/
void  Free_2D_Mat_flt (ptr_Mat, mRow)
float **ptr_Mat;
int  mRow;
{
   int i;
   
   for (i=0; i<mRow; i++)
      free (ptr_Mat[i]);
      
   free (ptr_Mat);   
}

/*============================== Flush () ================================
   Flush() uses getchar() to read characters from the input buffer until
   it reads in a carriage return.
=========================================================================*/
void  Flush ()
{
   while ( getchar() != '\n' )
      ;
}
/*************************************************************************/
