/*************************************************************************/
/*************************************************************************/
#include <stdio.h>
#include "local.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

/* 
 * External References
 */
extern char 	*optarg;
extern int	getopt();
extern double	pow();
extern double	sqrt();
/* 
 * Global References
 */
static char *progname;
int 	start_date = 1975;
int 	stop_date = 1995;
float	diffusion_coef, breed_coef, spread_coef, 
	slope_resistance;
float	road_gravity;
int	critical_slope;
float	critical_high, critical_low, boom, bust;
int	number_of_times, cumulate;

struct stat     stat_buf;

#ifdef DEBUG
FILE *DBG_FILE;
#endif


/* 
 * Forward References
 */
int append_results();

main(argc, argv)
int argc;
char *argv[];
{
int		errflag;
char		c;

#ifdef DEBUG
	DBG_FILE=fopen("./debug_calib", "w");
	(void)fprintf(DBG_FILE, "(%s %d) Enter Main\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	/* initialize */
	errflag = 0;
	RUN_FLAG = LOG_FLAG = FALSE;
	memset(RUN_DIR, '\0', MAX_LEN);
	memset(LOG_DIR, '\0', MAX_LEN);
	memset(OUTDIR, '\0', MAX_LEN);
	memset(SFX, '\0', MAX_LEN);
	PASS_LIST_P = NULL;
	NEI_RQMT = 0;
	NEI_HOOD = 0;

	/* process the command line */
	while ((c = getopt(argc, argv, "r:l:x:o:")) != EOF ) { switch (c) {
		case 'r':
			strcpy(RUN_DIR, optarg);
			RUN_FLAG = TRUE;
			break;
		case 'o':
			strcpy(OUTDIR, optarg);
			break;
		case 'l':
			strcpy(LOG_DIR, optarg);
			LOG_FLAG = TRUE;
			break;
		case 'x':
			strcpy(SFX, optarg);
			break;
		case '?':
			errflag++;
			break;
		} /* end switch */
		/* if there are errors, print the usage message and quit */
		if ( errflag ) {
			fprintf(stderr, "usage: %s [-r run_dir] [-l log_dir] [t tmplogdir] [-d logdir]\n\n", progname);
 			fprintf(stderr, "	-r	name of directory where grow program reside\n");
 			fprintf(stderr, "	-l	name of directory where logfiles reside\n");
 			fprintf(stderr, "	-x	suffix i.e. 1k, 120m, etc\n\n");
			exit(1);
		}
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) \n", __FILE__, __LINE__);
	(void)fprintf(DBG_FILE, "RUN:%s LOG:%s  SFX: %s\n",
	RUN_DIR, LOG_DIR, SFX);
	(void)fflush(DBG_FILE);
#endif

	if (strcmp(OUTDIR, "") == 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) -o outfile required!\n", __FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		(void)printf("-o outfile required!\n");
		exit(1);
	}
	if (strcmp(SFX, "") == 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) -s suffix required!\n", __FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		(void)printf("-s suffix required!\n");
		exit(1);
	}

	run_passes();

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Runs complete\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
#ifdef DEBUG
	fclose(DBG_FILE);
#endif
} /* end main */

/********************************************************************
********************************************************************/
run_passes()
{
	char	pathname[MAX_LEN], cmd[MAX_LEN], file_str[MAX_LEN];
	FILE	*fp_tmp;
	int	num_runs, rc;

	/*
	 * these values will default 
	 */
	slope_resistance = 5.0;
	critical_low = 0.3;
	boom = 1.01;
	bust = 0.99;
	critical_slope = 15;
	number_of_times = 1;
	cumulate = 0;
	critical_high = 10;

	for (NEI_RQMT = 4; NEI_RQMT >= 2 ; NEI_RQMT -= 2) {
	for (NEI_HOOD = 1; NEI_HOOD <= 4 ; NEI_HOOD += 3) {

	/* each loop represents a new control file */
	for (diffusion_coef = 5.0; diffusion_coef <= 95.0 ; diffusion_coef+= 30.0) {
	for (breed_coef = 5.0; breed_coef <= 95.0 ; breed_coef+= 30.0) {
	for (spread_coef = 5.0; spread_coef <= 95.0 ; spread_coef+= 30.0) {
	for (road_gravity = 5.0; road_gravity <= 95.0 ; road_gravity+= 30.0) {

		init_class_array();
		init_land_array();

		/* write the control for the "grow" program */
		strcpy(pathname, RUN_DIR);
		strcat(pathname, "/control");
		write_control(pathname);

		/* repeat the run with the same control file */
		for (num_runs=0; num_runs<MAX_C_RUNS; num_runs++) {
#ifdef DEBUG
			(void)fprintf(DBG_FILE, "(%s %d) Begin grow() for run #%d\n", __FILE__, __LINE__, num_runs+1);
			(void)fflush(DBG_FILE);
#endif
			run_grow();
			if ((rc = append_results(diffusion_coef, breed_coef, spread_coef, 
			road_gravity, NEI_RQMT, NEI_HOOD)) != 0) {
#ifdef DEBUG
				(void)fprintf(DBG_FILE, "(%s %d) Warning: grow() failed\n", __FILE__, __LINE__);
				(void)fflush(DBG_FILE);
#endif
				continue;
			}
		}
		if (PASS_LIST_P == NULL) {
#ifdef DEBUG
		
			(void)fprintf(DBG_FILE, "(%s %d) PASS is NULL, no data on entire control run\n",
			__FILE__, __LINE__);
			(void)fflush(DBG_FILE);
#endif
			continue;
		}
		dump_results();
		sum_data();
		calc_mean();
		calc_sdiffsqr();
		calc_variance_data();
		write_class_stats();
		write_land_stats();
		free_data();

	} /* end road gravity */
	} /* end spread loop */
	} /* end breed loop */
	} /* end diffusion loop */

	} /* end neighborhood */
	} /* end neighbor rqmt */
	return(0);
}

/********************************************************************
********************************************************************/
int
append_results(dc, bc, sc, rg, nr, nh)
double	dc, bc, sc, rg;
int	nr, nh;
{
	struct	PASS_LIST	*new_pass_elt;
	FILE	*fp_grow;
	char	fname[MAX_LEN],	line[MAX_LEN];
	int	ticktock;
	float	growth_rate, percent_road, pop, edges, clusters,
		average_slope, percent_urban, xmean, ymean, rad,
		d_coef, b_coef, s_coef, slp_res, road_grav, sng,
		sdg, sdc, og, rt, num_growth_pix, leesallee;
	float	mwfit_val;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) append results(%2.2f,%2.2f,%2.2f,%2.2f,%d,%d)\n",
	__FILE__, __LINE__, dc, bc, sc, rg, nr, nh);
	(void)fflush(DBG_FILE);
#endif
	/*
	 * This section will get the leesallee
	 */

	strcpy(fname, LOG_DIR);
	strcat(fname, "/grow1995.log");

	if ((fp_grow = fopen(fname, "r")) == NULL) {
		(void)fprintf(DBG_FILE, "grow1995.log NOT found\n");
		return(1);
	}
	new_pass_elt = (struct PASS_LIST *)calloc(1, sizeof(struct PASS_LIST));
	new_pass_elt->next_p = NULL;
	new_pass_elt->prev_p = NULL;
	new_pass_elt->gene[0] = dc;
	new_pass_elt->gene[1] = bc;
	new_pass_elt->gene[2] = sc;
	new_pass_elt->gene[3] = rg;
	new_pass_elt->gene[4] = (float)nr;
	new_pass_elt->gene[5] = (float)nh;
	new_pass_elt->leesallee = 0;
	new_pass_elt->mwfit = 0;
	init_stats(new_pass_elt);

	while ( fgetline(line, fp_grow) >= 0 ) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) LINE: %s\n", 
		__FILE__, __LINE__, line);
		(void)fflush(DBG_FILE);
#endif
		if (line[0] == '#' || line[0] == '\0')
			continue;
    		sscanf(line, "%d %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f", 
		&ticktock, &growth_rate, &percent_road, &pop, &edges, &clusters, &average_slope, 
		&percent_urban, &xmean, &ymean, &rad, &d_coef, &b_coef, 
		&s_coef, &slp_res, &road_grav, &sng, &sdg, &sdc, &og, 
		&rt, &num_growth_pix, &leesallee);
		break;
	}
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d)VARIABLE VALUES:\n", __FILE__, __LINE__);
	(void)fprintf(DBG_FILE, "%d %8.5f %8.5f %8f %8f %8f %8.5f %8.5f %8.5f %8.5f %8.5f %3.4f %3.4f %3.4f %3.4f %3.4f %6f %6f %6f %6f %6f %6f %6.4f\n",
	ticktock, growth_rate, percent_road, pop, edges, clusters, average_slope, percent_urban,
	xmean, ymean, rad, d_coef, b_coef, s_coef, slp_res, road_grav, sng, sdg, sdc, og, rt, 
	num_growth_pix, leesallee);
	(void)fflush(DBG_FILE);
#endif
	new_pass_elt->leesallee = leesallee;
	fclose(fp_grow);

	best_fit_mw(&mwfit_val);
	new_pass_elt->mwfit = mwfit_val;

	read_class_data(new_pass_elt);
	read_land_data(new_pass_elt);

	append_list(&new_pass_elt);

	return(0);
}

/********************************************************************/
/********************************************************************/
append_list(elt)
struct PASS_LIST	**elt;
{
	if (PASS_LIST_P == NULL) {
		/* empty list */
		PASS_LIST_P = *elt;
	} else {
		TMP_PASS_LIST_P = PASS_LIST_P;
		PASS_LIST_P = *elt;
		PASS_LIST_P->next_p = TMP_PASS_LIST_P;
		TMP_PASS_LIST_P->prev_p = PASS_LIST_P;
		TMP_PASS_LIST_P = NULL;
	}
	return(0);
}

/********************************************************************
* read the class data file
********************************************************************/
read_class_data(pass_elt)
struct PASS_LIST *pass_elt;
{
	FILE	*fp_class;
	char	data_line[MAX_LEN], tmp[MAX_LEN];
	char	LID[MAX_LEN], CWED[MAX_LEN], TECI[MAX_LEN], MECI[MAX_LEN], AWMECI[MAX_LEN], MPI[MAX_LEN];
	float	CA, TA, LAND, LPI, NP, PD, MPS, PSSD, PSCV, TE, ED, 
			LSI, MSI, AWMSI, DLFD, MPFD, AWMPFD, CLAND, TCA, NCA, CAD, MCA1, CASD1,
			CACV1, MCA2, CASD2, CACV2, TCAI, MCAI, MNN, NNSD, NNCV, IJI;
	int		TYPE;

	strcpy(tmp, LOG_DIR);
	strcat(tmp, "/M1995.class");
	if ((fp_class = fopen(tmp, "r")) == NULL) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) %s NOT found\n", __FILE__, __LINE__, tmp);
		(void)fflush(DBG_FILE);
#endif
		return(1);
	}
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) print contents of data_file\n", 
	__FILE__, __LINE__);
#endif
	while ( fgetline(data_line, fp_class) >= 0 ) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "%s\n", data_line);
		(void)fflush(DBG_FILE);
#endif
		if (strncmp(data_line, "         LID", 12) == 0)
			continue;
		/*
	 	 * parse the data line into variables and add them to summary stats
	 	 */
    		sscanf(data_line, "%s %d %f %f %f %f %f %f %f %f %f %f %f %s %s %s %s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %s %f",
		LID, &TYPE, &CA, &TA, &LAND, &LPI, &NP, &PD, &MPS, &PSSD, &PSCV, &TE, &ED, 
		CWED, TECI, MECI, AWMECI, &LSI, &MSI, &AWMSI, &DLFD, &MPFD, &AWMPFD,
		&CLAND, &TCA, &NCA, &CAD, &MCA1, &CASD1, &CACV1, &MCA2, &CASD2, &CACV2, &TCAI,
		&MCAI, &MNN, &NNSD, &NNCV, MPI, &IJI);

		/* look for urban class and add the data to the structure */
		if (TYPE == 4) {
			append_class_data( pass_elt, TYPE, CA, TA, LAND, LPI, NP, PD, MPS, PSSD, 
			PSCV, TE, ED, LSI, MSI, AWMSI, DLFD, MPFD, AWMPFD, CLAND, TCA,
			NCA, CAD, MCA1, CASD1, CACV1, MCA2, CASD2, CACV2, TCAI, MCAI,
			MNN, NNSD, NNCV, IJI);
		}
	}
	fclose(fp_class);
}
/********************************************************************
* insert the class data
********************************************************************/
append_class_data( pass_elt, TYPE, CA, TA, LAND, LPI, NP, PD, MPS, PSSD, 
PSCV, TE, ED, LSI, MSI, AWMSI, DLFD, MPFD, AWMPFD, CLAND, TCA,
NCA, CAD, MCA1, CASD1, CACV1, MCA2, CASD2, CACV2, TCAI, MCAI,
MNN, NNSD, NNCV, IJI)
struct PASS_LIST *pass_elt;
int 	TYPE;
float	CA, TA, LAND, LPI, NP, PD, MPS, PSSD, 
PSCV, TE, ED, LSI, MSI, AWMSI, DLFD, MPFD, AWMPFD, CLAND, TCA,
NCA, CAD, MCA1, CASD1, CACV1, MCA2, CASD2, CACV2, TCAI, MCAI,
MNN, NNSD, NNCV, IJI;
{
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "\n");
	(void)fprintf(DBG_FILE, "(%s %d) dump class values to be added\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
	(void)fprintf(DBG_FILE, 
	"TYPE, CA, TA,
	LAND, LPI, NP, PD,
	MPS, PSSD, PSCV, TE,
	ED, LSI, MSI, AWMSI,
	DLFD, MPFD, AWMPFD, CLAND,
	TCA, NCA, CAD, MCA1,
	CASD1, CACV1, MCA2, CASD2,
	CACV2, TCAI, MCAI, MNN,
	NNSD, NNCV, IJI \n");

	(void)fprintf(DBG_FILE, "%d	%8.5f	%8.5f
	%8.5f	%8.5f	%8.5f	%8.5f
	%8.5f	%8.5f	%8.5f	%8.5f
	%8.5f	%8.5f	%8.5f	%8.5f
	%8.5f	%8.5f	%8.5f	%8.5f
	%8.5f	%8.5f	%8.5f	%8.5f
	%8.5f	%8.5f	%8.5f	%8.5f
	%8.5f	%8.5f	%8.5f	%8.5f
	%8.5f	%8.5f	%8.5f\n",
	TYPE, CA, TA, LAND, LPI, NP, PD, MPS, PSSD, PSCV, TE, ED, LSI, MSI, AWMSI,
	DLFD, MPFD, AWMPFD, CLAND, TCA, NCA, CAD, MCA1, CASD1, CACV1, MCA2, CASD2,
	CACV2, TCAI, MCAI, MNN, NNSD, NNCV, IJI);
	fflush(DBG_FILE);
#endif
	pass_elt->class_urban_res.CA_VAL = CA;
	pass_elt->class_urban_res.TA_VAL = TA;
	pass_elt->class_urban_res.LAND_VAL = LAND;
	pass_elt->class_urban_res.LPI_VAL = LPI;
	pass_elt->class_urban_res.NP_VAL = NP;
	pass_elt->class_urban_res.PD_VAL = PD;
	pass_elt->class_urban_res.MPS_VAL = MPS;
	pass_elt->class_urban_res.PSSD_VAL = PSSD;
	pass_elt->class_urban_res.PSCV_VAL = PSCV;
	pass_elt->class_urban_res.TE_VAL = TE;
	pass_elt->class_urban_res.ED_VAL = ED;
	pass_elt->class_urban_res.LSI_VAL = LSI;
	pass_elt->class_urban_res.MSI_VAL = MSI;
	pass_elt->class_urban_res.AWMSI_VAL = AWMSI;
	pass_elt->class_urban_res.DLFD_VAL = DLFD;
	pass_elt->class_urban_res.MPFD_VAL = MPFD;
	pass_elt->class_urban_res.AWMPFD_VAL = AWMPFD;
	pass_elt->class_urban_res.CLAND_VAL = CLAND;
	pass_elt->class_urban_res.TCA_VAL = TCA;
	pass_elt->class_urban_res.NCA_VAL = NCA;
	pass_elt->class_urban_res.CAD_VAL = CAD;
	pass_elt->class_urban_res.MCA1_VAL = MCA1;
	pass_elt->class_urban_res.CASD1_VAL = CASD1;
	pass_elt->class_urban_res.CACV1_VAL = CACV1;
	pass_elt->class_urban_res.MCA2_VAL = MCA2;
	pass_elt->class_urban_res.CASD2_VAL = CASD2;
	pass_elt->class_urban_res.CACV2_VAL = CACV2;
	pass_elt->class_urban_res.TCAI_VAL = TCAI;
	pass_elt->class_urban_res.MCAI_VAL = MCAI;
	pass_elt->class_urban_res.MNN_VAL = MNN;
	pass_elt->class_urban_res.NNSD_VAL = NNSD;
	pass_elt->class_urban_res.NNCV_VAL = NNCV;
	pass_elt->class_urban_res.IJI_VAL = IJI;
	return(0);
}
/********************************************************************
* increment the mean component fields by new values and up the count
********************************************************************/
append_land_data( pass_elt, TA, LPI, NP, PD, MPS, PSSD, PSCV, TE, ED, LSI,
MSI, AWMSI, DFLD, MPFD, AWMPFD, TCA, NCA, CAD, MCA1, CASD1, CACV1,
MCA2, CASD2, CACV2, TCAI, MCAI, MNN, NNSD, NNCV, SHDI, SIDI, MSIDI,
PR, PRD, RPR, SHEI, SIEI, MSIEI, IJI, CONTAG)

struct PASS_LIST *pass_elt;
float	TA, LPI, NP, PD, MPS, PSSD, PSCV, TE, ED, LSI, MSI,
AWMSI, DFLD, MPFD, AWMPFD, TCA, NCA, CAD, MCA1, CASD1,
CACV1, MCA2, CASD2, CACV2, TCAI, MCAI, MNN, NNSD, NNCV,
SHDI, SIDI, MSIDI, PR, PRD, RPR, SHEI, SIEI, MSIEI, IJI, CONTAG;
{
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "\n");
	(void)fprintf(DBG_FILE, "(%s %d) dump land values to be added\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
	(void)fprintf(DBG_FILE, "	TA, LPI, NP, PD, MPS,
	PSSD, PSCV, TE, ED, LSI,
	MSI, AWMSI, DFLD, MPFD, AWMPFD,
	TCA, NCA, CAD, MCA1, CASD1,
	CACV1, MCA2, CASD2, CACV2, TCAI,
	MCAI, MNN, NNSD, NNCV, SHDI,
	SIDI, MSIDI, PR, PRD, RPR,
	SHEI, SIEI, MSIEI, IJI, CONTAG\n");
	(void)fprintf(DBG_FILE, "%8.5f %8.5f %8.5f %8.5f %8.5f
	%8.5f %8.5f %8.5f %8.5f %8.5f
	%8.5f %8.5f %8.5f %8.5f %8.5f
	%8.5f %8.5f %8.5f %8.5f %8.5f
	%8.5f %8.5f %8.5f %8.5f %8.5f
	%8.5f %8.5f %8.5f %8.5f %8.5f
	%8.5f %8.5f %8.5f %8.5f %8.5f
	%8.5f %8.5f %8.5f %8.5f %8.5f\n",
	TA, LPI, NP, PD, MPS, PSSD, PSCV, TE, ED, LSI, MSI, AWMSI, DFLD, MPFD, AWMPFD,
	TCA, NCA, CAD, MCA1, CASD1, CACV1, MCA2, CASD2, CACV2, TCAI, MCAI, MNN, NNSD,
	NNCV, SHDI, SIDI, MSIDI, PR, PRD, RPR, SHEI, SIEI, MSIEI, IJI, CONTAG);
	fflush(DBG_FILE);
#endif

	pass_elt->land_res.TA_VAL = TA;
	pass_elt->land_res.LPI_VAL = LPI;
	pass_elt->land_res.NP_VAL = NP;
	pass_elt->land_res.PD_VAL = PD;
	pass_elt->land_res.MPS_VAL = MPS;
	pass_elt->land_res.PSSD_VAL = PSSD;
	pass_elt->land_res.PSCV_VAL = PSCV;
	pass_elt->land_res.TE_VAL = TE;
	pass_elt->land_res.ED_VAL = ED;
	pass_elt->land_res.LSI_VAL = LSI;
	pass_elt->land_res.MSI_VAL = MSI;
	pass_elt->land_res.AWMSI_VAL = AWMSI;
	pass_elt->land_res.DFLD_VAL = DFLD;
	pass_elt->land_res.MPFD_VAL = MPFD;
	pass_elt->land_res.AWMPFD_VAL = AWMPFD;
	pass_elt->land_res.TCA_VAL = TCA;
	pass_elt->land_res.NCA_VAL = NCA;
	pass_elt->land_res.CAD_VAL = CAD;
	pass_elt->land_res.MCA1_VAL = MCA1;
	pass_elt->land_res.CASD1_VAL = CASD1;
	pass_elt->land_res.CACV1_VAL = CACV1;
	pass_elt->land_res.MCA2_VAL = MCA2;
	pass_elt->land_res.CASD2_VAL = CASD2;
	pass_elt->land_res.CACV2_VAL = CACV2;
	pass_elt->land_res.TCAI_VAL = TCAI;
	pass_elt->land_res.MCAI_VAL = MCAI;
	pass_elt->land_res.MNN_VAL = MNN;
	pass_elt->land_res.NNSD_VAL = NNSD;
	pass_elt->land_res.NNCV_VAL = NNCV;
	pass_elt->land_res.SHDI_VAL = SHDI;
	pass_elt->land_res.SIDI_VAL = SIDI;
	pass_elt->land_res.MSIDI_VAL = MSIDI;
	pass_elt->land_res.PR_VAL = PR;
	pass_elt->land_res.PRD_VAL = PRD;
	pass_elt->land_res.RPR_VAL = RPR;
	pass_elt->land_res.SHEI_VAL = SHEI;
	pass_elt->land_res.SIEI_VAL = SIEI;
	pass_elt->land_res.MSIEI_VAL = MSIEI;
	pass_elt->land_res.IJI_VAL = IJI;
	pass_elt->land_res.CONTAG_VAL = CONTAG;
	return(0);
}
/********************************************************************
* read the data files and calculate the means
********************************************************************/
read_land_data(pass_elt)
struct PASS_LIST *pass_elt;
{
	FILE	*fp_land;
	char	data_line[MAX_LEN], tmp[MAX_LEN];
	char	LID[MAX_LEN], CWED[MAX_LEN], TECI[MAX_LEN], MECI[MAX_LEN], AWMECI[MAX_LEN], MPI[MAX_LEN];
	float	TA, LPI, NP, PD, MPS, PSSD, PSCV, TE, ED, LSI, MSI,
		AWMSI, DFLD, MPFD, AWMPFD, TCA, NCA, CAD, MCA1, CASD1,
		CACV1, MCA2, CASD2, CACV2, TCAI, MCAI, MNN, NNSD, NNCV,
		SHDI, SIDI, MSIDI, PR, PRD, RPR, SHEI, SIEI, MSIEI, IJI, CONTAG;

	strcpy(tmp, LOG_DIR);
	strcat(tmp, "/M1995.land");
	if ((fp_land = fopen(tmp, "r")) == NULL) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) %s NOT found\n", 
		__FILE__, __LINE__, tmp);
		(void)fflush(DBG_FILE);
		return(1);
#endif
	}
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) print contents of data_file\n", 
	__FILE__, __LINE__);
#endif
	while ( fgetline(data_line, fp_land) >= 0 ) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "%s\n", data_line);
		(void)fflush(DBG_FILE);
#endif
		if (strncmp(data_line, "        LID", 11) == 0)
			continue;
		/*
	 	 * parse the data line into variables 
	 	 */
		sscanf(data_line, "%s %f %f %f %f %f %f %f %f %f %s %s %s %s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %s %f %f %f %f %f %f %f %f %f %f %f",
		LID, &TA, &LPI, &NP, &PD, &MPS, &PSSD, &PSCV, &TE,
		&ED, CWED, TECI, MECI, AWMECI, &LSI, &MSI, &AWMSI, &DFLD, &MPFD, 
		&AWMPFD, &TCA, &NCA, &CAD, &MCA1, &CASD1, &CACV1, &MCA2, &CASD2, 
		&CACV2, &TCAI, &MCAI, &MNN, &NNSD, &NNCV, MPI, &SHDI, &SIDI,
		&MSIDI, &PR, &PRD, &RPR, &SHEI, &SIEI, &MSIEI, &IJI, &CONTAG);

		append_land_data( pass_elt, TA, LPI, NP, PD, MPS, PSSD, PSCV, TE,
		ED, LSI, MSI, AWMSI, DFLD, MPFD, AWMPFD, TCA, NCA, CAD, MCA1,
		CASD1, CACV1, MCA2, CASD2, CACV2, TCAI, MCAI, MNN, NNSD, NNCV,
		SHDI, SIDI, MSIDI, PR, PRD, RPR, SHEI, SIEI, MSIEI, IJI, CONTAG);
	}
	fclose(fp_land);
	return(0);
}
/********************************************************************
********************************************************************/
write_control(pathname)
char	*pathname;
{	
	FILE	*fp;
	char	fname[MAX_LEN];

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Open %s \n", __FILE__, __LINE__, pathname);
	(void)fflush(DBG_FILE);
#endif
	fp = fopen(pathname, "w");
	fprintf(fp, "%d start_date\n", start_date);
	fprintf(fp, "%d stop_date\n", stop_date);
	fprintf(fp, "%2.2f diffusion_coefficient\n", diffusion_coef);
	fprintf(fp, "%2.2f breed_coefficient\n", breed_coef);
	fprintf(fp, "%2.2f spread_coefficient\n", spread_coef);
	fprintf(fp, "%2.2f slope_resistance\n", slope_resistance);
	fprintf(fp, "%2.2f road_gravity\n", road_gravity);
	fprintf(fp, "%2.2f critical_high\n", critical_high);
	fprintf(fp, "%.2f critical_low\n", critical_low);
	fprintf(fp, "%.2f boom\n", boom);
	fprintf(fp, "%.2f bust\n", bust);
	fprintf(fp, "%d critical_slope\n", critical_slope);
	fprintf(fp, "%d number_of_times\n", number_of_times);
	fprintf(fp, "%d cumulate\n", cumulate);
	fclose(fp);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) CONTROL\n", __FILE__, __LINE__);
	fprintf(DBG_FILE, "%d start_date\n", start_date);
	fprintf(DBG_FILE, "%d stop_date\n", stop_date);
	fprintf(DBG_FILE, "%2.2f diffusion_coefficient\n", diffusion_coef);
	fprintf(DBG_FILE, "%2.2f breed_coefficient\n", breed_coef);
	fprintf(DBG_FILE, "%2.2f spread_coefficient\n", spread_coef);
	fprintf(DBG_FILE, "%2.2f slope_resistance\n", slope_resistance);
	fprintf(DBG_FILE, "%2.2f road_gravity\n", road_gravity);
	fprintf(DBG_FILE, "%2.2f critical_high\n", critical_high);
	fprintf(DBG_FILE, "%.2f critical_low\n", critical_low);
	fprintf(DBG_FILE, "%.2f boom\n", boom);
	fprintf(DBG_FILE, "%.2f bust\n", bust);
	fprintf(DBG_FILE, "%d critical_slope\n", critical_slope);
	fprintf(DBG_FILE, "%d number_of_times\n", number_of_times);
	fprintf(DBG_FILE, "%d cumulate\n", cumulate);
	(void)fflush(DBG_FILE);
#endif
	return(0);
}

/********************************************************************
********************************************************************/
run_grow()
{
	char	cmd[MAX_LEN], temp[MAX_LEN];

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter run_grow()\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	strcpy(cmd, "cd ");
	strcat(cmd, RUN_DIR);
	strcat(cmd, "; ./run.");
	strcat(cmd, SFX);
	strcat(cmd, ".");
	sprintf(temp, "%d", start_date);
	strcat(cmd, temp);
	sprintf(temp, " -r%d", NEI_RQMT);
	strcat(cmd, temp);
	sprintf(temp, " -h%d", NEI_HOOD);
	strcat(cmd, temp);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Run: >%s<\n", __FILE__, __LINE__, cmd);
	(void)fflush(DBG_FILE);
#endif
	system(cmd);
	return(0);
}

/*************************************************************
*************************************************************/
/* fgetline: get a line from a file
 * Assumes the line array is long enough for the input and that the file
 * is open.
 */
int fgetline(line, fp)
char 	*line;
FILE	*fp;
{

	char	c;
	int	count = 0;

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter fgetline()\n", __FILE__, __LINE__);
#endif
	while (c = getc(fp))
	{
		*line = (char)c;
		if (*line == EOL || *line == EOF) break ;
		++line ;
		++count;
	}
	*line = '\0';
	/* return the length of the line */
	if (c == EOF) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) return -1 \n", __FILE__, __LINE__);
#endif
		return(-1);
	} else {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) return %d\n", 
		__FILE__, __LINE__, count);
#endif
		return(count);
	}
}
/********************************************************************
********************************************************************/
int
fatal_error(filename, line, msg)
char	*filename, *msg;
int		line;
{
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) ERROR: %s\n", filename, line, msg);
	(void)fflush(DBG_FILE);
#endif
	exit(1);
}
/********************************************************************
********************************************************************/
dump_results(pass_elt)
{

	if (PASS_LIST_P == NULL) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) PASS_LIST_P is EMPTY\n", __FILE__, __LINE__);
		fflush(DBG_FILE);
#endif
		return(0);
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "\n(%s %d) The resulting results structure\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	TMP_PASS_LIST_P = PASS_LIST_P;
	while (TMP_PASS_LIST_P != NULL) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "GENE: %2.2fd,%2.2fb,%2.2fs,%2.2fr,%2.2fnr,%2.2fnh\n",
		TMP_PASS_LIST_P->gene[0], TMP_PASS_LIST_P->gene[1],
		TMP_PASS_LIST_P->gene[2], TMP_PASS_LIST_P->gene[3],
		TMP_PASS_LIST_P->gene[4], TMP_PASS_LIST_P->gene[5]);
		(void)fprintf(DBG_FILE, "	LEESALLEE: %2.5f\n",TMP_PASS_LIST_P->leesallee);
		(void)fprintf(DBG_FILE, "	MW BEST FIT: %6.5f\n",TMP_PASS_LIST_P->mwfit);
		(void)fprintf(DBG_FILE, "	CLASS CA: %2.5f\n",TMP_PASS_LIST_P->class_urban_res.CA_VAL);
		(void)fprintf(DBG_FILE, "	CLASS TA: %2.5f\n",TMP_PASS_LIST_P->class_urban_res.TA_VAL);
		(void)fprintf(DBG_FILE, "	CLASS LAND: %2.5f\n",TMP_PASS_LIST_P->class_urban_res.LAND_VAL);
		(void)fprintf(DBG_FILE, "	LAND TA: %2.5f\n",TMP_PASS_LIST_P->land_res.TA_VAL);
		(void)fprintf(DBG_FILE, "	LAND LPI: %2.5f\n",TMP_PASS_LIST_P->land_res.LPI_VAL);
		(void)fprintf(DBG_FILE, "	LAND NP: %2.5f\n",TMP_PASS_LIST_P->land_res.NP_VAL);
		fflush(DBG_FILE);
#endif
		TMP_PASS_LIST_P = TMP_PASS_LIST_P->next_p;
	}
	return(0);
}
/********************************************************************
********************************************************************/
free_data()
{
	if (PASS_LIST_P == NULL) 
		return(0);

	TMP_PASS_LIST_P = PASS_LIST_P;
	while (TMP_PASS_LIST_P != NULL) {
		PASS_LIST_P = PASS_LIST_P->next_p;
		free(TMP_PASS_LIST_P);
		TMP_PASS_LIST_P = PASS_LIST_P;
	}
	return(0);
}
/********************************************************************
********************************************************************/
sum_data()
{

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter sum_data()\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	if (PASS_LIST_P == NULL)  {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Warning: No Class Data\n", __FILE__, __LINE__);
		fflush(DBG_FILE);
#endif
		return(0);
	}

	TMP_PASS_LIST_P = PASS_LIST_P;
	while (TMP_PASS_LIST_P != NULL) {
		/* fill class stuff here */
		CLASS_DATA.CA_SUM += TMP_PASS_LIST_P->class_urban_res.CA_VAL;
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) CLASS CA add %8.5f (tot: %8.5f)\n",
	 	__FILE__, __LINE__, TMP_PASS_LIST_P->class_urban_res.CA_VAL, CLASS_DATA.CA_SUM);
		fflush(DBG_FILE);
#endif
		CLASS_DATA.TA_SUM += TMP_PASS_LIST_P->class_urban_res.TA_VAL;
		CLASS_DATA.LAND_SUM += TMP_PASS_LIST_P->class_urban_res.LAND_VAL;
		CLASS_DATA.LPI_SUM += TMP_PASS_LIST_P->class_urban_res.LPI_VAL;
		CLASS_DATA.NP_SUM += TMP_PASS_LIST_P->class_urban_res.NP_VAL;
		CLASS_DATA.PD_SUM += TMP_PASS_LIST_P->class_urban_res.PD_VAL;
		CLASS_DATA.MPS_SUM += TMP_PASS_LIST_P->class_urban_res.MPS_VAL;
		CLASS_DATA.PSSD_SUM += TMP_PASS_LIST_P->class_urban_res.PSSD_VAL;
		CLASS_DATA.PSCV_SUM += TMP_PASS_LIST_P->class_urban_res.PSCV_VAL;
		CLASS_DATA.TE_SUM += TMP_PASS_LIST_P->class_urban_res.TE_VAL;
		CLASS_DATA.ED_SUM += TMP_PASS_LIST_P->class_urban_res.ED_VAL;
		CLASS_DATA.LSI_SUM += TMP_PASS_LIST_P->class_urban_res.LSI_VAL;
		CLASS_DATA.MSI_SUM += TMP_PASS_LIST_P->class_urban_res.MSI_VAL;
		CLASS_DATA.AWMSI_SUM += TMP_PASS_LIST_P->class_urban_res.AWMSI_VAL;
		CLASS_DATA.DLFD_SUM += TMP_PASS_LIST_P->class_urban_res.DLFD_VAL;
		CLASS_DATA.MPFD_SUM += TMP_PASS_LIST_P->class_urban_res.MPFD_VAL;
		CLASS_DATA.AWMPFD_SUM += TMP_PASS_LIST_P->class_urban_res.AWMPFD_VAL;
		CLASS_DATA.CLAND_SUM += TMP_PASS_LIST_P->class_urban_res.CLAND_VAL;
		CLASS_DATA.TCA_SUM += TMP_PASS_LIST_P->class_urban_res.TCA_VAL;
		CLASS_DATA.NCA_SUM += TMP_PASS_LIST_P->class_urban_res.NCA_VAL;
		CLASS_DATA.CAD_SUM += TMP_PASS_LIST_P->class_urban_res.CAD_VAL;
		CLASS_DATA.MCA1_SUM += TMP_PASS_LIST_P->class_urban_res.MCA1_VAL;
		CLASS_DATA.CASD1_SUM += TMP_PASS_LIST_P->class_urban_res.CASD1_VAL;
		CLASS_DATA.CACV1_SUM += TMP_PASS_LIST_P->class_urban_res.CACV1_VAL;
		CLASS_DATA.MCA2_SUM += TMP_PASS_LIST_P->class_urban_res.MCA2_VAL;
		CLASS_DATA.CASD2_SUM += TMP_PASS_LIST_P->class_urban_res.CASD2_VAL;
		CLASS_DATA.CACV2_SUM += TMP_PASS_LIST_P->class_urban_res.CACV2_VAL;
		CLASS_DATA.TCAI_SUM += TMP_PASS_LIST_P->class_urban_res.TCAI_VAL;
		CLASS_DATA.MCAI_SUM += TMP_PASS_LIST_P->class_urban_res.MCAI_VAL;
		CLASS_DATA.MNN_SUM += TMP_PASS_LIST_P->class_urban_res.MNN_VAL;
		CLASS_DATA.NNSD_SUM += TMP_PASS_LIST_P->class_urban_res.NNSD_VAL;
		CLASS_DATA.NNCV_SUM += TMP_PASS_LIST_P->class_urban_res.NNCV_VAL;
		CLASS_DATA.IJI_SUM += TMP_PASS_LIST_P->class_urban_res.IJI_VAL;
		CLASS_DATA.COUNT++;

		/* fill land stuff here */
		LAND_DATA.LEE_SUM += TMP_PASS_LIST_P->leesallee;
		LAND_DATA.MWFIT_SUM += TMP_PASS_LIST_P->mwfit;

		LAND_DATA.TA_SUM += TMP_PASS_LIST_P->land_res.TA_VAL;
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) LAND TA add %8.5f (tot: %8.5f)\n",
	 	__FILE__, __LINE__, TMP_PASS_LIST_P->land_res.TA_VAL, LAND_DATA.TA_SUM);
		fflush(DBG_FILE);
#endif
		LAND_DATA.LPI_SUM += TMP_PASS_LIST_P->land_res.LPI_VAL;
		LAND_DATA.NP_SUM += TMP_PASS_LIST_P->land_res.NP_VAL;
		LAND_DATA.PD_SUM += TMP_PASS_LIST_P->land_res.PD_VAL;
		LAND_DATA.MPS_SUM += TMP_PASS_LIST_P->land_res.MPS_VAL;
		LAND_DATA.PSSD_SUM += TMP_PASS_LIST_P->land_res.PSSD_VAL;
		LAND_DATA.PSCV_SUM += TMP_PASS_LIST_P->land_res.PSCV_VAL;
		LAND_DATA.TE_SUM += TMP_PASS_LIST_P->land_res.TE_VAL;
		LAND_DATA.ED_SUM += TMP_PASS_LIST_P->land_res.ED_VAL;
		LAND_DATA.LSI_SUM += TMP_PASS_LIST_P->land_res.LSI_VAL;
		LAND_DATA.MSI_SUM += TMP_PASS_LIST_P->land_res.MSI_VAL;
		LAND_DATA.AWMSI_SUM += TMP_PASS_LIST_P->land_res.AWMSI_VAL;
		LAND_DATA.DFLD_SUM += TMP_PASS_LIST_P->land_res.DFLD_VAL;
		LAND_DATA.MPFD_SUM += TMP_PASS_LIST_P->land_res.MPFD_VAL;
		LAND_DATA.AWMPFD_SUM += TMP_PASS_LIST_P->land_res.AWMPFD_VAL;
		LAND_DATA.TCA_SUM += TMP_PASS_LIST_P->land_res.TCA_VAL;
		LAND_DATA.NCA_SUM += TMP_PASS_LIST_P->land_res.NCA_VAL;
		LAND_DATA.CAD_SUM += TMP_PASS_LIST_P->land_res.CAD_VAL;
		LAND_DATA.MCA1_SUM += TMP_PASS_LIST_P->land_res.MCA1_VAL;
		LAND_DATA.CASD1_SUM += TMP_PASS_LIST_P->land_res.CASD1_VAL;
		LAND_DATA.CACV1_SUM += TMP_PASS_LIST_P->land_res.CACV1_VAL;
		LAND_DATA.MCA2_SUM += TMP_PASS_LIST_P->land_res.MCA2_VAL;
		LAND_DATA.CASD2_SUM += TMP_PASS_LIST_P->land_res.CASD2_VAL;
		LAND_DATA.CACV2_SUM += TMP_PASS_LIST_P->land_res.CACV2_VAL;
		LAND_DATA.TCAI_SUM += TMP_PASS_LIST_P->land_res.TCAI_VAL;
		LAND_DATA.MCAI_SUM += TMP_PASS_LIST_P->land_res.MCAI_VAL;
		LAND_DATA.MNN_SUM += TMP_PASS_LIST_P->land_res.MNN_VAL;
		LAND_DATA.NNSD_SUM += TMP_PASS_LIST_P->land_res.NNSD_VAL;
		LAND_DATA.NNCV_SUM += TMP_PASS_LIST_P->land_res.NNCV_VAL;
		LAND_DATA.SHDI_SUM += TMP_PASS_LIST_P->land_res.SHDI_VAL;
		LAND_DATA.SIDI_SUM += TMP_PASS_LIST_P->land_res.SIDI_VAL;
		LAND_DATA.MSIDI_SUM += TMP_PASS_LIST_P->land_res.MSIDI_VAL;
		LAND_DATA.PR_SUM += TMP_PASS_LIST_P->land_res.PR_VAL;
		LAND_DATA.PRD_SUM += TMP_PASS_LIST_P->land_res.PRD_VAL;
		LAND_DATA.RPR_SUM += TMP_PASS_LIST_P->land_res.RPR_VAL;
		LAND_DATA.SHEI_SUM += TMP_PASS_LIST_P->land_res.SHEI_VAL;
		LAND_DATA.SIEI_SUM += TMP_PASS_LIST_P->land_res.SIEI_VAL;
		LAND_DATA.MSIEI_SUM += TMP_PASS_LIST_P->land_res.MSIEI_VAL;
		LAND_DATA.IJI_SUM += TMP_PASS_LIST_P->land_res.IJI_VAL;
		LAND_DATA.CONTAG_SUM += TMP_PASS_LIST_P->land_res.CONTAG_VAL;
		LAND_DATA.COUNT++;

		TMP_PASS_LIST_P = TMP_PASS_LIST_P->next_p;
	}
	return(0);
}
/********************************************************************
********************************************************************/
calc_mean()
{

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter calc_mean_data()\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	CLASS_DATA.CA_MEAN = CLASS_DATA.CA_SUM /CLASS_DATA.COUNT;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) CLASS CA mean %8.5f (%8.5f/%d)\n",
 	__FILE__, __LINE__, CLASS_DATA.CA_MEAN, CLASS_DATA.CA_SUM, CLASS_DATA.COUNT);
	fflush(DBG_FILE);
#endif
	CLASS_DATA.TA_MEAN = CLASS_DATA.TA_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.LAND_MEAN = CLASS_DATA.LAND_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.LPI_MEAN = CLASS_DATA.LPI_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.NP_MEAN = CLASS_DATA.NP_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.PD_MEAN = CLASS_DATA.PD_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.MPS_MEAN = CLASS_DATA.MPS_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.PSSD_MEAN = CLASS_DATA.PSSD_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.PSCV_MEAN = CLASS_DATA.PSCV_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.TE_MEAN = CLASS_DATA.TE_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.ED_MEAN = CLASS_DATA.ED_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.LSI_MEAN = CLASS_DATA.LSI_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.MSI_MEAN = CLASS_DATA.MSI_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.AWMSI_MEAN = CLASS_DATA.AWMSI_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.DLFD_MEAN = CLASS_DATA.DLFD_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.MPFD_MEAN = CLASS_DATA.MPFD_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.AWMPFD_MEAN = CLASS_DATA.AWMPFD_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.CLAND_MEAN = CLASS_DATA.CLAND_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.TCA_MEAN = CLASS_DATA.TCA_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.NCA_MEAN = CLASS_DATA.NCA_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.CAD_MEAN = CLASS_DATA.CAD_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.MCA1_MEAN = CLASS_DATA.MCA1_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.CASD1_MEAN = CLASS_DATA.CASD1_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.CACV1_MEAN = CLASS_DATA.CACV1_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.MCA2_MEAN = CLASS_DATA.MCA2_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.CASD2_MEAN = CLASS_DATA.CASD2_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.CACV2_MEAN = CLASS_DATA.CACV2_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.TCAI_MEAN = CLASS_DATA.TCAI_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.MCAI_MEAN = CLASS_DATA.MCAI_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.MNN_MEAN = CLASS_DATA.MNN_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.NNSD_MEAN = CLASS_DATA.NNSD_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.NNCV_MEAN = CLASS_DATA.NNCV_SUM /CLASS_DATA.COUNT;
	CLASS_DATA.IJI_MEAN = CLASS_DATA.IJI_SUM /CLASS_DATA.COUNT;

	/* fill land stuff here */
	LAND_DATA.TA_MEAN = LAND_DATA.TA_SUM / LAND_DATA.COUNT;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) LAND TA mean %8.5f (%8.5f/%d)\n",
 	__FILE__, __LINE__, LAND_DATA.TA_MEAN, LAND_DATA.TA_SUM, LAND_DATA.COUNT);
	fflush(DBG_FILE);
#endif
	LAND_DATA.LPI_MEAN = LAND_DATA.LPI_SUM / LAND_DATA.COUNT;
	LAND_DATA.NP_MEAN = LAND_DATA.NP_SUM / LAND_DATA.COUNT;
	LAND_DATA.PD_MEAN = LAND_DATA.PD_SUM / LAND_DATA.COUNT;
	LAND_DATA.MPS_MEAN = LAND_DATA.MPS_SUM / LAND_DATA.COUNT;
	LAND_DATA.PSSD_MEAN = LAND_DATA.PSSD_SUM / LAND_DATA.COUNT;
	LAND_DATA.PSCV_MEAN = LAND_DATA.PSCV_SUM / LAND_DATA.COUNT;
	LAND_DATA.TE_MEAN = LAND_DATA.TE_SUM / LAND_DATA.COUNT;
	LAND_DATA.ED_MEAN = LAND_DATA.ED_SUM / LAND_DATA.COUNT;
	LAND_DATA.LSI_MEAN = LAND_DATA.LSI_SUM / LAND_DATA.COUNT;
	LAND_DATA.MSI_MEAN = LAND_DATA.MSI_SUM / LAND_DATA.COUNT;
	LAND_DATA.AWMSI_MEAN = LAND_DATA.AWMSI_SUM / LAND_DATA.COUNT;
	LAND_DATA.DFLD_MEAN = LAND_DATA.DFLD_SUM / LAND_DATA.COUNT;
	LAND_DATA.MPFD_MEAN = LAND_DATA.MPFD_SUM / LAND_DATA.COUNT;
	LAND_DATA.AWMPFD_MEAN = LAND_DATA.AWMPFD_SUM / LAND_DATA.COUNT;
	LAND_DATA.TCA_MEAN = LAND_DATA.TCA_SUM / LAND_DATA.COUNT;
	LAND_DATA.NCA_MEAN = LAND_DATA.NCA_SUM / LAND_DATA.COUNT;
	LAND_DATA.CAD_MEAN = LAND_DATA.CAD_SUM / LAND_DATA.COUNT;
	LAND_DATA.MCA1_MEAN = LAND_DATA.MCA1_SUM / LAND_DATA.COUNT;
	LAND_DATA.CASD1_MEAN = LAND_DATA.CASD1_SUM / LAND_DATA.COUNT;
	LAND_DATA.CACV1_MEAN = LAND_DATA.CACV1_SUM / LAND_DATA.COUNT;
	LAND_DATA.MCA2_MEAN = LAND_DATA.MCA2_SUM / LAND_DATA.COUNT;
	LAND_DATA.CASD2_MEAN = LAND_DATA.CASD2_SUM / LAND_DATA.COUNT;
	LAND_DATA.CACV2_MEAN = LAND_DATA.CACV2_SUM / LAND_DATA.COUNT;
	LAND_DATA.TCAI_MEAN = LAND_DATA.TCAI_SUM / LAND_DATA.COUNT;
	LAND_DATA.MCAI_MEAN = LAND_DATA.MCAI_SUM / LAND_DATA.COUNT;
	LAND_DATA.MNN_MEAN = LAND_DATA.MNN_SUM / LAND_DATA.COUNT;
	LAND_DATA.NNSD_MEAN = LAND_DATA.NNSD_SUM / LAND_DATA.COUNT;
	LAND_DATA.NNCV_MEAN = LAND_DATA.NNCV_SUM / LAND_DATA.COUNT;
	LAND_DATA.SHDI_MEAN = LAND_DATA.SHDI_SUM / LAND_DATA.COUNT;
	LAND_DATA.SIDI_MEAN = LAND_DATA.SIDI_SUM / LAND_DATA.COUNT;
	LAND_DATA.MSIDI_MEAN = LAND_DATA.MSIDI_SUM / LAND_DATA.COUNT;
	LAND_DATA.PR_MEAN = LAND_DATA.PR_SUM / LAND_DATA.COUNT;
	LAND_DATA.PRD_MEAN = LAND_DATA.PRD_SUM / LAND_DATA.COUNT;
	LAND_DATA.RPR_MEAN = LAND_DATA.RPR_SUM / LAND_DATA.COUNT;
	LAND_DATA.SHEI_MEAN = LAND_DATA.SHEI_SUM / LAND_DATA.COUNT;
	LAND_DATA.SIEI_MEAN = LAND_DATA.SIEI_SUM / LAND_DATA.COUNT;
	LAND_DATA.MSIEI_MEAN = LAND_DATA.MSIEI_SUM / LAND_DATA.COUNT;
	LAND_DATA.IJI_MEAN = LAND_DATA.IJI_SUM / LAND_DATA.COUNT;
	LAND_DATA.CONTAG_MEAN = LAND_DATA.CONTAG_SUM / LAND_DATA.COUNT;
	return(0);
}

/********************************************************************
* initialize the data arrays that will be used to accumulate the sums.
********************************************************************/
init_class_array()
{
	int	TYPE;

	CLASS_DATA.COUNT = 0;
	CLASS_DATA.CA_SUM = 0;
	CLASS_DATA.TA_SUM = 0;
	CLASS_DATA.LAND_SUM = 0;
	CLASS_DATA.LPI_SUM = 0;
	CLASS_DATA.NP_SUM = 0;
	CLASS_DATA.PD_SUM = 0;
	CLASS_DATA.MPS_SUM = 0;
	CLASS_DATA.PSSD_SUM = 0;
	CLASS_DATA.PSCV_SUM = 0;
	CLASS_DATA.TE_SUM = 0;
	CLASS_DATA.ED_SUM = 0;
	CLASS_DATA.LSI_SUM = 0;
	CLASS_DATA.MSI_SUM = 0;
	CLASS_DATA.AWMSI_SUM = 0;
	CLASS_DATA.DLFD_SUM = 0;
	CLASS_DATA.MPFD_SUM = 0;
	CLASS_DATA.AWMPFD_SUM = 0;
	CLASS_DATA.CLAND_SUM = 0;
	CLASS_DATA.TCA_SUM = 0;
	CLASS_DATA.NCA_SUM = 0;
	CLASS_DATA.CAD_SUM = 0;
	CLASS_DATA.MCA1_SUM = 0;
	CLASS_DATA.CASD1_SUM = 0;
	CLASS_DATA.CACV1_SUM = 0;
	CLASS_DATA.MCA2_SUM = 0;
	CLASS_DATA.CASD2_SUM = 0;
	CLASS_DATA.CACV2_SUM = 0;
	CLASS_DATA.TCAI_SUM = 0;
	CLASS_DATA.MCAI_SUM = 0;
	CLASS_DATA.MNN_SUM = 0;
	CLASS_DATA.NNSD_SUM = 0;
	CLASS_DATA.NNCV_SUM = 0;
	CLASS_DATA.IJI_SUM = 0;
	CLASS_DATA.CA_SDIFFSQR = 0;
	CLASS_DATA.TA_SDIFFSQR = 0;
	CLASS_DATA.LAND_SDIFFSQR = 0;
	CLASS_DATA.LPI_SDIFFSQR = 0;
	CLASS_DATA.NP_SDIFFSQR = 0;
	CLASS_DATA.PD_SDIFFSQR = 0;
	CLASS_DATA.MPS_SDIFFSQR = 0;
	CLASS_DATA.PSSD_SDIFFSQR = 0;
	CLASS_DATA.PSCV_SDIFFSQR = 0;
	CLASS_DATA.TE_SDIFFSQR = 0;
	CLASS_DATA.ED_SDIFFSQR = 0;
	CLASS_DATA.LSI_SDIFFSQR = 0;
	CLASS_DATA.MSI_SDIFFSQR = 0;
	CLASS_DATA.AWMSI_SDIFFSQR = 0;
	CLASS_DATA.DLFD_SDIFFSQR = 0;
	CLASS_DATA.MPFD_SDIFFSQR = 0;
	CLASS_DATA.AWMPFD_SDIFFSQR = 0;
	CLASS_DATA.CLAND_SDIFFSQR = 0;
	CLASS_DATA.TCA_SDIFFSQR = 0;
	CLASS_DATA.NCA_SDIFFSQR = 0;
	CLASS_DATA.CAD_SDIFFSQR = 0;
	CLASS_DATA.MCA1_SDIFFSQR = 0;
	CLASS_DATA.CASD1_SDIFFSQR = 0;
	CLASS_DATA.CACV1_SDIFFSQR = 0;
	CLASS_DATA.MCA2_SDIFFSQR = 0;
	CLASS_DATA.CASD2_SDIFFSQR = 0;
	CLASS_DATA.CACV2_SDIFFSQR = 0;
	CLASS_DATA.TCAI_SDIFFSQR = 0;
	CLASS_DATA.MCAI_SDIFFSQR = 0;
	CLASS_DATA.MNN_SDIFFSQR = 0;
	CLASS_DATA.NNSD_SDIFFSQR = 0;
	CLASS_DATA.NNCV_SDIFFSQR = 0;
	CLASS_DATA.IJI_SDIFFSQR = 0;
	CLASS_DATA.CA_MEAN = 0;
	CLASS_DATA.TA_MEAN = 0;
	CLASS_DATA.LAND_MEAN = 0;
	CLASS_DATA.LPI_MEAN = 0;
	CLASS_DATA.NP_MEAN = 0;
	CLASS_DATA.PD_MEAN = 0;
	CLASS_DATA.MPS_MEAN = 0;
	CLASS_DATA.PSSD_MEAN = 0;
	CLASS_DATA.PSCV_MEAN = 0;
	CLASS_DATA.TE_MEAN = 0;
	CLASS_DATA.ED_MEAN = 0;
	CLASS_DATA.LSI_MEAN = 0;
	CLASS_DATA.MSI_MEAN = 0;
	CLASS_DATA.AWMSI_MEAN = 0;
	CLASS_DATA.DLFD_MEAN = 0;
	CLASS_DATA.MPFD_MEAN = 0;
	CLASS_DATA.AWMPFD_MEAN = 0;
	CLASS_DATA.CLAND_MEAN = 0;
	CLASS_DATA.TCA_MEAN = 0;
	CLASS_DATA.NCA_MEAN = 0;
	CLASS_DATA.CAD_MEAN = 0;
	CLASS_DATA.MCA1_MEAN = 0;
	CLASS_DATA.CASD1_MEAN = 0;
	CLASS_DATA.CACV1_MEAN = 0;
	CLASS_DATA.MCA2_MEAN = 0;
	CLASS_DATA.CASD2_MEAN = 0;
	CLASS_DATA.CACV2_MEAN = 0;
	CLASS_DATA.TCAI_MEAN = 0;
	CLASS_DATA.MCAI_MEAN = 0;
	CLASS_DATA.MNN_MEAN = 0;
	CLASS_DATA.NNSD_MEAN = 0;
	CLASS_DATA.NNCV_MEAN = 0;
	CLASS_DATA.IJI_MEAN = 0;
	CLASS_DATA.CA_VARIANCE = 0;
	CLASS_DATA.TA_VARIANCE = 0;
	CLASS_DATA.LAND_VARIANCE = 0;
	CLASS_DATA.LPI_VARIANCE = 0;
	CLASS_DATA.NP_VARIANCE = 0;
	CLASS_DATA.PD_VARIANCE = 0;
	CLASS_DATA.MPS_VARIANCE = 0;
	CLASS_DATA.PSSD_VARIANCE = 0;
	CLASS_DATA.PSCV_VARIANCE = 0;
	CLASS_DATA.TE_VARIANCE = 0;
	CLASS_DATA.ED_VARIANCE = 0;
	CLASS_DATA.LSI_VARIANCE = 0;
	CLASS_DATA.MSI_VARIANCE = 0;
	CLASS_DATA.AWMSI_VARIANCE = 0;
	CLASS_DATA.DLFD_VARIANCE = 0;
	CLASS_DATA.MPFD_VARIANCE = 0;
	CLASS_DATA.AWMPFD_VARIANCE = 0;
	CLASS_DATA.CLAND_VARIANCE = 0;
	CLASS_DATA.TCA_VARIANCE = 0;
	CLASS_DATA.NCA_VARIANCE = 0;
	CLASS_DATA.CAD_VARIANCE = 0;
	CLASS_DATA.MCA1_VARIANCE = 0;
	CLASS_DATA.CASD1_VARIANCE = 0;
	CLASS_DATA.CACV1_VARIANCE = 0;
	CLASS_DATA.MCA2_VARIANCE = 0;
	CLASS_DATA.CASD2_VARIANCE = 0;
	CLASS_DATA.CACV2_VARIANCE = 0;
	CLASS_DATA.TCAI_VARIANCE = 0;
	CLASS_DATA.MCAI_VARIANCE = 0;
	CLASS_DATA.MNN_VARIANCE = 0;
	CLASS_DATA.NNSD_VARIANCE = 0;
	CLASS_DATA.NNCV_VARIANCE = 0;
	CLASS_DATA.IJI_VARIANCE = 0;
	return(0);
}
/********************************************************************
* initialize the data arrays that will be used to accumulate the sums.
********************************************************************/
init_land_array()
{
	LAND_DATA.COUNT = 0;
	LAND_DATA.LEE_SUM = 0;
	LAND_DATA.MWFIT_SUM = 0;
	LAND_DATA.TA_SUM = 0;
	LAND_DATA.LPI_SUM = 0;
	LAND_DATA.NP_SUM = 0;
	LAND_DATA.PD_SUM = 0;
	LAND_DATA.MPS_SUM = 0;
	LAND_DATA.PSSD_SUM = 0;
	LAND_DATA.PSCV_SUM = 0;
	LAND_DATA.TE_SUM = 0;
	LAND_DATA.ED_SUM = 0;
	LAND_DATA.LSI_SUM = 0;
	LAND_DATA.MSI_SUM = 0;
	LAND_DATA.AWMSI_SUM = 0;
	LAND_DATA.DFLD_SUM = 0;
	LAND_DATA.MPFD_SUM = 0;
	LAND_DATA.AWMPFD_SUM = 0;
	LAND_DATA.TCA_SUM = 0;
	LAND_DATA.NCA_SUM = 0;
	LAND_DATA.CAD_SUM = 0;
	LAND_DATA.MCA1_SUM = 0;
	LAND_DATA.CASD1_SUM = 0;
	LAND_DATA.CACV1_SUM = 0;
	LAND_DATA.MCA2_SUM = 0;
	LAND_DATA.CASD2_SUM = 0;
	LAND_DATA.CACV2_SUM = 0;
	LAND_DATA.TCAI_SUM = 0;
	LAND_DATA.MCAI_SUM = 0;
	LAND_DATA.MNN_SUM = 0;
	LAND_DATA.NNSD_SUM = 0;
	LAND_DATA.NNCV_SUM = 0;
	LAND_DATA.SHDI_SUM = 0;
	LAND_DATA.SIDI_SUM = 0;
	LAND_DATA.MSIDI_SUM = 0;
	LAND_DATA.PR_SUM = 0;
	LAND_DATA.PRD_SUM = 0;
	LAND_DATA.RPR_SUM = 0;
	LAND_DATA.SHEI_SUM = 0;
	LAND_DATA.SIEI_SUM = 0;
	LAND_DATA.MSIEI_SUM = 0;
	LAND_DATA.IJI_SUM = 0;
	LAND_DATA.CONTAG_SUM = 0;
	LAND_DATA.TA_MEAN = 0;
	LAND_DATA.LPI_MEAN = 0;
	LAND_DATA.NP_MEAN = 0;
	LAND_DATA.PD_MEAN = 0;
	LAND_DATA.MPS_MEAN = 0;
	LAND_DATA.PSSD_MEAN = 0;
	LAND_DATA.PSCV_MEAN = 0;
	LAND_DATA.TE_MEAN = 0;
	LAND_DATA.ED_MEAN = 0;
	LAND_DATA.LSI_MEAN = 0;
	LAND_DATA.MSI_MEAN = 0;
	LAND_DATA.AWMSI_MEAN = 0;
	LAND_DATA.DFLD_MEAN = 0;
	LAND_DATA.MPFD_MEAN = 0;
	LAND_DATA.AWMPFD_MEAN = 0;
	LAND_DATA.TCA_MEAN = 0;
	LAND_DATA.NCA_MEAN = 0;
	LAND_DATA.CAD_MEAN = 0;
	LAND_DATA.MCA1_MEAN = 0;
	LAND_DATA.CASD1_MEAN = 0;
	LAND_DATA.CACV1_MEAN = 0;
	LAND_DATA.MCA2_MEAN = 0;
	LAND_DATA.CASD2_MEAN = 0;
	LAND_DATA.CACV2_MEAN = 0;
	LAND_DATA.TCAI_MEAN = 0;
	LAND_DATA.MCAI_MEAN = 0;
	LAND_DATA.MNN_MEAN = 0;
	LAND_DATA.NNSD_MEAN = 0;
	LAND_DATA.NNCV_MEAN = 0;
	LAND_DATA.SHDI_MEAN = 0;
	LAND_DATA.SIDI_MEAN = 0;
	LAND_DATA.MSIDI_MEAN = 0;
	LAND_DATA.PR_MEAN = 0;
	LAND_DATA.PRD_MEAN = 0;
	LAND_DATA.RPR_MEAN = 0;
	LAND_DATA.SHEI_MEAN = 0;
	LAND_DATA.SIEI_MEAN = 0;
	LAND_DATA.MSIEI_MEAN = 0;
	LAND_DATA.IJI_MEAN = 0;
	LAND_DATA.CONTAG_MEAN = 0;
	LAND_DATA.TA_SDIFFSQR = 0;
	LAND_DATA.LPI_SDIFFSQR = 0;
	LAND_DATA.NP_SDIFFSQR = 0;
	LAND_DATA.PD_SDIFFSQR = 0;
	LAND_DATA.MPS_SDIFFSQR = 0;
	LAND_DATA.PSSD_SDIFFSQR = 0;
	LAND_DATA.PSCV_SDIFFSQR = 0;
	LAND_DATA.TE_SDIFFSQR = 0;
	LAND_DATA.ED_SDIFFSQR = 0;
	LAND_DATA.LSI_SDIFFSQR = 0;
	LAND_DATA.MSI_SDIFFSQR = 0;
	LAND_DATA.AWMSI_SDIFFSQR = 0;
	LAND_DATA.DFLD_SDIFFSQR = 0;
	LAND_DATA.MPFD_SDIFFSQR = 0;
	LAND_DATA.AWMPFD_SDIFFSQR = 0;
	LAND_DATA.TCA_SDIFFSQR = 0;
	LAND_DATA.NCA_SDIFFSQR = 0;
	LAND_DATA.CAD_SDIFFSQR = 0;
	LAND_DATA.MCA1_SDIFFSQR = 0;
	LAND_DATA.CASD1_SDIFFSQR = 0;
	LAND_DATA.CACV1_SDIFFSQR = 0;
	LAND_DATA.MCA2_SDIFFSQR = 0;
	LAND_DATA.CASD2_SDIFFSQR = 0;
	LAND_DATA.CACV2_SDIFFSQR = 0;
	LAND_DATA.TCAI_SDIFFSQR = 0;
	LAND_DATA.MCAI_SDIFFSQR = 0;
	LAND_DATA.MNN_SDIFFSQR = 0;
	LAND_DATA.NNSD_SDIFFSQR = 0;
	LAND_DATA.NNCV_SDIFFSQR = 0;
	LAND_DATA.SHDI_SDIFFSQR = 0;
	LAND_DATA.SIDI_SDIFFSQR = 0;
	LAND_DATA.MSIDI_SDIFFSQR = 0;
	LAND_DATA.PR_SDIFFSQR = 0;
	LAND_DATA.PRD_SDIFFSQR = 0;
	LAND_DATA.RPR_SDIFFSQR = 0;
	LAND_DATA.SHEI_SDIFFSQR = 0;
	LAND_DATA.SIEI_SDIFFSQR = 0;
	LAND_DATA.MSIEI_SDIFFSQR = 0;
	LAND_DATA.IJI_SDIFFSQR = 0;
	LAND_DATA.CONTAG_SDIFFSQR = 0;
	LAND_DATA.TA_VARIANCE = 0;
	LAND_DATA.LPI_VARIANCE = 0;
	LAND_DATA.NP_VARIANCE = 0;
	LAND_DATA.PD_VARIANCE = 0;
	LAND_DATA.MPS_VARIANCE = 0;
	LAND_DATA.PSSD_VARIANCE = 0;
	LAND_DATA.PSCV_VARIANCE = 0;
	LAND_DATA.TE_VARIANCE = 0;
	LAND_DATA.ED_VARIANCE = 0;
	LAND_DATA.LSI_VARIANCE = 0;
	LAND_DATA.MSI_VARIANCE = 0;
	LAND_DATA.AWMSI_VARIANCE = 0;
	LAND_DATA.DFLD_VARIANCE = 0;
	LAND_DATA.MPFD_VARIANCE = 0;
	LAND_DATA.AWMPFD_VARIANCE = 0;
	LAND_DATA.TCA_VARIANCE = 0;
	LAND_DATA.NCA_VARIANCE = 0;
	LAND_DATA.CAD_VARIANCE = 0;
	LAND_DATA.MCA1_VARIANCE = 0;
	LAND_DATA.CASD1_VARIANCE = 0;
	LAND_DATA.CACV1_VARIANCE = 0;
	LAND_DATA.MCA2_VARIANCE = 0;
	LAND_DATA.CASD2_VARIANCE = 0;
	LAND_DATA.CACV2_VARIANCE = 0;
	LAND_DATA.TCAI_VARIANCE = 0;
	LAND_DATA.MCAI_VARIANCE = 0;
	LAND_DATA.MNN_VARIANCE = 0;
	LAND_DATA.NNSD_VARIANCE = 0;
	LAND_DATA.NNCV_VARIANCE = 0;
	LAND_DATA.SHDI_VARIANCE = 0;
	LAND_DATA.SIDI_VARIANCE = 0;
	LAND_DATA.MSIDI_VARIANCE = 0;
	LAND_DATA.PR_VARIANCE = 0;
	LAND_DATA.PRD_VARIANCE = 0;
	LAND_DATA.RPR_VARIANCE = 0;
	LAND_DATA.SHEI_VARIANCE = 0;
	LAND_DATA.SIEI_VARIANCE = 0;
	LAND_DATA.MSIEI_VARIANCE = 0;
	LAND_DATA.IJI_VARIANCE = 0;
	LAND_DATA.CONTAG_VARIANCE = 0;
	return(0);
}
/********************************************************************
********************************************************************/
calc_sdiffsqr()
{
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter calc_sdiffsqr\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	if (PASS_LIST_P == NULL)  {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) WARNING: No Data\n", __FILE__, __LINE__);
		fflush(DBG_FILE);
#endif
		return(0);
	}
	TMP_PASS_LIST_P = PASS_LIST_P;
	while (TMP_PASS_LIST_P != NULL) {
		CLASS_DATA.CA_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.CA_VAL - CLASS_DATA.CA_MEAN), (double)2);
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) CLASS CA SDIFF VAL %8.5f MEAN: %8.5f\n", 
		__FILE__, __LINE__, TMP_PASS_LIST_P->class_urban_res.CA_VAL, CLASS_DATA.CA_MEAN);
		(void)fprintf(DBG_FILE, "(%s %d) CLASS CA SDIFF add %8.5f (tot: %8.5f)\n", 
		__FILE__, __LINE__, (pow((TMP_PASS_LIST_P->class_urban_res.CA_VAL - CLASS_DATA.CA_MEAN),
		(double)2)), CLASS_DATA.CA_SDIFFSQR);
		fflush(DBG_FILE);
#endif
		CLASS_DATA.TA_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.TA_VAL - CLASS_DATA.TA_MEAN), (double)2);
		CLASS_DATA.LAND_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.LAND_VAL - CLASS_DATA.LAND_MEAN), (double)2);
		CLASS_DATA.LPI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.LPI_VAL - CLASS_DATA.LPI_MEAN), (double)2);
		CLASS_DATA.NP_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.NP_VAL - CLASS_DATA.NP_MEAN), (double)2);
		CLASS_DATA.PD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.PD_VAL - CLASS_DATA.PD_MEAN), (double)2);
		CLASS_DATA.MPS_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.MPS_VAL - CLASS_DATA.MPS_MEAN), (double)2);
		CLASS_DATA.PSSD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.PSSD_VAL - CLASS_DATA.PSSD_MEAN), (double)2);
		CLASS_DATA.PSCV_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.PSCV_VAL - CLASS_DATA.PSCV_MEAN), (double)2);
		CLASS_DATA.TE_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.TE_VAL - CLASS_DATA.TE_MEAN), (double)2);
		CLASS_DATA.ED_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.ED_VAL - CLASS_DATA.ED_MEAN), (double)2);
		CLASS_DATA.LSI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.LSI_VAL - CLASS_DATA.LSI_MEAN), (double)2);
		CLASS_DATA.MSI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.MSI_VAL - CLASS_DATA.MSI_MEAN), (double)2);
		CLASS_DATA.AWMSI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.AWMSI_VAL - CLASS_DATA.AWMSI_MEAN), (double)2);
		CLASS_DATA.DLFD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.DLFD_VAL - CLASS_DATA.DLFD_MEAN), (double)2);
		CLASS_DATA.MPFD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.MPFD_VAL - CLASS_DATA.MPFD_MEAN), (double)2);
		CLASS_DATA.AWMPFD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.AWMPFD_VAL - CLASS_DATA.AWMPFD_MEAN), (double)2);
		CLASS_DATA.CLAND_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.CLAND_VAL - CLASS_DATA.CLAND_MEAN), (double)2);
		CLASS_DATA.TCA_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.TCA_VAL - CLASS_DATA.TCA_MEAN), (double)2);
		CLASS_DATA.NCA_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.NCA_VAL - CLASS_DATA.NCA_MEAN), (double)2);
		CLASS_DATA.CAD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.CAD_VAL - CLASS_DATA.CAD_MEAN), (double)2);
		CLASS_DATA.MCA1_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.MCA1_VAL - CLASS_DATA.MCA1_MEAN), (double)2);
		CLASS_DATA.CASD1_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.CASD1_VAL - CLASS_DATA.CASD1_MEAN), (double)2);
		CLASS_DATA.CACV1_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.CACV1_VAL - CLASS_DATA.CACV1_MEAN), (double)2);
		CLASS_DATA.MCA2_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.MCA2_VAL - CLASS_DATA.MCA2_MEAN), (double)2);
		CLASS_DATA.CASD2_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.CASD2_VAL - CLASS_DATA.CASD2_MEAN), (double)2);
		CLASS_DATA.CACV2_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.CACV2_VAL - CLASS_DATA.CACV2_MEAN), (double)2);
		CLASS_DATA.TCAI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.TCAI_VAL - CLASS_DATA.TCAI_MEAN), (double)2);
		CLASS_DATA.MCAI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.MCAI_VAL - CLASS_DATA.MCAI_MEAN), (double)2);
		CLASS_DATA.MNN_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.MNN_VAL - CLASS_DATA.MNN_MEAN), (double)2);
		CLASS_DATA.NNSD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.NNSD_VAL - CLASS_DATA.NNSD_MEAN), (double)2);
		CLASS_DATA.NNCV_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.NNCV_VAL - CLASS_DATA.NNCV_MEAN), (double)2);
		CLASS_DATA.IJI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->class_urban_res.IJI_VAL - CLASS_DATA.IJI_MEAN), (double)2);

		/* land data here */
		LAND_DATA.TA_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.TA_VAL - LAND_DATA.TA_MEAN), (double)2);
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) LAND TA SDIFF VAL: %8.5f MEAN: %8.5f\n", 
		__FILE__, __LINE__, TMP_PASS_LIST_P->land_res.TA_VAL, LAND_DATA.TA_MEAN);
		(void)fprintf(DBG_FILE, "(%s %d) LAND TA SDIFF add %8.5f (tot: %8.5f)\n", 
		__FILE__, __LINE__, (pow((TMP_PASS_LIST_P->land_res.TA_VAL - LAND_DATA.TA_MEAN),
		(double)2)), LAND_DATA.TA_SDIFFSQR);
		fflush(DBG_FILE);
#endif
		LAND_DATA.LPI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.LPI_VAL - LAND_DATA.LPI_MEAN), (double)2);
		LAND_DATA.NP_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.NP_VAL - LAND_DATA.NP_MEAN), (double)2);
		LAND_DATA.PD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.PD_VAL - LAND_DATA.PD_MEAN), (double)2);
		LAND_DATA.MPS_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MPS_VAL - LAND_DATA.MPS_MEAN), (double)2);
		LAND_DATA.PSSD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.PSSD_VAL - LAND_DATA.PSSD_MEAN), (double)2);
		LAND_DATA.PSCV_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.PSCV_VAL - LAND_DATA.PSCV_MEAN), (double)2);
		LAND_DATA.TE_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.TE_VAL - LAND_DATA.TE_MEAN), (double)2);
		LAND_DATA.ED_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.ED_VAL - LAND_DATA.ED_MEAN), (double)2);
		LAND_DATA.LSI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.LSI_VAL - LAND_DATA.LSI_MEAN), (double)2);
		LAND_DATA.MSI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MSI_VAL - LAND_DATA.MSI_MEAN), (double)2);
		LAND_DATA.AWMSI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.AWMSI_VAL - LAND_DATA.AWMSI_MEAN), (double)2);
		LAND_DATA.DFLD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.DFLD_VAL - LAND_DATA.DFLD_MEAN), (double)2);
		LAND_DATA.MPFD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MPFD_VAL - LAND_DATA.MPFD_MEAN), (double)2);
		LAND_DATA.AWMPFD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.AWMPFD_VAL - LAND_DATA.AWMPFD_MEAN), (double)2);
		LAND_DATA.TCA_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.TCA_VAL - LAND_DATA.TCA_MEAN), (double)2);
		LAND_DATA.NCA_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.NCA_VAL - LAND_DATA.NCA_MEAN), (double)2);
		LAND_DATA.CAD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.CAD_VAL - LAND_DATA.CAD_MEAN), (double)2);
		LAND_DATA.MCA1_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MCA1_VAL - LAND_DATA.MCA1_MEAN), (double)2);
		LAND_DATA.CASD1_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.CASD1_VAL - LAND_DATA.CASD1_MEAN), (double)2);
		LAND_DATA.CACV1_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.CACV1_VAL - LAND_DATA.CACV1_MEAN), (double)2);
		LAND_DATA.MCA2_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MCA2_VAL - LAND_DATA.MCA2_MEAN), (double)2);
		LAND_DATA.CASD2_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.CASD2_VAL - LAND_DATA.CASD2_MEAN), (double)2);
		LAND_DATA.CACV2_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.CACV2_VAL - LAND_DATA.CACV2_MEAN), (double)2);
		LAND_DATA.TCAI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.TCAI_VAL - LAND_DATA.TCAI_MEAN), (double)2);
		LAND_DATA.MCAI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MCAI_VAL - LAND_DATA.MCAI_MEAN), (double)2);
		LAND_DATA.MNN_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MNN_VAL - LAND_DATA.MNN_MEAN), (double)2);
		LAND_DATA.NNSD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.NNSD_VAL - LAND_DATA.NNSD_MEAN), (double)2);
		LAND_DATA.NNCV_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.NNCV_VAL - LAND_DATA.NNCV_MEAN), (double)2);
		LAND_DATA.SHDI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.SHDI_VAL - LAND_DATA.SHDI_MEAN), (double)2);
		LAND_DATA.SIDI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.SIDI_VAL - LAND_DATA.SIDI_MEAN), (double)2);
		LAND_DATA.MSIDI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MSIDI_VAL - LAND_DATA.MSIDI_MEAN), (double)2);
		LAND_DATA.PR_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.PR_VAL - LAND_DATA.PR_MEAN), (double)2);
		LAND_DATA.PRD_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.PRD_VAL - LAND_DATA.PRD_MEAN), (double)2);
		LAND_DATA.RPR_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.RPR_VAL - LAND_DATA.RPR_MEAN), (double)2);
		LAND_DATA.SHEI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.SHEI_VAL - LAND_DATA.SHEI_MEAN), (double)2);
		LAND_DATA.SIEI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.SIEI_VAL - LAND_DATA.SIEI_MEAN), (double)2);
		LAND_DATA.MSIEI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.MSIEI_VAL - LAND_DATA.MSIEI_MEAN), (double)2);
		LAND_DATA.IJI_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.IJI_VAL - LAND_DATA.IJI_MEAN), (double)2);
		LAND_DATA.CONTAG_SDIFFSQR +=
		pow((TMP_PASS_LIST_P->land_res.CONTAG_VAL - LAND_DATA.CONTAG_MEAN), (double)2);

		TMP_PASS_LIST_P = TMP_PASS_LIST_P->next_p;
	}
	return(0);
}
/********************************************************************
********************************************************************/
calc_variance_data()
{
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter calc_variance_data()\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	CLASS_DATA.CA_VARIANCE =
	CLASS_DATA.CA_SDIFFSQR / CLASS_DATA.COUNT;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) CLASS CA var %8.5f   (SDIF/CNT: %8.5f/%d)\n",
 	__FILE__, __LINE__, CLASS_DATA.CA_VARIANCE,
	CLASS_DATA.CA_SDIFFSQR, CLASS_DATA.COUNT);
	fflush(DBG_FILE);
#endif
	CLASS_DATA.TA_VARIANCE =
	CLASS_DATA.TA_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.LAND_VARIANCE =
	CLASS_DATA.LAND_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.LPI_VARIANCE =
	CLASS_DATA.LPI_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.NP_VARIANCE =
	CLASS_DATA.NP_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.PD_VARIANCE =
	CLASS_DATA.PD_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.MPS_VARIANCE =
	CLASS_DATA.MPS_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.PSSD_VARIANCE =
	CLASS_DATA.PSSD_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.PSCV_VARIANCE =
	CLASS_DATA.PSCV_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.TE_VARIANCE =
	CLASS_DATA.TE_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.ED_VARIANCE =
	CLASS_DATA.ED_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.LSI_VARIANCE =
	CLASS_DATA.LSI_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.MSI_VARIANCE =
	CLASS_DATA.MSI_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.AWMSI_VARIANCE =
	CLASS_DATA.AWMSI_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.DLFD_VARIANCE =
	CLASS_DATA.DLFD_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.MPFD_VARIANCE =
	CLASS_DATA.MPFD_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.AWMPFD_VARIANCE =
	CLASS_DATA.AWMPFD_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.CLAND_VARIANCE =
	CLASS_DATA.CLAND_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.TCA_VARIANCE =
	CLASS_DATA.TCA_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.NCA_VARIANCE =
	CLASS_DATA.NCA_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.CAD_VARIANCE =
	CLASS_DATA.CAD_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.MCA1_VARIANCE =
	CLASS_DATA.MCA1_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.CASD1_VARIANCE =
	CLASS_DATA.CASD1_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.CACV1_VARIANCE =
	CLASS_DATA.CACV1_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.MCA2_VARIANCE =
	CLASS_DATA.MCA2_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.CASD2_VARIANCE =
	CLASS_DATA.CASD2_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.CACV2_VARIANCE =
	CLASS_DATA.CACV2_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.TCAI_VARIANCE =
	CLASS_DATA.TCAI_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.MCAI_VARIANCE =
	CLASS_DATA.MCAI_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.MNN_VARIANCE =
	CLASS_DATA.MNN_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.NNSD_VARIANCE =
	CLASS_DATA.NNSD_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.NNCV_VARIANCE =
	CLASS_DATA.NNCV_SDIFFSQR / CLASS_DATA.COUNT;
	CLASS_DATA.IJI_VARIANCE =
	CLASS_DATA.IJI_SDIFFSQR / CLASS_DATA.COUNT;

	/* land data here */
	LAND_DATA.TA_VARIANCE =
	LAND_DATA.TA_SDIFFSQR / (LAND_DATA.COUNT);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) LAND TA var %8.5f   (SDIF/CNT: %8.5f/%d)\n",
 	__FILE__, __LINE__, LAND_DATA.TA_VARIANCE, LAND_DATA.TA_SDIFFSQR, LAND_DATA.COUNT);
	fflush(DBG_FILE);
#endif
	LAND_DATA.LPI_VARIANCE =
	LAND_DATA.LPI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.NP_VARIANCE =
	LAND_DATA.NP_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.PD_VARIANCE =
	LAND_DATA.PD_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MPS_VARIANCE =
	LAND_DATA.MPS_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.PSSD_VARIANCE =
	LAND_DATA.PSSD_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.PSCV_VARIANCE =
	LAND_DATA.PSCV_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.TE_VARIANCE =
	LAND_DATA.TE_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.ED_VARIANCE =
	LAND_DATA.ED_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.LSI_VARIANCE =
	LAND_DATA.LSI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MSI_VARIANCE =
	LAND_DATA.MSI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.AWMSI_VARIANCE =
	LAND_DATA.AWMSI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.DFLD_VARIANCE =
	LAND_DATA.DFLD_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MPFD_VARIANCE =
	LAND_DATA.MPFD_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.AWMPFD_VARIANCE =
	LAND_DATA.AWMPFD_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.TCA_VARIANCE =
	LAND_DATA.TCA_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.NCA_VARIANCE =
	LAND_DATA.NCA_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.CAD_VARIANCE =
	LAND_DATA.CAD_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MCA1_VARIANCE =
	LAND_DATA.MCA1_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.CASD1_VARIANCE =
	LAND_DATA.CASD1_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.CACV1_VARIANCE =
	LAND_DATA.CACV1_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MCA2_VARIANCE =
	LAND_DATA.MCA2_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.CASD2_VARIANCE =
	LAND_DATA.CASD2_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.CACV2_VARIANCE =
	LAND_DATA.CACV2_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.TCAI_VARIANCE =
	LAND_DATA.TCAI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MCAI_VARIANCE =
	LAND_DATA.MCAI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MNN_VARIANCE =
	LAND_DATA.MNN_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.NNSD_VARIANCE =
	LAND_DATA.NNSD_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.NNCV_VARIANCE =
	LAND_DATA.NNCV_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.SHDI_VARIANCE =
	LAND_DATA.SHDI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.SIDI_VARIANCE =
	LAND_DATA.SIDI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MSIDI_VARIANCE =
	LAND_DATA.MSIDI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.PR_VARIANCE =
	LAND_DATA.PR_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.PRD_VARIANCE =
	LAND_DATA.PRD_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.RPR_VARIANCE =
	LAND_DATA.RPR_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.SHEI_VARIANCE =
	LAND_DATA.SHEI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.SIEI_VARIANCE =
	LAND_DATA.SIEI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.MSIEI_VARIANCE =
	LAND_DATA.MSIEI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.IJI_VARIANCE =
	LAND_DATA.IJI_SDIFFSQR / (LAND_DATA.COUNT);
	LAND_DATA.CONTAG_VARIANCE =
	LAND_DATA.CONTAG_SDIFFSQR / (LAND_DATA.COUNT);
	return(0);
}
/************************************************
************************************************/
write_class_stats()
{
	int	i;
	FILE	*fp;
	char	filename[MAX_LEN], tmp[MAX_LEN];

	strcpy(filename, OUTDIR);
	strcat(filename, "/");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[0]);
	strcat(filename, tmp);
	strcat(filename, "d");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[1]);
	strcat(filename, tmp);
	strcat(filename, "b");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[2]);
	strcat(filename, tmp);
	strcat(filename, "s");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[3]);
	strcat(filename, tmp);
	strcat(filename, "r");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[4]);
	strcat(filename, tmp);
	strcat(filename, "r");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[5]);
	strcat(filename, tmp);
	strcat(filename, "h");
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) filename: %s\n", __FILE__, __LINE__, filename);
	fflush(DBG_FILE);
#endif

	if ((fp = fopen(filename, "w")) == NULL)
		return(1);

	(void)fprintf(fp, "GENES: %2.2f-diffu  %2.2f-breed  %2.2f-spread  %2.2f-road  %2.0f-nrqmt  %2.0f-nhood\n",
	PASS_LIST_P->gene[0], PASS_LIST_P->gene[1], PASS_LIST_P->gene[2], PASS_LIST_P->gene[3],
	PASS_LIST_P->gene[4], PASS_LIST_P->gene[5]);

	(void)fprintf(fp, "LEESALLEE   %2.6f  (%d,%d,%d,%d,%d,%d)\n",
	(LAND_DATA.LEE_SUM / LAND_DATA.COUNT), 
	(int)PASS_LIST_P->gene[0], (int)PASS_LIST_P->gene[1],
	(int)PASS_LIST_P->gene[2], (int)PASS_LIST_P->gene[3],
	(int)PASS_LIST_P->gene[4], (int)PASS_LIST_P->gene[5]);

	(void)fprintf(fp, "MW BESTFIT  %6.6f  (%d,%d,%d,%d,%d,%d)\n\n",
	(LAND_DATA.MWFIT_SUM / LAND_DATA.COUNT), 
	(int)PASS_LIST_P->gene[0], (int)PASS_LIST_P->gene[1],
	(int)PASS_LIST_P->gene[2], (int)PASS_LIST_P->gene[3],
	(int)PASS_LIST_P->gene[4], (int)PASS_LIST_P->gene[5]);

	(void)fprintf(fp, "\n");
	(void)fprintf(fp, "CLASS			MEAN		VARIANCE	STDDEV\n");
	(void)fprintf(fp, "CA			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.CA_MEAN, CLASS_DATA.CA_VARIANCE, sqrt(CLASS_DATA.CA_VARIANCE));
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) CLASS  CA Mean %8.5f  var %8.5f	stddev %8.5f\n", 
__FILE__, __LINE__, CLASS_DATA.CA_MEAN, CLASS_DATA.CA_VARIANCE, sqrt(CLASS_DATA.CA_VARIANCE));
fflush(DBG_FILE);
#endif
	(void)fprintf(fp, "TA			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.TA_MEAN, CLASS_DATA.TA_VARIANCE, sqrt(CLASS_DATA.TA_VARIANCE));
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) CLASS  TA Mean %8.5f  var %8.5f	stddev %8.5f\n", 
__FILE__, __LINE__, CLASS_DATA.TA_MEAN, CLASS_DATA.TA_VARIANCE, sqrt(CLASS_DATA.TA_VARIANCE));
fflush(DBG_FILE);
#endif
	(void)fprintf(fp, "LAND			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.LAND_MEAN, CLASS_DATA.LAND_VARIANCE, sqrt(CLASS_DATA.LAND_VARIANCE));
	(void)fprintf(fp, "LPI			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.LPI_MEAN, CLASS_DATA.LPI_VARIANCE, sqrt(CLASS_DATA.LPI_VARIANCE));
	(void)fprintf(fp, "NP			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.NP_MEAN, CLASS_DATA.NP_VARIANCE, sqrt(CLASS_DATA.NP_VARIANCE));
	(void)fprintf(fp, "PD			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.PD_MEAN, CLASS_DATA.PD_VARIANCE, sqrt(CLASS_DATA.PD_VARIANCE));
	(void)fprintf(fp, "MPS			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.MPS_MEAN, CLASS_DATA.MPS_VARIANCE, sqrt(CLASS_DATA.MPS_VARIANCE));
	(void)fprintf(fp, "PSSD			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.PSSD_MEAN, CLASS_DATA.PSSD_VARIANCE, sqrt(CLASS_DATA.PSSD_VARIANCE));
	(void)fprintf(fp, "PSCV			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.PSCV_MEAN, CLASS_DATA.PSCV_VARIANCE, sqrt(CLASS_DATA.PSCV_VARIANCE));
	(void)fprintf(fp, "TE			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.TE_MEAN, CLASS_DATA.TE_VARIANCE, sqrt(CLASS_DATA.TE_VARIANCE));
	(void)fprintf(fp, "ED			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.ED_MEAN, CLASS_DATA.ED_VARIANCE, sqrt(CLASS_DATA.ED_VARIANCE));
	(void)fprintf(fp, "LSI			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.LSI_MEAN, CLASS_DATA.LSI_VARIANCE, sqrt(CLASS_DATA.LSI_VARIANCE));
	(void)fprintf(fp, "MSI			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.MSI_MEAN, CLASS_DATA.MSI_VARIANCE, sqrt(CLASS_DATA.MSI_VARIANCE));
	(void)fprintf(fp, "AWMSI			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.AWMSI_MEAN, CLASS_DATA.AWMSI_VARIANCE, sqrt(CLASS_DATA.AWMSI_VARIANCE));
	(void)fprintf(fp, "DLFD			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.DLFD_MEAN, CLASS_DATA.DLFD_VARIANCE, sqrt(CLASS_DATA.DLFD_VARIANCE));
	(void)fprintf(fp, "MPFD			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.MPFD_MEAN, CLASS_DATA.MPFD_VARIANCE, sqrt(CLASS_DATA.MPFD_VARIANCE));
	(void)fprintf(fp, "AWMPFD			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.AWMPFD_MEAN, CLASS_DATA.AWMPFD_VARIANCE, sqrt(CLASS_DATA.AWMPFD_VARIANCE));
	(void)fprintf(fp, "CLAND			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.CLAND_MEAN, CLASS_DATA.CLAND_VARIANCE, sqrt(CLASS_DATA.CLAND_VARIANCE));
	(void)fprintf(fp, "TCA			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.TCA_MEAN, CLASS_DATA.TCA_VARIANCE, sqrt(CLASS_DATA.TCA_VARIANCE));
	(void)fprintf(fp, "NCA			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.NCA_MEAN, CLASS_DATA.NCA_VARIANCE, sqrt(CLASS_DATA.NCA_VARIANCE));
	(void)fprintf(fp, "CAD			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.CAD_MEAN, CLASS_DATA.CAD_VARIANCE, sqrt(CLASS_DATA.CAD_VARIANCE));
	(void)fprintf(fp, "MCA1			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.MCA1_MEAN, CLASS_DATA.MCA1_VARIANCE, sqrt(CLASS_DATA.MCA1_VARIANCE));
	(void)fprintf(fp, "CASD1			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.CASD1_MEAN, CLASS_DATA.CASD1_VARIANCE, sqrt(CLASS_DATA.CASD1_VARIANCE));
	(void)fprintf(fp, "CACV1			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.CACV1_MEAN, CLASS_DATA.CACV1_VARIANCE, sqrt(CLASS_DATA.CACV1_VARIANCE));
	(void)fprintf(fp, "MCA2			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.MCA2_MEAN, CLASS_DATA.MCA2_VARIANCE, sqrt(CLASS_DATA.MCA2_VARIANCE));
	(void)fprintf(fp, "CASD2			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.CASD2_MEAN, CLASS_DATA.CASD2_VARIANCE, sqrt(CLASS_DATA.CASD2_VARIANCE));
	(void)fprintf(fp, "CACV2			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.CACV2_MEAN, CLASS_DATA.CACV2_VARIANCE, sqrt(CLASS_DATA.CACV2_VARIANCE));
	(void)fprintf(fp, "TCAI			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.TCAI_MEAN, CLASS_DATA.TCAI_VARIANCE, sqrt(CLASS_DATA.TCAI_VARIANCE));
	(void)fprintf(fp, "MCAI			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.MCAI_MEAN, CLASS_DATA.MCAI_VARIANCE, sqrt(CLASS_DATA.MCAI_VARIANCE));
	(void)fprintf(fp, "MNN			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.MNN_MEAN, CLASS_DATA.MNN_VARIANCE, sqrt(CLASS_DATA.MNN_VARIANCE));
	(void)fprintf(fp, "NNSD			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.NNSD_MEAN, CLASS_DATA.NNSD_VARIANCE, sqrt(CLASS_DATA.NNSD_VARIANCE));
	(void)fprintf(fp, "NNCV			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.NNCV_MEAN, CLASS_DATA.NNCV_VARIANCE, sqrt(CLASS_DATA.NNCV_VARIANCE));
	(void)fprintf(fp, "IJI			%8.5f 	%8.5f	%8.5f\n",
	CLASS_DATA.IJI_MEAN, CLASS_DATA.IJI_VARIANCE, sqrt(CLASS_DATA.IJI_VARIANCE));
	fclose(fp);
	return(0);
}
/********************************************************************
* write stats to a file
********************************************************************/
write_land_stats()
{
	FILE	*fp;
	char	filename[MAX_LEN], tmp[MAX_LEN];

	if (PASS_LIST_P == NULL) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) ERROR: pass_list_p is null\n", __FILE__, __LINE__);
		fflush(DBG_FILE);
#endif
		exit(0);
	}
	strcpy(filename, OUTDIR);
	strcat(filename, "/");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[0]);
	strcat(filename, tmp);
	strcat(filename, "d");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[1]);
	strcat(filename, tmp);
	strcat(filename, "b");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[2]);
	strcat(filename, tmp);
	strcat(filename, "s");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[3]);
	strcat(filename, tmp);
	strcat(filename, "r");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[4]);
	strcat(filename, tmp);
	strcat(filename, "r");
	sprintf(tmp, "%d", (int)PASS_LIST_P->gene[5]);
	strcat(filename, tmp);
	strcat(filename, "h");
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) filename: %s\n", __FILE__, __LINE__, filename);
	fflush(DBG_FILE);
#endif

	if ((fp = fopen(filename, "a")) == NULL)
		return(1);
	(void)fprintf(fp, "\n\n\n");
	(void)fprintf(fp, "LANDSCAPE   		MEAN		VARIANCE	STDDEV\n");
	(void)fprintf(fp, "TA			%8.5f 	%8.5f	%8.5f\n",
	LAND_DATA.TA_MEAN, LAND_DATA.TA_VARIANCE, sqrt(LAND_DATA.TA_VARIANCE));
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) LAND  TA Mean %8.5f  var %8.5f   stddev %8.5f\n", 
__FILE__, __LINE__, LAND_DATA.TA_MEAN, LAND_DATA.TA_VARIANCE, sqrt(LAND_DATA.TA_VARIANCE));
fflush(DBG_FILE);
#endif
	(void)fprintf(fp, "LPI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.LPI_MEAN, LAND_DATA.LPI_VARIANCE, sqrt(LAND_DATA.LPI_VARIANCE));
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) LAND  LPI Mean %8.5f  var %8.5f   stddev %8.5f\n", 
__FILE__, __LINE__, LAND_DATA.LPI_MEAN, LAND_DATA.LPI_VARIANCE, sqrt(LAND_DATA.LPI_VARIANCE));
fflush(DBG_FILE);
#endif
	(void)fprintf(fp, "NP			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.NP_MEAN, LAND_DATA.NP_VARIANCE, sqrt(LAND_DATA.NP_VARIANCE));
	(void)fprintf(fp, "PD			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.PD_MEAN, LAND_DATA.PD_VARIANCE, sqrt(LAND_DATA.PD_VARIANCE));
	(void)fprintf(fp, "MPS			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MPS_MEAN, LAND_DATA.MPS_VARIANCE, sqrt(LAND_DATA.MPS_VARIANCE));
	(void)fprintf(fp, "PSSD			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.PSSD_MEAN, LAND_DATA.PSSD_VARIANCE, sqrt(LAND_DATA.PSSD_VARIANCE));
	(void)fprintf(fp, "PSCV			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.PSCV_MEAN, LAND_DATA.PSCV_VARIANCE, sqrt(LAND_DATA.PSCV_VARIANCE));
	(void)fprintf(fp, "TE			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.TE_MEAN, LAND_DATA.TE_VARIANCE, sqrt(LAND_DATA.TE_VARIANCE));
	(void)fprintf(fp, "ED			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.ED_MEAN, LAND_DATA.ED_VARIANCE, sqrt(LAND_DATA.ED_VARIANCE));
	(void)fprintf(fp, "LSI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.LSI_MEAN, LAND_DATA.LSI_VARIANCE, sqrt(LAND_DATA.LSI_VARIANCE));
	(void)fprintf(fp, "MSI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MSI_MEAN, LAND_DATA.MSI_VARIANCE, sqrt(LAND_DATA.MSI_VARIANCE));
	(void)fprintf(fp, "AWMSI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.AWMSI_MEAN, LAND_DATA.AWMSI_VARIANCE, sqrt(LAND_DATA.AWMSI_VARIANCE));
	(void)fprintf(fp, "DFLD			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.DFLD_MEAN, LAND_DATA.DFLD_VARIANCE, sqrt(LAND_DATA.DFLD_VARIANCE));
	(void)fprintf(fp, "MPFD			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MPFD_MEAN, LAND_DATA.MPFD_VARIANCE, sqrt(LAND_DATA.MPFD_VARIANCE));
	(void)fprintf(fp, "AWMPFD			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.AWMPFD_MEAN, LAND_DATA.AWMPFD_VARIANCE, sqrt(LAND_DATA.AWMPFD_VARIANCE));
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) LAND  AWMPFD Mean %8.8f  var %8.8f   stddev %8.8f\n", 
__FILE__, __LINE__, LAND_DATA.AWMPFD_MEAN, LAND_DATA.AWMPFD_VARIANCE, sqrt(LAND_DATA.AWMPFD_VARIANCE));
fflush(DBG_FILE);
#endif
	(void)fprintf(fp, "TCA			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.TCA_MEAN, LAND_DATA.TCA_VARIANCE, sqrt(LAND_DATA.TCA_VARIANCE));
	(void)fprintf(fp, "NCA			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.NCA_MEAN, LAND_DATA.NCA_VARIANCE, sqrt(LAND_DATA.NCA_VARIANCE));
	(void)fprintf(fp, "CAD			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.CAD_MEAN, LAND_DATA.CAD_VARIANCE, sqrt(LAND_DATA.CAD_VARIANCE));
	(void)fprintf(fp, "MCA1			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MCA1_MEAN, LAND_DATA.MCA1_VARIANCE, sqrt(LAND_DATA.MCA1_VARIANCE));
	(void)fprintf(fp, "CASD1			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.CASD1_MEAN, LAND_DATA.CASD1_VARIANCE, sqrt(LAND_DATA.CASD1_VARIANCE));
	(void)fprintf(fp, "CACV1			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.CACV1_MEAN, LAND_DATA.CACV1_VARIANCE, sqrt(LAND_DATA.CACV1_VARIANCE));
	(void)fprintf(fp, "MCA2			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MCA2_MEAN, LAND_DATA.MCA2_VARIANCE, sqrt(LAND_DATA.MCA2_VARIANCE));
	(void)fprintf(fp, "CASD2			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.CASD2_MEAN, LAND_DATA.CASD2_VARIANCE, sqrt(LAND_DATA.CASD2_VARIANCE));
	(void)fprintf(fp, "CACV2			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.CACV2_MEAN, LAND_DATA.CACV2_VARIANCE, sqrt(LAND_DATA.CACV2_VARIANCE));
	(void)fprintf(fp, "TCAI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.TCAI_MEAN, LAND_DATA.TCAI_VARIANCE, sqrt(LAND_DATA.TCAI_VARIANCE));
	(void)fprintf(fp, "MCAI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MCAI_MEAN, LAND_DATA.MCAI_VARIANCE, sqrt(LAND_DATA.MCAI_VARIANCE));
	(void)fprintf(fp, "MNN			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MNN_MEAN, LAND_DATA.MNN_VARIANCE, sqrt(LAND_DATA.MNN_VARIANCE));
	(void)fprintf(fp, "NNSD			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.NNSD_MEAN, LAND_DATA.NNSD_VARIANCE, sqrt(LAND_DATA.NNSD_VARIANCE));
	(void)fprintf(fp, "NNCV			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.NNCV_MEAN, LAND_DATA.NNCV_VARIANCE, sqrt(LAND_DATA.NNCV_VARIANCE));
	(void)fprintf(fp, "SHDI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.SHDI_MEAN, LAND_DATA.SHDI_VARIANCE, sqrt(LAND_DATA.SHDI_VARIANCE));
	(void)fprintf(fp, "SIDI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.SIDI_MEAN, LAND_DATA.SIDI_VARIANCE, sqrt(LAND_DATA.SIDI_VARIANCE));
	(void)fprintf(fp, "MSIDI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MSIDI_MEAN, LAND_DATA.MSIDI_VARIANCE, sqrt(LAND_DATA.MSIDI_VARIANCE));
	(void)fprintf(fp, "PR			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.PR_MEAN, LAND_DATA.PR_VARIANCE, sqrt(LAND_DATA.PR_VARIANCE));
	(void)fprintf(fp, "PRD			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.PRD_MEAN, LAND_DATA.PRD_VARIANCE, sqrt(LAND_DATA.PRD_VARIANCE));
	(void)fprintf(fp, "RPR			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.RPR_MEAN, LAND_DATA.RPR_VARIANCE, sqrt(LAND_DATA.RPR_VARIANCE));
	(void)fprintf(fp, "SHEI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.SHEI_MEAN, LAND_DATA.SHEI_VARIANCE, sqrt(LAND_DATA.SHEI_VARIANCE));
	(void)fprintf(fp, "SIEI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.SIEI_MEAN, LAND_DATA.SIEI_VARIANCE, sqrt(LAND_DATA.SIEI_VARIANCE));
	(void)fprintf(fp, "MSIEI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.MSIEI_MEAN, LAND_DATA.MSIEI_VARIANCE, sqrt(LAND_DATA.MSIEI_VARIANCE));
	(void)fprintf(fp, "IJI			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.IJI_MEAN, LAND_DATA.IJI_VARIANCE, sqrt(LAND_DATA.IJI_VARIANCE));
	(void)fprintf(fp, "CONTAG			%8.5f	%8.5f	%8.5f\n",
	LAND_DATA.CONTAG_MEAN, LAND_DATA.CONTAG_VARIANCE, sqrt(LAND_DATA.CONTAG_VARIANCE));
	fclose(fp);
	return(0);
}
/************************************************
************************************************/
best_fit_mw(mwfit_val)
float	*mwfit_val;
{
	int	nrow, ncol;
	FILE	*fp_mw;
	char	cmd[MAX_LEN], line[MAX_LEN], str[MAX_LEN], data_dir[MAX_LEN];
	char	empir_grid[MAX_LEN], model_grid[MAX_LEN];
	char	tmp[MAX_LEN];

	if (strcmp(SFX, "1k") == 0) {
		nrow = 101;
		ncol = 173;
	}
	if (strcmp(SFX, "480m") == 0) {
		nrow = 283;
		ncol = 361;
	}
	if (strcmp(SFX, "240m") == 0) {
		nrow = 577;
		ncol = 721;
	}
	if (strcmp(SFX, "120m") == 0) {
		nrow = 1000;
		ncol = 1443;
	}
	if (strcmp(SFX, "60m") == 0) {
		nrow = 1964;
		ncol = 2885;
	}
	strcpy(data_dir, DATA_DIR);
	strcat(data_dir, "/g");
	strcat(data_dir, SFX);
	strcat(data_dir, "_run");

	strcpy(empir_grid, LOG_DIR);
	strcat(empir_grid, "/seed_1995_r");
	sprintf(tmp, "%d", nrow);
	strcat(empir_grid, tmp);
	strcat(empir_grid, "_c");
	sprintf(tmp, "%d", ncol);
	strcat(empir_grid, tmp);
	strcat(empir_grid, ".asc");

	strcpy(model_grid, LOG_DIR);
	strcat(model_grid, "/urb_1995_r");
	sprintf(tmp, "%d", nrow);
	strcat(model_grid, tmp);
	strcat(model_grid, "_c");
	sprintf(tmp, "%d", ncol);
	strcat(model_grid, tmp);
	strcat(model_grid, ".asc");

	strcpy(cmd, MWFIT_PGM);
	strcat(cmd, " -r");
	sprintf(str, "%d", nrow);
	strcat(cmd, str);
	strcat(cmd, " -c");
	sprintf(str, "%d", ncol);
	strcat(cmd, str);
	strcat(cmd, " -i");
	strcat(cmd, empir_grid);
	strcat(cmd, ",");
	strcat(cmd, model_grid);
	strcat(cmd, " >mwfit.out");
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Run: >%s<\n", __FILE__, __LINE__, cmd);
	(void)fflush(DBG_FILE);
#endif
	system(cmd);
	if ((fp_mw = fopen("mwfit.out", "r")) == NULL) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) No MWFIT file\n", __FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		return(1);
	}
	if ( fgetline(line, fp_mw) >= 0 ) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) LINE: %s\n", 
		__FILE__, __LINE__, line);
		(void)fflush(DBG_FILE);
#endif
    	sscanf(line, "%f", mwfit_val); 
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) mwfit val: %4.4f\n", 
		__FILE__, __LINE__, *mwfit_val);
		(void)fflush(DBG_FILE);
#endif

	} else {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) No results for mwfit\n", __FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
	}
	fclose(fp_mw);
	return(0);
}
/************************************************
************************************************/
init_stats(elt)
struct	PASS_LIST	*elt;
{
	elt->class_urban_res.CA_VAL = 0;
	elt->class_urban_res.TA_VAL = 0;
	elt->class_urban_res.LAND_VAL = 0;
	elt->class_urban_res.LPI_VAL = 0;
	elt->class_urban_res.NP_VAL = 0;
	elt->class_urban_res.PD_VAL = 0;
	elt->class_urban_res.MPS_VAL = 0;
	elt->class_urban_res.PSSD_VAL = 0;
	elt->class_urban_res.PSCV_VAL = 0;
	elt->class_urban_res.TE_VAL = 0;
	elt->class_urban_res.ED_VAL = 0;
	elt->class_urban_res.LSI_VAL = 0;
	elt->class_urban_res.MSI_VAL = 0;
	elt->class_urban_res.AWMSI_VAL = 0;
	elt->class_urban_res.DLFD_VAL = 0;
	elt->class_urban_res.MPFD_VAL = 0;
	elt->class_urban_res.AWMPFD_VAL = 0;
	elt->class_urban_res.CLAND_VAL = 0;
	elt->class_urban_res.TCA_VAL = 0;
	elt->class_urban_res.NCA_VAL = 0;
	elt->class_urban_res.CAD_VAL = 0;
	elt->class_urban_res.MCA1_VAL = 0;
	elt->class_urban_res.CASD1_VAL = 0;
	elt->class_urban_res.CACV1_VAL = 0;
	elt->class_urban_res.MCA2_VAL = 0;
	elt->class_urban_res.CASD2_VAL = 0;
	elt->class_urban_res.CACV2_VAL = 0;
	elt->class_urban_res.TCAI_VAL = 0;
	elt->class_urban_res.MCAI_VAL = 0;
	elt->class_urban_res.MNN_VAL = 0;
	elt->class_urban_res.NNSD_VAL = 0;
	elt->class_urban_res.NNCV_VAL = 0;
	elt->class_urban_res.IJI_VAL = 0;

	elt->land_res.TA_VAL = 0;
	elt->land_res.LPI_VAL = 0;
	elt->land_res.NP_VAL = 0;
	elt->land_res.PD_VAL = 0;
	elt->land_res.MPS_VAL = 0;
	elt->land_res.PSSD_VAL = 0;
	elt->land_res.PSCV_VAL = 0;
	elt->land_res.TE_VAL = 0;
	elt->land_res.ED_VAL = 0;
	elt->land_res.LSI_VAL = 0;
	elt->land_res.MSI_VAL = 0;
	elt->land_res.AWMSI_VAL = 0;
	elt->land_res.DFLD_VAL = 0;
	elt->land_res.MPFD_VAL = 0;
	elt->land_res.AWMPFD_VAL = 0;
	elt->land_res.TCA_VAL = 0;
	elt->land_res.NCA_VAL = 0;
	elt->land_res.CAD_VAL = 0;
	elt->land_res.MCA1_VAL = 0;
	elt->land_res.CASD1_VAL = 0;
	elt->land_res.CACV1_VAL = 0;
	elt->land_res.MCA2_VAL = 0;
	elt->land_res.CASD2_VAL = 0;
	elt->land_res.CACV2_VAL = 0;
	elt->land_res.TCAI_VAL = 0;
	elt->land_res.MCAI_VAL = 0;
	elt->land_res.MNN_VAL = 0;
	elt->land_res.NNSD_VAL = 0;
	elt->land_res.NNCV_VAL = 0;
	elt->land_res.SHDI_VAL = 0;
	elt->land_res.SIDI_VAL = 0;
	elt->land_res.MSIDI_VAL = 0;
	elt->land_res.PR_VAL = 0;
	elt->land_res.PRD_VAL = 0;
	elt->land_res.RPR_VAL = 0;
	elt->land_res.SHEI_VAL = 0;
	elt->land_res.SIEI_VAL = 0;
	elt->land_res.MSIEI_VAL = 0;
	elt->land_res.IJI_VAL = 0;
	elt->land_res.CONTAG_VAL = 0;
	return(0);
}
