/*************************************************************************/
/*************************************************************************/
/*
 * Calibrate the model using a genetic algorithm. Must supply location of
 * executable and logfiles
 */

#include <stdio.h>
#include "defines.h"
#include "localdefs.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

/* 
 * External References
 */
extern int mk_gridname(char *, char *, char *);

/* 
 * Forward References
 */

/********************************************************************
********************************************************************/
run_fragstats(tag)
int		tag;
{
	char	gridname[MAX_LEN], prefix[MAX_LEN], f_gridname[MAX_LEN];
	char	OV_gridname[MAX_LEN];
	char	cmd[MAX_LEN];
	char	tmp_str[MAX_LEN];


	if (tag == MODEL) {
		/*
	 	 * run fragstats over the model 1995 output
	 	 */
		strcpy(prefix, LOG_DIR);
		strcat(prefix, "/urb");
		mk_gridname(prefix, "1995", gridname);
		
		overlay(gridname);
		strcpy(OV_gridname, gridname);
		strcat(OV_gridname, ".OV");
	
		strcpy(f_gridname, LOG_DIR);
		strcat(f_gridname, "/Mtmpgrid.xxx");
		do_frag(OV_gridname, f_gridname, "M1995");

	} else {
		if (tag == SEED) {
			/*
	 	 	 * run fragstats over the original seed
	 	 	 */
			strcpy(prefix, LOG_DIR);
			strcat(prefix, "/seed");
#ifdef DO_1912
			mk_gridname(prefix, "1912", gridname);
#endif
#ifdef DO_1955
			mk_gridname(prefix, "1955", gridname);
#endif
#ifdef DO_1975
			mk_gridname(prefix, "1975", gridname);
#endif
		
			overlay(gridname);
			strcpy(OV_gridname, gridname);
			strcat(OV_gridname, ".OV");

			strcpy(f_gridname, LOG_DIR);
			strcat(f_gridname, "/Stmpgrid.xxx");
#ifdef DO_1912
			do_frag(OV_gridname, f_gridname, "S1912");
#endif
#ifdef DO_1955
			do_frag(OV_gridname, f_gridname, "S1955");
#endif
#ifdef DO_1975
			do_frag(OV_gridname, f_gridname, "S1975");
#endif
		} else {
			/*
	 	 	 * run fragstats over the empirical 1995
	 	 	 */
			strcpy(prefix, LOG_DIR);
			strcat(prefix, "/seed");
			mk_gridname(prefix, "1995", gridname);
		
			overlay(gridname);
			strcpy(OV_gridname, gridname);
			strcat(OV_gridname, ".OV");

			strcpy(f_gridname, LOG_DIR);
			strcat(f_gridname, "/Rtmpgrid.xxx");
			do_frag(OV_gridname, f_gridname, "R1995");
		}
	}
	unlink(f_gridname);
	return(0);
}
/*****************************************************************************
*****************************************************************************/
do_frag(orig_grid, f_gridname, pref)
char	*orig_grid, *f_gridname, *pref;
{
	char	cmd[MAX_LEN];
	char	tmp_str[MAX_LEN];
	FILE	*fp_in, *fp_out;


#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter do_frag(%s, %s, %s)\n", 
	__FILE__, __LINE__, orig_grid, f_gridname, pref);
	(void)fflush(DBG_FILE);
#endif

#ifndef NORUN
	if ((fp_in = fopen(orig_grid, "r")) == NULL) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) ERROR can't open file %s\n", 
		__FILE__, __LINE__, orig_grid);
		(void)fflush(DBG_FILE);
#endif
		exit(-1);
	}
	if ((fp_out = fopen(f_gridname, "w")) == NULL) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) ERROR can't open file %s\n", 
		__FILE__, __LINE__, f_gridname);
		(void)fflush(DBG_FILE);
#endif
		exit(-1);
	}
	xfer_grid(fp_in, orig_grid, fp_out, f_gridname);

#endif /* NORUN */
	/*
	 * Run fragstats over the model output grid
	 */
	strcpy(cmd, FRAG_PRGM);
	strcat(cmd, " ");
	strcat(cmd, f_gridname);	/* input grid name */
	strcat(cmd, " ");
	strcat(cmd, LOG_DIR);		/* output prefix */
	strcat(cmd, "/");
	strcat(cmd, pref);
	strcat(cmd, " ");
	strcat(cmd, CELLSZ);     /* cell size */
	strcat(cmd, " ");
	strcat(cmd, EDGE_D);			/* edge distance */
	strcat(cmd, " ");
	strcat(cmd, " 2 ");			/* input type */
	sprintf(tmp_str, "%d", NROWS);
	strcat(cmd, tmp_str);
	strcat(cmd, " ");
	sprintf(tmp_str, "%d", NCOLS);
	strcat(cmd, tmp_str);
	strcat(cmd, " 5 ");			/* background character */
	strcat(cmd, " 2 ");			/* number of classes */
	strcat(cmd, " 1>/dev/null 2>&1");

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Run: >%s<\n", __FILE__, __LINE__, cmd);
	(void)fflush(DBG_FILE);
#endif
#ifndef NORUN
	system(cmd);
#endif

	return(0);
}
/*****************************************************************************
*****************************************************************************/
/*
 * strip out the first 6 lines of the file and copy the rest of
 * the data from the input file to the output file
 */
xfer_grid(fp_in, in_filename, fp_out, out_filename)
FILE	*fp_in, *fp_out;
char	*in_filename, out_filename;
{
	int		cnt;
	char	c;

	for (cnt = 0; cnt < 6; cnt++) {
		while ((c = getc(fp_in)) != '\n') { ;}
	}
	while ((c = getc(fp_in)) != EOF) {
		putc(c, fp_out);
	}
	return(0);
}

overlay(fname)
char	*fname;
{
	char 	unpdir[MAX_LEN];
	char	cmd[MAX_LEN];
	char	mask[MAX_LEN];
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter overlay with %s (SFX: %s)\n", 
__FILE__, __LINE__, fname, SFX);
(void)fflush(DBG_FILE);
#endif

	strcpy (mask, UNP_DIR);
	strcat (mask, "/g");
	strcat (mask, SFX);
	strcat (mask, "_unproc/g1995_");
	strcat (mask, SFX);
	strcat (mask, ".asc");

	strcpy (cmd, STRIP_PRGM);
	strcat (cmd, " -i");
	strcat (cmd, mask);
	strcat (cmd, " > " );
	strcat (cmd, LOG_DIR );
	strcat (cmd, "/mask" );
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) cmd: >%s< \n", __FILE__, __LINE__, cmd);
(void)fflush(DBG_FILE);
#endif
	system(cmd);

	strcpy (cmd, STRIP_PRGM);
	strcat (cmd, " -i");
	strcat (cmd, fname);
	strcat (cmd, " > " );
	strcat (cmd, LOG_DIR );
	strcat (cmd, "/base" );
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) cmd: >%s< \n", __FILE__, __LINE__, cmd);
(void)fflush(DBG_FILE);
#endif
	system(cmd);

	strcpy (cmd, OVERLAY_DIR);
	strcat (cmd, "/overlay_");
	strcat (cmd, SFX);
	strcat (cmd, " -i");
	strcat (cmd, LOG_DIR);
	strcat (cmd, "/base,");
	strcat (cmd, LOG_DIR);
	strcat (cmd, "/mask -o");
	strcat (cmd, LOG_DIR);
	strcat (cmd, "/tmp");

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) cmd: >%s< \n", __FILE__, __LINE__, cmd);
(void)fflush(DBG_FILE);
#endif
	system(cmd);

	strcpy (cmd, ADDEOL_PRGM);
	strcat (cmd, " -i");
	strcat (cmd, LOG_DIR);
	strcat (cmd, "/tmp > ");
	strcat (cmd, LOG_DIR);
	strcat (cmd, "/tmp2");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) cmd: >%s< \n", __FILE__, __LINE__, cmd);
(void)fflush(DBG_FILE);
#endif
	system(cmd);


	strcpy (cmd, "mv ");
	strcat (cmd, LOG_DIR);
	strcat (cmd, "/tmp2 ");
	strcat (cmd, fname);
	strcat (cmd, ".OV");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) cmd: >%s< \n", __FILE__, __LINE__, cmd);
(void)fflush(DBG_FILE);
#endif
	system(cmd);

	return(0);
}
