/*************************************************************************/
/* pixrect.c - CALIBRATION VERSION                                       */
/* converts a data file for use with sunview                             */
/* date: 6/27/90                                                         */
/* last revised: 7/2/90, added for file exchange at Ames                 */	
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>	/* added by Charles Wolff */
#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"
#define	RAS_MAGIC	0x59a66a95
#define RAS_DEPTH       32	/* Depth, defined as 1,8, or 24 */
#define BITS_IN_A_BYTE   8	/* Obvious */

/* Sun supported ras_type's */
#define RT_OLD		0	/* Raw pixrect image in 68000 byte order */
#define RT_STANDARD	1	/* Raw pixrect image in 68000 byte order */
#define RT_BYTE_ENCODED	2	/* Run-length compression of bytes */
#define RT_EXPERIMENTAL 0xffff	/* Reserved for testing */

/* Sun registered ras_maptype's */
#define RMT_RAW		0	/* No color table */
/* Sun supported ras_maptype's */
#define RMT_NONE	0	/* ras_maplength is expected to be 0 */

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

void
pixrect(outfile)
FILE *outfile;
{
	int             i, j, bit24;
	unsigned char   temp, zero;
	int             Temp;
	/*
	 * The defines and structure used here are from
	 * /usr/include/rasterfile.h
	 */
	typedef struct {
		int             ras_magic;	/* magic number */
		int             ras_width;	/* width (pixels) of image */
		int             ras_height;	/* height (pixels) of image */
		int             ras_depth;	/* depth (1, 8, or 24 bits)
						 * of pixel */
		int             ras_length;	/* length (bytes) of image */
		int             ras_type;	/* type of file; see RT_*
						 * below */
		int             ras_maptype;	/* type of colormap; see RMT_*
						 * below */
		int             ras_maplength;	/* length (bytes) of
						 * following map */
		/*
		 * color map follows for ras_maplength bytes, followed by
		 * image
		 */
		/* No color map in this version kcc */
	}           RASTERFILE;    
	RASTERFILE rastergrid, *rgrid_p;
	rgrid_p = &rastergrid;
	/*
	 * NOTES: Each line of the image is rounded out to a multiple of 16
	 * bits. This corresponds to the rounding convention used by the
	 * memory pixrect package (/usr/include/pixrect/memvar.h) of the
	 * SunWindows system. The ras_encoding field (always set to 0 by
	 * Sun's supported software) was renamed to ras_length in release
	 * 2.0.  As a result, rasterfiles of type 0 generated by the old
	 * software claim to have 0 length; for compatibility, code reading
	 * rasterfiles must be prepared to compute the true length from the
	 * width, height, and depth fields.
	 */
	/* First, set the variables in the structure */

	rastergrid.ras_magic = RAS_MAGIC;
	rastergrid.ras_depth = RAS_DEPTH;
	rastergrid.ras_type = RT_STANDARD;
	/* No color table map in this version */
	rastergrid.ras_maptype = RMT_NONE;
	rastergrid.ras_maplength = 0;
	/* Now the variables */
	/*
	 * Stopgap, if the number of columns is odd, the code doesn't work
	 * this is a very ugly fix, dated 4/95
	 */
	if ((ncols % 2) != 0) {
		/* Force even number of columns */
		for (i = 0; i < nrows; i++) {
			z[i][ncols] = 0;
		}
		ncols++;
	}
	rastergrid.ras_width = (int) ncols;
	rastergrid.ras_height = (int) nrows;
	zero = (unsigned char) 0;

	rastergrid.ras_depth = 8;
	rastergrid.ras_length = rastergrid.ras_depth * nrows * ncols;

	/*
	 * First, write out the pixrect header according to the structure
	 * type rasterfile defined in /usr/include/rasterfile.h
	 */
	fwrite(rgrid_p, sizeof(rastergrid), 1, outfile);
	/* Now just write all the data */
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			temp = (unsigned char) z[i][j];
			fwrite(&temp, sizeof(temp), 1, outfile);
		}
	}

}
