/*
 * Start with the ascii file input as generated by ArcView.
 * Depending on the kind of file - background,
 * slope, land use, etc, manipulate the entries to be of values that the
 * Clarke model expects to find.
 */
#include <stdio.h>
#include "local.h"
#define TRUE 1
#define FALSE 0

#define ORIG 1
#define PREP 0

extern char *optarg;
extern int getopt();
extern double atof();

#ifdef DEBUG
FILE *DBG_FILE;
#endif

char *orig_arr[NROW][NCOL];
char *prep_arr[NROW][NCOL];
char	DEST_TYPE[300];

main(argc, argv) 
int argc;
char *argv[];
{
	char	in_filename[300], out_filename[300];
	FILE	*fp_in, *fp_out;
	char 	c;

	/* initialize */
#ifdef DEBUG
	DBG_FILE=fopen("debug_prep", "w");
	(void)fprintf(DBG_FILE, "(%s %d) Enter Main\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	strcpy(in_filename, "");
	strcpy(out_filename, "");
	strcpy(DEST_TYPE, "");

	while ((c = getopt(argc, argv, "i:o:d:")) != EOF ) {
		switch (c) {
			case 'i':
				strcpy(in_filename, optarg);
				break;
			case 'o':
				strcpy(out_filename, optarg);
				break;
			case 'd':
				strcpy(DEST_TYPE, optarg);
				break;
			default:
				printf("bad option\n");
				exit(0);
				break;
		}
	}
	if ((strcmp(in_filename, "") == 0) 
	|| (strcmp(out_filename, "") == 0)
	|| (strcmp(DEST_TYPE, "") == 0)) {
#ifdef DEBUG
		fprintf(DBG_FILE, "Three arguments required, -i, -o and -d\n");
		fprintf(DBG_FILE, "input: %s  output: %s  dest: %s  row:%d  col:%d\n", 
		in_filename, out_filename, DEST_TYPE, NROW, NCOL);
		fflush(DBG_FILE);
#endif
		exit(0);
	}
#ifdef DEBUG
	fprintf(DBG_FILE, "input: %s  output: %s  dest: %s  row:%d  col:%d\n", 
	in_filename, out_filename, DEST_TYPE, NROW, NCOL);
	fflush(DBG_FILE);
#endif

	fp_in = fopen(in_filename, "r");
	/* writes into orig_arr */
	fill_orig_array(fp_in, in_filename, DEST_TYPE);
#ifdef DEBUG
	fprintf(DBG_FILE, "(%s %d)\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	dump_arr(ORIG);
#ifdef DEBUG
	fprintf(DBG_FILE, "(%s %d)\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	fclose(fp_in);
#ifdef DEBUG
	fprintf(DBG_FILE, "(%s %d)\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	/* convert original to prepped */
	convert_data(DEST_TYPE);
#ifdef DEBUG
	fprintf(DBG_FILE, "(%s %d)\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	dump_arr(PREP);
#ifdef DEBUG
	fprintf(DBG_FILE, "(%s %d)\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	fp_out = fopen(out_filename, "w");
#ifdef DEBUG
	fprintf(DBG_FILE, "(%s %d)\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	/* writes into prepped */
	write_output(fp_out, out_filename);
#ifdef DEBUG
	fprintf(DBG_FILE, "(%s %d)\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	fclose(fp_out);
#ifdef DEBUG
	fprintf(DBG_FILE, "(%s %d)\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	return(0);
}

fill_orig_array(fp_in, in_filename, DEST_TYPE)
FILE	*fp_in;
char	*in_filename;
char	*DEST_TYPE;
{
	int 	i, j, digit_cnt=0, tot_row, tot_col;
	char	c, word[15], new_word[15], *word_p;
	int		cur_row = 0, cur_col = 0;

#ifdef DEBUG
	fprintf(DBG_FILE, "Enter fill_orig()\n");
	fflush(DBG_FILE);
#endif
	if (fp_in == NULL) {
#ifdef DEBUG
		fprintf(DBG_FILE, "opening input %s failed\n", in_filename);
#endif
		exit(0);
	}
	/*
	 * find the next character position after the 6th newline
	 */
	for (i = 0; i < 6; i++) {
		while ((c = getc(fp_in)) != '\n') { ;}
	}

	cur_row = 0;
	cur_col = 0;

	tot_row = 0;
	tot_col = 0;

	while ((c = getc(fp_in)) != EOF) {
        if ( c == '\015' ) 
        	continue;
		if (( c == ' ') || ( c == '\n')) {
			/* end of word */
			if (digit_cnt == 0) {
				/*
				 * this is the end of the line or just a space; no word.
				 */
				continue;
			}
			word[digit_cnt]  = '\0';
#ifdef xDEBUG
			fprintf(DBG_FILE, "found word %s\n", word);
			fflush(DBG_FILE);
#endif
			if (cur_row < NROW && cur_col < NCOL) {

				/* do substitutions */
#ifdef EX_ND
			if (((strncmp(DEST_TYPE, "landuse1912", 11) == 0) 
			|| (strncmp(DEST_TYPE, "landuse1934", 11) == 0)
			|| (strncmp(DEST_TYPE, "landuse1955", 11) == 0)
			|| (strncmp(DEST_TYPE, "landuse1975", 11) == 0) 
			|| (strncmp(DEST_TYPE, "landuse1995", 11) == 0))
			&& (strcmp(word, "-9999") == 0)){
					strcpy(word, "100");
					digit_cnt = 3;
			}
#else /* it's not EX_ND */
				if ((strncmp(DEST_TYPE, "roads", 5) != 0) 
				&& (strcmp(word, "-9999") == 0)){
					strcpy(word, "0");
					digit_cnt = 1;
				}
#endif /* end EX_ND */
				/* record the new word */
				word_p = (char *)calloc(1, strlen(word)+1);
				strcpy(word_p, word);
				orig_arr[cur_row][cur_col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Write to orig row/col %d/%d (%s) \n", 
				cur_row + 1, cur_col + 1, word_p);
				fflush(DBG_FILE);
#endif
				tot_col++;
			}
			cur_col++;
			if (cur_col >= NCOL) {
				cur_row++;
				tot_row++;
				cur_col = 0;
				tot_col = 0;
			}
			digit_cnt = 0;
		}else {
			word[digit_cnt++]  = c;
		} /* it's not space or EOL */
	} /* end while */
}

/* write global prepped array into out file */
write_output(fp_out, out_filename)
FILE	*fp_out;
char	*out_filename;
{
	char	word[15], col_str[30], row_str[30], cell_str[30];
	int		row, col, k;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter write_output()\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	if (fp_out == NULL) {
#ifdef DEBUG
		fprintf(DBG_FILE, "opening output %s failed\n", out_filename);
		fflush(DBG_FILE);
#endif
		exit(0);
	}
	/* write the header info */
	sprintf(col_str, "ncols         %d", NCOL);
	sprintf(row_str, "nrows         %d", NROW);
	fprintf(fp_out, "%s\n", col_str);
	fprintf(fp_out, "%s\n", row_str);
	fprintf(fp_out,   "xllcorner     %s\n", XLL);
	fprintf(fp_out,   "yllcorner     %s\n", YLL);
	sprintf(cell_str, "cellsize      %s", CELL_SZ);
	fprintf(fp_out, "%s\n", cell_str);
#ifdef EX_ND
	if ((strncmp(DEST_TYPE, "landuse1912", 11) == 0) 
	|| (strncmp(DEST_TYPE, "landuse1934", 11) == 0)
	|| (strncmp(DEST_TYPE, "landuse1955", 11) == 0)
	|| (strncmp(DEST_TYPE, "landuse1975", 11) == 0) 
	|| (strncmp(DEST_TYPE, "landuse1995", 11) == 0))
		fprintf(fp_out, "NODATA_value  100\n");
	else
		fprintf(fp_out, "NODATA_value  0\n");
#else
		fprintf(fp_out, "NODATA_value  0\n");
#endif

	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			strcpy(word, prep_arr[row][col]);
			if (col < NCOL - 1) {
#ifdef DEBUG
				/*fprintf(DBG_FILE, "Write word %s<sp>\n", word);*/
#endif
				fprintf(fp_out, "%s ", word);
			} else {
#ifdef DEBUG
				/*fprintf(DBG_FILE, "Write word %s<cr>\n", word);*/
#endif
				fprintf(fp_out, "%s\n", word);
			}
		}
	}
}

/* turn global orig into global prepped */
convert_data(DEST_TYPE)
char	*DEST_TYPE;
{
	int		flag, row, col;
	int		i_val;
	char	*word_p;
	double	f_val;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter convert_data()\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {

#ifdef xDEBUG
	fprintf(DBG_FILE, "(%s %d)row %d col %d\n", __FILE__, __LINE__, row, col);
	fflush(DBG_FILE);
#endif
			/* for model, urban needs to be type 4, agriculture is 2 */
			if (strncmp(DEST_TYPE, "landuse1912", 11) == 0) {
				/* desert 1 ag 2 urban 3 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
#ifdef ALL_TYPES
				switch (i_val) {
					case 0:
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
					case 1:
						word_p = (char *)calloc(1, strlen("1")+1);
						strcpy(word_p, "1");
						break;
					case 2:
						word_p = (char *)calloc(1, strlen("2")+1);
						strcpy(word_p, "2");
						break;
					case 3:
						word_p = (char *)calloc(1, strlen("4")+1);
						strcpy(word_p, "4");
						break;
					case 100:
						word_p = (char *)calloc(1, strlen("100")+1);
						strcpy(word_p, "100");
						break;
					default:
#ifdef DEBUG
						(void)fprintf(DBG_FILE, 
						"(%s %d) ERROR: Unknown value\n", __FILE__, __LINE__);
#endif
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
				}
#else
				switch (i_val) {
					case 0:
					case 1:
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
					case 2:
						word_p = (char *)calloc(1, strlen("2")+1);
						strcpy(word_p, "2");
						break;
					case 3:
						word_p = (char *)calloc(1, strlen("4")+1);
						strcpy(word_p, "4");
						break;
					case 100:
						word_p = (char *)calloc(1, strlen("100")+1);
						strcpy(word_p, "100");
						break;
					default:
#ifdef DEBUG
						(void)fprintf(DBG_FILE, 
						"(%s %d) ERROR: Unknown value\n", __FILE__, __LINE__);
#endif
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
				}
#endif /* ALL_TYPES */
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if ((strncmp(DEST_TYPE, "landuse1934", 11) == 0)
			|| (strncmp(DEST_TYPE, "landuse1955", 11) == 0)) {
				/* desert 1 ag 2 urban 3 rec 4 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
#ifdef ALL_TYPES
				switch (i_val) {
					case 0:
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
					case 1:
						word_p = (char *)calloc(1, strlen("1")+1);
						strcpy(word_p, "1");
						break;
					case 4:
						word_p = (char *)calloc(1, strlen("3")+1);
						strcpy(word_p, "3");
						break;
					case 2:
						word_p = (char *)calloc(1, strlen("2")+1);
						strcpy(word_p, "2");
						break;
					case 3:
						word_p = (char *)calloc(1, strlen("4")+1);
						strcpy(word_p, "4");
						break;
					case 100:
						word_p = (char *)calloc(1, strlen("100")+1);
						strcpy(word_p, "100");
						break;
					default:
#ifdef DEBUG
						(void)fprintf(DBG_FILE, 
						"(%s %d) ERROR: Unknown value\n", __FILE__, __LINE__);
#endif
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
				}
#else
				switch (i_val) {
					case 0:
					case 1:
					case 4:
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
					case 2:
						word_p = (char *)calloc(1, strlen("2")+1);
						strcpy(word_p, "2");
						break;
					case 3:
						word_p = (char *)calloc(1, strlen("4")+1);
						strcpy(word_p, "4");
						break;
					case 100:
						word_p = (char *)calloc(1, strlen("100")+1);
						strcpy(word_p, "100");
						break;
					default:
#ifdef DEBUG
						(void)fprintf(DBG_FILE, 
						"(%s %d) ERROR: Unknown value\n", __FILE__, __LINE__);
#endif
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
				}
#endif /* ALL_TYPES */
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if ((strncmp(DEST_TYPE, "landuse1975", 11) == 0) 
			|| (strncmp(DEST_TYPE, "landuse1995", 11) == 0)) {
				/* desert 1 urban 2 ag 3 rec 4 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
#ifdef ALL_TYPES
				switch (i_val) {
					case 0:
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
					case 1:
						word_p = (char *)calloc(1, strlen("1")+1);
						strcpy(word_p, "1");
						break;
					case 4:
						word_p = (char *)calloc(1, strlen("3")+1);
						strcpy(word_p, "3");
						break;
					case 3:
						word_p = (char *)calloc(1, strlen("2")+1);
						strcpy(word_p, "2");
						break;
					case 2:
						word_p = (char *)calloc(1, strlen("4")+1);
						strcpy(word_p, "4");
						break;
					case 100:
						word_p = (char *)calloc(1, strlen("100")+1);
						strcpy(word_p, "100");
						break;
					default:
#ifdef DEBUG
						(void)fprintf(DBG_FILE, 
						"(%s %d) ERROR: Unknown value\n", __FILE__, __LINE__);
#endif
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
				}
#else
				switch (i_val) {
					case 0:
					case 1:
					case 4:
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
					case 3:
						word_p = (char *)calloc(1, strlen("2")+1);
						strcpy(word_p, "2");
						break;
					case 2:
						word_p = (char *)calloc(1, strlen("4")+1);
						strcpy(word_p, "4");
						break;
					case 100:
						word_p = (char *)calloc(1, strlen("100")+1);
						strcpy(word_p, "100");
						break;
					default:
#ifdef DEBUG
						(void)fprintf(DBG_FILE, 
						"(%s %d) ERROR: Unknown value\n", __FILE__, __LINE__);
#endif
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
				}
#endif /* ALL_TYPES */
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "recreation", 10) == 0) { 
				/* all years:  rec 4 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
#ifdef ALL_TYPES
				switch (i_val) {
					case 0:
					case 1:
					case 2:
					case 3:
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
					case 4:
						word_p = (char *)calloc(1, strlen("1")+1);
						strcpy(word_p, "1");
						break;
					default:
#ifdef DEBUG
						(void)fprintf(DBG_FILE, 
						"(%s %d) ERROR: Unknown value\n", __FILE__, __LINE__);
#endif
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
				}
#else
				switch (i_val) {
					case 0:
					case 1:
					case 2:
					case 3:
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
					case 4:
						word_p = (char *)calloc(1, strlen("1")+1);
						strcpy(word_p, "1");
						break;
					default:
#ifdef DEBUG
						(void)fprintf(DBG_FILE, 
						"(%s %d) ERROR: Unknown value\n", __FILE__, __LINE__);
#endif
						word_p = (char *)calloc(1, strlen("0")+1);
						strcpy(word_p, "0");
						break;
				}
#endif /* ALL_TYPES */
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}

			if (strncmp(DEST_TYPE, "exclusions", 10) == 0) {
				/* exclusion files, types are 0 and 1 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
				if (i_val > 0) {
					word_p = (char *)calloc(1, strlen("1")+1);
					strcpy(word_p, "1");
				} else {
					word_p = (char *)calloc(1, strlen("0")+1);
					strcpy(word_p, "0");
				}
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "rec_exclusions", 14) == 0) {
				/* exclusion files, types are 0 and 1 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
				/* all years:  rec 4 in unprocessed files */
				if (i_val == 4) {           /* recreation in land use data */
					word_p = (char *)calloc(1, strlen("1")+1);
					strcpy(word_p, "1");
				} else {
					word_p = (char *)calloc(1, strlen("0")+1);
					strcpy(word_p, "0");
				}
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "ownership", 9) == 0) {
				/* private land is type 1 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
				if (i_val > 1) {           /* other than private land */
					word_p = (char *)calloc(1, strlen("1")+1);
					strcpy(word_p, "1");
				} else {
					word_p = (char *)calloc(1, strlen("0")+1);
					strcpy(word_p, "0");
				}
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "(owner)Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "owner_nostates", 14) == 0) {
				/* private land is type 1, state trust is 2, BLM is 3 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
				if (i_val > 3) {           /* other than private, state or BLM land */
					word_p = (char *)calloc(1, strlen("1")+1);
					strcpy(word_p, "1");
				} else {
					word_p = (char *)calloc(1, strlen("0")+1);
					strcpy(word_p, "0");
				}
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "owner_noblm", 11) == 0) {
				/* private land is type 1, BLM is 3 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
				if (i_val != 1 && i_val != 3) {       /* other than private or BLM land */
					word_p = (char *)calloc(1, strlen("1")+1);
					strcpy(word_p, "1");
				} else {
					word_p = (char *)calloc(1, strlen("0")+1);
					strcpy(word_p, "0");
				}
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "owner_notrust", 13) == 0) {
				/* private land is type 1, state is 2 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
				if (i_val != 1 && i_val != 2) {       /* other than private or trust land */
					word_p = (char *)calloc(1, strlen("1")+1);
					strcpy(word_p, "1");
				} else {
					word_p = (char *)calloc(1, strlen("0")+1);
					strcpy(word_p, "0");
				}
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "zeros", 5) == 0) {
				/* make everything a zero */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
				word_p = (char *)calloc(1, strlen("0")+1);
				strcpy(word_p, "0");
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "roads", 5) == 0) {
				/* road files, types are 0 and 255 */
				i_val = atoi(orig_arr[row][col]);
				free(orig_arr[row][col]);
				if (i_val > 0) {
					word_p = (char *)calloc(1, strlen("255")+1);
					strcpy(word_p, "255");
				} else {
					word_p = (char *)calloc(1, strlen("0")+1);
					strcpy(word_p, "0");
				}
				prep_arr[row][col] = word_p;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Replace  %d/%d (%d) with (%s) \n", 
				row + 1, col + 1, i_val, word_p);
				fflush(DBG_FILE);
#endif
				continue;
			}
			if (strncmp(DEST_TYPE, "background", 10) == 0) {
				/* background files are all 0 for now */
				free(orig_arr[row][col]);
				word_p = (char *)calloc(1, strlen("0")+1);
				strcpy(word_p, "0");
				prep_arr[row][col] = word_p;
				continue;
			}
			if (strncmp(DEST_TYPE, "slope", 5) == 0) {
				/* will be floating pt value between 0 and 90 */
				f_val = atof(orig_arr[row][col]);
				free(orig_arr[row][col]);
				word_p = (char *)calloc(1, strlen("00.000")+1);
				/*
				sprintf(word_p, "%2.3f", f_val);
				*/
				sprintf(word_p, "00.000", f_val);
				prep_arr[row][col] = word_p;

				continue;
			}
		}
	}
	return(0);
}

dump_arr(flag)
int		flag;
{
#ifdef DUMP
#ifdef DEBUG
	int		row, col;

	if (flag == ORIG) {
		(void)fprintf(DBG_FILE, "Here's what orig_arr[][] looks like\n");
		for (row = 0; row < NROW; row++) {
			for (col = 0; col < NCOL; col++) {
				fprintf(DBG_FILE, "%s ", orig_arr[row][col]);
			}
			fprintf(DBG_FILE, "\n");
		}
		fflush(DBG_FILE);
	} else {
		(void)fprintf(DBG_FILE, "Here's what prep_arr[][] looks like\n");
		for (row = 0; row < NROW; row++) {
			for (col = 0; col < NCOL; col++) {
				fprintf(DBG_FILE, "%s ", prep_arr[row][col]);
			}
			fprintf(DBG_FILE, "\n");
		}
		fflush(DBG_FILE);
	}
#endif
#endif /* DUMP */
	return(0);
}
