/*
 * get the best fit of an output grid by passing a window across it and
 * comparing it to a base grid.
 */
#include <stdio.h>
#include "local.h"

extern char *optarg;
extern int getopt();
extern double atof();

int 	GRID_FLAG;
char	GRID_NAME[MAX_LEN];
char	GRID_STR[MAX_LEN];
int	NROW, NCOL, WIN_SZ, WIN_AREA, TOT_WIN;
int	grid1[MAX_ROWS][MAX_COLS];
int	grid2[MAX_ROWS][MAX_COLS];

#ifdef DEBUG
FILE *DBG_FILE;
#endif

main(argc, argv) 
int argc;
char *argv[];
{
	char	in_filename[MAX_LEN], tmp_str[MAX_LEN];
	char	map1[MAX_LEN], map2[MAX_LEN], c;
	FILE	*fp_in;
	int	row, col;

	/* initialize */
	NROW = 0;
	NCOL = 0;
	WIN_SZ = 0;
	WIN_AREA = 0;
	memset(GRID_NAME, '\0', MAX_LEN);
	memset(GRID_STR, '\0', MAX_LEN);
	GRID_FLAG = FALSE;

#ifdef DEBUG
	DBG_FILE=fopen("debug_mwfit", "w");
	(void)fprintf(DBG_FILE, "(%s %d) Enter Main\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	while ((c = getopt(argc, argv, "i:r:c:")) != EOF ) {
		switch (c) {
			case 'i':
				strcpy(GRID_STR, optarg);
				GRID_FLAG = TRUE;
				break;
			case 'r':
				strcpy(tmp_str, optarg);
				NROW = atoi(tmp_str);
				break;
			case 'c':
				strcpy(tmp_str, optarg);
				NCOL = atoi(tmp_str);
				break;
			default:
				printf("bad option\n");
				exit(0);
				break;
		}
	}
	if ((strcmp(GRID_STR, "") == 0)||(NROW == 0)||(NCOL == 0)) {
#ifdef DEBUG
		fprintf(DBG_FILE, "Arguments required, -ifile_list -rrows -ccols\n");
		fflush(DBG_FILE);
#endif
		exit(1);
	}
#ifdef DEBUG
	fprintf(DBG_FILE, "input: %s  rows: %d   cols: %d\n", 
	GRID_STR, NROW, NCOL);
	fflush(DBG_FILE);
#endif

	get_name();  /* fills in GRID_NAME */
	strcpy(map1, GRID_NAME);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) map1: %s\n", __FILE__, __LINE__, map1);
	fflush(DBG_FILE);
#endif
	get_name();  /* fills in GRID_NAME */
	strcpy(map2, GRID_NAME);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) map2: %s\n", __FILE__, __LINE__, map2);
	fflush(DBG_FILE);
#endif
	if ((strcmp(map1, "") == 0) || (strcmp(map2, "") == 0)) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Use grid: >%s<\n", __FILE__, __LINE__, GRID_NAME);
		(void)fflush(DBG_FILE);
#endif
		(void)printf("One or Both grid names are invalid\n");
		exit(1);
	}
	if ((fp_in = fopen(map1, "r")) != 0) {
		read_array(fp_in, map1, grid1);
		fclose(fp_in);
		/* dump_array(grid1); */
	} else {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Open map1 failed \n", __FILE__, __LINE__);
#endif
		(void)printf("Can't open map1\n");
		exit(1);
	}
	if ((fp_in = fopen(map2, "r")) != 0) {
		read_array(fp_in, map2, grid2);
		fclose(fp_in);
		/* dump_array(grid2); */
	} else {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Open map2 failed \n", __FILE__, __LINE__);
#endif
		(void)printf("Can't open map2\n");
		exit(1);
	}
	compare_grids(grid1, grid2);
	return(0);
}

/***************************************************/
/***************************************************/
compare_grids(grid1, grid2)
int		grid1[MAX_ROWS][MAX_COLS];
int		grid2[MAX_ROWS][MAX_COLS];
{
	double		onesize_tot, Fw, sum_numer, sum_denom, total_fit;
	int		grid1_urb, grid2_urb;
	int		grid1_ag, grid2_ag;
	int		grid1_des, grid2_des;
	int		grid1_excld, grid2_excld;
	int		grid1_nodata, grid2_nodata;
	int		m_row, m_col, row, col, MAX_SZ, total_count;
	double		k, onewin_fit;

	/* k = 0.01; */
	k = 0.0;
	sum_numer = 0.0; 
	sum_denom = 0.0;
	MAX_SZ = min(NROW, NCOL);
	WIN_SZ = 1;
	total_fit = 0.0;
	total_count = 0;
	while (WIN_SZ <= MAX_SZ) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) NEW WINDOW SIZE: %d\n",
		__FILE__, __LINE__, WIN_SZ);
		fflush(DBG_FILE);
#endif
		WIN_AREA = WIN_SZ * WIN_SZ;
		TOT_WIN = 0;
		onesize_tot = 0.0;
		/* m_row and m_col position the upper left corner of each window */
		for (m_row = 0; m_row <= NROW - WIN_SZ; m_row++)
		{
			for (m_col = 0; m_col <= NCOL - WIN_SZ; m_col++)
			{
				
				/* initialize the counts for each new window */
				grid1_urb = 0;
				grid2_urb = 0;
				grid1_ag = 0;
				grid2_ag = 0;
				grid1_des = 0;
				grid2_des = 0;
				grid1_excld = 0;
				grid2_excld = 0;
				/* count urban, ag, desert and exclusions */
				for (row = m_row; row < m_row + WIN_SZ; row++) {
					for (col = m_col; col < m_col + WIN_SZ; col++) {
						switch (grid1[row][col]) {
						case NODATA:
							grid1_nodata++;
							break;
						case REC_EX:
						case EXCLD:
							grid1_excld++;
							break;
						case AG:
							grid1_ag++;
							break;
						case URBAN:
							grid1_urb++;
							break;
						case DESERT:
							grid1_des++;
							break;
						}

						switch (grid2[row][col]) {
						case NODATA:
							grid2_nodata++;
							break;
						case EXCLD:
						case REC_EX:
							grid2_excld++;
							break;
						case AG:
							grid2_ag++;
							break;
						case URBAN:
							grid2_urb++;
							break;
						case DESERT:
							grid2_des++;
							break;
						}
					} /* end col */
				} /* end row */

#ifdef TURNER
				/* for the Turner cases, there are 3 classes */
				onewin_fit = (1.0 - ((double)(
				(abs(grid1_urb - grid2_urb) + 
				abs(grid1_ag - grid2_ag) + abs(grid1_des - grid2_des)))
				/ (2.0 * (double)WIN_AREA)));
#else
				/* 
				 * for the PHX-HILT cases, there are 4 classes, 
				 * urban, desert, agriculture and exclusions
				 */
				onewin_fit = (1.0 - 
				((double)((abs(grid1_urb - grid2_urb) + abs(grid1_ag - grid2_ag)
				+ abs(grid1_des - grid2_des) + abs(grid1_excld - grid2_excld)))
				/ (2.0 * (double)WIN_AREA)));
#endif
				TOT_WIN++;
				onesize_tot += onewin_fit;
			} /* end m_col */
		} /* end m_row */
		Fw = (onesize_tot) / (double)TOT_WIN;
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d)Fit for window sz %d is %6.6f\n", 
		__FILE__, __LINE__, WIN_SZ, Fw);
		fflush(DBG_FILE);
#endif
		(void)printf("Fit for window size of %d is %6.6f\n", WIN_SZ, Fw);

		sum_numer += Fw * log((k * -1.0) * (double)(WIN_SZ - 1));
		sum_denom += log((k * -1.0) * (double)(WIN_SZ - 1));
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d)numer:%6.6f     denom:%6.6f\n", 
		__FILE__, __LINE__, (Fw * log((k * -1.0) * (double)(WIN_SZ - 1))),
		(log((k * -1.0) * (double)(WIN_SZ - 1))));
		fflush(DBG_FILE);
#endif
		/* keep track of fit for each window size */
		total_fit += Fw;
		total_count++;
#ifdef TURNER
		WIN_SZ = WIN_SZ + 1;
		if (WIN_SZ == 11)
			WIN_SZ = 20;
#else
		if (WIN_SZ == 64)
			WIN_SZ = NROW;
		else
			WIN_SZ = WIN_SZ * 2;
#endif
	} /* continue with the next window size */

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) AVG FIT: %6.6f        WEIGHTED FIT (%1.1f): %6.6f\n",
	__FILE__, __LINE__, total_fit/(double)total_count, k, sum_numer/sum_denom);
	fflush(DBG_FILE);
#endif
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) \n",__FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	(void)printf("AVG FIT: %6.6f   WEIGHTED FIT (%1.1f): %6.6f\n",
	total_fit/(double)total_count, k, sum_numer/sum_denom);

	return(0);
}

/***************************************************/
/***************************************************/
read_array(fp_in, in_filename, grid)
FILE	*fp_in;
char	*in_filename;
int		grid[MAX_ROWS][MAX_COLS];
{
	int 	i, j, digit_cnt=0, tot_row, tot_col;
	char	c, word[15];
	int		cur_row = 0, cur_col = 0;

#ifdef xDEBUG
	fprintf(DBG_FILE, "Enter read_array()\n");
	fflush(DBG_FILE);
#endif
	if (fp_in == NULL) {
#ifdef DEBUG
		fprintf(DBG_FILE, "opening input %s failed\n", in_filename);
#endif
		exit(1);
	}
	/*
	 * for the PHX-HILT grids find the next character position after the 6th newline.
	 * Those lines contain ArcView positional information
	 */
#ifndef TURNER
	for (i = 0; i < 6; i++) {
		while ((c = getc(fp_in)) != '\n') { ;}
	}
#endif

	cur_row = 0;
	cur_col = 0;
	tot_row = 0;
	tot_col = 0;

	while ((c = getc(fp_in)) != EOF) {
        if ( c == '\015' ) 
        	continue;
		if (( c == ' ') || ( c == '\n')) {
			/* end of word */
			if (digit_cnt == 0) {
				/*
				 * this is the end of the line or just a space; no word.
				 */
				continue;
			}
			word[digit_cnt]  = '\0';
#ifdef xDEBUG
			fprintf(DBG_FILE, "found word %s\n", word);
			fflush(DBG_FILE);
#endif
			if (cur_row < NROW && cur_col < NCOL) {
				/* record the new word */
				grid[cur_row][cur_col] = atoi(word);
				tot_col++;
			}
			cur_col++;
			if (cur_col >= NCOL) {
				cur_row++;
				tot_row++;
				cur_col = 0;
				tot_col = 0;
			}
			digit_cnt = 0;
		}else {
			word[digit_cnt++]  = c;
		} /* it's not space or EOL */
	} /* end while */
	return(0);
}

/************************************************************/
/************************************************************/
dump_array(grid)
int		grid[MAX_ROWS][MAX_COLS];
{
#ifdef DEBUG
	int		row, col;
	(void)fprintf(DBG_FILE, "Here's what array looks like\n");
	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			fprintf(DBG_FILE, "%d ", grid[row][col]);
		}
		fprintf(DBG_FILE, "\n");
	}
	fflush(DBG_FILE);
#endif
	return(0);
}
/************************************************************/
/************************************************************/
get_name()
{
	char	*c_p, *out_p;
	int		cnt = 0, i = 0;

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter get_name()\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
	memset(GRID_NAME, '\0', MAX_LEN);
	if ((GRID_STR == NULL) || (*GRID_STR == '\0')) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Last grid \n", __FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		strcpy(GRID_STR, "");
		return(0);
	}
	c_p = GRID_STR;
	out_p = GRID_NAME;
	/* read until EOL or comma */
	while (*c_p != '\0' && *c_p != ',') {
		*out_p++ = *c_p++;
		cnt++;
	}
	*out_p = '\0';

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) cnt: %d\n", __FILE__, __LINE__, cnt);
	(void)fflush(DBG_FILE);
#endif
	/* if we ended because of an EOL, we are done */
	if (*c_p == '\0') {
		*GRID_STR = '\0';
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Using last name\n", __FILE__, __LINE__);
		(void)fprintf(DBG_FILE, "(%s %d) return string >%s<\n", __FILE__, __LINE__, GRID_NAME);
		(void)fflush(DBG_FILE);
#endif
		memset(GRID_STR, '\0', MAX_LEN);
		return(0);
	}
	
	/* throw out comma and move the entire string array left cnt characters */
	for (i=cnt+1; i<MAX_LEN; i++) {
		GRID_STR[i - (cnt + 1)] = GRID_STR[i];
	}
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) remainder: >%s<\n", 
	__FILE__, __LINE__, GRID_STR);
	(void)fflush(DBG_FILE);
#endif

	return(0);
}
