/*
 * find 1/2 the state lands to open
 */
#include <stdio.h>
#include "local.h"
#define TRUE 1
#define FALSE 0
#define MAX_LEN 1000

extern char *optarg;
extern int getopt();
extern double atof();

#ifdef DEBUG
FILE *DBG_FILE;
#endif

/* slope is not trimmed to county */
int slope_arr[NROW][NCOL];
/* 1995 layer, types rec, ag and urban only */
int lu_arr[NROW][NCOL];
/* all owner, trimmed to county */
int owner_arr[NROW][NCOL];
/* exclusions lay, without state trust lands, trimmed to county */
int excld_arr[NROW][NCOL];
/* place to put the rebuilt state trust lands data */
int target_arr[NROW][NCOL];

/***************************************************************************************
***************************************************************************************/
main(argc, argv) 
int argc;
char *argv[];
{
	char	out_filename[300];
	FILE	*fp_in, *fp_out;
	char 	c;
	int	row, col;

	/* initialize */
	strcpy(out_filename, "");

#ifdef DEBUG
	DBG_FILE=fopen("debug_open", "w");
	(void)fprintf(DBG_FILE, "(%s %d) Enter Main\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	while ((c = getopt(argc, argv, "o:")) != EOF ) {
		switch (c) {
			case 'o':
				strcpy(out_filename, optarg);
				break;
			default:
				printf("bad option\n");
				exit(0);
				break;
		}
	}
	if (strcmp(out_filename, "") == 0) {
#ifdef DEBUG
		fprintf(DBG_FILE, "argument required, -o \n");
		fflush(DBG_FILE);
#endif
		exit(0);
	}
#ifdef DEBUG
	fprintf(DBG_FILE, "output: %s \n", out_filename);
	fflush(DBG_FILE);
#endif

	/* fill the target_array with zeros. That's the default */
	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			target_arr[row][col] = 0;
		}
	}

	fp_in = fopen("1995use.cut", "r");
	read_array(fp_in, "1995use.cut", lu_arr);
	fclose(fp_in);

	fp_in = fopen("sub.slope", "r");
	read_array(fp_in, "sub.slope", slope_arr);
	fclose(fp_in);

	fp_in = fopen("owner.cut", "r");
	read_array(fp_in, "owner.cut", owner_arr);
	fclose(fp_in);

	fp_in = fopen("sub.ex.nostl.cut", "r");
	read_array(fp_in, "sub.ex.nostl.cut", excld_arr);
	fclose(fp_in);


	crunch();

	fp_out = fopen(out_filename, "w");
	/* writes out merged array */
	write_output(fp_out, out_filename);
	fclose(fp_out);
	return(0);
}

/***************************************************************************************
***************************************************************************************/
crunch()
{
	int	row, col;
	int	state_cnt = 0, excld_cnt = 0, conv_cnt = 0;

	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			if ((excld_arr[row][col] == 0) && (owner_arr[row][col] == 2))
				state_cnt++;
			if (excld_arr[row][col] == 1)
				excld_cnt++;
			if (excld_arr[row][col] == 1)
				target_arr[row][col] = 1;

#ifdef STOP
			/* do some converting */
			if ((owner_arr[row][col] == 2) && (lu_arr[row][col] == 0)) {
				/*
				if (search_slope(row, col, 2) == TRUE) {
					target_arr[row][col] = 1;
				}
				*/
				protect();
			}
#endif
		}
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) INIT> state:%d  excld:%d\n", 
	__FILE__, __LINE__, state_cnt, excld_cnt);
	fflush(DBG_FILE);
#endif
	protect();

	/* print the final tally */
	state_cnt = excld_cnt = conv_cnt = 0;
	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			if ((target_arr[row][col] == 0) && (owner_arr[row][col] == 2))
				state_cnt++;
			if ((target_arr[row][col] == 1) && (owner_arr[row][col] == 2) 
			&& (lu_arr[row][col] == 0)) {    /* state parks excluded */
				conv_cnt++;
			}
			if (excld_arr[row][col] == 1)
				excld_cnt++;
		}
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) FINAL> state:%d  excld:%d  conv:%d\n", 
	__FILE__, __LINE__, state_cnt, excld_cnt, conv_cnt);
	fflush(DBG_FILE);
#endif
	return(0);
}
/***************************************************************************************
***************************************************************************************/
protect()
{
	int	row, col, loop = TRUE, cpass, r, c, pass_cnt = 0, change_cnt, 
		sum, toggle, tot_changed = 0;
	
	while (loop) {
		pass_cnt++;
		change_cnt = 0;
		cpass = FALSE;
		for (row = 0; row < NROW; row++) {
			for (col = 0; col < NCOL; col++) {
				/* unprotected state land, not a park already */
				if ((target_arr[row][col] == 0) && (owner_arr[row][col] == 2) 
				&& (lu_arr[row][col] == 0)) {
					/* see if there are protected lands adjacent */
					sum = 0;
					for (r = row-1; r <= row+1; r++) {
						for (c = col-1; c <= col+1; c++) {
							if (r>=0 && r<NROW && c>=0 && c<NCOL) {
								if ((target_arr[r][c] == 1) && valid(r,c))
									sum++;
							}
							toggle = flip();
							if ((toggle == 0) && (tot_changed < 3300)) {
								if ((sum > 2) && (target_arr[row][col] == 0)){
									target_arr[row][col] = 1;
									change_cnt++;
									tot_changed++;
									cpass = TRUE;
								}
							} 
							if (toggle == 1) {
								if ((sum > 3) && (target_arr[row][col] == 0)){
									target_arr[row][col] = 1;
									change_cnt++;
									tot_changed++;
									cpass = TRUE;
								}
							} 
						}
					}
				}
			}
		}
		/* if there were no changes on this pass, finish */
		if (cpass == FALSE)
			loop = FALSE;
		else {
#ifdef xDEBUG
			(void)fprintf(DBG_FILE, "(%s %d) Pass %d, %d changes\n", 
			__FILE__, __LINE__, pass_cnt, change_cnt);
			fflush(DBG_FILE);
#endif
		}
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Took %d pass(es)\n", __FILE__, __LINE__, pass_cnt-1);
	fflush(DBG_FILE);
#endif
	return(0);
}
/***************************************************************************************
***************************************************************************************/
search_slope(r, c, dist)
{
	int	row, col, answer = FALSE, slp_cnt = 0;
	int	slp_limit = 4, slp_max = 10;

	for (row = r-dist; row <= r+dist; row++) {
		if (row < 0 || row >= NROW)
			continue;   /* out of bounds */
		for (col = c-dist; col <= c+dist; col++) {
			if (col < 0 || col >= NCOL)
				continue;   /* out of bounds */
			if (slope_arr[row][col] >= slp_max)
				slp_cnt++;
		}
	}
	if (slp_cnt >= slp_limit) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) %d+ cells within dist of %d from %d,%d have slope > %d\n",
		__FILE__, __LINE__, slp_limit, dist, r, c, slp_max);
		fflush(DBG_FILE);
#endif
		return(TRUE);
	}
	return(FALSE);
}

/***************************************************************************************
***************************************************************************************/
read_array(fp_in, in_filename, array)
FILE	*fp_in;
char	*in_filename;
int	array[NROW][NCOL];
{
	int 	i, j, digit_cnt=0, tot_row, tot_col;
	char	c, word[15];
	int	cur_row = 0, cur_col = 0;
	int	word_val;

#ifdef DEBUG
	fprintf(DBG_FILE, "Enter read_array()\n");
	fflush(DBG_FILE);
#endif
	if (fp_in == NULL) {
#ifdef DEBUG
		fprintf(DBG_FILE, "opening input %s failed\n", in_filename);
#endif
		exit(0);
	}
	/*
	 * find the next character position after the 6th newline
	 */
	for (i = 0; i < 6; i++) {
		while ((c = getc(fp_in)) != '\n') { ;}
	}

	cur_row = 0;
	cur_col = 0;

	tot_row = 0;
	tot_col = 0;

	while ((c = getc(fp_in)) != EOF) {
        if ( c == '\015' ) 
        	continue;
		if (( c == ' ') || ( c == '\n')) {
			/* end of word */
			if (digit_cnt == 0) {
				/*
				 * this is the end of the line or just a space; no word.
				 */
				continue;
			}
			word[digit_cnt]  = '\0';
#ifdef xDEBUG
			fprintf(DBG_FILE, "found word %s\n", word);
			fflush(DBG_FILE);
#endif
			if (cur_row < NROW && cur_col < NCOL) {

				/* record the new word */
				word_val = atoi(word);
				array[cur_row][cur_col] = word_val;
#ifdef xDEBUG
				fprintf(DBG_FILE, "Write to array row/col %d/%d \n", 
				cur_row + 1, cur_col + 1);
				fflush(DBG_FILE);
#endif
				tot_col++;
			}
			cur_col++;
			if (cur_col >= NCOL) {
				cur_row++;
				tot_row++;
				cur_col = 0;
				tot_col = 0;
			}
			digit_cnt = 0;
		}else {
			word[digit_cnt++]  = c;
		} /* it's not space or EOL */
	} /* end while */
}

/***************************************************************************************
***************************************************************************************/
/* write global target array into out file */
write_output(fp_out, out_filename)
FILE	*fp_out;
char	*out_filename;
{
	char	col_str[30], row_str[30], cell_str[30];
	int		row, col, k;
	int		word_val;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter write_output()\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	if (fp_out == NULL) {
#ifdef DEBUG
		fprintf(DBG_FILE, "opening output %s failed\n", out_filename);
#endif
		exit(0);
	}
	/* write the header info */
	sprintf(col_str, "ncols         %d", NCOL);
	sprintf(row_str, "nrows         %d", NROW);
	fprintf(fp_out, "%s\n", col_str);
	fprintf(fp_out, "%s\n", row_str);
	fprintf(fp_out,   "xllcorner     %s\n", XLL);
	fprintf(fp_out,   "yllcorner     %s\n", YLL);
	sprintf(cell_str, "cellsize      %s", CELL_SZ);
	fprintf(fp_out, "%s\n", cell_str);
	fprintf(fp_out, "NODATA_value  0\n");

	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			word_val = target_arr[row][col];
			if (col < NCOL - 1) {
#ifdef xDEBUG
				/*fprintf(DBG_FILE, "Write word %d<sp>\n", word_val);*/
#endif
				fprintf(fp_out, "%d ", word_val);
			} else {
#ifdef xDEBUG
				/*fprintf(DBG_FILE, "Write word %d<cr>\n", word_val);*/
#endif
				fprintf(fp_out, "%d\n", word_val);
			}
		}
	}
}

/***************************************************************************************
***************************************************************************************/
dump_arr(flag, array, ar_name)
int	flag;
char	*array, *ar_name;
{
	int	row, col;
#ifdef DUMP
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "Here's what %s array looks like\n", ar_name);
	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			fprintf(DBG_FILE, "%s ", array[row][col]);
		}
		fprintf(DBG_FILE, "\n");
	}
	fflush(DBG_FILE);
#endif
#endif /* DUMP */
	return(0);
}

/*****************************************************************
*****************************************************************/
int
flip()
{
	/* returns 0 or 1 */
	return(rand() % 2);   
}
	
/*****************************************************************
*****************************************************************/
int
valid(row,col)
int	row, col;
{
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) val: %d\n", __FILE__, __LINE__, owner_arr[row][col]);
	fflush(DBG_FILE);
#endif
	switch (owner_arr[row][col]) {
		case 3:
		case 8:
		case 14:
		case 71:
		case 72:
		case 86:
		case 21:
		case 18:
		case 22:
			return(TRUE);
			break;
		default:
			if ((target_arr[row][col] == 1) && (owner_arr[row][col] == 2) 
			&& (lu_arr[row][col] == 0)) {
				return(TRUE);
			}
			return(FALSE);
			break;
	}
}
