/*
 * Take the first grid and fill in the new array. Take the second array and 
 * anywhere a 0 exists, put a -1 in the new array
 */
#include <stdio.h>
#include "local.h"
#define TRUE 1
#define FALSE 0
#define MAX_LEN 1000

#define DEFAULT_NUM	1

extern char *optarg;
extern int getopt();
extern double atof();

int 	GRID_FLAG;
int 	PARK_TAG;
int 	KEEP_ZERO;
char	GRID_NAME[MAX_LEN];
char	GRID_STR[MAX_LEN];
int		use_num = DEFAULT_NUM;
int		USE_ORIG_VAL = FALSE;

#ifdef DEBUG
FILE *DBG_FILE;
#endif

int prep_arr[NROW][NCOL];

main(argc, argv) 
int argc;
char *argv[];
{
	char	in_filename[300], out_filename[300], dest_type[300];
	FILE	*fp_in, *fp_out;
	char 	c, use_num_str[10];
	int		argc_rem, row, col;

	/* initialize */
	memset(GRID_NAME, '\0', MAX_LEN);
	memset(GRID_STR, '\0', MAX_LEN);
	GRID_FLAG = FALSE;
	KEEP_ZERO = FALSE;
	PARK_TAG = FALSE;
	strcpy(out_filename, "");
	argc_rem = argc;

#ifdef DEBUG
	DBG_FILE=fopen("debug_overlay", "w");
	(void)fprintf(DBG_FILE, "(%s %d) Enter Main\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	strcpy(dest_type, "");

	while ((c = getopt(argc, argv, "i:o:f")) != EOF ) {
		switch (c) {
			case 'i':
				strcpy(GRID_STR, optarg);
				GRID_FLAG = TRUE;
				argc_rem -= 2;
				break;
			case 'o':
				strcpy(out_filename, optarg);
				break;
			case 'f':
				PARK_TAG = TRUE;
				break;
			default:
				printf("bad option\n");
				exit(0);
				break;
		}
	}
	if ((strcmp(GRID_STR, "") == 0) 
	|| (strcmp(out_filename, "") == 0)) {
#ifdef DEBUG
		fprintf(DBG_FILE, "Two arguments required, -i and -o \n");
		fflush(DBG_FILE);
#endif
		exit(0);
	}
#ifdef DEBUG
	fprintf(DBG_FILE, "input: %s  output: %s  num: %d\n", 
	GRID_STR, out_filename, use_num);
	fflush(DBG_FILE);
#endif

	/* fill the prep_array with zeros. That's the default */
	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			prep_arr[row][col] = 0;
		}
	}
	get_name();
	while (strcmp(GRID_NAME, "") != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Use grid: >%s<\n", __FILE__, __LINE__, GRID_NAME);
		(void)fflush(DBG_FILE);
#endif

		fp_in = fopen(GRID_NAME, "r");
		merge_array(fp_in, GRID_NAME);
		fclose(fp_in);

		get_name();
	}
	fp_out = fopen(out_filename, "w");
	/* writes out merged array */
	write_output(fp_out, out_filename);
	fclose(fp_out);
	return(0);
}

merge_array(fp_in, in_filename)
FILE	*fp_in;
char	*in_filename;
{
	int 	i, j, digit_cnt=0, tot_row, tot_col;
	char	c, word[15];
	int		cur_row = 0, cur_col = 0;
	int		word_val;
	static	int pass = 1;

#ifdef DEBUG
	fprintf(DBG_FILE, "Enter merge_array()\n");
	fflush(DBG_FILE);
#endif
	if (fp_in == NULL) {
#ifdef DEBUG
		fprintf(DBG_FILE, "opening input %s failed\n", in_filename);
#endif
		exit(0);
	}
	/*
	 * find the next character position after the 6th newline
	 */
	for (i = 0; i < 6; i++) {
		while ((c = getc(fp_in)) != '\n') { ;}
	}

	cur_row = 0;
	cur_col = 0;

	tot_row = 0;
	tot_col = 0;

	while ((c = getc(fp_in)) != EOF) {
        if ( c == '\015' ) 
        	continue;
		if (( c == ' ') || ( c == '\n')) {
			/* end of word */
			if (digit_cnt == 0) {
				/*
				 * this is the end of the line or just a space; no word.
				 */
				continue;
			}
			word[digit_cnt]  = '\0';
#ifdef xDEBUG
			fprintf(DBG_FILE, "found word %s\n", word);
			fflush(DBG_FILE);
#endif
			if (cur_row < NROW && cur_col < NCOL) {

				if (strcmp(word, "-9999") == 0){
					strcpy(word, "0");
					digit_cnt = 1;
				}
				/* record the new word */
				word_val = atoi(word);
				if (pass == 1) {
					prep_arr[cur_row][cur_col] = word_val;
					use_num = word_val;
				} else {
					/* this would be NO DATA in the original grids */
					if (word_val == 0)
						prep_arr[cur_row][cur_col] = -1;
						use_num = -1;
					/* this would be recreation in the original grids */
					if (word_val == 4)
						prep_arr[cur_row][cur_col] = 1;
						use_num = 1;
				}
#ifdef xDEBUG
				fprintf(DBG_FILE, "Write to prep row/col %d/%d (%d) \n", 
				cur_row + 1, cur_col + 1, use_num);
				fflush(DBG_FILE);
#endif
				tot_col++;
			}
			cur_col++;
			if (cur_col >= NCOL) {
				cur_row++;
				tot_row++;
				cur_col = 0;
				tot_col = 0;
			}
			digit_cnt = 0;
		}else {
			word[digit_cnt++]  = c;
		} /* it's not space or EOL */
	} /* end while */
	pass++;
}

/* write global prepped array into out file */
write_output(fp_out, out_filename)
FILE	*fp_out;
char	*out_filename;
{
	char	col_str[30], row_str[30], cell_str[30];
	int		row, col, k;
	int		word_val;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter write_output()\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	if (fp_out == NULL) {
#ifdef DEBUG
		fprintf(DBG_FILE, "opening output %s failed\n", out_filename);
#endif
		exit(0);
	}
	/* write the header info */
	sprintf(col_str, "ncols         %d", NCOL);
	sprintf(row_str, "nrows         %d", NROW);
	fprintf(fp_out, "%s\n", col_str);
	fprintf(fp_out, "%s\n", row_str);
	fprintf(fp_out,   "xllcorner     %s\n", XLL);
	fprintf(fp_out,   "yllcorner     %s\n", YLL);
	sprintf(cell_str, "cellsize      %s", CELL_SZ);
	fprintf(fp_out, "%s\n", cell_str);
	fprintf(fp_out, "NODATA_value  -1\n");

	for (row = 0; row < NROW; row++) {
		for (col = 0; col < NCOL; col++) {
			word_val = prep_arr[row][col];
			if (col < NCOL - 1) {
#ifdef xDEBUG
				/*fprintf(DBG_FILE, "Write word %d<sp>\n", word_val);*/
#endif
				fprintf(fp_out, "%d ", word_val);
			} else {
#ifdef xDEBUG
				/*fprintf(DBG_FILE, "Write word %d<cr>\n", word_val);*/
#endif
				fprintf(fp_out, "%d\n", word_val);
			}
		}
	}
}

dump_arr(flag)
int		flag;
{
#ifdef DUMP
#ifdef DEBUG
	int		row, col;

	if (flag == ORIG) {
		(void)fprintf(DBG_FILE, "Here's what orig_arr[][] looks like\n");
		for (row = 0; row < NROW; row++) {
			for (col = 0; col < NCOL; col++) {
				fprintf(DBG_FILE, "%s ", orig_arr[row][col]);
			}
			fprintf(DBG_FILE, "\n");
		}
		fflush(DBG_FILE);
	} else {
		(void)fprintf(DBG_FILE, "Here's what prep_arr[][] looks like\n");
		for (row = 0; row < NROW; row++) {
			for (col = 0; col < NCOL; col++) {
				fprintf(DBG_FILE, "%s ", prep_arr[row][col]);
			}
			fprintf(DBG_FILE, "\n");
		}
		fflush(DBG_FILE);
	}
#endif
#endif /* DUMP */
	return(0);
}
get_name()
{
	char	*c_p, *out_p;
	int		cnt = 0, i = 0;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter get_name(%s)\n", __FILE__, __LINE__, GRID_STR);
	(void)fflush(DBG_FILE);
#endif
	memset(GRID_NAME, '\0', MAX_LEN);
	if ((GRID_STR == NULL) || (*GRID_STR == '\0')) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Last grid \n", __FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		strcpy(GRID_STR, "");
		return(0);
	}
	c_p = GRID_STR;
	out_p = GRID_NAME;
	/* read until EOL or comma */
	while (*c_p != '\0' && *c_p != ',') {
		*out_p++ = *c_p++;
		cnt++;
	}
	*out_p = '\0';

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) cnt: %d\n", __FILE__, __LINE__, cnt);
	(void)fflush(DBG_FILE);
#endif
	/* if we ended because of an EOL, we are done */
	if (*c_p == '\0') {
		*GRID_STR = '\0';
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Using last name\n", __FILE__, __LINE__);
		(void)fprintf(DBG_FILE, "(%s %d) return string >%s<\n", __FILE__, __LINE__, GRID_NAME);
		(void)fflush(DBG_FILE);
#endif
		memset(GRID_STR, '\0', MAX_LEN);
		return(0);
	}
	
	/* throw out comma and move the entire string array left cnt characters */
	for (i=cnt+1; i<MAX_LEN; i++) {
		GRID_STR[i - (cnt + 1)] = GRID_STR[i];
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) remainder: >%s<\n", 
	__FILE__, __LINE__, GRID_STR);
	(void)fflush(DBG_FILE);
#endif

	return(0);
}
