/*************************************************************************/
/* Main.c - PREDICTION VERSION                                          */
/* last revised: 11/02 sbw
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include <signal.h>
#include <sys/stat.h>
#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"


#ifdef DEBUG
FILE *DBG_FILE;
#endif

/* program name */
static char *progname;

extern char 	*optarg;
extern int	getopt();
extern int	run_fragstats();
/* for some reason, this compiler will core dump unless this is global. Size? */
int 		cumulate[NROWS][NCOLS];

/* forward references */
int	checksubs(int, int, char *, int);

main(argc, argv)
int 	argc;
char	*argv[];
{
	void            growth(), pixrect();
	char            c, label[30], junk1[25], junk2[25], junk3[25], 
			junk4[25], junk5[25], junk6[25], junk7[25], 
			junk8[25], junk9[25], junk10[25], junk11[25], 
			junk12[25], junk13[25], junk14[25];
	int             k, time, i, j;
	FILE		*control, *cumul_file;
	int 		temp1, temp2;
	float		temp3, temp4, temp5, temp6, temp7, temp8, temp9, temp10, temp11;
	int 		temp12, temp13, temp14;
	int		errflag, argc_rem;
	char    	fname[MAX_LEN];

#ifdef DEBUG
	DBG_FILE=fopen("./debug_file", "w");
#endif
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter main\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
	/* initialize */
	errflag = 0;
	WEX_FLAG = FALSE;
	DATA_FLAG = LOG_FLAG = FALSE;
	argc_rem = argc;
	memset(DATA_DIR, '\0', MAX_LEN);
	memset(LOG_DIR, '\0', MAX_LEN);
	memset(OPEN_TAG, '\0', MAX_LEN);
	memset(RATE_TAG, '\0', MAX_LEN);
	AGRI_FACTOR = AGRI_DEFAULT;
	MEAN_FACTOR = MEAN_DEFAULT;
	NEI_RQMT = NEI_RQMT_DEF;
	NEI_HOOD = NEI_HOOD_DEF;
	ABOUT_FULL = FALSE;
	MAGIC_YEAR = MAGIC_YEAR_DEF;
	OPEN_TYPE = OPENALL;
	RATE_TYPE = FIXED;
	PICKMAX = 100;
	BRIEF = FALSE;
	DONE_SCENARIO = FALSE;

	/* process the command line */
	while ((c = getopt(argc, argv, "qd:l:g:r:h:s:m:t:")) != EOF ) { switch (c) {
		case 'd':
			strcpy(DATA_DIR, optarg);
			DATA_FLAG = TRUE;
			argc_rem -= 2;
			break;
		case 'l':
			strcpy(LOG_DIR, optarg);
			LOG_FLAG = TRUE;
			argc_rem -= 2;
			break;
		case 'g':
			AGRI_FACTOR = atoi(optarg);
			argc_rem -= 2;
			break;
		case 'r':
			NEI_RQMT = atoi(optarg);
			argc_rem -= 2;
			break;
		case 'w':
			WEX_FLAG = TRUE;
			argc_rem -= 2;
			break;
		case 'h':
			NEI_HOOD = atoi(optarg);
			argc_rem -= 2;
			break;
		case 'm':
			MAGIC_YEAR = atoi(optarg);
			argc_rem -= 2;
			break;
		case 'q':
			BRIEF = TRUE;
			argc_rem -= 2;
			break;
		case 's':
			strcpy(OPEN_TAG, optarg);
			argc_rem -= 2;
			break;
		case 't':
			strcpy(RATE_TAG, optarg);
			argc_rem -= 2;
			break;
		case '?':
			errflag++;
			break;
		} /* end switch */
		/* if there are errors, print the usage message and quit */
		if ( errflag ) {
			fprintf(stderr, "usage: \n\n");
 			fprintf(stderr, "	-d	name of directory where input files reside\n");
 			fprintf(stderr, "	-l	name of directory where logfiles will go\n");
 			fprintf(stderr, "	-q	Quit running when cells are really hard to find\n");
 			fprintf(stderr, "	-m	year to open state trust lands\n");
 			fprintf(stderr, "	-s	percent state lands to open: all|half|none\n");
 			fprintf(stderr, "	-t	growth rate action: fixed|dcons|dultra|custom\n");
 			fprintf(stderr, "	-r	# cells reqd urban to consider spread\n");
 			fprintf(stderr, "	-h	how far out cells are considered neighbors\n");
 			fprintf(stderr, "	-t	can leap frog up to this many cells away\n");
 			fprintf(stderr, "	-g	default value for agriculture (0-99)\n\n");
			exit(1);
		}
	}
	if (MAGIC_YEAR < 1995 || MAGIC_YEAR > 2099) {
		(void)printf("Bad magic year (%d)\n", MAGIC_YEAR);
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Bad magic_year (%d)\n", 
	        __FILE__, __LINE__, MAGIC_YEAR);
#endif
		exit(1);
	}
	if (strcmp(OPEN_TAG, "") != 0) {
		if ((strcmp(OPEN_TAG, "all") == 0) || (strcmp(OPEN_TAG, "a") == 0)) 
			OPEN_TYPE = OPENALL;
		else {
			if ((strcmp(OPEN_TAG, "half") == 0) || (strcmp(OPEN_TAG, "h") == 0)) 
				OPEN_TYPE = OPENHALF;
			else {
				if ((strcmp(OPEN_TAG, "none") == 0) || (strcmp(OPEN_TAG, "n") == 0)) 
					OPEN_TYPE = OPENNONE;
				else {
					(void)printf("Bad open type (%s)\n", OPEN_TAG);
#ifdef DEBUG
					(void)fprintf(DBG_FILE, "(%s %d) Bad open type (%s)\n", 
	        			__FILE__, __LINE__, OPEN_TAG);
#endif
					exit(1);
				}
			}
		}
	}
	if (strcmp(RATE_TAG, "") != 0) {
		if ((strcmp(RATE_TAG, "fixed") == 0) || (strcmp(RATE_TAG, "f") == 0)) 
			RATE_TYPE = FIXED;
		else {
			if ((strcmp(RATE_TAG, "dcons") == 0) || (strcmp(RATE_TAG, "c") == 0)) 
				RATE_TYPE = DCONS;
			else {
				if ((strcmp(RATE_TAG, "dultra") == 0) || (strcmp(RATE_TAG, "u") == 0)) 
					RATE_TYPE = DULTRA;
				else {
					if (strcmp(RATE_TAG, "custom") == 0) 
						RATE_TYPE = CUSTOM;
					else {
						(void)printf("Bad rate type (%s)\n", RATE_TAG);
#ifdef DEBUG
						(void)fprintf(DBG_FILE, "(%s %d) Bad rate type (%s)\n", 
	        				__FILE__, __LINE__, RATE_TAG);
#endif
						exit(1);
					}
				}
			}
		}
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) \n", __FILE__, __LINE__);
	(void)fprintf(DBG_FILE, "	DATA: %s   LOG: %s\n",
	DATA_DIR, LOG_DIR);
	(void)fprintf(DBG_FILE, "	NEI_RQMT: %d   NEI_HOOD: %d\n",
	NEI_RQMT, NEI_HOOD);
	(void)fprintf(DBG_FILE, "	MAGIC_YEAR: %d   OPEN_TYPE: %d   RATE_TYPE: %d\n",
	MAGIC_YEAR, OPEN_TYPE, RATE_TYPE);
	(void)fflush(DBG_FILE);
#endif

	if (verify_tools_and_dirs() != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Missing something we need. Exit.\n",
			      __FILE__, __LINE__);
#endif
		exit(0);
	}

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Read control file\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
		/* read in values from control to initialize the parameters */
		if ((control = fopen("control", "r")) == NULL) {
			printf("no control file\n");
			exit(1);
		}
		fscanf(control, "%d%s%*1c %d%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %d%s%*1c %d%s%*1c %d%s%*1c",&temp1, junk1, &temp2, junk2, &temp3, junk3, &temp4, junk4, &temp5, junk5, &temp6, junk6, &temp7, junk7, &temp8, junk8, &temp9, junk9, &temp10, junk10, &temp11, junk11, &temp12, junk12, &temp13, junk13, &temp14, junk14);

#ifdef DEBUG
(void)fprintf(DBG_FILE, "%d %s \n", temp1, junk1);
(void)fprintf(DBG_FILE, "%d %s \n", temp2, junk2);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp3, junk3);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp4, junk4);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp5, junk5);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp6, junk6);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp7, junk7);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp8, junk8);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp9, junk9);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp10, junk10);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp11, junk11);
(void)fprintf(DBG_FILE, "%d %s \n", temp12, junk12);
(void)fprintf(DBG_FILE, "%d %s \n", temp13, junk13);
(void)fprintf(DBG_FILE, "%d %s \n", temp14, junk14);
(void)fflush(DBG_FILE);
#endif

	       startstop.start_date = temp1;
	       startstop.stop_date = temp2;
	       param.diffusion_coefficient = (double) temp3;
	       param.breed_coefficient = (double) temp4;
	       param.spread_coefficient = (double) temp5;
	       param.slope_resistance = (double) temp6;
	       param.road_gravity = (double) temp7;
	       selfmod.critical_high = (float)temp8;
	       selfmod.critical_low = (float)temp9;
	       selfmod.boom = (float)temp10;
	       selfmod.bust = (float)temp11;
	       selfmod.critical_slope = temp12;
	       montecarlo.number_of_times = temp13;
	       montecarlo.cumulate = temp14;
		fclose (control);

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) randomize with process ID\n", 
__FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* Seed the random number generator with the process id number */
	srand(getpid()); 

	/* start the simulation */
	for (time = 0; time < montecarlo.number_of_times; time++) {
		(void) growth();
	}

	do_overlay(SEED, 0, NULL);
	/*
	do_overlay(MODEL, startstop.stop_date, NULL);
	*/
	return (0);
}

/* fgetline: get a line from a file
 * Assumes the line array is long enough for the input and that the file
 * is open.
 */
int fgetline(line, fp)
char 	*line;
FILE	*fp;
{

	char	c;
	int	count = 0;

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter fgetline()\n", __FILE__, __LINE__);
#endif
	while (c = getc(fp))
	{
		*line = (char)c;
		if (*line == EOL || *line == EOF) break ;
		++line ;
		++count;
	}
	*line = '\0';
	/* return the length of the line */
	if (c == EOF) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) return -1 \n", __FILE__, __LINE__);
#endif
		return(-1);
	} else {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) return %d\n", 
		__FILE__, __LINE__, count);
#endif
		return(count);
	}
}

checksubs(row, col, file_loc, line_num)
int		row, col, line_num;
char	*file_loc;
{
	if (row < 0 || row >= NROWS) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) NOTE row out of bounds\n",
		file_loc, line_num);
		(void)fflush(DBG_FILE);
#endif
		return(-1);
	}
	if (col < 0 || col >= NCOLS) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) NOTE column out of bounds\n",
		file_loc, line_num);
		(void)fflush(DBG_FILE);
#endif
		return(-1);
	}
	return(0);
}

/************************************************************************
************************************************************************/
verify_tools_and_dirs() 
{
	char    tmp_fname[MAX_LEN];
	struct 	stat stat_buf;
	int	ret;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter verify_tools()\n", __FILE__, __LINE__);
#endif
	/* check OVERLAY_PRGM */
	strcpy(tmp_fname, OVERLAY_DIR);
	strcat(tmp_fname, "/overlay_");
	strcat(tmp_fname, SFX);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, tmp_fname);
#endif
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check UNP_DIR */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, UNP_DIR);
#endif
	strcpy(tmp_fname, UNP_DIR);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check DATA_DIR */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, DATA_DIR);
#endif
	strcpy(tmp_fname, DATA_DIR);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check LOG_DIR */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, LOG_DIR);
#endif
	strcpy(tmp_fname, LOG_DIR);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check FRAG_PRGM */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, FRAG_PRGM);
#endif
	strcpy(tmp_fname, FRAG_PRGM);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check STRIP_PGM */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, STRIP_PRGM);
#endif
	strcpy(tmp_fname, STRIP_PRGM);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}
	/* check ADDEOL_PGM */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, ADDEOL_PRGM);
#endif
	strcpy(tmp_fname, ADDEOL_PRGM);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}
	return(0);
}
/*****************************************************************
*****************************************************************/
lost_tool_msg(file)
char	*file;
{
	(void)fprintf(stderr, "\n%s could not be found. Either change what is being \n", file);
	(void)fprintf(stderr, "looked for (likely place to look is localdefs.h or the command line)\n");
	(void)fprintf(stderr, "or create the directory/utility that is required\n\n");
	return(0);
}
