/*************************************************************************/
/* growth.c - PREDICTION VERSION                                         */
/* last revised: 11/13 SBW						 */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

/* 
 * If you want to customize the growth rate and density rate formulas, search for the
 * word CUSTOM in this file. There will be 2 places, one for the density and one for
 * rate. If you make no source code changes but use the custom option, the defaults are
 * to keep a constant rate and density.
 *
 * The functions to be concerned with are get_est_density() and calc_new_rate()
 */

#include <stdio.h>
#include <string.h>
#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

unsigned short int       roads[NROWS][NCOLS];
unsigned short int       slp[NROWS][NCOLS];
unsigned short int       excld[NROWS][NCOLS];
unsigned short int       new_parks[NROWS][NCOLS];
unsigned short int       bkgrn[NROWS][NCOLS];
unsigned short int       seed[NROWS][NCOLS];
unsigned short int       urban[NROWS][NCOLS];
unsigned short int       my_lone[NROWS][NCOLS];
unsigned short int       grd[NROWS][NCOLS];  /* generic grid */
unsigned short int       grd2[NROWS][NCOLS];  /* generic grid */
unsigned short int       grd3[NROWS][NCOLS];  /* generic grid */

char            tmp_gname[MAX_LEN];

int		years_remain = 0;
short int       elev_max;
extern int      nrows, ncols;
int             num_growth_pix;
float           growth_rate, last_growth_rate, percent_road, percent_urban;
int             roadcount, noncount;
float           average_slope;
int             change_all;
double		calc_rate(), calc_new_rate();
int		max_urban;

float		INIT_RATE;
int		INIT_DENSITY;

void
growth()
{
	register int    r, c, i, j;
	int             time = 5, changed, circ_color;
	FILE           *fp, *fp_grid;
	int             spread();
	void            show_status(), read_erdas();
	void            Edge();
	int             iteration = 0;
	ECO_HEADER      save_head, head;
	char           *getenv();
	char            fname[MAX_LEN], name[MAX_LEN], date[6];
	int             n;
	int             total_pixels;
	char            logname[MAX_LEN], tmp_str[MAX_LEN];

	long            dev;
	short           value;
	short           attached = 1;
	short           item = 0;
	int             nbuttons;
	void            pixrect ();
	int		z_counter = 0, urban_counter = 0;
	int		actual_agri = 0, model_agri = 0, seed_agri_cnt = 0, 
			seed_urban_cnt = 0, seed_park_cnt = 0;
	STATS           stats[1000];
	GSTATS          gstats;
 	int		initial_year, initial_pop;
 	int		initial_density, initial_num_cells;
 	int		num_cells_before;
 	int		pop_now;
 	int		start_year, stop_year, current_density;
 	double  	initial_rate, current_rate;
#ifdef COUNTING
	int nd, des, rec, ag, urb, rd, exc, slo, nexc, mcslo;
#endif


#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter growth()\n",
	 __FILE__, __LINE__);
	 (void)fflush(DBG_FILE);
#endif
#ifdef COUNTING
	nd = des = rec = ag = urb = rd = nexc = exc = mcslo = 0;
#endif

	/* initialize the externals */
	num_growth_pix = 0;
	ticktock = startstop.start_date;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) ticktock initialized to %d\n", 
__FILE__, __LINE__, ticktock);
(void)fflush(DBG_FILE);
#endif
	growth_rate = 1.0;
	PARK_PEND = FALSE;

	/* read in the input arrays */

	/* empty the string */
	fname[0] = '\0';
	strcpy(fname, DATA_DIR);
	strcat(fname, "/sub");
	strcpy(name, fname);
	strcat(name, ".slope");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) open: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	fp = fopen(name, "r");
	if (fp == NULL) {
		printf("ERROR: File %s not found...\n", name);
		exit(1);
	}
	read_header(&head, fp);
	save_head = head;
	fclose(fp);

	nrows = head.nl;
	ncols = head.ns;
	total_pixels = nrows * ncols;

	/* slope layer */
	strcpy(name, fname);
	strcat(name, ".slope");
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, slp, NOFIX);
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) return from readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif

	/* print slope grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/slope");
	mk_gridname(tmp_gname, "1995", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, slp, I_NOCHANGE);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	/* END print slope grid */

	/* road layer */
	strcpy(name, fname);
	strcat(name, ".roads");
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, roads, NOFIX);

	/* print road grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/roads");
	mk_gridname(tmp_gname, "1978", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, roads, I_NOCHANGE);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	/* END print road grid */


	/* excluded areas layer */
	strcpy(name, fname);
	strcat(name, ".excld");
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, excld, NOFIX);
	read_grid(&head, name, EXCLD, NOFIX);

#ifdef OWNERS
	/* ownership areas layer */
	strcpy(name, fname);
	strcat(name, ".owners");
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, OWN, NOFIX);
#endif

	/* print excld grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/excld");
	mk_gridname(tmp_gname, "1995", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, excld, I_NOCHANGE);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	/* END print excld grid */


	/* write original grid to match format for output */
	write_early_grid("1995", 1);


	/* seed layer */
	strcpy(date, SEED_DATE);

	strcpy(name, fname);
	strcat(name, ".seed");
	strcat(name, date);
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) read SEED: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, seed, EXFIX);

	/* print seed grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/seed");
	mk_gridname(tmp_gname, date, NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, seed, I_NOCHANGE);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	/* END print seed grid */

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) count road and urban pixels\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* count the number of road pixels and the number of nonurban pixels */
	roadcount = 0;
	noncount = 0;
	max_urban = 0;
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			z[i][j] = 0;
			if (roads[i][j] == 255)
				roadcount++;
			if (excld[i][j] >= 1)
				noncount++;
			if ((seed[i][j] >= URBAN)|| ((roads[i][j] != 255) && (excld[i][j] == 0)))
				max_urban++;
			/*
			 * in our changes, urban is 4 (URBAN) so check for 4 or above. Will
			 * use 2 (AGRIC) for agriculture
			 */
			if (seed[i][j] >= URBAN) {
				z[i][j] = (unsigned short) PHASE0G;
				seed_urban_cnt++;
			}
			if (seed[i][j] == AGRIC) {
				z[i][j] = (unsigned short) AGRIC;
				seed_agri_cnt++;
			}
			if (seed[i][j] == PARKS) {
				z[i][j] = (unsigned short) PARKS;
				seed_park_cnt++;
			}
#ifdef COUNTING
			/* do some counting separately */
			{
				if (roads[i][j] == 255)
					rd++;
				if (excld[i][j] >= 1)
					exc++;
				if (excld[i][j] == 0)
					nexc++;
				if (slp[i][j] >= 15)
					slo++;
				/* grd is stuff that can still be protected */
				if ((excld[i][j] == 0) && (slp[i][j] >= 15) && (seed[i][j] != URBAN)) {
					mcslo++;
					grd[i][j] = slp[i][j];
				} else
					grd[i][j] = 0;
				if ((slp[i][j] >= 15) && (seed[i][j] != URBAN)) {
					grd2[i][j] = slp[i][j];
				} else
					grd2[i][j] = 0;
				if (slp[i][j] >= 15) {
					grd3[i][j] = slp[i][j];
				} else
					grd3[i][j] = 0;
				if (seed[i][j] == URBAN)
					urb++;
				if (seed[i][j] == AGRIC)
					ag++;
				if (seed[i][j] == 0)
					nd++;
			}
#endif
		}
	}
	/* related to separate counting */
#ifdef COUNTING
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) road count: %d\n", __FILE__, __LINE__, rd);
	(void)fprintf(DBG_FILE, "excld count: %d\n", exc);
	(void)fprintf(DBG_FILE, "non-excld count: %d\n", nexc);
	(void)fprintf(DBG_FILE, "big slope count: %d\n", slo);
	(void)fprintf(DBG_FILE, "big slope non-excld: %d\n", mcslo);
	(void)fprintf(DBG_FILE, "urb count: %d\n", urb);
	(void)fprintf(DBG_FILE, "ag count: %d\n", ag);
	(void)fprintf(DBG_FILE, "nodata count: %d\n", nd);
	(void)fprintf(DBG_FILE, "tot seed count: %d\n", urb + ag + nd);
	(void)fflush(DBG_FILE);
#endif
	/* print special slope grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/avbigslope");
	mk_gridname(tmp_gname, "1995", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, grd, PASS_THRU);
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/allbigslope");
	mk_gridname(tmp_gname, "1995", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, grd2, PASS_THRU);
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/slope3");
	mk_gridname(tmp_gname, "1995", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, grd3, PASS_THRU);
	exit(0);
#endif

	percent_road = (100.0 * roadcount) / (total_pixels - noncount);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) SEED urban: %d agric: %d  park: %d   MAX URBAN: %d\n", 
	__FILE__, __LINE__, seed_urban_cnt, seed_agri_cnt, seed_park_cnt, max_urban);
	(void)fflush(DBG_FILE);
#endif

	/* zero the growth stats structure */
	gstats.sng = 0;
	gstats.sdg = 0;
	gstats.sdc = 0;
	gstats.og = 0;
	gstats.rt = 0;
	gstats.pop = 0;

	changed = TRUE;

	TOT_URBAN_RUN = seed_urban_cnt;  /* keeps track of total urban to date */

	start_year = startstop.start_date;
	stop_year = startstop.stop_date;
	initial_year = start_year;
	setup_initials(&initial_pop, &initial_num_cells, &initial_density, &initial_rate);

	/* initialize the changing variables */
	num_cells_before = initial_num_cells;
	pop_now = initial_pop;
	current_rate = initial_rate;
	get_est_density(ticktock, &current_density);

	while (changed) {
		iteration++;
		
		/* apply the cellular automaton rules for this year */
		if ((changed = begin_spread(&pop_now, &num_cells_before, 
		&current_rate, &current_density, &gstats, &stats[iteration])) == -1) {
			for (i = 0; i < nrows; i++) {
				for (j = 0; j < ncols; j++) {
					if ((z[i][j] >= PHASE0G) && (seed[i][j] <= PHASE5G))
						my_lone[i][j] = URBAN;
					if (z[i][j] == AGRIC) 
						my_lone[i][j] = AGRIC;
					if (z[i][j] == PARKS) 
						my_lone[i][j] = PARKS;
				}
			}
			/* write out the current year's ecology grid */
			sprintf(tmp_str, "%d", ticktock);
			strcpy(tmp_gname, LOG_DIR);
			strcat(tmp_gname, "/urb");
			mk_gridname(tmp_gname, tmp_str, NAME_ARR);
			fp_grid = fopen(NAME_ARR, "w");
			write_int(fp_grid, my_lone, U_A_D_E);
			do_overlay(ITER_IMAGE, 1, NAME_ARR);

			/* write out the current year's grid */
			sprintf(tmp_str, "%d", ticktock);
			strcpy(tmp_gname, LOG_DIR);
			strcat(tmp_gname, "/urb");
			mk_gridname(tmp_gname, tmp_str, NAME_ARR);
			fp_grid = fopen(NAME_ARR, "w");
			write_int(fp_grid, my_lone, U_A_COMBO_ROAD);
			do_overlay(ITER_IMAGE, 0, NAME_ARR);
			changed = FALSE;
			continue;
		}
		if (changed == -2) {
			/* don't write out the grid; it's incomplete */
			changed = FALSE;
#ifdef DEBUG
			(void)fprintf(DBG_FILE, "(%s %d)Quit early with about %d years left.\n",
		        __FILE__, __LINE__, years_remain);
			(void)fflush(DBG_FILE);
#endif
			continue;
		}
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) year: %d   pop_now: %d   nc_before: %d   cur_rate: %.5f  cur_den: %d\n",
__FILE__, __LINE__, ticktock, pop_now, num_cells_before, current_rate, current_density);
fflush(DBG_FILE);
#endif

		growth_rate = ((float) num_growth_pix / gstats.pop * 100.0);
		if (num_growth_pix == 0)
			growth_rate = 0.0;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Growth Rate: %3.4f\n", __FILE__, __LINE__, growth_rate);
(void)fprintf(DBG_FILE, "%d / (%d * 100)\n", num_growth_pix, gstats.pop);
(void)fflush(DBG_FILE);
#endif

		/* calculate the percentage of land that has been urbanized */
		percent_urban = (100.0 * gstats.pop) / (total_pixels - roadcount - noncount);

#ifdef HILT 
{

#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) do self-mods\n", __FILE__, __LINE__);
(void)fprintf(DBG_FILE, "(%s %d) Cur g-rate: %8.2f (hi:%8.2f lo:%8.2f)\n", 
__FILE__, __LINE__, growth_rate, selfmod.critical_high, selfmod.critical_low);
(void)fflush(DBG_FILE);
#endif

		/* self-modification: occurs if growth_rate > critical_high or < critical_low */
		if (growth_rate > selfmod.critical_high) {
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) WARNING: above critical\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif

			param.slope_resistance += (double) (percent_urban / 20.0);
			if (param.slope_resistance > 100) param.slope_resistance = 100.0;

			param.road_gravity += (double) (percent_urban / 20.0);
			if (param.road_gravity > 100) param.road_gravity = 100.0;

			if (param.diffusion_coefficient < 100)
			{
				param.diffusion_coefficient *= selfmod.boom;
				if (param.diffusion_coefficient > 100) param.diffusion_coefficient = 100.0;

				param.spread_coefficient *= selfmod.boom;
				if (param.spread_coefficient > 100) param.spread_coefficient = 100.0;

				param.breed_coefficient *= selfmod.boom;
				if (param.breed_coefficient > 100) param.breed_coefficient = 100.0;

			}

		} 
		if (growth_rate < selfmod.critical_low){

#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) WARNING: below critical\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
			param.slope_resistance -= (double) (percent_urban / 20.0);
			if (param.slope_resistance <= 0.01) param.slope_resistance = 1.0;

			param.road_gravity -= (double) (percent_urban / 20.0);
			if (param.road_gravity <= 0.01) param.road_gravity = 1.0;

			if (growth_rate < selfmod.critical_low)  {
				if (param.diffusion_coefficient > 0.0) 
				{
					param.diffusion_coefficient *= selfmod.bust;
					if (param.diffusion_coefficient <= 0.01) param.diffusion_coefficient = 1.0;

					param.spread_coefficient *= selfmod.bust;
					if (param.spread_coefficient <= 0.01) param.spread_coefficient = 1.0;

					param.breed_coefficient *= selfmod.bust;
					if (param.breed_coefficient <= 0.01) param.breed_coefficient = 1.0;

				}
			}
		}
}
#endif      /* self mods for HILT only */

		/* check to see if STOP_DATE has been reached yet */
		if (ticktock >= startstop.stop_date) {
			changed = FALSE;
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) Reached stop date (tick:%d stop:%d)\n", __FILE__, __LINE__, ticktock, startstop.stop_date);
(void)fflush(DBG_FILE);
#endif
		}
		else {
			changed = TRUE;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Haven't reached stop date (tick:%d stop:%d)\n", 
__FILE__, __LINE__, ticktock, startstop.stop_date);
(void)fflush(DBG_FILE);
#endif
		}
#ifdef xDEBUG
		(void)fprintf(DBG_FILE,"(%s %d) Yr:%d",__FILE__,__LINE__,ticktock);
		fprintf(DBG_FILE, "	  P1: %d P2: %d P3: %d P4: %d P5: %d\n", 
		gstats.sng, gstats.sdg, gstats.sdc, gstats.og, gstats.rt);
		(void)fflush(DBG_FILE);
#endif
		/* write out stats for designated year */

		/*
		 * any urban cells, regardless of growth type, now need to be of
		 * type URBAN so we write the grid out with just urban values.
		 * We want to include agriculture here.
		 */
		for (i = 0; i < nrows; i++) {
			for (j = 0; j < ncols; j++) {
				if ((z[i][j] >= PHASE0G) && (seed[i][j] <= PHASE5G))
					my_lone[i][j] = URBAN;
				if (z[i][j] == AGRIC) 
					my_lone[i][j] = AGRIC;
				if (z[i][j] == PARKS) 
					my_lone[i][j] = PARKS;
			}
		}
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d)Write out grids for year\n", __FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		/* write out the current year's ecology grid */
		sprintf(tmp_str, "%d", ticktock);
		strcpy(tmp_gname, LOG_DIR);
		strcat(tmp_gname, "/urb");
		mk_gridname(tmp_gname, tmp_str, NAME_ARR);
		fp_grid = fopen(NAME_ARR, "w");
		write_int(fp_grid, my_lone, U_A_D_E);
		do_overlay(ITER_IMAGE, 1, NAME_ARR);

		/* write out the current year's grid */
		sprintf(tmp_str, "%d", ticktock);
		strcpy(tmp_gname, LOG_DIR);
		strcat(tmp_gname, "/urb");
		mk_gridname(tmp_gname, tmp_str, NAME_ARR);
		fp_grid = fopen(NAME_ARR, "w");
		write_int(fp_grid, my_lone, U_A_COMBO_ROAD);
		do_overlay(ITER_IMAGE, 0, NAME_ARR);
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) replace seeds\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
		/* place urban seeds back */
		for (i = 0; i < nrows; i++)  {
			for (j = 0; j < ncols; j++) {
				if (seed[i][j] >= URBAN)
					z[i][j] = (unsigned short) PHASE0G;
			}
		}
   
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Final urban: %d         final pop: %d\n",
	__FILE__, __LINE__, num_cells_before, pop_now);
	fflush(DBG_FILE);
#endif
	return;
}

/*******************************************************************************
Tags or 0 to just write the grid and 1 to compute area, edges and clusters too
*******************************************************************************/
write_early_grid(year_str, tag)
char	*year_str;
int		tag;
{
	char			fname[MAX_LEN];
	ECO_HEADER      head;
	FILE           *fp_grid;
	float			area, edges, clusters;

	strcpy(fname, DATA_DIR);
	strcat(fname, "/sub.seed");
	strcat(fname, year_str);
	read_grid(&head, fname, grd, EXFIX);

	/* print grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/grid");
	mk_gridname(tmp_gname, year_str, NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, grd, PASS_THRU_EXCLD);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	return(0);
}
/*****************************************************************
*****************************************************************/
int
begin_spread(pop_now, num_cells_before, current_rate, current_density,
gstats, mstats)
int 	*pop_now, *num_cells_before, *current_density;
double	*current_rate;
GSTATS  *gstats;
STATS   *mstats;
{
	int	pop_after, changed;
	int	num_cells_after, actual_added;
	int	cells2add;
	int	new_population;
	int	end_pop, year_pass;
	double  new_rate;

	/* Let time pass */
	ticktock++; 

	if (ticktock == MAGIC_YEAR) {
		open_statelands();
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "\n(%s %d) New Year: %d\n", __FILE__, __LINE__, ticktock);
	(void)fflush(DBG_FILE);
#endif

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Iteration %d: Apply rules\n", __FILE__, __LINE__, iteration);
(void)fflush(DBG_FILE);
#endif

#ifndef HILT
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) start pop: %d  start num cells: %d\n",
	__FILE__, __LINE__, *pop_now, *num_cells_before);
	(void)fprintf(DBG_FILE, "(%s %d) cur rate: %.5f   cur dens: %d\n",
	__FILE__, __LINE__, *current_rate, *current_density);
	fflush(DBG_FILE);
#endif
	pop_after = project_pop(1, *pop_now, *current_rate);

	/* figure out how many cells need to be added */
	num_cells_after = pop_after / *current_density;
	cells2add = num_cells_after - *num_cells_before;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Cells now: %d  needed: %d   to add: %d\n",
	__FILE__, __LINE__, *num_cells_before, num_cells_after, cells2add);
	fflush(DBG_FILE);
#endif
#else  /* HILT */
	cells2add = 0;
#endif /* HILT */
	/*
	 * before calling spread another year, find out how many cells are left
	 * available to populate. Stop if the number is too low
	 */
	if ( ! verify_avail(cells2add)) {
		/* there just aren't enough cells left */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) There aren't enough cells for another year\n",
	__FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
		return(-1);
	}

	changed = spread(gstats, mstats, cells2add, &actual_added);

#ifndef HILT
	/* reset the variables for the next loop */
	if (changed < 0)
		return(changed);  /* too hard to find cells. Quit. */
	if (actual_added > 0)
		*num_cells_before += actual_added;
	new_population = *num_cells_before * *current_density;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Actual add: %d    new_pop: %d\n",
	__FILE__, __LINE__, actual_added, new_population);
	fflush(DBG_FILE);
#endif
	get_est_density(ticktock, current_density);
	
#endif /* HILT */

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "\n(%s %d) New Year: %d\n", __FILE__, __LINE__, ticktock);
	(void)fflush(DBG_FILE);
#endif

#ifndef HILT
	new_rate = calc_new_rate(current_rate, new_population, current_density, ticktock);
	*current_rate = new_rate;
	*pop_now = new_population;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) new_rate %f\n", __FILE__, __LINE__, *current_rate);
	(void)fflush(DBG_FILE);
#endif
#endif /* HILT */

	return(changed);
}
/*****************************************************************
*****************************************************************/
int
get_est_density(ticktock, est_density)
int	ticktock, *est_density;
{
	static int prev_density, first_time = TRUE;
	static	yrs_passed = 1;



#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter est_dens (tick: %d)\n", __FILE__, __LINE__, ticktock);
(void)fflush(DBG_FILE);
#endif
	if (strcmp(SFX, "1k") == 0) {
		/* constant density */
		*est_density = 1263;
	}

	if (strcmp(SFX, "480m") == 0) {
		if (first_time == TRUE) {
			prev_density = 296;
			first_time = FALSE;
			*est_density = 296;
		}
		if (ticktock <= 2002)
			return(0);   /* no changes until 2002 */
		yrs_passed++;
		if (RATE_TYPE == FIXED) {
			/* no change */
			*est_density = 296;
#ifdef DEBUG
			(void)fprintf(DBG_FILE, 
	        	"(%s %d) No change in density\n", __FILE__, __LINE__);
#endif
		} else {
			if (RATE_TYPE == DCONS) {
				/* increase density 10% over 10 years */
				*est_density = 
				(INIT_DENSITY + ((INIT_DENSITY * 0.10)/(11-MIN(10, yrs_passed))));
				prev_density = *est_density;
#ifdef DEBUG
				(void)fprintf(DBG_FILE, 
		        	"(%s %d) Increase density 10 perc over 10 years\n", 
		        	__FILE__, __LINE__);
#endif
			} else {
				if (RATE_TYPE == DULTRA) {
					/* increase density 25% over 10 years */
					*est_density = 
					(INIT_DENSITY + ((INIT_DENSITY * 0.25)/(11-MIN(10, yrs_passed))));
					prev_density = *est_density;
#ifdef DEBUG
					(void)fprintf(DBG_FILE, 
		        		"(%s %d) Increase density 25 perc over 10 years\n", 
		        		__FILE__, __LINE__);
#endif
				} else {
					/* 
					 * if you have requested a CUSTOM type, this is where
					 * you put the source code to implement the density change
					 */
#ifdef DEBUG
					(void)fprintf(DBG_FILE, "(%s %d) CUSTOM density rate\n", 
	        			__FILE__, __LINE__);
#endif
					*est_density = 296;  /* default */
				}
			}
		}
	}

	if (strcmp(SFX, "240m") == 0)
		*est_density = 73;
	if (strcmp(SFX, "120m") == 0)
		*est_density = 18;
	if (strcmp(SFX, "60m") == 0)
		*est_density = 4;
	
	return(0);
}
/*****************************************************************
*****************************************************************/
int
project_pop(year, init_pop, rate)
int	init_pop, year;
double	rate;
{
	int	projected_pop;
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter project_pop() year: %d  init: %d   rate: %.5f\n",
__FILE__, __LINE__, year, init_pop, rate);
(void)fflush(DBG_FILE);
#endif
	/* Nt = N0*e^(rate*t) */
	projected_pop = (int)((double)init_pop * pow(2.718,rate * (double)year));
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) Projected population: %d\n", __FILE__, __LINE__, projected_pop);
(void)fflush(DBG_FILE);
#endif

	return(projected_pop);
}
/*****************************************************************
*****************************************************************/
double
calc_rate(init_pop, seek_pop, years)
int	init_pop, seek_pop, years;
{
	double e = 2.7183;
	double	e_val = 0, rate = 0, last_rate;

	if (strcmp(SFX, "480m") == 0) {
		last_rate = 0.03458;
		return(last_rate);
	}
	
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter calc_rate with init_pop %d   seek: %d   yrs: %d\n", 
	__FILE__, __LINE__, init_pop, seek_pop, years);
	fflush(DBG_FILE);
#endif
	while (e_val < (double)seek_pop) {
		last_rate = rate;
		rate++;
		e_val = (double)init_pop * pow(e, (rate * (double)years));
	}
	/* the last_rate value is the one we want */
	rate = last_rate;
	e_val = (double)init_pop * pow(e, (last_rate * (double)years));
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Start rate: %3.5f   exp: %3.5f    eval: %3.5f\n", 
	__FILE__, __LINE__, rate, pow(e, (rate * (double)years)), e_val);
	fflush(DBG_FILE);
#endif
	while (e_val < (double)seek_pop) {
		last_rate = rate;
		rate += .000001;
		e_val = (double)init_pop * pow(e, (rate * (double)years));
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) rate: %3.5f   eval: %3.5f\n", 
		__FILE__, __LINE__, rate, e_val);
		fflush(DBG_FILE);
#endif
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) NEW rate: %3.5f\n", 
	__FILE__, __LINE__, last_rate);
	fflush(DBG_FILE);
#endif
	return(last_rate);
}
/*****************************************************************
*****************************************************************/
int
setup_initials(initial_pop, initial_num_cells, initial_density, initial_rate)
int	*initial_pop, *initial_num_cells, *initial_density;
double	*initial_rate;
{

	*initial_pop = 2528700;  /* population in 1995 */
	*initial_rate = calc_rate(POP_1975, POP_1997, 22);
	INIT_RATE = *initial_rate;
	if (strcmp(SFX, "1k") == 0) {
		*initial_num_cells = 2002;
		*initial_density = 1263;
	}
	if (strcmp(SFX, "480m") == 0) {
		*initial_num_cells = 8522;
		*initial_density = 296;
		INIT_DENSITY = 296;
	}
	if (strcmp(SFX, "240m") == 0) {
		*initial_num_cells = 34347;
		*initial_density = 73;
	}
	if (strcmp(SFX, "120m") == 0) {
		*initial_num_cells = 137154;
		*initial_density = 18;
	}
	if (strcmp(SFX, "60m") == 0) {
		*initial_num_cells = 548265;
		*initial_density = 4;
	}
	return(0);
}
/*****************************************************************
calculate a new rate based on the current population, the
current density and the current year. This will allow changes
in rate as years go by 
*****************************************************************/
double
calc_new_rate(rate, pop, density, year)
double	*rate;
int	pop, year, *density;
{
	double	new_rate;
	static	yrs_passed = 1;

	if (year <= 2002)
		return(*rate);   /* no changes until 2002 */
	yrs_passed++;
	if (RATE_TYPE == FIXED) {
		/* no change */
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) No change in growth rate\n", 
	        __FILE__, __LINE__);
#endif
		return(*rate);
	} else {
		if (RATE_TYPE == DCONS) {
			/* decrease rate 5% over 10 years */
			*rate = (INIT_RATE - ((INIT_RATE * 0.05)/(11-MIN(10, yrs_passed))));
#ifdef DEBUG
			(void)fprintf(DBG_FILE, 
		        "(%s %d) Decrease growth rate 5 perc over 10 years\n", 
		        __FILE__, __LINE__);
#endif
			return(*rate);
		} else {
			if (RATE_TYPE == DULTRA) {
				/* decrease rate 15% over 10 years */
				*rate = (INIT_RATE - ((INIT_RATE * 0.15)/(11-MIN(10, yrs_passed))));
#ifdef DEBUG
				(void)fprintf(DBG_FILE, 
			        "(%s %d) Decrease growth rate 15 perc over 10 years\n", 
		        	__FILE__, __LINE__);
#endif
				return(*rate);
			} else {
				/* 
				 * if you have requested a CUSTOM type, this is where
				 * you put the source code to implement the rate change
				 */
#ifdef DEBUG
				(void)fprintf(DBG_FILE, "(%s %d) CUSTOM growth rate\n", 
	        		__FILE__, __LINE__);
#endif
				return(*rate);  /* default */
			}
		}
	}
	return(*rate);
}
/*****************************************************************
* in 2007 we want to open the state trust lands. Either all will be open 
* for development or half will open and half turned into parks. In either
* case, there is a new exclusions layer to be overlayed and the parks that
* have been added to the map have to be carried over into the new exclusions
* layer. If half are becoming parks, the new parks need to be incorporated 
* into the master map (z[][]) by reading in the change grid in and placing
* all changes (new parks) in the z grid.
*****************************************************************/
int
open_statelands()
{
	char	fname[MAX_LEN], name[MAX_LEN];
	int	i, j;
	ECO_HEADER      head;

	/*
	 * Replace the exclusions layer
	 */
	strcpy(fname, DATA_DIR);
	if (OPEN_TYPE == OPENHALF)  {
		strcat(fname, "/sub.ex.halfstate");
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) OPEN HALF STATE LANDS\n", __FILE__, __LINE__);
#endif
	}
	if (OPEN_TYPE == OPENALL)  {
		strcat(fname, "/sub.ex.nostate");
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) OPEN ALL STATE LANDS\n", __FILE__, __LINE__);
#endif
	}
	/* if neither OPEN_TYPE test above passes, there is nothing to do */
	if (strcmp(fname, DATA_DIR) == 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) all STATE LANDS stay excluded\n", __FILE__, __LINE__);
#endif
		return(0);
	}

	strcpy(name, fname);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
	(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, excld, NOFIX);
	read_grid(&head, name, EXCLD, NOFIX);

	/*
	 * go through the master grid (z[][]) and any PARKS should be 
	 * carried over into the new exclusion layer. There were parks
	 * created during run that must continue to be excluded.
	 */
	for (i = 0; i < NROWS; i++) {
		for (j = 0; j < NCOLS; j++) {
			if (z[i][j] == PARKS) {
				excld[i][j] = PARKS;
				EXCLD[i][j] = PARKS;
			}
		}
	}

	if (OPEN_TYPE == OPENHALF) {
	/*
	 * read in the new set of park cells created from state trust lands
	 * and add them to the master layer. They are already in the exclusion
	 * layer (just read in).
	 */
	strcpy(fname, DATA_DIR);
	strcat(fname, "/new_state_parks.asc");
	strcpy(name, fname);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
	(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, new_parks, NOFIX);
	for (i = 0; i < NROWS; i++) {
		for (j = 0; j < NCOLS; j++) {
			if (new_parks[i][j] == 1) {
				z[i][j] = PARKS;
			}
		}
	}
	}
	return(0);
}
/*****************************************************************
*****************************************************************/
int
verify_avail(need)
int	need;
{
	int	i, j, tally = 0;

	for (i = 0; i < NROWS; i++) {
		for (j = 0; j < NCOLS; j++) {
			if ((excld[i][j] == 0) && (z[i][j] < PHASE0G) && (z[i][j] != PARKS))
				tally++;
		}
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Need:%d   Have:%d\n", __FILE__, __LINE__, need, tally);
	(void)fflush(DBG_FILE);
#endif
	/*
	 * the years remaining value will become more accurate as the scenario comes to
	 * a close. In the early years, it will be over-estimated because the count is
	 * based on the current year's need
	 */
	years_remain = tally / need;
	/*
	if (tally > (need + 1500))
	*/
	if (tally > (need + 500))
		return(TRUE);
	else
		return(FALSE);
}
