/*************************************************************************/
/* read.c - PREDICTION VERSION                                          */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern	FILE *DBG_FILE;
#endif


/**** READ ROUTINES **********************************************************/
    
void read_header(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    int ftype;

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter read_header(fp:%d)\n",
__FILE__, __LINE__, fp);
(void)fflush(DBG_FILE);
#endif
    get_filetype(&ftype,fp);
    switch (ftype) {
	case ECOSAT:
	    read_ecohead(head,fp);
	    break;
	case ERDAS:
	    read_erdashead(head,fp);
	    break;
	case LAS:
	    read_lashead(head,fp);
	    break;
	case PEDIT:
	    read_pedithead(head,fp);
	    break;
	case BIL_RAW:
	    head->data_format = BIL;
	    read_rawhead(head,fp);
	    break;
	case BSQ_RAW:
	    head->data_format = BSQ;
	    read_rawhead(head,fp);
	    break;
	case PIL_RAW:
	    head->data_format = PIL;
	    read_rawhead(head,fp);
	    break;
	case TARGA:
	    read_targahead(head,fp);
	    break;
	case ASCII:
   	    read_asciihead(head, fp);
	    break;
	default:
	    printf("ERROR: Cannot read file header.  \n");
	    exit(0);
	    }
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Exit read_header()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
return;
}



ask_filetype(ftype)
int *ftype;
{
    char fmt[10];
    int done=FALSE;
    int slen;

    printf("Cannot Determine File Type...\n");
    printf("Please Give Me a Hint...\n");
    while (!done) {
	printf("	ascii:\n");
	printf("	bsq_raw:\n");
	printf("	bil_raw:\n");
	printf("	pil_raw:\n");
	printf("	targa:\n");
	printf("File type = ");
	getline(fmt,&slen);
	if (slen == 0) exit(1);
	switch (fmt[0]) {
	    case 'a': case 'A':
		printf("Assuming Ascii Data!\n");
		*ftype = ASCII;
		done = TRUE;
		break;
	    case 'b': case 'B':
		switch (fmt[1]) {
		    case 'i': case 'I':
			printf("Assuming bil_raw Data Without Header!\n");
			*ftype = BIL_RAW;
			done = TRUE;
			break;
		    case 's': case 'S':
			printf("Assuming bsq_raw Data Without Header!\n");
			*ftype = BSQ_RAW;
			done = TRUE;
			break;
		}
		break;
	    case 'p': case 'P':
		printf("Assuming pil_raw Data Without Header!\n");
		*ftype = PIL_RAW;
		done = TRUE;
		break;
	    case 't': case 'T':
		printf("Assuming TARGA format file!\n");
		*ftype = TARGA;
		done = TRUE;
		break;
	    default:
		printf("Unknown File Type, please reenter\n");
		break;
	}
    }
}
    

read_rawhead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    char ttystr[40];
    int strlen;
 
    head->nl = 0;
    head->ns = 0;
    head->data_type = 0;
    head->nc = 0;
    head->nbih = -1;
    
    while (head->nl == 0) {
	printf("Enter number of lines: ");
	scanf("%ld",&head->nl);
	if (head->nl < 1) {
	    printf("Illegal number of lines!\n");
	    head->nl = 0;
	}
    }
    while (head->ns == 0) {
	printf("Enter number of samples: ");
	scanf("%ld",&head->ns);
	if (head->ns < 1) {
	    printf("Illegal number of samples!\n");
	    head->ns = 0;
	}
    }
    while (head->data_type == 0) {
	printf("Enter Data Type: ");
	scanf("%ld",&head->data_type);
	if (head->data_type < 1) {
	    printf("Illegal Data Type!\n");
	    head->data_type = 0;
	}
    }
    while (head->nc == 0) {
	printf("Enter Number of Channels: ");
	scanf("%ld",&head->nc);
	if (head->nc < 1) {
	    printf("Illegal Number of Channels!\n");
	    head->nc = 0;
	}
    }
    while (head->nbih < 0) {
	printf("Enter number of bytes in header: ");
	scanf("%ld",&head->nbih);
	if (head->nbih < 0) {
	    printf("Illegal Number of bytes in header!\n");
	    head->nbih = -1;
	}
    }
    getline(ttystr,&strlen);
    
    switch (head->data_format) {
	case BIL:
	case BSQ:
	    if (head->data_type != FLOAT)
		head->nbpr =(head->ns * head->data_type);
	    else head->nbpr = (head->ns * 4);
	    break;
	case PIL:
	    if (head->data_type != FLOAT)
		head->nbpr =(head->ns * head->data_type * head->nc);
	    else head->nbpr = (head->ns * 4 * head->nc);
	    break;
	case ASCII:
	    break;
    }
    head->sl = 1;
    head->nl = head->nl;
    head->ss = 1;
    head->ns = head->ns;
    head->nc = head->nc;
    head->data_type = head->data_type;
    head->max = 0;
    head->min = 0;
    
    fseek(fp,0,BEGIN);
}


read_pedithead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    int nitems;
    BUF_UNION buf, Wbuf, Hbuf;

    printf("PEDIT file format\n");
    fseek(fp,0,BEGIN);
    nitems = fread(buf.c,sizeof(*buf.c),PEDITHEAD,fp);
    rd_err(nitems,PEDITHEAD,"Header");

    head->nbih = PEDITHEAD;
    head->sl = Wbuf.w[21];
    head->nl = Wbuf.w[23] - Wbuf.w[21] + 1;
    head->ss = Wbuf.w[22];
    head->ns = Wbuf.w[24] - Wbuf.w[22] + 1;
    head->nc = Wbuf.w[6];
    head->magic_number = Wbuf.w[1];
    head->data_type = 1;
    head->max = Wbuf.w[8];
    head->min = 0;
    head->data_format = Wbuf.w[3];
    switch (head->data_format) {
	case PIL:
	    head->nbpr = (Wbuf.w[24] - Wbuf.w[22] + 1) * Wbuf.w[6];
	    break;
	case BSQ: case BIL:
	    head->nbpr = (Wbuf.w[24] - Wbuf.w[22] + 1);
	    break;
    }
    
    fseek(fp,PEDITHEAD,BEGIN);
}


		
read_ecohead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    int nitems;
    BUF_UNION buf, Wbuf, Hbuf;

    fseek(fp,0,BEGIN);
    nitems = fread(head,sizeof(ECO_HEADER),1,fp);
    rd_err(nitems,1,"Header");
	/* reverse them in place */
	reverse_ints((char *)head, (char *)head);
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Magic number: %d\n", 
__FILE__, __LINE__, head->magic_number);
(void)fflush(DBG_FILE);
#endif

    switch (head->magic_number) {
	case ECOSAT:
	    /*printf("ECOSAT file format\n");*/
	    break;
	default:
	    printf("ERROR: Not an ECOSAT file\n");
	    head->magic_number = UNKNOWN;
	    exit(1);
	    break;
    }
    /* temporary fix for old data */
   /* head->data_format = BSQ;*/
    
    fseek(fp,ECOSATHEAD,BEGIN);
}
    

read_lashead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    int nitems;
    BUF_UNION buf, Wbuf, Hbuf;
    int linesize,hsize;

    printf("LAS file format\n");
    fseek(fp,0,BEGIN);
    nitems = fread(buf.c,sizeof(*buf.c),LASHEAD,fp);
    rd_err(nitems,LASHEAD,"Header");
    
    head->nl = Wbuf.w[0];
    head->nc = Wbuf.w[1];
    head->ns = Wbuf.w[2];
    printf("buf 0= %d\n",Wbuf.w[0]);
    printf("buf 1= %d\n",Wbuf.w[1]);
    printf("buf 2= %d\n",Wbuf.w[2]);
    printf("buf 3= %d\n",Wbuf.w[3]);
    printf("buf 4= %d\n",Wbuf.w[4]);
    printf("buf 5= %d\n",Wbuf.w[5]);
    printf("buf 6= %d\n",Wbuf.w[6]);
    printf("buf 7= %d\n",Wbuf.w[7]);
    printf("buf 8= %d\n",Wbuf.w[8]);
    printf("buf 9= %d\n",Wbuf.w[9]);
    printf("buf10= %d\n",Wbuf.w[10]);
    printf("buf11= %d\n",Wbuf.w[11]);
    printf("buf12= %d\n",Wbuf.w[12]);
    head->nbih = (Wbuf.w[3] + 1) * Wbuf.w[4];
    if (head->nbih != LASHEAD) printf("LAS header size = %d\n",head->nbih);
    head->magic_number = Wbuf.w[12];
    
    head->sl = 1;
    head->ss = 1;
    switch (Wbuf.w[7]) {
	case 26002:
	    head->nbpr = (int)(((Wbuf.w[2] - 1) / Wbuf.w[4] + 1) * Wbuf.w[4]);
	    head->data_type = BYTE;
	    break;
	case 52003:
	    head->ns = head->ns / 2;
	    head->nbpr = (int)(((Wbuf.w[2] - 1) / Wbuf.w[4] + 1) * Wbuf.w[4]);
	    head->data_type = SHORT;
	    break;
    }
    
    head->max = 0.0;
    head->min = 0.0;
    head->data_format = BSQ;
}


read_elashead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    int nitems;
    BUF_UNION buf, Wbuf, Hbuf;

    printf("\nELAS file format\n");
    fseek(fp,0,BEGIN);
    nitems = fread(buf.c,sizeof(*buf.c),ELASHEAD,fp);
    rd_err(nitems,ELASHEAD,"Header");
    
    head->nbih = Wbuf.w[0];
    head->nbpr = Wbuf.w[1];
    head->sl = Wbuf.w[2];
    head->nl = Wbuf.w[3];
    head->ss = Wbuf.w[4];
    head->ns = Wbuf.w[5];
    head->nc = Wbuf.w[6];
    head->magic_number = Wbuf.w[7];
    
    head->nl = Wbuf.w[3] - head->sl + 1;
    head->ns = Wbuf.w[5] - head->ss + 1;
    if (head->ns > 512) head->data_type = (int)(Wbuf.w[1]/head->ns);
    else if (head->ns < 256) head->data_type = 2;
       else head->data_type = 1;
    head->data_format = BSQ;
}


read_targahead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    int nitems;
    BUF_UNION buf, Wbuf, Hbuf;
    int i;

    printf("\nTARGA file format\n");
    fseek(fp,0,BEGIN);
    nitems = fread(buf.c,sizeof(*buf.c),TARGAHEAD,fp);
    rd_err(nitems,TARGAHEAD,"Header");
    
    for (i=0;i<18;i++)
	printf("c %d = %d\n",i,buf.c[i]);
    
    head->nbih = TARGAHEAD;
    head->sl = 1;
    head->ss = 1;
    head->ns = (buf.c[13] * 256) + buf.c[12];
    head->nl = (buf.c[15] * 256) + buf.c[14];
    head->nc = buf.c[16] / 8;

    head->nbpr = head->ns * head->nc;
    head->data_format = PIL;
    
    head->magic_number = TARGA;
    head->data_type = BYTE;
}


read_erdashead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    int nitems;
    BUF_UNION buf,tmp, Wbuf, Hbuf;

    fseek(fp,0,BEGIN);
    nitems = fread(buf.c,sizeof(*buf.c),ERDASHEAD,fp);
    rd_err(nitems,ERDASHEAD,"Header");
    
    printf("\nERDAS file format: Version %c%c%c%c%c%c\n",buf.c[0],buf.c[1],buf.c[2],buf.c[3],buf.c[4],buf.c[5]);
    head->nbih = ERDASHEAD;
    head->magic_number = Wbuf.w[0];
    HeadSwap(&Hbuf.h[3],2);
    switch(Hbuf.h[3]) {
	case 0:
	    head->data_type = BYTE;
	    break;
	case 1:
	    printf("4 bit data conversion necessary...\n");
	    head->data_type = 9;
	    break;
	case 2:
	    head->data_type = SHORT;
	    break;
	default:
	    printf("Warning: Unknown data type...\n");
	    break;
    }

    HeadSwap(&Hbuf.h[4],2);
    head->nc = Hbuf.h[4];

    WordSwap(&Wbuf.w[4],16);
    head->ns = Wbuf.w[4];
    switch (head->data_type) {
	case BYTE:
	    head->nbpr = head->ns;
	    break;
	case SHORT:
	    head->nbpr = (head->ns * 2);
	    break;
	case 9:
	    head->nbpr = (int)(head->ns / 2.0 +.5);
	    break;
    }

    head->nl = Wbuf.w[5];
    head->ss = Wbuf.w[6];
    head->sl = Wbuf.w[7];

    HeadSwap(&Hbuf.h[45],2);
    head->max = Hbuf.h[45] - 1;
    head->min = 0;

    head->data_format = BIL;
}


read_asashead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    BUF_UNION buf, Wbuf, Hbuf;
    int nitems;

    printf("\nASAS file format\n");
    fseek(fp,0,BEGIN);
    nitems = fread(buf.c,sizeof(*buf.c),ASASHEAD,fp);
    rd_err(nitems,ASASHEAD,"Header");

    head->ns = (int)Hbuf.h[2];
    head->nl = (int)Hbuf.h[3];
    head->nc = (int)buf.c[8];
    head->data_type = (int)buf.c[9];
}


read_avshead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
    int nitems;
    BUF_UNION buf, Wbuf, Hbuf;

    printf("\nAVS file format\n");
    fseek(fp,0,BEGIN);
    nitems = fread(buf.c,sizeof(*buf.c),AVSHEAD,fp);
    rd_err(nitems,AVSHEAD,"Header");

    head->ns = Wbuf.w[0];
    head->nl = Wbuf.w[1];

    head->sl = 1;
    head->ss = 1;
    head->nc = 3;
    head->data_type = BYTE;
}


void get_filetype(type,fp)
int *type;
FILE *fp;
{
    int word0,word1,word7;
    int short0;
    BUF_UNION buf, Wbuf, Hbuf;
    int nitems;

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter get_filetype()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
#ifndef SAMPLE
	*type = ASCII;
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) File Type is %s\n",
	__FILE__, __LINE__, ((*type == ASCII)?"ascii":"non-ascii"));
	(void)fflush(DBG_FILE);
#endif
	return;
#endif
    nitems = fread(buf.c,sizeof(*buf.c),HEADERSIZE,fp);
    rd_err(nitems,HEADERSIZE,"header");
#ifdef SAMPLE
	reverse_ints((char *)&buf, (char *)&Wbuf);
	reverse_shorts((char *)&buf, (char *)&Hbuf);
#endif

    word0 = Wbuf.w[0];
    word1 = Wbuf.w[1];
    word7 = Wbuf.w[7];
    short0 = Hbuf.h[0];
    /*printf("Magic Numbers= %d %d %u\n",word0,word7,short0);*/
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d)Magic Numbers= %d %d %d %u\n", __FILE__, __LINE__,
word0, word1, word7,short0);
(void)fflush(DBG_FILE);
#endif
    
	if ((word0 == 0) && (word1 == 0) 
    && (word7 == 0) && (short0 == 0)) {
		*type = ASCII;
	} else {
    switch(word7) {
	case ECOSAT:
	    *type = ECOSAT;
	    break;
	case ELAS:
	    *type = ELAS;
	    break;
	default:
	    switch (word0) {
		case SUN:
		    *type = SUN;
		    break;
		default:  
		    if ((word0 == 0) && (word1 == 51)) {
			*type = PEDIT;
			}
		    else if (short0 == IRIS) {
			*type = IRIS;
			}
			else
			{
				*type = ASCII;

			}
	    }
	    break;
    }
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) File Type is %s\n",
	__FILE__, __LINE__, ((*type == ASCII)?"ascii":"non-ascii"));
	(void)fflush(DBG_FILE);
#endif
	return;
}
    
reverse_ints(buf_p, Wbuf_p)
char	*buf_p, *Wbuf_p;
{
	int	tot_swap;
	char	c1, c2, c3, c4;
	char 	*loc_p, *loc2_p;

	loc_p = buf_p;
	loc2_p = Wbuf_p;
	tot_swap = 0;
	while (tot_swap < HEADERSIZE)
	{
		c1 = *loc_p;
		c2 = *(loc_p + 1);
		c3 = *(loc_p + 2);
		c4 = *(loc_p + 3);

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) c1:%o c2:%o c3:%o c4:%o\n",
__FILE__, __LINE__, c1, c2, c3, c4);
#endif
		*(loc2_p + 3) = c1;
		*(loc2_p + 2) = c2;
		*(loc2_p + 1) = c3;
		*(loc2_p) = c4;

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Turn %o %o %o %o into %o %o %o %o\n",
__FILE__, __LINE__, *(loc_p + 0), *(loc_p + 1),  *(loc_p + 2),  *(loc_p + 3),   
*(loc2_p + 0), *(loc2_p + 1),  *(loc2_p + 2),  *(loc2_p + 3));
#endif
		loc_p += 4;
		loc2_p += 4;
		tot_swap += 4;

	}
}

reverse_shorts(buf_p, Hbuf_p)
char	*buf_p, *Hbuf_p;
{
	int	tot_swap;
	char	c1, c2;
	char 	*loc_p, *loc2_p;

	loc_p = buf_p;
	loc2_p = Hbuf_p;
	tot_swap = 0;
	while (tot_swap < HEADERSIZE)
	{
		c1 = *loc_p;
		c2 = *(loc_p + 1);

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) c1:%o c2:%o\n",
__FILE__, __LINE__, c1, c2);
#endif
		*(loc2_p + 1) = c1;
		*(loc2_p) = c2;

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Turn %o %o into %o %o\n",
__FILE__, __LINE__, *(loc_p + 0), *(loc_p + 1),    
*(loc2_p + 0), *(loc2_p + 1));
#endif
		loc_p += 2;
		loc2_p += 2;
		tot_swap += 2;

	}
}
read_asciihead(head,fp)
ECO_HEADER *head;
FILE *fp;
{
	int 	cnt;
	char	c;

	head->nl = ASC_NROWS;
	head->ns = ASC_NCOLS;
	head->data_type = ASCII;

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter read_asciihead()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/*
	 * find the character position of the 6th newline
	 */
    fseek(fp,0,BEGIN);
	for (cnt = 0; cnt < 6; cnt++) {
		while ((c = getc(fp)) != '\n') { ;}
	}
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Exit read_asciihead()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	return(0);
}
