/*************************************************************************/
/* spread.c - VISUAL and NON-VISUAL PREDICTION/CALIBRATION VERSION       */
/* last revised 11/02 sbw
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include <math.h>
#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

extern int      ticktock;
extern int      nrows, ncols;

extern short int roads[NROWS][NCOLS];
extern short int excld[NROWS][NCOLS];
extern short int slp[NROWS][NCOLS];

extern int      num_growth_pix;
extern float    growth_rate, average_slope;
extern int 		influences();

int 		delta[NROWS][NCOLS];

extern	int	checksubs();

int     growth_row[NROWS*NCOLS], growth_col[NROWS*NCOLS];

/* This routine actually iterates over the array */
int
spread(gstats, mstats, cells2add, tot_added)
	GSTATS         	*gstats;
	STATS          	*mstats;
	int		cells2add, *tot_added;
{

	register int    ii, jj, i, j, doit;
	int             nrows_minus_two;
	int             ncols_minus_two;
	int             row, col, road_trip, stop_here, run;
	int             time, changed;
	int             growth_count, rep_loops, rep_cnt;
	int             row_func, col_func;
	int		TMP_TOT;
	/* 
	 *	note: swght array was 90 but in weights.c, array is set from 0-90
	 *	or a sum of 91
	 */
	double          swght[91];
	int             start_r, start_c;
	int             r, c, temp, pixel, pick_val = 0;
	int		phase_cnt = 0;
	float 		cut;
	void 		pixrect();
	FILE 		*fp;
	int		row_dist, col_dist, spread_d, xx, yy;
	int		Tot_Ex_Del, Tot_Rd_Del, loop_cnt, zz;
	int		check_val;

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter spread()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	nrows_minus_two = nrows - 2;
	ncols_minus_two = ncols - 2;

	/* Zero the growth array for this time period */
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			delta[i][j] = 0;
		}
	}

	temp = (int) param.slope_resistance;
	get_slp_weights(temp, swght);


#ifndef HILT
	/*
	 * we will calculate the number of cells that need to be urbanized based
	 * on the projected population (exponential growth). Allow the loop to stop
	 * if it is within 5% of the target.
	 */
	loop_cnt = 0;
	TMP_TOT = 0;
	/* reset the actual growth count for this time period */
	while (TMP_TOT < cells2add)
	{
		loop_cnt++;
		if (loop_cnt%500 == 0) {
#ifdef DEBUG
			(void)fprintf(DBG_FILE, "(%s %d) Another 500 passes\n",
			      __FILE__, __LINE__);
			(void)fflush(DBG_FILE);
#endif
		}
		if (ABOUT_FULL == FALSE && loop_cnt >= 5000) {
			if (BRIEF) {
#ifdef DEBUG
				(void)fprintf(DBG_FILE, "(%s %d) Hard to find cells. Quit Early.\n",
			      	__FILE__, __LINE__);
				(void)fflush(DBG_FILE);
#endif
				return(-2);
			} else {
#ifdef DEBUG
				(void)fprintf(DBG_FILE, "(%s %d) Hard to find cells. Relax restrictions.\n",
			      __FILE__, __LINE__);
				(void)fflush(DBG_FILE);
#endif
				ABOUT_FULL = TRUE;
				param.spread_coefficient = 99;
				param.diffusion_coefficient = 99;
				param.breed_coefficient = 99;
				param.road_gravity = 99;
				NEI_RQMT = 1;
				NEI_HOOD = 4;
				PICKMAX = 10000;
			}
		}
#endif   /* orig HILT */


	/* Select a random point on the map */
	check_val = FALSE;
	while (check_val == FALSE) {
		i = (int) pickrow();
		j = (int) pickcol();
		if (checksubs(i, j, __FILE__, __LINE__) != 0) 
			continue;
		check_val = TRUE;
	}

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "\n(%s %d) Phase 1: diffusive growth\n", 
	__FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
	/* Phase 1 growth: Random weighted by slope */

	if ((z[i][j] < PHASE0G) && (delta[i][j] == 0)) {
		/* Test for map edge */
		if ((i < nrows_minus_two) && (j < ncols_minus_two) && (i > 0) && (j > 0)) {
			if (pickr() < PICKMAX * swght[slp[i][j]]) 
			{

				/* Allow spontaneous growth based on diffusion coefficient */
				if (pickr() < param.diffusion_coefficient + influences(i, j)) {
					if (chk_excld(i,j) == 0) {
						delta[i][j] =  PHASE1G;
						TMP_TOT++;
						check_park(TMP_TOT, i, j);
						gstats->sng++;
					}

				}
			}
		}
	}

#ifndef HILT
	if (TMP_TOT >= cells2add)
		break;
#endif /* HILT */

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Phase 2 and 3: spread centers\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* Grow (Phase 2) and spread (phase 3) new centers : Weighted by breed */
	/* Select a new random point on the map */
	check_val = FALSE;
	while (check_val == FALSE) {
		i = (int) pickrow();
		j = (int) pickcol();
		if (checksubs(i, j, __FILE__, __LINE__) != 0)
			continue;
		check_val = TRUE;
	}

	if (pickr() < param.breed_coefficient + influences(i,j)) {
#ifdef DECAY_SPREAD
		if ((z[i][j] < PHASE0G) && (delta[i][j] == 0)) 
#else
		if ((z[i][j] < PHASE0G) && (delta[i][j] == 0)) 
#endif
		{
			/* Allow a new center to form spontaneously */
			if (pickr() < PICKMAX * swght[slp[i][j]]) {
				if (chk_excld(i,j) == 0) {
					delta[i][j] =  PHASE2G;
					TMP_TOT++;
					check_park(TMP_TOT, i, j);
					gstats->sdg++;
				}
				/* Spread newly diffused center */
				temp = 0;
				while (temp < 3) {
					static int wh_cnt_2 = 0;
					wh_cnt_2++;
					if (wh_cnt_2 > 10) {
#ifdef xDEBUG
						(void)fprintf(DBG_FILE, "(%s %d) wh_cnt_2 over. \n", 
						__FILE__, __LINE__);
						(void)fflush(DBG_FILE);
#endif
						break;
					}
					ii = i + ((int) (pickr() * 0.03) - 1);
					jj = j + ((int) (pickr() * 0.03) - 1);

					/* don't let the subscripts go past the array bounds */
					if ((ii < 0 || ii >= NROWS) || (jj < 0 || jj >= NCOLS)) {
						continue;
					}

					if ((z[ii][jj] < PHASE0G) && (delta[ii][jj] == 0)) {
						if (pickr() < PICKMAX * swght[slp[ii][jj]]) {
							if (chk_excld(ii,jj) == 0) {
								delta[ii][jj] =  PHASE3G ;
								TMP_TOT++;
								check_park(TMP_TOT, ii, jj);
								gstats->sdc++;
							}
						}
						temp++;
					}
				}
			}
		}
	}

#ifndef HILT
	if (TMP_TOT >= cells2add)
		break;
#endif /* HILT */

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Phase 4: spread growth\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* Phase 4 : Spread out the entire growth pattern (organic growth) */
	for (row = 1; row < nrows_minus_two; row++) {
#ifndef HILT
		if (TMP_TOT >= cells2add) {
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) BREAK OUT %d,%d\n", __FILE__, __LINE__, row, col);
(void)fflush(DBG_FILE);
#endif
			break;
		}
#endif /* HILT */
		for (col = 1; col < ncols_minus_two; col++) {
			if (TMP_TOT >= cells2add){
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) BREAK OUT  %d,%d\n", __FILE__, __LINE__, row, col);
(void)fflush(DBG_FILE);
#endif
				break;
			}
			if (checksubs(row, col, __FILE__, __LINE__) != 0)
				continue;
			/* First, is this already an urban pixel? */
			if ((z[row][col] >= PHASE0G) && (pickr()  <  
			param.spread_coefficient + influences(row,col))) { 
#ifndef HILT
{
				temp = 0;
				/*
				 * Examine neighbors out distance of NEI_HOOD.
				 */
				spread_d = sprd();
				for (ii = row - spread_d; ii <= row + spread_d; ii++) {
					for (jj = col - spread_d; jj <= col + spread_d; jj++) {
						if (checksubs(ii, jj, __FILE__, __LINE__) != 0)
							continue;
						if (z[ii][jj] >= PHASE0G)
							temp++;
					}
				}
				/*
				 * temp is the number of urban neighbors. Need NEI_RQMT to spread. 
				 */
				if (ABOUT_FULL)
					rep_loops = 10;
				else
					rep_loops = 1;

				/* 
				 * when the grid is almost completely urbanized, we need to be
				 * more generous or it will loop just about forever
				 */
				for (rep_cnt = 1; rep_cnt <= rep_loops; rep_cnt++) {
				if (temp >= NEI_RQMT) {
#ifdef xDEBUG
					(void)fprintf(DBG_FILE,"(%s %d)Spread up to %d cells from %d,%d\n", 
					__FILE__, __LINE__, NEI_HOOD, row, col);
					(void)fflush(DBG_FILE);
#endif
					/*
					 * It meets all specs, go ahead and diffuse
					 */
					row_func = pick_fn(); /* will be ADD or SUBTRACT */
					col_func = pick_fn(); /* will be ADD or SUBTRACT */
					row_dist = dst();	/* will be 0 to NEI_HOOD inclusive */
					col_dist = dst();	/* will be 0 to NEI_HOOD inclusive */
					if (row_func == ADD && col_func == ADD) {
						if (checksubs(row + row_dist, col + col_dist, __FILE__, __LINE__) != 0)
							continue;
#ifdef xDEBUG
						(void)fprintf(DBG_FILE, "(%s %d) try %d,%d\n", 
						__FILE__, __LINE__, row+row_dist, col+col_dist);
						(void)fflush(DBG_FILE);
#endif
						if ((z[row + row_dist][col + col_dist] < PHASE0G) &&
						    (delta[row + row_dist][col + col_dist] == 0)) {
							if (pickr() < PICKMAX * swght[slp[row + row_dist][col + col_dist]])
							if (chk_excld(row + row_dist,col + col_dist) == 0) {
								delta[row + row_dist][col + col_dist] =  PHASE4G;
								TMP_TOT++;
								check_park(TMP_TOT, row + row_dist, col + col_dist);
								gstats->og++;
							}
						}
					}
					if (row_func == ADD && col_func == SUBTRACT) {
						if (checksubs(row + row_dist, col - col_dist, __FILE__, __LINE__) != 0)
							continue;
#ifdef xDEBUG
						(void)fprintf(DBG_FILE, "(%s %d) try %d,%d\n", 
						__FILE__, __LINE__, row+row_dist, col-col_dist);
						(void)fflush(DBG_FILE);
#endif
						if ((z[row + row_dist][col - col_dist] < PHASE0G) &&
						    (delta[row + row_dist][col - col_dist] == 0)) {
							if (pickr() < PICKMAX * swght[slp[row + row_dist][col - col_dist]])
							if (chk_excld(row + row_dist,col - col_dist) == 0) {
								delta[row + row_dist][col - col_dist] =  PHASE4G;
								TMP_TOT++;
								check_park(TMP_TOT, row + row_dist, col - col_dist);
								gstats->og++;
							}
						}
					}
					if (row_func == SUBTRACT && col_func == ADD) {
						if (checksubs(row - row_dist, col + col_dist, __FILE__, __LINE__) != 0)
							continue;
#ifdef xDEBUG
						(void)fprintf(DBG_FILE, "(%s %d) try %d,%d\n", 
						__FILE__, __LINE__, row-row_dist, col+col_dist);
						(void)fflush(DBG_FILE);
#endif
						if ((z[row - row_dist][col + col_dist] < PHASE0G) &&
						    (delta[row - row_dist][col + col_dist] == 0)) {
							if (pickr() < PICKMAX * swght[slp[row - row_dist][col + col_dist]])
							if (chk_excld(row - row_dist,col + col_dist) == 0) {
								delta[row - row_dist][col + col_dist] =  PHASE4G;
								TMP_TOT++;
								check_park(TMP_TOT, row - row_dist, col + col_dist);
								gstats->og++;
							}
						}
					}
					if (row_func == SUBTRACT && col_func == SUBTRACT) {
						if (checksubs(row - row_dist, col - col_dist, __FILE__, __LINE__) != 0)
							continue;
#ifdef xDEBUG
						(void)fprintf(DBG_FILE, "(%s %d) try %d,%d\n", 
						__FILE__, __LINE__, row-row_dist, col-col_dist);
						(void)fflush(DBG_FILE);
#endif
						if ((z[row - row_dist][col - col_dist] < PHASE0G) &&
						    (delta[row - row_dist][col - col_dist] == 0)) {
							if (pickr() < PICKMAX * swght[slp[row - row_dist][col - col_dist]])
							if (chk_excld(row - row_dist,col - col_dist) == 0) {
								delta[row - row_dist][col - col_dist] =  PHASE4G;
								TMP_TOT++;
								check_park(TMP_TOT, row - row_dist, col - col_dist);
								gstats->og++;
							}
						}
					}

				} /* End of neighbors test */
				} /* end of repeated loops when almost full */
}
#else	/* use orig HILT code */
{
				temp = 0;
				/*
				 * Examine the eight cell neighbors
				 * Spread at random if at least four are urban
				 * Pixel itself must be urban (3)
				 */
				for (ii = row - 1; ii < row + 2; ii++) {
					for (jj = col - 1; jj < col + 2; jj++) {
						if (z[ii][jj] > 0)
							temp++;
					}
				}
				if ((temp >= 4) && (temp < 9)) {
					cut = (float) PICKMAX / 8.0;
					/*
					 * It meets all specs, go
					 * ahead and diffuse
					 */

					pixel = (int) ((1 + pickr()) / cut);
					if (pixel == 8) pixel = 0;
					switch (pixel) {
					case 0:
						if ((z[row - 1][col - 1] == 0) &&
						    (delta[row - 1][col - 1] == 0))
							if (pickr() < PICKMAX * swght[slp[row - 1][col - 1]])
							if (chk_excld(row - 1,col - 1) == 0) {
								delta[row - 1][col - 1] =  PHASE4G;
								TMP_TOT++;
								check_park(TMP_TOT, row - 1, col - 1);
								gstats->og++;
							}
						break;
					case 1:
						if ((z[row - 1][col] == 0) &&
							    (delta[row - 1][col] == 0))
						if (pickr() < PICKMAX * swght[slp[row - 1][col]])
							if (chk_excld(row - 1,col) == 0) {
								delta[row - 1][col] =  PHASE4G ;
								TMP_TOT++;
								check_park(TMP_TOT, row -1 , col);
								gstats->og++;
						}
						break;
					case 2:
						if ((z[row - 1][col + 1] == 0) &&
						    (delta[row - 1][col + 1] == 0))
							if (pickr() < PICKMAX * swght[slp[row - 1][col + 1]])
							if (chk_excld(row - 1,col + 1) == 0) {
								delta[row - 1][col + 1] =  PHASE4G ;
								TMP_TOT++;
								check_park(TMP_TOT, row - 1, col + 1);
								gstats->og++;
							}
						break;
					case 3:
						if ((z[row][col - 1] == 0) &&
						    (delta[row][col - 1] == 0))
							if (pickr() < PICKMAX * swght[slp[row][col - 1]])
								if (chk_excld(row,col - 1) == 0) {
									delta[row][col - 1] =  PHASE4G ; 
									TMP_TOT++;
									check_park(TMP_TOT, row, col - 1);
									gstats->og++;
								}
						break;
					case 4:
						if ((z[row][col + 1] == 0) &&
						    (delta[row][col + 1] == 0))
						if (pickr() < PICKMAX * swght[slp[row][col + 1]])
						if (chk_excld(row,col + 1) == 0) {
							delta[row][col + 1] =  PHASE4G ;
							TMP_TOT++;
							check_park(TMP_TOT, row, col + 1);
							gstats->og++;
							}
						break;
					case 5:
						if ((z[row + 1][col - 1] == 0) &&
						    (delta[row + 1][col - 1] == 0))
							if (pickr() < PICKMAX * swght[slp[row + 1][col - 1]])
								if (chk_excld(row + 1,col - 1) == 0) {
									delta[row + 1][col - 1] =  PHASE4G;
									TMP_TOT++;
									check_park(TMP_TOT, row + 1, col - 1);
									gstats->og++;
								}
						break;
					case 6:
						if ((z[row + 1][col] == 0) &&
						    (delta[row + 1][col] == 0))
							if (pickr() < PICKMAX * swght[slp[row + 1][col]])
								if (chk_excld(row + 1,col) == 0) {
									delta[row + 1][col] =  PHASE4G ;
									TMP_TOT++;
									check_park(TMP_TOT, row + 1, col);
									gstats->og++;
								}
						break;
					case 7:
						if ((z[row + 1][col + 1] == 0) &&
						    (delta[row + 1][col + 1] == 0))
							if (pickr() < PICKMAX * swght[slp[row + 1][col + 1]])
								if (chk_excld(row + 1,col + 1) == 0) {
									delta[row + 1][col + 1] =  PHASE4G ;
									TMP_TOT++;
									check_park(TMP_TOT, row + 1, col + 1);
									gstats->og++;
								}
						break;
					default:
					printf("Unmatched case in switch statement, function spread()\n");
					} /* End of switch */
				} /* End of neighbors test */
}
#endif  /* orig HILT */
			 } /* End of urban test */
		} /* Columns loop */
	} /* Rows loop */


#ifndef HILT
	if (TMP_TOT >= cells2add) {
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) BREAK OUT\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
		break;
	}
#endif /* HILT */

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Phase 5: road trips\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
	/* Phase 5: Road trips */
	/* Check for new growth */
	growth_count = 0;
	for (row = 0; row < nrows; row++) {
		for (col = 0; col < ncols; col++) {
			if (delta[row][col] >= PHASE0G) {
				if (pickr() < param.diffusion_coefficient) {
					growth_row[growth_count] = row;
					growth_col[growth_count] = col;
					growth_count++;
				}
			}
		}
	}

	if (growth_count > 0) {
		road_trip = 0;

      		doit = (int) ((double) growth_count * pickr() / 100.);
		start_r = growth_row[doit];
		start_c = growth_col[doit];
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) start road trip at %d,%d\n", __FILE__, __LINE__, start_r, start_c);
(void)fflush(DBG_FILE);
#endif

		/* Starting at this location, seek outwards  */
		temp = (int) param.road_gravity;

		for (ii = start_r - temp; ii < start_r + temp; ii++) {
			for (jj = start_c - temp; jj < start_c + temp; jj++) {
				if ((ii >= 0) && (jj >= 0) && (ii < nrows - 1) &&
			    	(jj < nrows - 1)) {
					/*
				 	 * Is there a road in the
				 	 * neighborhood?
				 	 */
					if (roads[ii][jj] == 255) {
						road_trip = 1;
						r = ii;
						c = jj;
					}
				}
			}
		}
		/*
	 	 * If there's a road around, go walkies!
	 	 */
		if (road_trip) {
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) ROAD TRIP!\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif

			i = r;
			j = c;
			row = r;
			col = c;
			stop_here = 0;
			run = 0;
			while (!stop_here) {
#ifndef HILT
{
				static int wh_cnt_3 = 0;
				wh_cnt_3++;
				if (wh_cnt_3 > 100) {
#ifdef xDEBUG
					(void)fprintf(DBG_FILE, "(%s %d) wh_cnt_3 over. \n", 
					__FILE__, __LINE__);
					(void)fflush(DBG_FILE);
#endif
					stop_here = 1;
					continue;
				}
}
#endif   /* orig HILT */
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) old i: %d    old j: %d\n", __FILE__, __LINE__, i, j);
(void)fflush(DBG_FILE);
#endif
				i += ((int) (pickr() * 0.03) - 1);
				j += ((int) (pickr() * 0.03) - 1);
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) new i: %d    new j: %d\n", __FILE__, __LINE__, i, j);
(void)fflush(DBG_FILE);
#endif
				if (checksubs(i, j, __FILE__, __LINE__) != 0) {
					stop_here = 1;
					continue;
				}
				if (roads[i][j] == 255) {
					/*
					 * Found a road, move along
					 * it
					 */
					run++;
					row = i;
					col = j;
				} else {
					i = row;
					j = col;
				}
				if (run > ((int) param.diffusion_coefficient * 2)) {
					stop_here = 1;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) ran road: %d (stop at: %d)\n", 
__FILE__, __LINE__, run, ((int) param.diffusion_coefficient * 2));
(void)fflush(DBG_FILE);
#endif
				}
			}

			/*
			 * spread new center found by road trip,
			 * otherwise it won't act as a new center
			 */
		}
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Begin spread new center after road trip\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
		if ((i < nrows_minus_two) && (j < ncols_minus_two) && (i > 0)
		    && (j > 0)) {
			for (ii = i - 1; ii < i + 2; ii++) {
				for (jj = j - 1; jj < j + 2; jj++) {
					if (checksubs(ii, jj, __FILE__, __LINE__) != 0)
						continue;

					if ((z[ii][jj] < PHASE0G) && (delta[ii][jj] == 0)) {
						if (chk_excld(ii,jj) == 0) {
							if (pickr() < (float) PICKMAX * 0.5 * swght[slp[ii][jj]]) {
								delta[ii][jj] =  PHASE5G ;
								TMP_TOT++;
								check_park(TMP_TOT, ii, jj);
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Phase5G in %d,%d\n", __FILE__, __LINE__, ii, jj);
(void)fflush(DBG_FILE);
#endif
								gstats->rt++;
							}
						}
					} 
				}
			}
		}
	}

	/* counts growth at each pass (may do more than one pass per year) */
	/*
	actual_growth = 0;
	for (xx = 0; xx < nrows; xx++) {
		for (yy = 0; yy < ncols; yy++) {
			if (delta[xx][yy] > PHASE0G) actual_growth++;
		}
	}
	*/
#ifndef HILT
	} /* end exponential loop */ 
#ifndef HILT
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) year sum: %d cells (%d needed) (%d LOOPS)\n",
	__FILE__, __LINE__, TMP_TOT, cells2add, loop_cnt);
	(void)fflush(DBG_FILE);
#endif
#endif /* HILT */

#endif   /* orig HILT */
	/**************************************************
	Now place growth array into current array
	***************************************************/
	changed = 0;
	num_growth_pix = 0;

	gstats->pop = 0;
	average_slope = 0.0;
	Tot_Ex_Del = 0;
	Tot_Rd_Del = 0;
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			/* clear out anything higher than PHASE5 growth */
			if (delta[i][j] > PHASE5G) {
				delta[i][j] = 0;
				Tot_Rd_Del++;
			}

			/* clear anything that may have gone into an excluded area */
			if (chk_excld(i,j) == 1) {
				if (delta[i][j] != 0) {
					delta[i][j] = 0;
					Tot_Ex_Del++;
				}
			}

			if (delta[i][j] > PHASE0G)
				changed = 1;

			/*
			 * if there is a non-urban cell in z that is now an urban growth cell
			 * (ie > PHASE0), save it, count it and recompute slope average
			 */
			if ((z[i][j] < PHASE0G) && (delta[i][j] > PHASE0G)) {
				/* New growth being placed into array */
				average_slope += (float) slp[i][j];
				z[i][j] = delta[i][j];
				num_growth_pix++;
			}
			if (z[i][j] >=  PHASE0G )
				gstats->pop++;
		}
	}
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Urban del from chk_excld: %d    Del from Roads: %d\n",
__FILE__, __LINE__, Tot_Ex_Del, Tot_Rd_Del);
(void)fflush(DBG_FILE);
#endif
	if (num_growth_pix == 0)
		average_slope = 0.0;
	else 
		average_slope /= (float) num_growth_pix;

	*tot_added = TMP_TOT;
	return (changed);
}

/*****************************************************************
*****************************************************************/
int
sprd()
{
	/* go out randomly as far as NEI_HOOD */
	if (ABOUT_FULL)
		return( NEI_HOOD ); /* number between 1 and NEI_HOOD incl */
	else
		return( rand() % NEI_HOOD + 1); /* number between 1 and NEI_HOOD incl */
}
/*****************************************************************
*****************************************************************/
int
dst()
{
	/* can grow into a cell up to NEI_HOOD away, leave empty lot */
	return(rand() % (NEI_HOOD + 1));   /* it will be a 0 up to NEI_HOOD inclu */
	
}
/*****************************************************************
*****************************************************************/
int
pick_fn()
{
	/* returns 0 or 1, just like a coin flip; test as ADD or SUBTRACT */
	return(rand() % 2);   
}
	
/*****************************************************************
*****************************************************************/
int
chk_excld(i,j)
{
	int res = 0;
	if (WEX_FLAG == FALSE) {
		if (excld[i][j] == 0)
			res = 0;
		if (excld[i][j] >= 1)
			res = 1;
	} else {
		if (pickr() < excld[i][j])
			res = 1;
		else
			res = 0;
	}
	return(res);
}
/*****************************************************************
*****************************************************************/
int
check_park(TMP_TOT, i, j)
int	TMP_TOT, i, j;
{

	/* 
	 * If we are either waiting for a good park spot or our number has come up,
	 * add a park. If we can't add one because of location, set the flag to try
	 * next time.
	 */
	if (PARK_PEND) {  
		if (add_park(i, j)) 
			PARK_PEND = FALSE;
		return(0);
	}
	/* 
	 * separate these two cases in case we loop many times on a pending park; in that case 
	 * we'd end up putting too many parks 
	 */
	if ((rand() % 74) == 0) {  
		if (add_park(i, j)) 
			PARK_PEND = FALSE;
		else
			PARK_PEND = TRUE;
	}
	return(0);
}
/*****************************************************************
*****************************************************************/
int
add_park(i, j)
int	i, j;
{
	int	row, col, x_row, x_col;

	for (row = i-PARK_DIST; row <= i+PARK_DIST; row++) {
		for (col = j-PARK_DIST; col <= j+PARK_DIST; col++) {
			if (row<0 || row>=NROWS || col<0 || col>=NCOLS)
				continue;
			if ( z[row][col] == PARKS) {
#ifdef xDEBUG
				(void)fprintf(DBG_FILE, "(%s %d) Park exits nearby; wait until next time\n", 
				__FILE__, __LINE__);
				(void)fflush(DBG_FILE);
#endif
				return(FALSE); /* failed to add a park */
			}
			/* no parks nearby so add one in an adjoining cell */
			for (x_row = i-1; x_row <= i+1; x_row++) {
				for (x_col = j-1; x_col <= j+1; x_col++) {
					if (x_row<0 || x_row>=NROWS || x_col<0 || x_col>=NCOLS)
						continue;
					if ((z[x_row][x_col] < PHASE0G) && (delta[x_row][x_col] == 0) 
					&& (chk_excld(x_row,x_col) == 0)) {
						/* put the park in the master array */
						z[x_row][x_col] = PARKS;
						/* put the park in the exclusion array */
						excld[x_row][x_col] = PARKS;
#ifdef xDEBUG
						(void)fprintf(DBG_FILE, "(%s %d) ADDED Park at %d,%d\n",
						__FILE__, __LINE__, x_row, x_col);
						(void)fflush(DBG_FILE);
#endif
						return(TRUE); /* successfully added a park */
					}
				}
			}
		}
	}
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Failed to Add Park\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
	return(FALSE); /* failed to add a park */
}
