/*************************************************************************/
/* Circle.c - VISUAL.CALIB VERSION                                       */
/* Compute the  circle of equal area located at the mean center          */
/* last revised                                                          */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include <math.h>
#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern	FILE *DBG_FILE;
#endif

STATS 
Circle(Stat)
	STATS           Stat;
{
	FILE           *fp;
	int             i, j, number = 0;
	double          xmean = 0.0, ymean = 0.0;

	/* First, compute the means */
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			if (z[i][j] >= PHASE0G) {
				xmean += (double) j;
				ymean += (double) i;
				number++;
			}
		}
	}
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) %d Urban pixels found\n", __FILE__, __LINE__, number);
	(void)fflush(DBG_FILE);
#endif
	/* printf("%d urban pixels found\n", number); */

	xmean /= (double) number;
	ymean /= (double) number;
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Mean center (%f %f)\n", __FILE__, __LINE__, xmean, ymean);
	(void)fflush(DBG_FILE);
#endif
/*
	fp = fopen("record", "a");
	fprintf(fp, "Mean center (%f,%f)", xmean, ymean);
	fclose (fp);
*/

	Stat.area = number;
	Stat.xmean = xmean;
	Stat.ymean = ymean;

	/* Compute the radius of the circle with same area as number */
	Stat.rad = pow((Stat.area / PI), 0.5);
	return (Stat);
}
