/*************************************************************************/
/* Main.c - VISUAL.CALIB VERSION                                         */
/* revised 02-5-96 Sal Perez                                             */
/* last revised 6/9/96, Stacy Hoppen                                     */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include <sys/stat.h>
#include "defines.h"
#include "typedefs.h"
#include "graphics.h"
#include "localdefs.h"


#ifdef DEBUG
FILE *DBG_FILE;
#endif

/* internal and external functions */
void init();
void set_hints();
void create_window_set_properties();
extern void close_frames();
extern void show_status_window();
void get_colors();
extern void plot_circle(double xc, double yc, double distance, unsigned long color);
void make_cursor ();
Pixmap  make_pixmap(char* data,int width, int height);
void select_events();
void draw_button (int win);
void take_events ();
void delay ();
void paint_grid();
extern void growth(), pixrect(FILE *cum_file);
extern void draw_text();/* legend text */
void initial_status(); /* display initial status */

void initialize_param(); 
void begin_simulation(); 

	char theiconname[] = "Growth";
	char thewindowname[] = "G r o w t h    C o n t r o l    W i n d o w";
	char *display_name=NULL;
	int pressedColor   =	WHITE;
	int unpressedColor =	BLACK;
	int disabledColor  =	NOCOLOR;

	/* buttons' windows, Begin, Quit, Growth, their id, x, y, width, 
	   and height values. */
	OpaqueFrame Buttons[NBUTTONS] = {
		{ 0, 5, growth_icon_height + 20, begin_icon_width, begin_icon_height}, 

		{ 0, 5, 2*growth_icon_height + 30, quit_icon_width, quit_icon_height},

		{ 0, 5, 10, growth_icon_width, growth_icon_height}, };

	/* colors definition windows */
	OpaqueFrame legend[LEGENDWINDOWS] = { { 0, 5, 550, 18, 15 }, { 0, 5, 570, 18, 15},
				 { 0, 5, 590, 18, 15}, { 0, 5, 610, 18, 15}, 
				 { 0, 5, 630, 18, 15}, { 0, 5, 650, 18, 15},
				 { 0, 5, 670, 18, 15}, { 0, 5, 690, 18, 15}, };


	/* status window */
	OpaqueFrame status = { 0, 2, 230, 135, 320};

	/* changes buttons' colors when pressed */
		struct windata {
	int	color;
	} windata[NBUTTONS] = {
		{ NOCOLOR }, { NOCOLOR }, { NOCOLOR }, };


extern char 	*optarg;
extern int	getopt();
/* program name */
static char *progname;

int		grid_repeat_val = 0;

main(argc, argv)
int argc;
char *argv[];
{
int		errflag, argc_rem;
char	c;
char	grid_repeat_str[MAX_LEN];

#ifdef DEBUG
	DBG_FILE=fopen("./debug_file", "w");
#endif
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter Main\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	/* initialize */
	errflag = 0;
	WEX_FLAG = FALSE;
	DATA_FLAG = LOG_FLAG = YEAR_FLAG = FALSE;
	GRID_FLAG = FALSE;
	SELFMOD_FLAG = TRUE;
	argc_rem = argc;
	strcpy(LOG_DIR, ".");
	strcpy(DATA_DIR, ".");
	memset(YEAR_FILE, '\0', MAX_LEN);
	memset(GRID_NAME, '\0', MAX_LEN);
	memset(GRID_STR, '\0', MAX_LEN);
	memset(ORIG_GRID_STR, '\0', MAX_LEN);
	memset(grid_repeat_str, '\0', MAX_LEN);
	AGRI_FACTOR = AGRI_DEFAULT;
	MEAN_FACTOR = MEAN_DEFAULT;
	YEAR_CNT = 0;
	NEI_RQMT = NEI_RQMT_DEF;
	NEI_HOOD = NEI_HOOD_DEF;
	NEI_DIST = NEI_DIST_DEF;

	/* process the command line */
	while ((c = getopt(argc, argv, "d:y:l:g:r:h:w")) != EOF ) { switch (c) {
		case 'd':
			strcpy(DATA_DIR, optarg);
			DATA_FLAG = TRUE;
			argc_rem -= 2;
			break;
		case 'w':
			WEX_FLAG = TRUE;
			argc_rem -= 2;
			break;
		case 'l':
			strcpy(LOG_DIR, optarg);
			LOG_FLAG = TRUE;
			argc_rem -= 2;
			break;
		case 'y':
			strcpy(YEAR_FILE, optarg);
			YEAR_FLAG = TRUE;
			YEAR_CNT = read_years(YEAR_FILE);
			argc_rem -= 2;
			break;
		case 'g':
			AGRI_FACTOR = atoi(optarg);
			argc_rem -= 2;
			break;
		case 'r':
			NEI_RQMT = atoi(optarg);
			argc_rem -= 2;
			break;
		case 'h':
			NEI_HOOD = atoi(optarg);
			argc_rem -= 2;
			break;
		case '?':
			errflag++;
			break;
		} /* end switch */
		/* if there are errors, print the usage message and quit */
		if ( errflag ) {
			fprintf(stderr, "usage: %s [-d data_dir] [-y year_file] [-l log_dir]\n\n", progname);
 			fprintf(stderr, "	-d	name of directory where input files reside\n");
 			fprintf(stderr, "	-y	name of file which contains years for which data exists\n");
 			fprintf(stderr, "	-g	default value for agriculture (0-99)\n");
 			fprintf(stderr, "	-r	# cells reqd urban to consider spread\n");
 			fprintf(stderr, "	-h	how far out cells are considered neighbors\n");
 			fprintf(stderr, "	-t	can leap frog up to this many cells away\n");
 			fprintf(stderr, "	-l	name of directory where logfiles will go\n\n");
			exit(1);
		}
	}
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) \n", __FILE__, __LINE__);
	(void)fprintf(DBG_FILE, "	DATA: %s   LOG: %s   YEAR: %s\n",
	DATA_DIR, LOG_DIR, YEAR_FILE);
	(void)fprintf(DBG_FILE, "	NEI_RQMT: %d   NEI_HOOD: %d\n",
	NEI_RQMT, NEI_HOOD);
	(void)fflush(DBG_FILE);
#endif

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) \n", __FILE__, __LINE__);
	(void)fprintf(DBG_FILE, "DATA: %s   LOG: %s   YEAR: %s\n",
	DATA_DIR, LOG_DIR, YEAR_FILE);
	(void)fprintf(DBG_FILE, "(%s %d) GRID_FLAG: %s\n", 
	__FILE__, __LINE__, (GRID_FLAG == TRUE)?"TRUE":"FALSE");
	(void)fflush(DBG_FILE);
#endif

	if (GRID_FLAG == TRUE) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Input grid_str: >%s<\n", 
		__FILE__, __LINE__, GRID_STR);
		(void)fflush(DBG_FILE);
#endif
		if (get_name(GRID_NAME, GRID_STR) == 0){
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) GRID_STR: >%s<\n", __FILE__, __LINE__, GRID_STR);
	(void)fflush(DBG_FILE);
#endif
			if (strcmp(GRID_STR, "") == 0)
				exit(0);
			else {
#ifdef xDEBUG
				(void)fprintf(DBG_FILE, "(%s %d) Use grid >%s<\n", 
				__FILE__, __LINE__, GRID_NAME);
				(void)fflush(DBG_FILE);
#endif
			}
		} else {
#ifdef DEBUG
			(void)fprintf(DBG_FILE, "(%s %d) NO grid found. Exit \n", 
			__FILE__, __LINE__);
			(void)fflush(DBG_FILE);
#endif
			exit(0);
		}
	}

	/* Connect to server, initialize display, and
	   buttons windows */
	init(argv);

	/* Get Image Colors */
	get_colors();

	/* Set windows sizes and locations */
	set_hints();

	/* Set windows names, default graphic contexts,
	   and pixmaps for buttons and logo. */
	create_window_set_properties(argc,argv);

	/* make a standard cursor */
	make_cursor ();

	/* Set various input properties for all windows */
	select_events();

	/* Display initial windows */
	XMapSubwindows(thedisplay,thewindow);
	XMapWindow(thedisplay,thewindow);

	/* Loop through all windows taking care of
	   the different events occuring on all windows */
	take_events ();

} /* end main */




/* initialize display */
void init(argv)
char *argv[];
{
	register int win;

	/*  Establish a connection */
	if ( (thedisplay=XOpenDisplay(display_name)) == NULL )
  	{ 
	   (void) fprintf(stderr, "DISPLAY OPENING ERROR: cannot connect to X server %s\n", XDisplayName(display_name));
	   exit(-1);
	}

	/*  Prepare and create the two windows */
	thescreen = DefaultScreen (thedisplay);
	visual = DefaultVisual(thedisplay, thescreen);
	colormap = DefaultColormap(thedisplay, thescreen);

	thebackground = WhitePixel (thedisplay, thescreen);
	theforeground = BlackPixel (thedisplay, thescreen);
	/*
	 * Loop through buttons, setting disabled buttons
	 * to Color Black.
	 */
	for (win = 0; win < NBUTTONS; win++) {
		windata[win].color = unpressedColor;  
	}

} /* end init() */



/* Get colors  */
void get_colors()
{
	int depth, gray_map_ok;
	Visual *visual;
	static char *name[] = {"brown", "white", "black", "grey","green","blue", "red",
				"dim gray", "dark slate gray", "gray", "khaki","yellow",
				"light gray", "light grey", "dark slate grey", "dim grey",
				"green yellow", "aquamarine", "medium aquamarine", "wheat",
				"orange", "cyan", "blue violet", "navy blue", "sky blue",
				"pink", "maroon","magenta",};
	XColor exact_def,gray_def;
	Colormap cmap;
	int ncolors = MAX_COLORS;
	int colors[MAX_COLORS];
	int i,j;
	depth  = DisplayPlanes(thedisplay,thescreen);
	visual = DefaultVisual(thedisplay,thescreen);
	cmap   = DefaultColormap(thedisplay,thescreen);
	if (depth == 1) { /* one-plane monochrome */
		/* Use BlackPixel and WhitePixel only. 
		 * We must do this first if depth is 1, since
		 * it may allocat colors succesfully but the
		 * colors might all be black */;
		border_pixel = BlackPixel(thedisplay,thescreen);
		background_pixel = WhitePixel(thedisplay,thescreen);
		foreground_pixel = BlackPixel(thedisplay,thescreen);
		}
	else {
	     for (i = 0; i<MAX_COLORS; i++) {
		if (!XParseColor(thedisplay,cmap,name[i],&exact_def))
		 { fprintf(stderr, "\n\tError:: Color name %s not in database ", name[i]);
		       	 exit(0);
		 }


		if (!XAllocColor(thedisplay,cmap,&exact_def))
		 { fprintf(stderr, "\n\tError:: all colorcells allocated and read/write\n");
		       	 exit(0);
		 }
		colors[i] = exact_def.pixel;
		} /* end for loop */
	     border_pixel       = colors[0]; /* brown */
	     background_pixel   = colors[1]; /* white */
	     white_pixel   		= colors[1]; /* white */
	     foreground_pixel   = colors[2]; /* black */
	     grey_pixel         = colors[3]; /* grey */
	     green_pixel        = colors[4]; /* green */
	     blue_pixel         = colors[5]; /* blue */
	     roads_pixel        = colors[6]; /* red */
	     dim_gray_pixel     = colors[7]; /* dim gray */
	     dark_slate_gray_pixel = colors[8]; /* dark slate gray */
	     gray_pixel         = colors[9]; /* gray */
	     khaki_pixel        = colors[10]; /* khaki */
	     yellow_pixel       = colors[11]; /* yellow */		
	     light_gray_pixel   = colors[12]; /* light gray*/
	     light_grey_pixel   = colors[13]; /* light grey*/
	     dark_slate_grey_pixel = colors[14]; /* dk slate grey*/
	     dim_grey_pixel     = colors[15]; /* dim grey*/
	     green_yellow_pixel = colors[16]; /* green yellow*/
	     aquamarine_pixel   = colors[17]; /* aquamarine*/
	     medium_aquamarine_pixel = colors[18]; 
	     wheat_pixel        = colors[19]; /* wheat*/
	     orange_pixel       = colors[20]; /* orange*/
	     cyan_pixel         = colors[21]; /* cyan */
	     blue_violet_pixel  = colors[22]; /* blue violet */
	     navy_blue_pixel    = colors[23]; /* navy blue */
	     sky_blue_pixel     = colors[24]; /* sky blue */
	     pink_pixel         = colors[25]; /* pink */
	     maroon_pixel       = colors[26]; /* maroon */
	     magenta_pixel      = colors[27]; /* magenta */


	} /* end else */

} /* end get_colors */


/* Set windows Hints (values for main and image windows' sizes and locations) */
void set_hints()
{
	thehint.x = 10;     /* moves window left or right  */
	thehint.y = 10;   /* moves window up or down  */
#ifdef SAMPLE
	thehint.width = 820;  /* window width */
	thehint.height = 740; /* window height */
#else
	/*thehint.width = 1600;*/  /* window width */
	/*thehint.height = 740;*/ /* window height */
	thehint.width = SIZE * NCOLS + 250;  /* window width */
	thehint.height = SIZE * NROWS + 150; /* window height */
#endif
	thehint.flags = PPosition | PSize | PMinSize;


	imagehint.x = 230;  /* image x coord on screen. moves window left or right */
	imagehint.y = 10;   /* image y coodr on screen. moves window up or down  */
#ifdef SAMPLE
	imagehint.width = 570;  /* (SIZE * NCOLS).  image width  */
	imagehint.height = 720; /* (SIZE * NROWS).  image height */
#else
	/*imagehint.width = 8806;*/  /* (SIZE * NCOLS).  image width  */
	/*imagehint.height = 2710;*/ /* (SIZE * NROWS).  image height */
	imagehint.width = SIZE * NCOLS;  /* (SIZE * NCOLS).  image width  */
	imagehint.height = SIZE * NROWS; /* (SIZE * NROWS).  image height */
#endif
	imagehint.flags = PPosition | PSize | PMinSize;

} /* end set_hints() */



/* Specify x,y coordinates, width and height for window, 
 * create pixmaps for logo, begin, and quit buttons with their
 * respective locations, widths, and heights.
 * Create the different graphic values, set windows and 
 * icons names.
*/
	/* This is needed before phisically mapping the windows */

void create_window_set_properties(argc,argv)
int argc;  char *argv[];
{
	XSetWindowAttributes attrib_begin;
	XSetWindowAttributes attrib_quit;
	XSetWindowAttributes attrib_growth;
	XSetWindowAttributes temp_attrib;

	register int i;
	attrib.cursor = thecursor;
	/* background for Growth main window */
	attrib.background_pixel = background_pixel + 2;/* excellent */
	attrib.border_pixel = theforeground;

	/****** Create image window  *******/

	thewindow = XCreateWindow (thedisplay,
			DefaultRootWindow (thedisplay),
			thehint.x, thehint.y,
			thehint.width, thehint.height,
			BORDER_WIDTH, DefaultDepth(thedisplay,thescreen),
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &attrib);


	thegc = XCreateGC (thedisplay, thewindow, 0, 0);

	if (thegc == NULL) {
		fprintf(stderr, "XCreateGC failed ... exiting... \n");
		exit(1);
	}
	

	XSetSubwindowMode(thedisplay,thegc,IncludeInferiors);

	/* background for status window */
	temp_attrib.cursor = thecursor;
	temp_attrib.background_pixel = background_pixel + 2;
	temp_attrib.border_pixel = thebackground + 2;
	status.self = XCreateWindow (thedisplay,
			thewindow,
			status.x, status.y,
			status.width, status.height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);

	/* Create the windows for the legend with their associated colors
		and locations. */
	temp_attrib.cursor = thecursor;
	temp_attrib.background_pixel = foreground_pixel;/* ROADS */
	temp_attrib.border_pixel = theforeground;
	legend[0].self = XCreateWindow (thedisplay,
			thewindow,
			legend[0].x, legend[0].y,
			legend[0].width, legend[0].height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);

#ifdef SAMPLE
	temp_attrib.background_pixel = dark_slate_grey_pixel;/**/
#else /* SAMPLE */
	temp_attrib.background_pixel = white_pixel;/* excluded */
#endif /* SAMPLE */
	temp_attrib.border_pixel = theforeground;
	legend[1].self = XCreateWindow (thedisplay,
			thewindow,
			legend[1].x, legend[1].y,
			legend[1].width, legend[1].height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);


#ifdef SAMPLE
	temp_attrib.background_pixel = orange_pixel;/*was magenta; seed*/
#else /* SAMPLE */
	temp_attrib.background_pixel = roads_pixel;/* SEED */
#endif /* SAMPLE */
	temp_attrib.border_pixel = theforeground;
	legend[2].self = XCreateWindow (thedisplay,
			thewindow,
			legend[2].x, legend[2].y,
			legend[2].width, legend[2].height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);


#ifdef SAMPLE
	temp_attrib.background_pixel = magenta_pixel;/*phases 1g, 2g, 3g*/
#else /* SAMPLE */
	temp_attrib.background_pixel = yellow_pixel;/**/
#endif /* SAMPLE */
	temp_attrib.border_pixel = theforeground;
	legend[3].self = XCreateWindow (thedisplay,
			thewindow,
			legend[3].x, legend[3].y,
			legend[3].width, legend[3].height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);

#ifdef SAMPLE
	temp_attrib.background_pixel = background_pixel;/*phase4g*/
#else /* SAMPLE */
	temp_attrib.background_pixel = orange_pixel;/* spontaneous growth and spread */
#endif /* SAMPLE */
	temp_attrib.border_pixel = theforeground;
	legend[4].self = XCreateWindow (thedisplay,
			thewindow,
			legend[4].x, legend[4].y,
			legend[4].width, legend[4].height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);


#ifdef SAMPLE
	temp_attrib.background_pixel = pink_pixel;/*phase3g*/
#else /* SAMPLE */
	temp_attrib.background_pixel = green_pixel;/* agriculture */
#endif /* SAMPLE */
	temp_attrib.border_pixel = theforeground;
	legend[5].self = XCreateWindow (thedisplay,
			thewindow,
			legend[5].x, legend[5].y,
			legend[5].width, legend[5].height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);


#ifdef SAMPLE
	temp_attrib.background_pixel = aquamarine_pixel;/*phase4g*/
#else /* SAMPLE */
	temp_attrib.background_pixel = gray_pixel;/* available */
#endif /* SAMPLE */
	temp_attrib.border_pixel = theforeground;
	legend[6].self = XCreateWindow (thedisplay,
			thewindow,
			legend[6].x, legend[6].y,
			legend[6].width, legend[6].height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);


#ifdef SAMPLE
	temp_attrib.background_pixel = yellow_pixel;/*phase5g*/
#else /* SAMPLE */
	temp_attrib.background_pixel = foreground_pixel;/* gone */
#endif /* SAMPLE */
	temp_attrib.border_pixel = theforeground;
	legend[7].self = XCreateWindow (thedisplay,
			thewindow,
			legend[7].x, legend[7].y,
			legend[7].width, legend[7].height,
			1, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &temp_attrib);

	/* Load buttons images*/
	growthpixmap  = make_pixmap(growth_icon_bits,growth_icon_width,
				growth_icon_height);
	beginpixmap  = make_pixmap(begin_icon_bits,begin_icon_width,
				begin_icon_height);
	beginpixmap2 = make_pixmap(light_begin_icon_bits,light_begin_icon_width,
				light_begin_icon_height);
	quitpixmap   = make_pixmap(quit_icon_bits,quit_icon_width,
				quit_icon_height);
	quitpixmap2  = make_pixmap(light_quit_icon_bits,light_quit_icon_width,
				light_quit_icon_height);

	/* Create the buttons as subwindows. */
	attrib_begin.background_pixmap = beginpixmap;
	attrib_begin.border_pixel = thebackground;
	attrib_quit.background_pixmap = quitpixmap;
	attrib_quit.border_pixel = thebackground;
	attrib_growth.background_pixmap = growthpixmap;
	attrib_growth.border_pixel = thebackground;

	for (i = 0; i < NBUTTONS; i++)
	   switch (windata[i].color) {
		case WHITE:
			Buttons[i].self = XCreateSimpleWindow(thedisplay,
				thewindow, Buttons[i].x, Buttons[i].y,
				Buttons[i].width, Buttons[i].height,
				1, theforeground, thebackground);
			break;
		case BLACK: /* unpressed color. Initial state. */
			if (i==0) {
			Buttons[i].self = XCreateWindow(thedisplay,
				thewindow, Buttons[i].x, Buttons[i].y,
				Buttons[i].width, Buttons[i].height,
				1, CopyFromParent, InputOutput, CopyFromParent,
				CWBorderPixel | CWBackPixmap,
				&attrib_begin);
			} else if (i==1) {
			Buttons[i].self = XCreateWindow(thedisplay,
				thewindow, Buttons[i].x, Buttons[i].y,
				Buttons[i].width, Buttons[i].height,
				1, CopyFromParent, InputOutput, CopyFromParent,
				CWBorderPixel | CWBackPixmap,
				&attrib_quit);
			} else if (i==2) {
			Buttons[i].self = XCreateWindow(thedisplay,
				thewindow, Buttons[i].x, Buttons[i].y,
				Buttons[i].width, Buttons[i].height,
				1, CopyFromParent, InputOutput, CopyFromParent,
				CWBorderPixel | CWBackPixmap,
				&attrib_growth);
			};

			break;
		}




	/*  Notify the other applications about the created windows */
	XSetStandardProperties (thedisplay, thewindow, thewindowname,
			theiconname, None, argv,
			argc, &thehint);

	attrib.background_pixel = thebackground;
	attrib.border_pixel = theforeground;

	/****** Create image window  *******/
	imagewindow = XCreateWindow (thedisplay,
			thewindow,
			imagehint.x, imagehint.y,
			imagehint.width, imagehint.height,
			BORDER_WIDTH, CopyFromParent,
			InputOutput, CopyFromParent,CWBorderPixel |
			CWBackPixel | CWCursor, &attrib);


	imagegc = XCreateGC (thedisplay, imagewindow, 0, 0);

	if (imagegc == NULL) {
		fprintf(stderr, "XCreateGC failed ... exiting... \n");
		exit(1);
	}
	

}  /* end create_window_set_properties() */



/* Make a pixmap. This section is used to create the logo, begin, and quit buttons 
   the buttons were created using iconedit using black and white colors with  
   64 x 64 dimensions.*/
Pixmap  make_pixmap(data,width,height)
char *data;
int width, height;
{
    Pixmap pid;

    pid = XCreatePixmapFromBitmapData(thedisplay, DefaultRootWindow(thedisplay),
            data, width, height, theforeground, thebackground, DefaultDepth(thedisplay,
            thescreen));
    return(pid);
}



 /* Make the cursor */
void make_cursor ()
{	/* make a hand cursor pointing left, use XC_hand1 to point right */
	thecursor = XCreateFontCursor (thedisplay, XC_hand2);
	XDefineCursor(thedisplay,thewindow,thecursor);/* only in main window */
}



/* Set up the selection of events for each window. */
void select_events ()
{
	int win;
	XSelectInput(thedisplay,thewindow,KeyPressMask | ExposureMask);
	XSelectInput(thedisplay,imagewindow,ExposureMask);
	XSelectInput(thedisplay,status.self,ExposureMask);

	/* Begin and Quit only */
	for (win = 0; win < NBUTTONS - 1; win++) 
		XSelectInput(thedisplay, Buttons[win].self,  ExposureMask
		|ButtonPressMask | ButtonReleaseMask
		|EnterWindowMask | LeaveWindowMask);
	/* growth only */
	XSelectInput(thedisplay, Buttons[win].self,ExposureMask);

	/* Legend windows */
	for (win = 0; win < LEGENDWINDOWS; win++)
		XSelectInput(thedisplay, legend[win].self, ExposureMask);

} /* end select_events */



/*  Get events and process them. */
/*  This is the main loop where processes such as cursor movement, keystrokes, 
    buttons pressing, refreshig of screen, iconization of window, etc., are 
    accounted for.  Once the simulation begins, Growth returns control
    of the program after iterating NUMBER_OF_TIMES times.  If the simulation is
    to be stopped, press Control+C to cancel the simulation.  Otherwise just
    have a sit and wait, until Begin button becomes depressed, and press Quit.
*/
    
void take_events ()
{
	void begin_simulation();
	FILE           *control;
	char            junk[50];
	int 		temp1, temp2;
	int 		temp3, temp4, temp5, temp6, temp7;
	double 		temp8, temp9, temp10, temp11, temp12;
	int 		temp13, temp14;
	int             time;

	register int win;
	register int Pressed = False;
	register int InWindow = False;
	register char *KeyChars = thebuffer;
	int old_color,keycount,finished;

	finished = FALSE;
	initialize_param();
	initial_status();

	while (finished==FALSE) 
	{
	XNextEvent (thedisplay, &theevent);

	for (win = 0; win < NBUTTONS - 1; win++)
		if (Buttons[win].self == theevent.xbutton.window)
		break;
	switch (theevent.type) {
		case KeyPress:
			break;
		case EnterNotify:
			old_color = windata[win].color;
			InWindow = True;
			draw_button (win);
			break;
		case ButtonPress:
			Pressed = True;
			windata[win].color = pressedColor;
			draw_button (win);
			break;
		case LeaveNotify:
			InWindow = False;
			windata[win].color = old_color;
			draw_button (win);
			break;
		case ButtonRelease:
			draw_button (win);
			if ((win==0) && (windata[win].color == pressedColor))
			{ draw_button (win);
				if (GRID_FLAG == TRUE)
					display_single_grid();
			  	begin_simulation();
			}
			if ((win==1) && (windata[win].color == pressedColor))
				{draw_button (win);finished=TRUE;}
			
			break;
		case Expose:
			draw_text();initial_status();break;
	}/* end case */
	}/* end while */
 XFlush(thedisplay);

 (void) close_frames();

} /* end take_events */



void initialize_param() {

	char            junk[25];
	char            junk1[25], junk2[25], junk3[25], 
			junk4[25], junk5[25], junk6[25], junk7[25], 
			junk8[25], junk9[25], junk10[25], junk11[25], 
			junk12[25], junk13[25], junk14[25];
	int             k, time, i, j;
	FILE		*control;
	int 		temp1, temp2;
	float		temp3, temp4, temp5, temp6, temp7, temp8, temp9, temp10, temp11;
	int 		temp12, temp13, temp14;
	static		int	first_time = TRUE;

	if (verify_tools_and_dirs() != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Missing something we need. Exit.\n",
			      __FILE__, __LINE__);
#endif
		exit(0);
	}

/* initialize the parameters and values set in control file */

#ifdef xDEBUG
	if (first_time == TRUE) {
	(void)fprintf(DBG_FILE, "(%s %d) Read control file\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
	}
#endif
		/* read in values from control to initialize the parameters */
		/* read in values from control to initialize the parameters */
		if ((control = fopen("control", "r")) == NULL) {
			printf("no control file\n");
			exit(1);
		}
		fscanf(control, "%d%s%*1c %d%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %f%s%*1c %d%s%*1c %d%s%*1c %d%s%*1c",&temp1, junk1, &temp2, junk2, &temp3, junk3, &temp4, junk4, &temp5, junk5, &temp6, junk6, &temp7, junk7, &temp8, junk8, &temp9, junk9, &temp10, junk10, &temp11, junk11, &temp12, junk12, &temp13, junk13, &temp14, junk14);

#ifdef xDEBUG

if (first_time == TRUE) {

(void)fprintf(DBG_FILE, "%d %s \n", temp1, junk1);
(void)fprintf(DBG_FILE, "%d %s \n", temp2, junk2);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp3, junk3);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp4, junk4);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp5, junk5);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp6, junk6);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp7, junk7);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp8, junk8);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp9, junk9);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp10, junk10);
(void)fprintf(DBG_FILE, "%.2f %s \n", temp11, junk11);
(void)fprintf(DBG_FILE, "%d %s \n", temp12, junk12);
(void)fprintf(DBG_FILE, "%d %s \n", temp13, junk13);
(void)fprintf(DBG_FILE, "%d %s \n", temp14, junk14);
(void)fflush(DBG_FILE);

first_time = FALSE;

}
#endif

	      startstop.start_date = temp1;
	      startstop.stop_date = temp2;
	      param.diffusion_coefficient = (double) temp3;
	      param.breed_coefficient = (double) temp4;
	      param.spread_coefficient = (double) temp5;
	      param.slope_resistance = (double) temp6;
	      param.road_gravity = (double) temp7;
	      selfmod.critical_high = (float)temp8;
	      selfmod.critical_low = (float)temp9;
	      selfmod.boom = (float)temp10;
	      selfmod.bust = (float)temp11;
	      selfmod.critical_slope = temp12;
	      montecarlo.number_of_times = temp13;
	      montecarlo.cumulate = temp14;
	
	      fclose (control);

} /* end initialize_param() */



void begin_simulation() {

	int             time;

/* Seed the random number generator with the process id number */
	srand(getpid());
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) begin simulation\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif

	/* start the simulation */
	for (time = 0; time < montecarlo.number_of_times; time++) {
		initialize_param();	
		/*growth(time);*/
		(void) growth();
	}
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) exit begin simulation\n", __FILE__, __LINE__);
	fflush(DBG_FILE);
#endif
	if (GRID_FLAG == FALSE) {
		if (startstop.stop_date == 1995) {
			run_fragstats(MODEL);
			run_fragstats(SEED);
			run_fragstats(REAL);
		}
	}
return;

} /* end begin_simulation */




/* Draw the pressed button. */

void draw_button (win)
register int win;
{
	XSetWindowAttributes attrib_begin,attrib_quit;
	unsigned long valuemask;

	switch (windata[win].color) {

	   case BLACK: /* unpressed button */
		attrib_begin.background_pixmap = beginpixmap;
		attrib_begin.border_pixel = theforeground;
		attrib_quit.background_pixmap = quitpixmap;
		attrib_quit.border_pixel = thebackground;
		valuemask = CWBorderPixel | CWBackPixmap;
           break;
	  case WHITE: /* pressed button */
		attrib_begin.background_pixmap = beginpixmap2;
		attrib_begin.border_pixel = theforeground;
		attrib_quit.background_pixmap = quitpixmap2;
		attrib_quit.border_pixel = thebackground;
		valuemask = CWBorderPixel | CWBackPixmap;
           break;
	} /* end case */
	if (win==0) {
		XChangeWindowAttributes(thedisplay, Buttons[win].self,
			valuemask, &attrib_begin);
		XClearWindow(thedisplay, Buttons[win].self);	
	} else if (win==1) {
		XChangeWindowAttributes(thedisplay, Buttons[win].self,
			valuemask, &attrib_quit);
		XClearWindow(thedisplay, Buttons[win].self);
	}/* end if else */
}/* end draw_button */




/* Display status and control variables with their initial values */
void initial_status()
{
	char            text[12];

	/* create structure for graphics context, prepare for image */
	
	if (!(font = XLoadQueryFont(thedisplay, "fixed"))) {
		puts("cannot load fixed font file");
		exit(1);
	}

	statgcvalues.font = font->fid;
	statgcvalues.function = GXcopy;
	statgcvalues.fill_style = FillSolid;
	statgcvaluemask = GCFunction | GCFillStyle;
	

	/*printf("\n\tIn draw_status_window ...\n");*/
	/*  Create a Graphics context */
	statgc = XCreateGC(thedisplay,status.self, statgcvaluemask, &statgcvalues);

	if (statgc == NULL) {
		fprintf(stderr, "XCreateGC failed ... exiting... \n");
		exit(1);
	}
	

	/* First, the text strings */
	/* Erase the last text strings */

	XSetForeground (thedisplay, statgc, blue_pixel);

	XClearWindow(thedisplay,status.self);


	(void) f_to_str((double) startstop.start_date, 0, text); 
	XDrawString(thedisplay, status.self, statgc, 0, 20, "DATE       :", 12);
	XDrawString(thedisplay, status.self, statgc, 80, 20, text, strlen(text) - 2);

	XDrawString(thedisplay, status.self, statgc, 0, 40, "GROWTH RATE:", 12);
	XDrawString(thedisplay, status.self, statgc, 80, 40, "1.0", 3); 


	/* display initial control parameters */
	XDrawString(thedisplay, status.self, statgc, 0, 70, " GROWTH COEFFICIENTS", 20);

	(void) f_to_str((double) param.diffusion_coefficient, 5, text);
	XDrawString(thedisplay, status.self, statgc, 0,  85, "   Diffusion:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 85, text, strlen(text) - 1);

	(void) f_to_str((double) param.breed_coefficient, 5, text);
	XDrawString(thedisplay, status.self, statgc, 0, 105, "       Breed:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 105, text, strlen(text) - 1);


	(void) f_to_str((double) param.spread_coefficient, 5, text);
	XDrawString(thedisplay, status.self, statgc, 0, 125, "      Spread:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 125, text, strlen(text) - 1);


	(void) f_to_str((double) param.slope_resistance, 5, text);
	XDrawString(thedisplay, status.self, statgc, 0, 145, "Slope Factor:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 145, text, strlen(text) - 1);


	(void) f_to_str((double) param.road_gravity, 5, text);
	XDrawString(thedisplay, status.self, statgc, 0, 165, "Road Gravity:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 165, text, strlen(text) - 1);


	/*
	XDrawString(thedisplay, status.self, statgc, 0, 200, "   SELF-MODIFICATION", 20);
	XDrawString(thedisplay, status.self, statgc, 0, 215, "     COEFFICIENTS", 17);


	(void) f_to_str((double) selfmod.critical_high, 6, text);
	XDrawString(thedisplay, status.self, statgc, 0, 230, "Critical High:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 230, text, strlen(text) - 1);

	(void) f_to_str((double) selfmod.critical_low, 6, text);
	XDrawString(thedisplay, status.self, statgc, 0, 250, " Critical Low:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 250, text, strlen(text) - 1);

	(void) f_to_str((double) selfmod.boom, 6, text);
	XDrawString(thedisplay, status.self, statgc, 0, 270, "         Boom:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 270, text, strlen(text) - 1);

	(void) f_to_str((double) selfmod.bust, 6, text);
	XDrawString(thedisplay, status.self, statgc, 0, 290, "         Bust:", 13);
	XDrawString(thedisplay, status.self, statgc, 85, 290, text, strlen(text) - 1);
	*/

} /* end initial_status() */



/* Delay a little while  */
void delay ()
{
    long tic;

    for (tic = 0; tic < 5000000; tic++)
        ;
}

read_years(year_file)
char *year_file;
{

	FILE * fp;
	struct yr_elt *curr_p, *tmp_p;
	int		year_val, count = 0;
	char	line[MAX_LEN];

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter read_years(%s)\n", 
	__FILE__, __LINE__, year_file);
	(void)fflush(DBG_FILE);
#endif
	/* read in year values from file. Create linked list. */
	if ((fp = fopen(year_file, "r")) == NULL) {
		printf("no year input file\n");
		exit(1);
	}
	/* read a year and make it an integer */
	while ( fgetline(line, fp) >= 0 ) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) LINE: %s\n", 
		__FILE__, __LINE__, line);
		(void)fflush(DBG_FILE);
#endif

		if (line[0] == '#' || line[0] == ' ' || line[0] == '\0')
			continue;

		/* if atoi fails, it will return a 0 and will be ignored*/
		year_val = atoi(line);
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) year value: %d\n", 
		__FILE__, __LINE__, year_val);
		(void)fflush(DBG_FILE);
#endif
		if (year_val > 3000 || year_val < 1700) {
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Year out of range (%d)\n", 
__FILE__, __LINE__, year_val);
#endif
			continue;
		}

#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) build year linked list\n",
		__FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		/* put it into a structure and add to list */
		curr_p = (struct yr_elt *)calloc(1, sizeof(struct yr_elt));
		curr_p->year = year_val;

		if (START_p == NULL) {
			START_p = curr_p;
			curr_p->next_p = NULL;
			curr_p->prev_p = NULL;
			curr_p = NULL;
		} else {
			tmp_p = START_p;
			while (tmp_p->next_p != NULL)
				tmp_p = tmp_p->next_p;
			curr_p->prev_p = tmp_p;
			curr_p->next_p = NULL;
			tmp_p->next_p = curr_p;
		}
		count++;

	}
#ifdef xDEBUG
	dump_list();
#endif
	return(count);
}

#ifdef DEBUG
dump_list()
{
	struct yr_elt	*curr_p;

	(void)fprintf(DBG_FILE, " Here's what the YEAR list looks like:\n");
	curr_p = START_p;
	while (curr_p != NULL) {
		(void)fprintf(DBG_FILE, "    %d\n", curr_p->year);
		(void)fflush(DBG_FILE);
		curr_p = curr_p->next_p;
	}
}
#endif

/* fgetline: get a line from a file
 * Assumes the line array is long enough for the input and that the file
 * is open.
 */
int fgetline(line, fp)
char 	*line;
FILE	*fp;
{

	char	c;
	int	count = 0;

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter fgetline()\n", __FILE__, __LINE__);
#endif
	while (c = getc(fp))
	{
		*line = (char)c;
		if (*line == EOL || *line == EOF) break ;
		++line ;
		++count;
	}
	*line = '\0';
	/* return the length of the line */
	if (c == EOF) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) return -1 \n", __FILE__, __LINE__);
#endif
		return(-1);
	} else {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) return %d\n", 
		__FILE__, __LINE__, count);
#endif
		return(count);
	}
}

display_single_grid()
{
	char	fname[MAX_LEN];
	ECO_HEADER      save_head, head;
	FILE           *fp;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter display_single_grid()\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif

	/* Empty the string */
	fname[0] = '\0';
	strcpy(fname, GRID_NAME);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) GRID FILE: >%s<\n", 
	__FILE__, __LINE__, fname);
#endif
	fp = fopen(fname, "r");
	if (fp == NULL) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) File %s not found...\n", 
		__FILE__, __LINE__, fname);
		(void)fflush(DBG_FILE);
#endif
		exit(1);
	}
	read_header(&head, fp);
	save_head = head;
	fclose(fp);

	read_grid(&head, fname, s_grid);

	/*
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) paint grid\n", 
		__FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
	(void) paint_grid();
	*/
	return(0);
}


void paint_grid()
{
	int             i, j;
	unsigned long this_color;
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter paint_grid()\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
				if (GRID_FLAG == TRUE) {
#ifdef ALL_USE
					switch(s_grid[i][j]) {
				  		case 1:	this_color = green_pixel;break; /* desert */
				  		case 2: this_color = blue_pixel;break;	/* urban */
				  		case 3: this_color = orange_pixel;break;/* agric */
				  		case 4: this_color = pink_pixel;break;	/* rec */
				  	}
#else
					this_color = orange_pixel;
#endif
				} else {

			switch(s_grid[i][j]) {
			  case 0: this_color = dark_slate_grey_pixel; 
			  case 1: this_color = background_pixel;break;
			  case 2: this_color = pink_pixel;break;
			  case 3: this_color = aquamarine_pixel;break;
			  case 4: this_color = aquamarine_pixel;break;
			  default: this_color = yellow_pixel;break;
			} /* end switch */
				}
			XSetForeground(thedisplay, imagegc, this_color);
			XFillRectangle(thedisplay, imagewindow, imagegc, j * SIZE, i * SIZE, SIZE, SIZE);
		} /* end columns */
	} /* end rows */
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Exit paint_grid()\n", __FILE__, __LINE__);
	(void)fflush(DBG_FILE);
#endif
	return;

} /* end paint_grid */

get_name(lone_name, name_str)
char	*lone_name, *name_str;
{
	char	*c_p, *out_p;
	int		cnt = 0, i = 0;
	static	int		grid_repeat_cnt = 1;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter get_name(%s)\n", __FILE__, __LINE__, name_str);
	(void)fflush(DBG_FILE);
#endif
	memset(lone_name, '\0', MAX_LEN);
	if ((name_str == NULL) || (*name_str == '\0')) {
		if (grid_repeat_cnt < grid_repeat_val) {
#ifdef DEBUG
			(void)fprintf(DBG_FILE, "(%s %d) NO grid (cnt:%d val:%d). Start over\n", 
			__FILE__, __LINE__, grid_repeat_cnt, grid_repeat_val);
			(void)fflush(DBG_FILE);
#endif
			strcpy(name_str, ORIG_GRID_STR);
			grid_repeat_cnt++;
		} else {
#ifdef DEBUG
			(void)fprintf(DBG_FILE, "(%s %d) Last grid. Quit\n", __FILE__, __LINE__);
			(void)fflush(DBG_FILE);
#endif
			exit(0);
		}
	}
	c_p = name_str;
	out_p = lone_name;
	/* read until EOL or comma */
	while (*c_p != '\0' && *c_p != ',') {
		*out_p++ = *c_p++;
		cnt++;
	}
	*out_p = '\0';

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) cnt: %d\n", __FILE__, __LINE__, cnt);
	(void)fflush(DBG_FILE);
#endif
	/* if we ended because of an EOL, we are done */
	if (*c_p == '\0') {
		*name_str = '\0';
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Using last name\n", __FILE__, __LINE__);
		(void)fprintf(DBG_FILE, "(%s %d) return string >%s<\n", __FILE__, __LINE__, lone_name);
		(void)fflush(DBG_FILE);
#endif
		memset(GRID_STR, '\0', MAX_LEN);
		return(0);
	}
		
	/* throw out comma and move the entire string array left cnt characters */
	for (i=cnt+1; i<MAX_LEN; i++) {
		name_str[i - (cnt + 1)] = name_str[i];
	}
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) remainder: >%s<\n", 
	__FILE__, __LINE__, name_str);
	(void)fflush(DBG_FILE);
#endif

	return(0);
}

/*************************************************************
*************************************************************/
checksubs(row, col, file_loc, line_num)
int		row, col, line_num;
char	*file_loc;
{
	if (row < 0 || row >= NROWS) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) NOTE row out of bounds\n",
		file_loc, line_num);
		(void)fflush(DBG_FILE);
#endif
		return(-1);
	}
	if (col < 0 || col >= NCOLS) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) NOTE column out of bounds\n",
		file_loc, line_num);
		(void)fflush(DBG_FILE);
#endif
		return(-1);
	}
	return(0);
}
/************************************************************************
************************************************************************/
verify_tools_and_dirs() 
{
	char    tmp_fname[MAX_LEN];
	struct 	stat stat_buf;
	int	ret;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter verify_tools()\n", __FILE__, __LINE__);
#endif
	/* check OVERLAY_PRGM */
	strcpy(tmp_fname, OVERLAY_DIR);
	strcat(tmp_fname, "/overlay_");
	strcat(tmp_fname, SFX);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, tmp_fname);
#endif
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check UNP_DIR */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, UNP_DIR);
#endif
	strcpy(tmp_fname, UNP_DIR);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check DATA_DIR */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, DATA_DIR);
#endif
	strcpy(tmp_fname, DATA_DIR);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check LOG_DIR */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, LOG_DIR);
#endif
	strcpy(tmp_fname, LOG_DIR);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check FRAG_PRGM */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, FRAG_PRGM);
#endif
	strcpy(tmp_fname, FRAG_PRGM);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}

	/* check STRIP_PGM */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, STRIP_PRGM);
#endif
	strcpy(tmp_fname, STRIP_PRGM);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}
	/* check ADDEOL_PGM */
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Check for >%s<\n", __FILE__, __LINE__, ADDEOL_PRGM);
#endif
	strcpy(tmp_fname, ADDEOL_PRGM);
	if ((ret = stat(tmp_fname, &stat_buf)) != 0) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) stat() failed (%d)\n", __FILE__, __LINE__, ret);
#endif
		lost_tool_msg(tmp_fname);
		return(-1);
	}
	return(0);
}
/*****************************************************************
*****************************************************************/
lost_tool_msg(file)
char	*file;
{
	(void)fprintf(stderr, "\n%s could not be found. Either change what is being \n", file);
	(void)fprintf(stderr, "looked for (likely place to look is localdefs.h or the command line)\n");
	(void)fprintf(stderr, "or create the directory/utility that is required\n\n");
	return(0);
}
