/*************************************************************************/
/* cluster.c - VISUAL.CALIB VERSION                                      */
/* Computes the  number of clusters in the distribution                  */
/* last revised                                                          */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include <math.h>
#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern	FILE *DBG_FILE;
#endif

STATS
cluster(Stat)
	STATS           Stat;
{
	/*
	 * Keith Clarke's highly Inefficient Cluster Algorithm
	 * 
	 */
	unsigned short  cluster[NROWS][NCOLS];
	int             erode = 1;
	int             i, j, last;
	int             sum = 0, cross;
	int             ii, jj;
Stat.pop = 0;

	/* Make binary array */
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			if (z[i][j] != 0) {
				cluster[i][j] = 1;
				Stat.pop++;
			} else
				cluster[i][j] = 0;
			/* Zero the edges, necessary for later on */
			if (i == 0)
				cluster[i][j] = 0;
			if (j == 0)
				cluster[i][j] = 0;
			if (i == nrows - 1)
				cluster[i][j] = 0;
			if (j == ncols - 1)
				cluster[i][j] = 0;
		}
	}
	/* this step fills in any holes, which otherwise become circles */
	for (i = 1; i < nrows - 1; i++) {
		for (j = 1; j < ncols - 1; j++) {
			sum = 0;
			for (ii = i - 1; ii <= i + 1; ii++) {
				for (jj = j - 1; jj <= j + 1; jj++) {
					if (!((i == ii) && (j == jj)))
						sum += cluster[ii][jj];
				}
			}
			if (sum == 8)
				cluster[i][j] = 1;
		}
	}

	/* Iterate while the operator modifies the grid */
	while (erode) {
		erode = 0;
		for (i = 1; i < nrows - 1; i++) {
			for (j = 1; j < ncols - 1; j++) {
				/* Only do this for white pixels */
				if (cluster[i][j]) {
					/* save the lower left bit status */
					last = cluster[i - 1][j - 1];
					cross = 0;
					if (cluster[i - 1][j] != last) {
						cross++;
						last = !last;
					}
					if (cluster[i - 1][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster[i][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster[i + 1][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster[i + 1][j] != last) {
						cross++;
						last = !last;
					}
					if (cluster[i + 1][j - 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster[i][j - 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster[i - 1][j - 1] != last) {
						cross++;
						last = !last;
					}
					/* Test for pixel deletion */
					if (cross == 2) {
						cluster[i][j] = 0;
						erode = 1;
					}
				}
			}
		}
	}
	/* All done, now count how many clusterss are left */
	cross = 0;
	for (i = 0; i < nrows; i++)
		for (j = 0; j < ncols; j++)
			if (cluster[i][j])
				cross++;
	Stat.clusters = cross;
	return (Stat);
}
