/*************************************************************************/
/* defines.h - VISUAL.CALIB VERSION                                      */
/* last revised 6/9/96, Stacy Hoppen                                     */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

/* Now the includes */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <values.h>
/* The following are in /usr/openwin/include/X11 */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>

#ifdef SAMPLE
#define SEED_DATE "1900"
#else

#ifdef DO_1912
#define SEED_DATE "1912"
#endif
#ifdef DO_1934
#define SEED_DATE "1934"
#endif
#ifdef DO_1955
#define SEED_DATE "1955"
#endif
#ifdef DO_1975
#define SEED_DATE "1975"
#endif

#endif

#ifdef SAMPLE
#define ITER0 1900
#define ITER1 1940
#define ITER2 1954
#define ITER3 1962
#define ITER4 1974
#define ITER5 1990
#else
#define ITER0 1900
#define ITER1 1912
#define ITER2 1934
#define ITER3 1955
#define ITER4 1975
#define ITER5 1995
#endif

/* number of rows and columns in the input arrays */
#ifdef SAMPLE
#define NROWS 360
#define NCOLS 285
#else

/* number of rows and columns in the input arrays */
/* the EDGE_D is 5% of the NROWS value; this is the edge distance for fragstats */

/*
#define NCOLS         	173
#define NROWS         	101
#define ASC_NCOLS     	173
#define ASC_NROWS     	101
#define XLL	     	"317723.326562"
#define YLL	     	"3667823.723033"
#define CELLSZ       	"1000"
#define EDGE_D       	"5"
#define SFX		"1k"
*/

#define ASC_NCOLS       361
#define ASC_NROWS       283
#define NCOLS         	361
#define NROWS         	283
#define XLL     	"317723.326562"
#define YLL     	"3650222.010936"
#define CELLSZ      	"480"
#define EDGE_D       	"14"
#define SFX		"480m"

/*
#define ASC_NCOLS       721
#define ASC_NROWS       577
#define NCOLS         	721
#define NROWS         	577
#define XLL     	"317723.326562"
#define YLL     	"3648932.269096"
#define CELLSZ      	"240"
#define EDGE_D       	"28"
#define SFX		"240m"
*/

/*
#define ASC_NCOLS       1443
#define ASC_NROWS       1000
#define NCOLS         	1443
#define NROWS         	1000
#define XLL     	"317723.326562"
#define YLL     	"3658171.834556"
#define CELLSZ      	"120"
#define EDGE_D       	"50"
#define SFX		"120m"
*/

/*
#define ASC_NCOLS     	2885
#define ASC_NROWS     	1964
#define NCOLS         	2885
#define NROWS         	1964
#define XLL	     	"317723.326562"
#define YLL	     	"3659229.198877"
#define CELLSZ       	"60"
#define EDGE_D       	"98"
#define SFX		"60m"
*/

#ifdef DO_1912
#define SEED_DATE "1912"
#endif
#ifdef DO_1934
#define SEED_DATE "1934"
#endif
#ifdef DO_1955
#define SEED_DATE "1955"
#endif
#ifdef DO_1975
#define SEED_DATE "1975"
#endif
#endif


#define PICKMAX 100

/* maximum length for strings */
#define MAX_LABEL 15

/*
 * Number of colors in the color table, either stored in file color_table or
 * in Main.c
 */
#define NUM_COLORS 255 /* Sal Perez. Not used on this graphic version */

/* Height and width of the status window in pixels, necessary for scaling */
#ifdef SAMPLE
#define HEIGHT 400
#define WIDTH 300
#define SIZE 2 /* Use two x two pixels per cell */
#else
/* here is where you might want to play with the image resolution */
#define HEIGHT 400
#define WIDTH 300
#define SIZE 2
#endif

	
#define WATER_COLOR  0   /* neutral. Not used anymore. Sal Perez 1-22-96 */
#define CIRCLE_COLOR 1   /* white. Not used anymore. Sal Perez 1-22-96  */
#define ROAD_COLOR 2    /* brown. Not used anymore. Sal Perez 1-22-96  */

/* colors for different phases of urban growth */
#define PHASE0G	3	
#define PHASE1G 4
#define PHASE2G	5
#define PHASE3G 6
#define PHASE4G 7
#define PHASE5G 8

#define URBAN 4
#define AGRIC 2

#define MAXRBUF		100


/*********** Important Sizes *************/

#ifdef SAMPLE
#define MAXC		32768
#else
#define MAXC		132768
#endif
/* these next 4 appear to be unused. For last 2, use NROWS and NCOLS instead */
#define MAXBANDS	12
#define MAXV            8192	/* Max vertices in rd_xy */
#define ROWS            4096
#define COLS            4096

/*********** General Purpose Defines *************/

#ifndef TRUE
#define TRUE		1
#define FALSE		0
#endif

#define BOOLEAN		int

#define PI		3.1415927
#define RADIAN		57.29577951
#define BIG		99999999.0

#define EOS		'\0'
#define EOL		'\n'


#define BEGIN		0
#define	CURRENT		1

#define EXFIX		0
#define	NOFIX		1

/************** Magic numbers for different file types ****************/
#define DBASE		6732		/* word 7 */
#define ECOSAT		4322		/* word 7 */
#define ELAS		4321		/* word 7 */
#define ERDAS		1212498244      /* word 0 "HEAD" */
#define IDIMS		50
#define IRIS		474		/* short 0 */
#define LAS		1229799751	/* word 12 */
#define SUN 		1504078485	/* word 0 */
#define TARGA		10
#define PEDIT		51
#define VAS		8888
#define VIFF		171		/* byte 0 */
#define NEWECOSAT	4323		/* word 124 */

#define UNKNOWN		100
#define AVS		110
#define ASAS		120
#define ASCII		130
/*#define RAW		140*/
#define BIL_RAW		141
#define BSQ_RAW		142
#define PIL_RAW		143
#define NLIO		150
#define EPABINARY	160

#define PIL		0	
#define BIL		1	
#define BSQ		2	
#define TILE		3	


/************** Header sizes in bytes *************************/
#define ASASHEAD	1024
#define AVSHEAD		8
#define DBASEHEAD	512
#define ECOSATHEAD	512
#define NEWECOSATHEAD	1024
#define ELASHEAD	512
#define ERDASHEAD	128
#define HEADERSIZE	512
#define IRISHEAD	512
#define LASHEAD		512
#define PEDITHEAD	6400
#define RAWHEAD		0
#define SUNHEAD		32
#define VASHEAD		512
#define VIFFHEAD	1024
#define TARGAHEAD	18


/************** ECOSAT Data Types ****************/
#define BYTE		1
#define SHORT		2
#define LONG		4
#define FLOAT		6
#define DOUBLE		8


/************** VAS Data Types ****************/
#define POINT           91
#define LINE            92
#define AREA            93
#define TIME            95
#define MULTI           96
#define MAGNITUDE       97
#define DEPTH           98
#define CROSS           99
#define PALL            100
 
#define MAX(a, b)       ((a) > (b) ? (a) : (b))
#define MIN(a, b)       ((a) < (b) ? (a) : (b))

/************** Magic numbers for different file types ****************/
#define DBASE		6732		/* word 7 */
#define ECOSAT		4322		/* word 7 */
#define ELAS		4321		/* word 7 */
#define ERDAS		1212498244      /* word 0 "HEAD" */
#define IDIMS		50
#define IRIS		474		/* short 0 */
#define LAS		1229799751	/* word 12 */
#define SUN 		1504078485	/* word 0 */
#define TARGA		10
#define PEDIT		51
#define VAS		8888
#define VIFF		171		/* byte 0 */
#define NEWECOSAT	4323		/* word 124 */

#define UNKNOWN		100
#define AVS		110
#define ASAS		120
#define ASCII		130
/*#define RAW		140*/
#define BIL_RAW		141
#define BSQ_RAW		142
#define PIL_RAW		143
#define NLIO		150
#define EPABINARY	160

#define PIL		0	
#define BIL		1	
#define BSQ		2	
#define TILE		3	


/************** Header sizes in bytes *************************/
#define ASASHEAD	1024
#define AVSHEAD		8
#define DBASEHEAD	512
#define ECOSATHEAD	512
#define NEWECOSATHEAD	1024
#define ELASHEAD	512
#define ERDASHEAD	128
#define HEADERSIZE	512
#define IRISHEAD	512
#define LASHEAD		512
#define PEDITHEAD	6400
#define RAWHEAD		0
#define SUNHEAD		32
#define VASHEAD		512
#define VIFFHEAD	1024
#define TARGAHEAD	18


/************** ECOSAT Data Types ****************/
#define BYTE		1
#define SHORT		2
#define LONG		4
#define FLOAT		6
#define DOUBLE		8


/************** VAS Data Types ****************/
#define POINT           91
#define LINE            92
#define AREA            93
#define TIME            95
#define MULTI           96
#define MAGNITUDE       97
#define DEPTH           98
#define CROSS           99
#define PALL            100
 
#define MAX(a, b)       ((a) > (b) ? (a) : (b))
#define MIN(a, b)       ((a) < (b) ? (a) : (b))
