/*************************************************************************/
/* growth.c - CALIBRATION VERSION                                        */
/* last revised: 6/5/96, Stacy Hoppen                                    */	
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include <string.h>
#include "defines.h"
#include "typedefs.h"
#include "graphics.h"
#include "localdefs.h"

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

unsigned short int       roads[NROWS][NCOLS];
unsigned short int       slp[NROWS][NCOLS];
unsigned short int       excld[NROWS][NCOLS];
unsigned short int       bkgrn[NROWS][NCOLS];
unsigned short int       seed[NROWS][NCOLS];
unsigned short int       urban[NROWS][NCOLS];
unsigned short int       single_displ_grid[NROWS][NCOLS]; /* the single display grid */
unsigned short int       my_lone[NROWS][NCOLS];
unsigned short int       grd[NROWS][NCOLS];  /* generic grid */

char            tmp_gname[MAX_LEN];
short int       elev_max;
extern int      nrows, ncols;
int             num_growth_pix;
float           growth_rate, last_growth_rate, percent_road, percent_urban;
int             roadcount, noncount;
float           average_slope;
int             change_all;

double		calc_rate();

void
growth()
{
	register int    r, c, i, j;
	int		intersection, together;
	int		append_num;
	float 		leesalee[5];
	int             time = 5, changed, circ_color;
	FILE           *fp, *fp_grid;
	int             spread();
	void            read_erdas();
	void            Edge();
	void            show_status_window(), read_erdas();
	void		repaint();
	void		plot_circle(double xc, double yc, double distance,
					unsigned long color);
	STATS           stats[1000], Circle(STATS);
	STATS		cluster(STATS);
	int             iteration = 0;
	ECO_HEADER      save_head, head;
	GSTATS          gstats;
	char           *getenv();
	char            fname[MAX_LEN], name[MAX_LEN], date[6];
	int             n;
	int             total_pixels;
	char            logname[MAX_LEN], tmp[MAX_LEN];

	long            dev;
	short           value;
	short           attached = 1;
	short           item = 0;
	int             nbuttons;
	void            pixrect ();
	struct {
			double          x;
			double          y;
			double          r;
	}               circle[10];
	int		z_counter = 0, urban_counter = 0;
	int		actual_agri = 0, model_agri = 0, seed_agri_cnt = 0, 
			seed_urban_cnt = 0;
	int		initial_year, initial_pop;
	int		initial_density, initial_num_cells;
	int		num_cells_before;
	int		pop_now;
	int		start_year, stop_year, current_density;
	double  	initial_rate, current_rate;


#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter growth()\n",
	 __FILE__, __LINE__);
	 (void)fflush(DBG_FILE);
#endif
	/* initialize the externals */
	num_growth_pix = 0;
	ticktock = startstop.start_date;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) ticktock initialized to %d\n", 
__FILE__, __LINE__, ticktock);
(void)fflush(DBG_FILE);
#endif
	growth_rate = 1.0;

	/* read in the input arrays */

	/* empty the string */
	fname[0] = '\0';
	strcpy(fname, DATA_DIR);
	strcat(fname, "/sub");
	strcpy(name, fname);
	strcat(name, ".slope");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) open: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	fp = fopen(name, "r");
	if (fp == NULL) {
		printf("ERROR: File %s not found...\n", name);
		exit(1);
	}
	read_header(&head, fp);
	save_head = head;
	fclose(fp);

	nrows = head.nl;
	ncols = head.ns;
	total_pixels = nrows * ncols;

	/* slope layer */
	strcpy(name, fname);
	strcat(name, ".slope");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, slp, NOFIX);
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) return from readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif

	/* print slope grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/slope");
	mk_gridname(tmp_gname, "1975", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, slp, I_NOCHANGE);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	/* END print slope grid */

	/* road layer */
	strcpy(name, fname);
	strcat(name, ".roads");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, roads, NOFIX);

	/* print road grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/roads");
	mk_gridname(tmp_gname, "1978", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, roads, I_NOCHANGE);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	/* END print road grid */


	/* excluded areas layer */
	strcpy(name, fname);
	strcat(name, ".excld");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) readgrid: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, excld, NOFIX);
	read_grid(&head, name, EXCLD, NOFIX);

	/* print excld grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/excld");
	mk_gridname(tmp_gname, "1975", NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, excld, I_NOCHANGE);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	/* END print excld grid */


	/*
	 * We want to read in the 1912, 1934, 1955, 1975 and 1995 grids so
	 * we can write them out in a matching format
	 */
	write_early_grid("1912", 0);
	write_early_grid("1934", 0);
	write_early_grid("1955", 0);
	write_early_grid("1975", 0);
	write_early_grid("1995", 1);


	/* seed layer */
	strcpy(date, SEED_DATE);

	strcpy(name, fname);
	strcat(name, ".seed");
	strcat(name, date);
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) read SEED: %s\n", __FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	read_grid(&head, name, seed, EXFIX);

	/* print seed grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/seed");
	mk_gridname(tmp_gname, date, NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, seed, I_NOCHANGE);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	/* END print seed grid */

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) count road and urban pixels\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* count the number of road pixels and the number of nonurban pixels */
	roadcount = 0;
	noncount = 0;
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			z[i][j] = 0;
			if (roads[i][j] == 255)
				roadcount++;
			if (excld[i][j] >= 1)
				noncount++;
			/*
			 * in our changes, urban is 4 (URBAN) so check for 4 or above. Will
			 * use 2 (AGRIC) for agriculture
			 */
			if (seed[i][j] >= URBAN) {
				z[i][j] = (unsigned short) PHASE0G;
				seed_urban_cnt++;
			}
			if (seed[i][j] == AGRIC) {
				z[i][j] = (unsigned short) AGRIC;
				seed_agri_cnt++;
			}
		}
	}
	percent_road = (100.0 * roadcount) / (total_pixels - noncount);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) SEED urban: %d agric: %d\n", 
	__FILE__, __LINE__, seed_urban_cnt, seed_agri_cnt);
	(void)fflush(DBG_FILE);
#endif

	/* zero the growth stats structure */
	gstats.sng = 0;
	gstats.sdg = 0;
	gstats.sdc = 0;
	gstats.og = 0;
	gstats.rt = 0;
	gstats.pop = 0;

	changed = TRUE;

	TOT_URBAN_RUN = seed_urban_cnt;  /* keeps track of total urban to date */

	start_year = startstop.start_date;
	stop_year = startstop.stop_date;
	initial_year = start_year;
	setup_initials(&initial_pop, &initial_num_cells, &initial_density, &initial_rate);

	/* initialize the changing variables */
	num_cells_before = initial_num_cells;
	pop_now = initial_pop;
	current_rate = initial_rate;
	current_density = initial_density;

#ifdef DEBUG
	(void)fprintf(DBG_FILE, "\n(%s %d) NEW YEAR: %d\n", __FILE__, __LINE__, ticktock + 1);
	(void)fflush(DBG_FILE);
#endif

	while (changed) {
		iteration++;
		
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Iteration %d: Apply rules\n", __FILE__, __LINE__, iteration);
(void)fflush(DBG_FILE);
#endif
		/* Compute the circle stats */
		stats[iteration] = Circle(stats[iteration]);

			/* Plot circle: Sal Perez 01-17-96 */

			circle[0].x = stats[iteration].xmean;
			circle[0].y = stats[iteration].ymean;
			circle[0].r = stats[iteration].rad;
			for (time = 9; time > 0; time--)
				circle[time] = circle[time - 1];
			for (time = 0; time < 10; time++) {
				plot_circle(circle[time].x, circle[time].y,
					circle[time].r, aquamarine_pixel);
			}

		/* apply the cellular automaton rules for this year */
		changed = begin_spread(&pop_now, &num_cells_before, 
		&current_rate, &current_density, &gstats, &stats[iteration]);
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) year: %d   pop_now: %d   nc_before: %d   cur_rate: %.5f  cur_den: %d\n",
__FILE__, __LINE__, ticktock, pop_now, num_cells_before, current_rate, current_density);
fflush(DBG_FILE);
#endif

		/* Compute the number of edge pixels */
		Edge(&stats[iteration]);

		/* Compute the number of clusters */
		stats[iteration] = (STATS) cluster(stats[iteration]);


		growth_rate = ((float) num_growth_pix / gstats.pop * 100.0);
		if (num_growth_pix == 0)
			growth_rate = 0.0;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Growth Rate: %3.4f\n", __FILE__, __LINE__, growth_rate);
(void)fprintf(DBG_FILE, "%d / (%d * 100)\n", num_growth_pix, gstats.pop);
(void)fflush(DBG_FILE);
#endif

		/* calculate the percentage of land that has been urbanized */
		percent_urban = (100.0 * gstats.pop) / (total_pixels - roadcount - noncount);

#ifdef HILT 
{

#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) do self-mods\n", __FILE__, __LINE__);
(void)fprintf(DBG_FILE, "(%s %d) Cur g-rate: %8.2f (hi:%8.2f lo:%8.2f)\n", 
__FILE__, __LINE__, growth_rate, selfmod.critical_high, selfmod.critical_low);
(void)fflush(DBG_FILE);
#endif

		/* self-modification: occurs if growth_rate > critical_high or < critical_low */
		if (growth_rate > selfmod.critical_high) {
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) WARNING: above critical\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif

			param.slope_resistance += (double) (percent_urban / 20.0);
			if (param.slope_resistance > 100) param.slope_resistance = 100.0;

			param.road_gravity += (double) (percent_urban / 20.0);
			if (param.road_gravity > 100) param.road_gravity = 100.0;

			if (param.diffusion_coefficient < 100)
			{
				param.diffusion_coefficient *= selfmod.boom;
				if (param.diffusion_coefficient > 100) param.diffusion_coefficient = 100.0;

				param.spread_coefficient *= selfmod.boom;
				if (param.spread_coefficient > 100) param.spread_coefficient = 100.0;

				param.breed_coefficient *= selfmod.boom;
				if (param.breed_coefficient > 100) param.breed_coefficient = 100.0;

			}

		} 
		if (growth_rate < selfmod.critical_low){

#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) WARNING: below critical\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
			param.slope_resistance -= (double) (percent_urban / 20.0);
			if (param.slope_resistance <= 0.01) param.slope_resistance = 1.0;

			param.road_gravity -= (double) (percent_urban / 20.0);
			if (param.road_gravity <= 0.01) param.road_gravity = 1.0;

			if (growth_rate < selfmod.critical_low)  {
				if (param.diffusion_coefficient > 0.0) 
				{
					param.diffusion_coefficient *= selfmod.bust;
					if (param.diffusion_coefficient <= 0.01) param.diffusion_coefficient = 1.0;

					param.spread_coefficient *= selfmod.bust;
					if (param.spread_coefficient <= 0.01) param.spread_coefficient = 1.0;

					param.breed_coefficient *= selfmod.bust;
					if (param.breed_coefficient <= 0.01) param.breed_coefficient = 1.0;

				}
			}
		}
}
#endif      /* self mods for HILT only */

		/* check to see if STOP_DATE has been reached yet */
		if (ticktock >= startstop.stop_date) {
			changed = FALSE;
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) Reached stop date (tick:%d stop:%d)\n", __FILE__, __LINE__, ticktock, startstop.stop_date);
(void)fflush(DBG_FILE);
#endif
		}
		else {
			changed = TRUE;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Haven't reached stop date (tick:%d stop:%d)\n", 
__FILE__, __LINE__, ticktock, startstop.stop_date);
(void)fflush(DBG_FILE);
#endif
		}
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) plot stats\n", 
		__FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
  		/* Plot Statistics and image */
		(void) show_status_window(&gstats);

#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) repaint\n", 
		__FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		(void) repaint();
#ifdef DEBUG
		(void)fprintf(DBG_FILE,"(%s %d) Yr:%d",__FILE__,__LINE__,ticktock);
		fprintf(DBG_FILE, "	  P1: %d P2: %d P3: %d P4: %d P5: %d\n", 
		gstats.sng, gstats.sdg, gstats.sdc, gstats.og, gstats.rt);
		(void)fflush(DBG_FILE);
#endif
		/* write out stats for designated year */
		switch (ticktock) {
			case ITER1:
				append_num = 1;
				break;
			case ITER2:
				append_num = 2;
				break;
			case ITER3:
				append_num = 3;
				break;
			case ITER4:
				append_num = 4;
				break;
			case ITER5:
				append_num = 5;
				break;
			default:
				continue;
				break;
		}
		if ((ticktock == ITER5) || (ticktock == startstop.stop_date)) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "\n(%s %d) write stats ITER%d\n", 
		__FILE__, __LINE__, append_num);
		(void)fflush(DBG_FILE);
#endif

		/* compute leesalee */
		together = 0;
		intersection = 0;
        	strcpy(name, DATA_DIR);
        	strcat(name, "/sub.seed");
		append_year(name, append_num);
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) readgrid %d: %s\n", __FILE__, __LINE__, 
		append_num, name);
		(void)fflush(DBG_FILE);
#endif
		read_grid(&head, name, urban, EXFIX);

		/* print stop year grid to the log directory */
		strcpy(tmp_gname, LOG_DIR);
		strcat(tmp_gname, "/seed");
		if (ticktock == ITER5) 
			mk_gridname(tmp_gname, "1995", NAME_ARR);
		else {
			sprintf(tmp, "%d", startstop.stop_date);
			mk_gridname(tmp_gname, tmp, NAME_ARR);
		}
		fp_grid = fopen(NAME_ARR, "w");
		write_int(fp_grid, urban, I_NOCHANGE);
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
		__FILE__, __LINE__, NAME_ARR);
		(void)fflush(DBG_FILE);
#endif
		/* END stop year slope grid */

		for (i = 0; i < nrows; i++) {
			for (j = 0; j < ncols; j++) {
				if (z[i][j] >= PHASE0G)  z_counter++;
				if (urban[i][j] >= URBAN) urban_counter++;
				if ((z[i][j] >= PHASE0G) || (urban[i][j] >= URBAN)) together++;
				if ((z[i][j] >= PHASE0G) && (urban[i][j] >= URBAN)) 
					intersection++;

				if (urban[i][j] == AGRIC) actual_agri++;
				if (z[i][j] == AGRIC) model_agri++;
			}
		}
		leesalee[0] = (float) intersection / together;
#ifdef DEBUG
		(void)fprintf(DBG_FILE, 
		"(%s %d) seed_urb:%d   real_urb:%d   mod_urb:%d  tot: %d  olap:%d  \n", 
		__FILE__, __LINE__, seed_urban_cnt, urban_counter, z_counter, together, intersection);

		(void)fprintf(DBG_FILE, "(%s %d) seed_agri:%d   real_agri:%d   mod_agri:%d\n", 
		__FILE__, __LINE__, seed_agri_cnt, actual_agri, model_agri);

		(void)fprintf(DBG_FILE, "(%s %d) leesalee: %8.8f (%s)\n", 
		__FILE__, __LINE__,  leesalee[0], name);

		(void)printf("NEW RUN:\n");
		(void)printf("	leesalee: %8.8f\n", leesalee[0]);
		(void)printf("	seed_urb:%d   real_urb:%d  mod_urb:%d  tot: %d  olap:%d\n",
		seed_urban_cnt, urban_counter, z_counter, together, intersection);
		(void)printf("	seed_agri:%d    real_agri:%d   mod_agri:%d\n", 
		seed_agri_cnt, actual_agri, model_agri);


		(void)fflush(DBG_FILE);
#endif

		/* Compute number of edge and area pixels */
		Edge(&stats[iteration]);

		/* Compute the number of clusters */
		stats[iteration] = cluster(stats[iteration]);

		/* Compute the circle stats */
		stats[iteration] = Circle(stats[iteration]);

		/* write out stats to grow.log */

		build_log(logname, "grow", append_num);
		fp = fopen(logname, "a");
		if (fp == NULL) {
			printf("ERROR: File %s not created...\n", logname);
#ifdef DEBUG
			(void)fprintf(DBG_FILE, 
			"(%s %d) ERROR: File %s not created\n", __FILE__, __LINE__, logname);
			(void)fflush(DBG_FILE);
#endif
			exit(1);
		}
		fprintf(fp, "%d %8.2f %8.2f", ticktock, growth_rate, percent_road);
		fprintf(fp, " %8f %8f %8f", (float) gstats.pop, 
		(float) stats[iteration].edges, (float) stats[iteration].clusters);
		fprintf(fp, " %8.2f %8.2f", average_slope, percent_urban);
		fprintf(fp, " %8.2f %8.2f %8.2f", stats[iteration].xmean, 
		stats[iteration].ymean, stats[iteration].rad);
		fprintf(fp, " %3.4f %3.4f %3.4f %3.4f %3.4f",
		(float) param.diffusion_coefficient,
		(float) param.breed_coefficient, (float) param.spread_coefficient,
		(float) param.slope_resistance, (float) param.road_gravity);
		fprintf(fp, " %6f %6f %6f %6f %6f %6f %6.4f %8f\n", (float) gstats.sng,
		(float) gstats.sdg, (float) gstats.sdc, (float) gstats.og, 
		(float) gstats.rt, (float) num_growth_pix, (float) leesalee[0], 
		(float) stats[iteration].area);
		fclose(fp);

		/* write out some of these statistics to individual files */
		build_log(logname, "param", append_num);
		fp = fopen(logname, "a");
		if (fp == NULL) {
			printf("ERROR: File %s not created...\n", logname);
			exit(1);
		}
		fprintf(fp, "%3.4f %3.4f %3.4f %3.4f %3.4f\n", 
		(float) param.diffusion_coefficient, (float) param.breed_coefficient, 
		(float) param.spread_coefficient, (float) param.slope_resistance, 
		(float) param.road_gravity); 
		fclose(fp);


		build_log(logname, "avslope", append_num);
		fp = fopen(logname, "a");
		if (fp == NULL) {
			printf("ERROR: File %s not created...\n", logname);
			exit(1);
		}
		fprintf(fp, "%3.2f\n", average_slope);
		fclose(fp);


		build_log(logname, "rate", append_num);
		fp = fopen(logname, "a");
		if (fp == NULL) {
			printf("ERROR: File %s not created...\n", logname);
			exit(1);
		}
		fprintf(fp, "%3.2f\n", growth_rate);
		fclose(fp);


		build_log(logname, "urban", append_num);
		fp = fopen(logname, "a");
		if (fp == NULL) {
			printf("ERROR: File %s not created...\n", logname);
			exit(1);
		}
		fprintf(fp, "%3.2f\n", percent_urban);
		fclose(fp);


		build_log(logname, "pop", append_num);
		fp = fopen(logname, "a");
		if (fp == NULL) {
			printf("ERROR: File %s not created...\n", logname);
			exit(1);
		}
		fprintf(fp, "%d\n", gstats.pop);
		fclose(fp);


		build_log(logname, "edge", append_num);
		fp = fopen(logname, "a");
		if (fp == NULL) {
			printf("ERROR: File %s not created...\n", logname);
			exit(1);
		}
		fprintf(fp, "%d\n", stats[iteration].edges);
		fclose(fp);


		build_log(logname, "clust", append_num);
		fp = fopen(logname, "a");
		if (fp == NULL) {
			printf("ERROR: File %s not created...\n", logname);
			exit(1);
		}
		fprintf(fp, "%d\n", stats[iteration].clusters);
		fclose(fp);

		/*
		 * any urban cells, regardless of growth type, now need to be of
		 * type URBAN so we write the grid out with just urban values.
		 * We want to include agriculture here.
		 */
		{
			char	tmp_year_str[5];
			for (i = 0; i < nrows; i++) {
				for (j = 0; j < ncols; j++) {
					if ((z[i][j] >= PHASE0G) && (seed[i][j] <= PHASE5G))
						my_lone[i][j] = URBAN;
					if (z[i][j] == AGRIC) 
						my_lone[i][j] = AGRIC;
					/*
					if (roads[i][j] == 255) 
						my_lone[i][j] = 255;
					*/
				}
			}
			memset(tmp_year_str, '\0', 5);
			append_year(tmp_year_str, append_num);
			strcpy(tmp_gname, LOG_DIR);
			strcat(tmp_gname, "/urb");
			mk_gridname(tmp_gname, tmp_year_str, NAME_ARR);
			fp_grid = fopen(NAME_ARR, "w");
			write_int(fp_grid, my_lone, U_A_COMBO_ROAD);
#ifdef xDEBUG
			(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
			__FILE__, __LINE__, NAME_ARR);
			(void)fflush(DBG_FILE);
#endif
		}
		}
   
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) replace seeds\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
		/* place urban seeds back */
		for (i = 0; i < nrows; i++)  {
			for (j = 0; j < ncols; j++) {
				if (seed[i][j] >= URBAN)
					z[i][j] = (unsigned short) PHASE0G;
			}
		}
	}
	return;
}

/********************************************************************************
 build up a logname file of the format LOG_DIR/paramyear.log where LOG_DIR is the
 global argument, param is the function argument, posn is the position of the year on the
 year linked list and the suffix is .log.  For example, LOG_DIR=/ and param=slope 
 and the posn is 1, and the first year on the list is 1940, it will build /slope1940.log
*********************************************************************************/
build_log(logname, param, year)
char	*logname, *param;
int	year;
{
	struct yr_elt	*curr_p;
	int	posn;
	char	year_str[5];

	if (year < 1 || year > YEAR_CNT) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Wants non-existent posn\n",
		__FILE__, __LINE__);
		(void)fflush(DBG_FILE);
#endif
		(void)fprintf(stderr, 
		"Can't build logfile for requested year (%d)\n", year);
		exit(1);
	}
	strcpy(logname, LOG_DIR);
	strcat(logname, "/");
	strcat(logname, param);

	/* get year off the list */
	posn  = 1;
	curr_p = START_p;
	while (posn != year) {
		curr_p = curr_p->next_p;
		if (curr_p == NULL) {
#ifdef DEBUG
			(void)fprintf(DBG_FILE, "(%s %d) Error in year list\n",
			__FILE__, __LINE__);
			(void)fflush(DBG_FILE);
#endif
			(void)fprintf(stderr, "Error in year list\n");
			exit(1);
		}
		posn++;
	}
	sprintf(year_str, "%4d", curr_p->year);
	strcat(logname, year_str);
	strcat(logname, ".log");
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Final log name: %s\n",
	__FILE__, __LINE__, logname);
	(void)fflush(DBG_FILE);
#endif
	return(0);

}
/********************************************************************************
 given a string, cat the year number of the given position of the year linked list
 to it.  For example, if given year is 1 and the first year on the linked list
 is 1940, it will cat 1940 to the given input string
*********************************************************************************/
append_year(string, year) 
char	*string;
int	year;
{
	struct yr_elt	*curr_p;
	int	posn;
	char	year_str[5];

	if (year < 1 || year > YEAR_CNT) {
#ifdef DEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Wants non-existent posn\n",
		__FILE__, __LINE__);
		(void)fprintf(DBG_FILE, "	Choices 1 - %d: entered %d\n",YEAR_CNT, year);
		(void)fflush(DBG_FILE);
#endif
		(void)fprintf(stderr, 
		"Can't concatenate requested year (%d)\n", year);
		exit(1);
	}

	/* get year off the list */
	posn = 1;
	curr_p = START_p;
	while (posn != year) {
		curr_p = curr_p->next_p;
		if (curr_p == NULL) {
#ifdef DEBUG
			(void)fprintf(DBG_FILE, "(%s %d) Error in year list\n", 
			__FILE__, __LINE__);
			(void)fflush(DBG_FILE);
#endif
			exit(1);
		}
		posn++;
	}
	sprintf(year_str, "%4d", curr_p->year);
	strcat(string, year_str);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Cat string: %s\n", 
	__FILE__, __LINE__, string);
	(void)fflush(DBG_FILE);
#endif
	return(0);
}
/*******************************************************************************
Tags or 0 to just write the grid and 1 to compute area, edges and clusters too
*******************************************************************************/
write_early_grid(year_str, tag)
char	*year_str;
int		tag;
{
	char			fname[MAX_LEN];
	ECO_HEADER      head;
	FILE           *fp_grid;
	float			area, edges, clusters;

	strcpy(fname, DATA_DIR);
	strcat(fname, "/sub.seed");
	strcat(fname, year_str);
	read_grid(&head, fname, grd, EXFIX);

	if (tag == 1) {
		area_stats(grd, &area, &edges, &clusters);
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Grid %s\n", __FILE__, __LINE__, fname);
		(void)fprintf(DBG_FILE, "	area: %8.4f    edges: %8.4f    clusters: %8.4f\n", 
		area, edges, clusters);
		(void)fflush(DBG_FILE);
#endif
	}

	/* print grid to the log directory */
	strcpy(tmp_gname, LOG_DIR);
	strcat(tmp_gname, "/grid");
	mk_gridname(tmp_gname, year_str, NAME_ARR);
	fp_grid = fopen(NAME_ARR, "w");
	write_int(fp_grid, grd, PASS_THRU_EXCLD);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Success in grid write >%s<\n", 
	__FILE__, __LINE__, NAME_ARR);
	(void)fflush(DBG_FILE);
#endif
	return(0);
}
/*******************************************************************************
*******************************************************************************/
/*****************************************************************
*****************************************************************/
int
begin_spread(pop_now, num_cells_before, current_rate, current_density,
gstats, mstats)
int 	*pop_now, *num_cells_before, *current_density;
double	*current_rate;
GSTATS  *gstats;
STATS   *mstats;
{
	int	pop_after, changed;
	int	num_cells_after, actual_added;
	int	cells2add;
	int	new_population;
	int	end_pop, year_pass;
	double  new_rate;

	/* Let time pass */
	ticktock++; 

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) start pop: %d  start num cells: %d\n",
	__FILE__, __LINE__, *pop_now, *num_cells_before);
	(void)fprintf(DBG_FILE, "(%s %d) cur rate: %.5f   cur dens: %d\n",
	__FILE__, __LINE__, *current_rate, *current_density);
	fflush(DBG_FILE);
#endif
	pop_after = project_pop(1, *pop_now, *current_rate);

	/* figure out how many cells need to be added */
	num_cells_after = (int)(((float)pop_after / (float)*current_density) + 1);
	cells2add = num_cells_after - *num_cells_before;
	if (cells2add < 0)
		cells2add = 0;
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Cells now: %d  needed: %d   to add: %d\n",
	__FILE__, __LINE__, *num_cells_before, num_cells_after, cells2add);
	fflush(DBG_FILE);
#endif
	changed = spread(gstats, mstats, cells2add, &actual_added);

	/* reset the variables for the next loop */
	if (actual_added > 0)
		*num_cells_before += actual_added;
	new_population = *num_cells_before * *current_density;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Actual add: %d    new_pop: %d\n",
	__FILE__, __LINE__, actual_added, new_population);
	fflush(DBG_FILE);
#endif
	end_pop = POP_1997;
	year_pass = 1997 - (ticktock + 1);
	if (ticktock < 1934) {
		get_est_density(ticktock + 1, current_density);
	}
	if ((ticktock >= 1934) && (ticktock < 1955)) { 
		get_est_density(ticktock + 1, current_density);
	}
	if ((ticktock >= 1955) && (ticktock < 1975)) { 
		get_est_density(ticktock + 1, current_density);
	}
	if (ticktock >= 1975) {
		get_est_density(ticktock + 1, current_density);
	}
	if (ticktock == startstop.stop_date)
		return(changed);
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "\n(%s %d) NEW YEAR: %d\n", __FILE__, __LINE__, ticktock + 1);
	(void)fflush(DBG_FILE);
#endif
	new_rate = calc_rate(new_population, end_pop, year_pass);
	*current_rate = new_rate;
	*pop_now = new_population;

	return(changed);
}
/*****************************************************************
*****************************************************************/
int
project_pop(year, init_pop, rate)
int	init_pop, year;
double	rate;
{
	int	projected_pop;
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter project_pop() year: %d  init: %d   rate: %.5f\n",
__FILE__, __LINE__, year, init_pop, rate);
(void)fflush(DBG_FILE);
#endif
	/* Nt = N0*e^(rate*t) */
	projected_pop = (int)((double)init_pop * pow(2.718,rate * (double)year));
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) Projected population: %d\n", __FILE__, __LINE__, projected_pop);
(void)fflush(DBG_FILE);
#endif

	return(projected_pop);
}
/*****************************************************************
*****************************************************************/

double
calc_rate(init_val, seek_val, years)
int init_val, seek_val, years;
{
	double rate;

	rate = log((double)seek_val/(double)init_val) / (double)years;
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d) New rate: %.5f based on start: %d stop: %d over %d year(s)\n", 
	__FILE__, __LINE__, rate, init_val, seek_val, years);
	fflush(DBG_FILE);
#endif
	return(rate);
}
/*****************************************************************
*****************************************************************/
int
setup_initials(initial_pop, initial_num_cells, initial_density, initial_rate)
int	*initial_pop, *initial_num_cells, *initial_density;
double	*initial_rate;
{

#ifdef DO_1912
	*initial_pop = 39030;
	*initial_rate = calc_rate(POP_1912, POP_1997, 97-12);
	if (strcmp(SFX, "1k") == 0) {
		*initial_num_cells = 19; 
		*initial_density = 2054;
	}
	if (strcmp(SFX, "480m") == 0) {
		*initial_num_cells = 83;
		*initial_density = 470;
	}
	if (strcmp(SFX, "240m") == 0) {
		*initial_num_cells = 348;
		*initial_density = 112;
	}
	if (strcmp(SFX, "120m") == 0) {
		*initial_num_cells = 1390;
		*initial_density = 28;
	}
	if (strcmp(SFX, "60m") == 0) {
		*initial_num_cells = 5550;
		*initial_density = 7;
	}
#endif
#ifdef DO_1934
	*initial_pop = 138667;
	*initial_rate = calc_rate(POP_1934, POP_1997, 97-34);
	if (strcmp(SFX, "1k") == 0) {
		*initial_num_cells = 44;
		*initial_density = 3151;
	}
	if (strcmp(SFX, "480m") == 0) {
		*initial_num_cells = 199;
		*initial_density = 696;
	}
	if (strcmp(SFX, "240m") == 0) {
		*initial_num_cells = 804;
		*initial_density = 172;
	}
	if (strcmp(SFX, "120m") == 0) {
		*initial_num_cells = 3215;
		*initial_density = 43;
	}
	if (strcmp(SFX, "60m") == 0) {
		*initial_num_cells = 12976;
		*initial_density = 10;
	}
#endif
#ifdef DO_1955
	*initial_pop = 465000;
	*initial_rate = calc_rate(POP_1955, POP_1997, 97-55);
	if (strcmp(SFX, "1k") == 0) {
		*initial_num_cells = 395;
		*initial_density = 1177;
	}
	if (strcmp(SFX, "480m") == 0) {
		*initial_num_cells = 1702;
		*initial_density = 273;
	}
	if (strcmp(SFX, "240m") == 0) {
		*initial_num_cells = 6807;
		*initial_density = 68;
	}
	if (strcmp(SFX, "120m") == 0) {
		*initial_num_cells = 27351;
		*initial_density = 14;
	}
	if (strcmp(SFX, "60m") == 0) {
		*initial_num_cells = 109305;
		*initial_density = 4;
	}
#endif
#ifdef DO_1975
	*initial_pop = 1253400;
	*initial_rate = calc_rate(POP_1975, POP_1997, 97-75);
	if (strcmp(SFX, "1k") == 0) {
		*initial_num_cells = 717;
		*initial_density = 1748;
	}
	if (strcmp(SFX, "480m") == 0) {
		*initial_num_cells = 3240;
		*initial_density = 386;
	}
	if (strcmp(SFX, "240m") == 0) {
		*initial_num_cells = 13009;
		*initial_density = 96;
	}
	if (strcmp(SFX, "120m") == 0) {
		*initial_num_cells = 51589;
		*initial_density = 24;
	}
	if (strcmp(SFX, "60m") == 0) {
		*initial_num_cells = 205913;
		*initial_density = 6;
	}
#endif
#ifdef DEBUG
	(void)fprintf(DBG_FILE, "(%s %d)Initialize Values\n", __FILE__, __LINE__);
	(void)fprintf(DBG_FILE, "	Initialize density: %d\n", *initial_density);
	(void)fprintf(DBG_FILE, "	Initialize num cells: %d\n", *initial_num_cells);
	(void)fprintf(DBG_FILE, "	Initialize population: %d\n", *initial_pop);
	(void)fprintf(DBG_FILE, "	Initialize rate: %.5f\n", *initial_rate);
	(void)fflush(DBG_FILE);
#endif
	return(0);
}
/*****************************************************************
*****************************************************************/
int
get_est_density(ticktock, current_density)
int 	ticktock, *current_density;
{
	double b, m;
	
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter est_dens (tick: %d)\n", __FILE__, __LINE__, ticktock);
(void)fflush(DBG_FILE);
#endif
	if (ticktock >= 1900 && ticktock < 1934) {

		if (strcmp(SFX, "1k") == 0) {
			m = ((float)3151 - (float)2054) / ((float)1934 - (float)1912);
			b = (float)2054 - (m * (float)1912);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "480m") == 0) {
			m = ((float)696 - (float)470) / ((float)1934 - (float)1912);
			b = (float)470 - (m * (float)1912);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "240m") == 0) {
			m = ((float)172 - (float)112) / ((float)1934 - (float)1912);
			b = (float)112 - (m * (float)1912);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "120m") == 0) {
			m = ((float)43 - (float)28) / ((float)1934 - (float)1912);
			b = (float)28 - (m * (float)1912);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "60m") == 0) {
			m = ((float)10 - (float)7) / ((float)1934 - (float)1912);
			b = (float)7 - (m * (float)1912);
			*current_density = (m * (float)ticktock) + b;
		}
	}
	if (ticktock >= 1934 && ticktock < 1955) {
		if (strcmp(SFX, "1k") == 0) {
			m = ((float)1177 - (float)3151) / ((float)1955 - (float)1934);
			b = (float)3151 - (m * (float)1934);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "480m") == 0) {
			m = ((float)273 - (float)696) / ((float)1955 - (float)1934);
			b = (float)696 - (m * (float)1934);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "240m") == 0) {
			m = ((float)68 - (float)172) / ((float)1955 - (float)1934);
			b = (float)172 - (m * (float)1934);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "120m") == 0) {
			m = ((float)14 - (float)43) / ((float)1955 - (float)1934);
			b = (float)43 - (m * (float)1934);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "60m") == 0) {
			m = ((float)4 - (float)10) / ((float)1955 - (float)1934);
			b = (float)10 - (m * (float)1934);
			*current_density = (m * (float)ticktock) + b;
		}
	}
	if (ticktock >= 1955 && ticktock < 1975) {
		if (strcmp(SFX, "1k") == 0) {
			m = ((float)1748 - (float)1177) / ((float)1975 - (float)1955);
			b = (float)1177 - (m * (float)1955);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "480m") == 0) {
			m = ((float)386 - (float)273) / ((float)1975 - (float)1955);
			b = (float)273 - (m * (float)1955);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "240m") == 0) {
			m = ((float)96 - (float)68) / ((float)1975 - (float)1955);
			b = (float)68 - (m * (float)1955);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "120m") == 0) {
			m = ((float)24 - (float)24) / ((float)1975 - (float)1955);
			b = (float)24 - (m * (float)1955);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "60m") == 0) {
			m = ((float)6 - (float)4) / ((float)1975 - (float)1955);
			b = (float)4 - (m * (float)1955);
			*current_density = (m * (float)ticktock) + b;
		}
	}
	if (ticktock >= 1975 ) {
		if (strcmp(SFX, "1k") == 0) {
			m = ((float)1263 - (float)1748) / ((float)1995 - (float)1975);
			b = (float)1748 - (m * (float)1975);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "480m") == 0) {
			m = ((float)296 - (float)386) / ((float)1995 - (float)1975);
			b = (float)386 - (m * (float)1975);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "240m") == 0) {
			m = ((float)73 - (float)96) / ((float)1995 - (float)1975);
			b = (float)96 - (m * (float)1975);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "120m") == 0) {
			m = ((float)18 - (float)24) / ((float)1995 - (float)1975);
			b = (float)24 - (m * (float)1975);
			*current_density = (m * (float)ticktock) + b;
		}
		if (strcmp(SFX, "60m") == 0) {
			m = ((float)4 - (float)6) / ((float)1995 - (float)1975);
			b = (float)6 - (m * (float)1975);
			*current_density = (m * (float)ticktock) + b;
		}
	}
	return(0);
}
