/*************************************************************************/
/* read_grid.c - CALIBRATION VERSION                                     */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

read_grid(head,fname,grid, tag)
ECO_HEADER *head;
char *fname;
short int grid[NROWS][NCOLS];
int tag;
{
    FILE *fp;
    BUF_UNION buf;
    int i,j;

    /*printf("Reading file %s...\n",fname);*/
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Read file: %s\n", 
	__FILE__, __LINE__, fname);
	(void)fflush(DBG_FILE);
#endif
    fp = fopen(fname, "r");
    if (fp == NULL) {
		printf("ERROR: File %s not found...\n",fname);
		exit(1);
    }

    read_header(head,fp);
    /*print_header(head);*/

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) return from read_header(%d)\n", 
__FILE__, __LINE__, head->data_type);
(void)fflush(DBG_FILE);
#endif
    switch (head->data_type) {
	case BYTE:
	    for (i = 0; i < head->nl; i++) {
		fread(buf.c,sizeof(*buf.c),head->ns,fp);
		for (j = 0; j < head->ns; j++) grid[i][j] = (short)buf.c[j];
	    }
	    break;
	case SHORT:
	    for (i = 0; i < head->nl; i++) {
		fread(buf.h,sizeof(*buf.h),head->ns,fp);
		for (j = 0; j < head->ns; j++) grid[i][j] = buf.h[j];
	    }
	    break;
	case ASCII:
		{          /* START ASCII */
		char	arr[15], c;
		int		arr_cnt = 0, rdigit = 0, cdigit = 0;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) rows: %d  cols: %d\n", __FILE__, __LINE__,
head->nl, head->ns);
(void)fflush(DBG_FILE);
#endif
	    for (rdigit = 0; rdigit < head->nl; rdigit++) {
			for (cdigit = 0; cdigit < head->ns; cdigit++) {
				while ((c = getc(fp)) != EOF) {
					if ( c == ' ') {
						/* end of word */
						arr[arr_cnt]  = '\0';
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Word: %s\n", __FILE__, __LINE__, arr);
(void)fflush(DBG_FILE);
#endif
						/* do substitutions */
						if (tag == EXFIX) {
							int tmpv;
							tmpv = atoi(arr);
							if (tmpv == 100)
								grid[rdigit][cdigit] = 0;
							else
								grid[rdigit][cdigit] = tmpv;
						} else {
							grid[rdigit][cdigit] = atoi(arr);
						}
#ifdef xDEBUG
						if (grid[rdigit][cdigit] != 0) {
						(void)fprintf(DBG_FILE, "(%s %d) Non-0 at %d,%d\n", 
						__FILE__, __LINE__, rdigit, cdigit);
						fflush(DBG_FILE);
						}
#endif
						arr_cnt = 0;
						break;
					}else {
						arr[arr_cnt++]  = c;
					}
				} /* end while */
			}  /* end cdigit loop */
	    }  /* end rdigit loop */
		} /* END ASCII */
	    break;
	default:
#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) UNKNOWN TYPE\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	    break;
    }
    fclose(fp);
#ifdef EBUG
(void)fprintf(DBG_FILE, "(%s %d) exit read_grid()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	return(0);
}


print_grid(grid)
short int grid[NROWS][NCOLS];
{
    int i,j;

#ifdef DEBUG
(void)fprintf(DBG_FILE, "(%s %d) PRINT GRID\n", __FILE__, __LINE__);
    for (i = 0; i < NROWS; i++) {
		for (j = 0; j < NCOLS; j++)
	    	fprintf(DBG_FILE, "%d ",grid[i][j]);
		fprintf(DBG_FILE, "\n");
    }
(void)fflush(DBG_FILE);
#endif
}
