/*************************************************************************/
/* repaint.c - VISUAL.CALIB VERSION                                      */
/* Draws onto the canvas using Xlib drawing functions.                   */
/* last revised: 01-25-96, Sal Perez                                     */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include "defines.h"
#include "typedefs.h"
#include "graphics.h"
#include "localdefs.h"

#ifdef DEBUG
extern	FILE *DBG_FILE;
#endif

extern int	z[NROWS][NCOLS];
extern short    slp[NROWS][NCOLS];
extern short    roads[NROWS][NCOLS];
extern short    excld[NROWS][NCOLS]; /* excluded areas - parks, water */
extern short    bkgrn[NROWS][NCOLS];
extern short    seed[NROWS][NCOLS];
extern short 	single_displ_grid[NROWS][NCOLS]; 
extern int      nrows, ncols;


void repaint()
{
	int             i, j;
	unsigned long this_color;
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Enter repaint()\n", __FILE__, __LINE__);
	(void)fprintf(DBG_FILE, "(%s %d) GRID_FLAG: %s\n", __FILE__, __LINE__,
	(GRID_FLAG==TRUE)?"true":"false");
	(void)fflush(DBG_FILE);
#endif
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) nrows:%d  ncols:%d\n", __FILE__, __LINE__, nrows, ncols);
	(void)fflush(DBG_FILE);
#endif
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			/* Background color first */
			/*this_color = 9 + (long) (240. * (float) bkgrn[i][j] / 255.);*/

			if (bkgrn[i][j] == 0 ){
				/*this_color = thebackground + 2; 1-22-96corners */
				/*this_color = dark_slate_grey_pixel;*/ /*corners */
				this_color = blue_pixel; /*corners */
			}
			if (bkgrn[i][j] == 1)/* 1-23-96*/{
				this_color = background_pixel + 104;/*water or excluded 19*/

				/*this_color = dark_slate_grey_pixel;water or excluded 19*/
				/*this_color = thebackground + 2;water or excluded 19*/
			}
			if (bkgrn[i][j] > 2 && bkgrn[i][j] <= 30){
				this_color = background_pixel + 104;/*shadow 14*/
			}
			if (bkgrn[i][j] > 30 && bkgrn[i][j] <= 70)/*shadow 14*/{
				this_color = dim_gray_pixel;/* 104*/
			}
			if (bkgrn[i][j] > 70 && bkgrn[i][j] <= 85)/*57*/{
				this_color = background_pixel + 57;
			}
			if (bkgrn[i][j] > 85 && bkgrn[i][j] <= 95)/*shadow */{
				this_color = background_pixel + 98;/* 4,14,93,96-99,104,106-113*/
				/* 98 is very good */
			}
			if (bkgrn[i][j] > 95 && bkgrn[i][j] <= 100){/* dark shadow*/
				this_color = background_pixel + 118;/*54 14 104 dim_gray47*/
			}
			if (bkgrn[i][j] > 100 && bkgrn[i][j] <= 118){/* dark shadow*/
				this_color = background_pixel + 118;/*54 14 104 dim_gray98*/
			}
			if (bkgrn[i][j] > 118 && bkgrn[i][j] <= 123){
				this_color = background_pixel + 98;/*98 102*/
			}
			if (bkgrn[i][j] > 123 && bkgrn[i][j] <= 126)/*95 1-22-96*/{
				this_color = dim_gray_pixel;/*dim_gray_pixel low elev 57 18*/
			}
			if (bkgrn[i][j] > 126 && bkgrn[i][j] <= 131)/* low elev 95 1-22-96*/{
				this_color = dim_gray_pixel;/* 95 is good also*/
			}
			if (bkgrn[i][j] >131 && bkgrn[i][j] <= 135)/*low elev mts skirts*/{
				this_color = background_pixel + 102;/* 19,47,55,54,95,102*/
			}
			if (bkgrn[i][j] > 145 && bkgrn[i][j] <= 165){
				 /* face of mts 47,46 is very good */
			/*grey_pixel; 105,103,102,90,81,65,54,48,45,44,38,19,18*/
				this_color = background_pixel + 102;/* 47 102*/
			}
			if (bkgrn[i][j] > 165 && bkgrn[i][j] <= 230)/* peaks 230*/{
				this_color = background_pixel + 102;/*38,88,102wheat*/
			}

			/* Overlay the ocean */
			if (excld[i][j] > 0) 
				/* Excluded areas ( water, parks) */
				this_color = dark_slate_grey_pixel;
				/*this_color = thebackground + 2;*/
			if (GRID_FLAG == TRUE) {
				switch(single_displ_grid[i][j]) {
				 	case 255:	this_color = roads_pixel;break; /* roads */
					/* normal grid display */
				  	case -1:	this_color = dark_slate_grey_pixel;break;
				  	case 0: this_color = white_pixel;break;
				  	case 1: this_color = orange_pixel;break;
				  	case 2: this_color = green_pixel;break;
				  	case 3: this_color = magenta_pixel;break;
				  	case 4: this_color = roads_pixel;break;
				  	case 100: this_color = blue_pixel;break;
				}
			} else {
				if (z[i][j] >= 0){
				switch(z[i][j]) {
#ifdef ALL_USE
				  case 1: 	this_color = dim_gray_pixel;break; /* desert */
				  case 2: this_color = blue_pixel;break;	/* urban */
				  case 3: this_color = orange_pixel;break;	/* agric */
				  case 4: this_color = pink_pixel;break;	/* rec */
#else
				  case 0: this_color = gray_pixel;break;
				  case 1: this_color = magenta_pixel;break;
				  case AGRIC: 	this_color = green_pixel;break;
				  case PHASE0G: this_color = roads_pixel;break;
				  case PHASE1G: 
				  case PHASE2G: 
				  case PHASE3G: this_color = yellow_pixel;break;
				  case PHASE4G: this_color = orange_pixel;break;
				  case PHASE5G: this_color = orange_pixel;break;
#endif
				} /* end switch */
				}
				/* for normal run, this shows the roads */
				/*
				if (roads[i][j] > 0) 
					this_color = roads_pixel;*/  /* for now red */
				/* for normal run, this shows the exclusions */
				if (excld[i][j] == 1) 
					this_color = white_pixel; 
				if (excld[i][j] == 2) 
					this_color = dark_slate_grey_pixel;
			}
			XSetForeground(thedisplay, imagegc, this_color);
			XFillRectangle(thedisplay, imagewindow, imagegc, j * SIZE, i * SIZE, SIZE, SIZE);
		} /* end columns */
	} /* end rows */
	return;

} /* end repaint */
