/*************************************************************************/
/* typedefs.h - VISUAL.CALIB VERSION                                     */
/* last revised 2/11/96                                                  */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

int		ticktock;  /* clock */

unsigned short 	pickr();
void 		read_header();
void 		get_filetype();
int 		nrows, ncols; 
int 		z[NROWS][NCOLS];
int       	s_grid[NROWS][NCOLS]; 


typedef struct {
    int       row;
    int       col;
    int       date;
    char      label[MAX_LABEL];
} SEED;

typedef struct {
    int area;
    int edges;
    int clusters;
    int pop;
    double xmean;
    double ymean;
	double rad;
} STATS;

typedef struct {
    int sng;
    int sdg;
    int sdc;
    int og;
    int rt;
    int pop;
} GSTATS;

typedef struct {
    double diffusion_coefficient;
    double spread_coefficient;
    double breed_coefficient;
    double slope_resistance;
    double road_gravity;
} PARAM;
PARAM param;

typedef struct {
    double critical_high;
    double critical_low;
    double boom;
    double bust;
    double critical_slope;
} SELFMOD;
SELFMOD selfmod;

typedef struct {
    int start_date;
    int stop_date;
} STARTSTOP;
STARTSTOP startstop;

typedef struct {
    int number_of_times;
    int cumulate;
} MONTECARLO;
MONTECARLO montecarlo;

typedef struct {		/* Header for ECOSAT Files */
    long        nbih;           /* num of bytes in header */
    long        nbpr;           /* num of bytes per record */
    long        sl;             /* starting line */
    long        nl;             /* number of lines */
    long        ss;             /* starting sample */
    long        ns;             /* number of samples */
    long        nc;             /* number of channels */
    long        magic_number;   /* SEDS magic number */
    long        notused1;       /* not currently used */
    long        data_type;      /* data type: byte, int, float, double */
    float	min;
    float	max;
    int		data_format;
    char	tmp[460];
    } ECO_HEADER;


typedef union {
    unsigned char   c[MAXC];
    short  h[MAXC/2];
    int    w[MAXC/4];
    float  r[MAXC/4];
    double d[MAXC/8];
    } BUF_UNION;

