/*************************************************************************/
/* weights.c - VISUAL.CALIB VERSION                                      */
/* last revised:                                                         */	
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include "defines.h"
#include "typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

get_slp_weights(slope_resistance,lut)
int slope_resistance;
double lut[91];
{
    int i;
    float maxslope;
    float pcval;

    if (slope_resistance >= 50) {
		pcval = (slope_resistance - 50) / 50.0;
		maxslope = 90.0 / (1.0 - pcval);
    } else 
		maxslope = slope_resistance * (90.0 / 50.0);
    
    for (i=0;i<=90;i++) {
		lut[i] = 1.0 - i / maxslope;
		if (lut[i] < 0) lut[i] = 0.0;
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) maxslope %f probability %d = %f\n", 
		__FILE__, __LINE__, maxslope, i, lut[i]);
		(void)fflush(DBG_FILE);
#endif
		/*if (i <= maxslope) 
	    	printf("maxslope %f probability %d = %f\n",maxslope,i,lut[i]);*/
    }
    /*printf("maxslope %f\n",maxslope);*/
}

/*******************************************************************
*******************************************************************/
int
influences(a, b)
int		a, b;
{
	int		agri_val, mean_val;

	agri_val = agri_factor(a,b);

	mean_val = mean_factor(a,b);

#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) agri val: %d         mean val: %d\n", 
	__FILE__, __LINE__, agri_val, mean_val);
	(void)fflush(DBG_FILE);
#endif
	return(MAX(agri_val, mean_val));
}

/*******************************************************************
*******************************************************************/
int
agri_factor(a, b)
int		a, b;
{
#ifdef FAVOR_AGRIC
	if (z[a][b] == AGRIC) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) Found Agriculture. Incr by %d\n", 
		__FILE__, __LINE__, AGRI_FACTOR);
		(void)fflush(DBG_FILE);
#endif
		return(AGRI_FACTOR); /* increase factor by this count */
	}
	else
		return(-99);
#else
	return(0);
#endif
		
}
/*******************************************************************
*******************************************************************/
int
mean_factor(a, b)
int		a, b;
{
	static	int		allow_xdist = 0;
	static	int		allow_ydist = 0;
	static	int		call_cnt = 0;
	int				number = 0;
	static	double		xmean = 0, ymean = 0;
	int		i, j;

	
#ifdef FAVOR_MEAN
	/* recalculate after every 100 calls */
	if ((call_cnt++ == 0) || (call_cnt++ % 100 == 0)) {
		xmean = ymean = number = 0;
		/* compute the means and distances */
		for (i = 0; i < nrows; i++) {
			for (j = 0; j < ncols; j++) {
				if (z[i][j] >= PHASE0G) {
					xmean += (double) j;
					ymean += (double) i;
					number++;
				}
			}
		}
		xmean /= (double) number;
		ymean /= (double) number;
		/*
	 	 * if the x and y values are within 10% of 
		 * the mean, prefer them
	 	 */
		allow_xdist = (int)(xmean * 0.1);
		allow_ydist = (int)(ymean * 0.1);
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) a/b %d/%d  mean %3.2f/%3.2f   dist: %d/%d\n", 
		__FILE__, __LINE__, a, b, xmean, ymean, allow_xdist, allow_ydist);
		(void)fflush(DBG_FILE);
#endif
	}
	if (((a >= xmean - allow_xdist) && (a <= xmean + allow_xdist))
	&& ((b >= ymean - allow_ydist) && (b <= ymean + allow_ydist))) {
#ifdef xDEBUG
		(void)fprintf(DBG_FILE, "(%s %d) x,y within mean range. Incr by %d\n", 
		__FILE__, __LINE__, MEAN_FACTOR);
		(void)fflush(DBG_FILE);
#endif
		return(MEAN_FACTOR); /* increase factor by this count */
	} else {
		return(-99);
	}
#else /* favor mean */
	return(0);
#endif /* favor mean */
}

get_road_weights(road_coefficient,lut)
int road_coefficient;
double lut[MAXRBUF + 1];
{
    int i;

    for (i=0;i<=MAXRBUF;i++) {
	lut[i] = 0.1;
    }
    lut[0] = 1.0;

    for (i=1;i<=road_coefficient;i++) {
		lut[i] = (road_coefficient - (i-1.0)) / road_coefficient; 
		if (lut[i] < 0.1) lut[i] = 0.1;
		/*printf("road coefficient %d = %f\n",i,lut[i]);*/
    }
}
