/*************************************************************************/
/* write_int.c  - VISUAL CALIBRATION VERSION                                    */
/* outputs data in raw integer format file                               */
/* date: 12/7/94                                                         */
/* revised for compatibility with urban modeling code DO NOT COPY BACK!  */	
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include "defines.h"
#include "typedefs.h"
#include <ctype.h>
#include "localdefs.h"

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

void
write_int (fp, grid, tag)
	FILE *fp;
	unsigned short grid[NROWS][NCOLS];
{

	int         i, j;
	char		tmp[MAX_LEN];
	int			urb_cnt = 0, agr_cnt = 0;


	/* Write out 6 lines of header */
	fprintf(fp, "ncols         ");
	sprintf(tmp, "%d", NCOLS);
	fprintf(fp, "%s\n", tmp);
	fprintf(fp, "nrows         ");
	sprintf(tmp, "%d", NROWS);
	fprintf(fp, "%s\n", tmp);
	fprintf(fp, "xllcorner     %s\n", XLL);
	fprintf(fp, "yllcorner     %s\n", YLL);
	fprintf(fp, "cellsize      %s\n", CELLSZ);
	fprintf(fp, "NODATA_value  0\n");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) nrow: %d  ncol: %d\n", __FILE__, __LINE__, nrows, ncols);
(void)fflush(DBG_FILE);
#endif
	/* Write out the data */
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			switch (tag) {
				case URBAN:
					/*
				 	* Want all non-0 values to be 4 so they are all type urban
				 	*/
					if ((grid[i][j] >= PHASE0G) && (grid[i][j] <= PHASE5G)) {
						fprintf(fp, "4 ");
						urb_cnt++;
					}
					else {
						fprintf(fp, "0 ");
					}
					break;
				case I_NOCHANGE:

					if (EXCLD[i][j] == 1) {
						fprintf(fp, "100 ");
					} else {
						if ((grid[i][j] >= PHASE0G) && (grid[i][j] <= PHASE5G))
							urb_cnt++;
						if (grid[i][j] == AGRIC)
							agr_cnt++;
						fprintf(fp, "%d ", grid[i][j]);
					}
					break;
				case I_NOCHG_ROAD:
					if (EXCLD[i][j] == 1) {
						fprintf(fp, "100 ");
					} else {
					if ((grid[i][j] >= PHASE0G) && (grid[i][j] <= PHASE5G))
						urb_cnt++;
					if (grid[i][j] == AGRIC)
						agr_cnt++;
					if (OVER_ROAD == TRUE) {
						if (ROAD[i][j] == 255)
							fprintf(fp, "255 ");
						else
							fprintf(fp, "%d ", grid[i][j]);
					} else {
						fprintf(fp, "%d ", grid[i][j]);
					}
					}
					break;
				case U_A_COMBO_ROAD:
					if (EXCLD[i][j] == 1) {
						fprintf(fp, "100 ");
					} else {
					if ((grid[i][j] >= PHASE0G) && (grid[i][j] <= PHASE5G)) {
						if (OVER_ROAD == TRUE) {
							if (ROAD[i][j] == 255)
								fprintf(fp, "255 ");
							else {
								fprintf(fp, "4 ");
								urb_cnt++;
							}
						} else {
							fprintf(fp, "4 ");
							urb_cnt++;
						}
					}
					else {
						if (grid[i][j] == AGRIC) {
							if (OVER_ROAD == TRUE) {
								if (ROAD[i][j] == 255)
									fprintf(fp, "255 ");
								else {
									fprintf(fp, "2 ");
									agr_cnt++;
								}
							} else {
								fprintf(fp, "2 ");
								agr_cnt++;
							}
						} else
							if (OVER_ROAD == TRUE) {
								if (ROAD[i][j] == 255)
									fprintf(fp, "255 ");
								else 
									fprintf(fp, "0 ");
							} else {
								fprintf(fp, "0 ");
							}
					}
					}
					break;
				case F_NOCHANGE:
					if ((grid[i][j] >= PHASE0G) && (grid[i][j] <= PHASE5G))
						urb_cnt++;
					if (grid[i][j] == AGRIC)
						agr_cnt++;
					fprintf(fp, "%.3f ", grid[i][j]);
					break;
				case PASS_THRU:
					switch (grid[i][j]) {
						case 1:
							fprintf(fp, "1 ", grid[i][j]);
							break;
						case 2:
							fprintf(fp, "2 ", grid[i][j]);
							break;
						case 3:
							fprintf(fp, "3 ", grid[i][j]);
							break;
						case 4:
							fprintf(fp, "4 ", grid[i][j]);
							break;
						default:
							fprintf(fp, "0 ", grid[i][j]);
							break;
					}
					break;
				default:
					if ((grid[i][j] >= PHASE0G) && (grid[i][j] <= PHASE5G)) {
						fprintf(fp, "4 ");
						urb_cnt++;
					}
					else {
						if (grid[i][j] == AGRIC) {
							fprintf(fp, "2 ");
							agr_cnt++;
						} else
							fprintf(fp, "0 ");
					}
					break;
			}
		}
		fprintf(fp, "\n");
	}

	/* Now close up the file and quit */
	fclose(fp);
#ifdef xDEBUG
	(void)fprintf(DBG_FILE, "(%s %d) Wrote file urb:%d  agr:%d\n", 
	__FILE__, __LINE__, urb_cnt, agr_cnt);
	(void)fflush(DBG_FILE);
#endif
	return;
}

mk_gridname(prefix, year, name)
char	*prefix, *year, *name;
{
	char	tmp[10];

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) Enter mk_gridname\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	strcpy(name, prefix);
	strcat(name, "_");
	strcat(name, year);
	strcat(name, "_r");
	sprintf(tmp, "%d", NROWS);
	strcat(name, tmp);
	strcat(name, "_c");
	sprintf(tmp, "%d", NCOLS);
	strcat(name, tmp);
	strcat(name, ".asc");
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) exit mk_grid with >%s<\n", 
__FILE__, __LINE__, name);
(void)fflush(DBG_FILE);
#endif
	return(0);
}
