
/*************************************************************************
 Area_stats.c
 Based on the Edge.c and Cluster.c functions
*************************************************************************/

#include <stdio.h>
#include <math.h>
#include "./defines.h"
#include "./typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

unsigned short  cluster_arr[NROWS][NCOLS];

area_stats(area_grd, area, edges, clusters)
unsigned short area_grd[NROWS][NCOLS];
float	*area, *edges, *clusters;
{
	calc_edges(area_grd, area, edges);
	calc_clusters(area_grd, clusters);
	return(0);
}
/**********************************************************************
**********************************************************************/
calc_edges(area_grd, area_p, edges_p)
unsigned short area_grd[NROWS][NCOLS];
float	*area_p, *edges_p;
{
	int             i, j;
	int             edge, edges;
	int             area;
	int             x, y;

	edges = 0;
	area = 0;
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			edge = FALSE;
			if (area_grd[i][j] != 0) {
				area++;

		 		/* this does a 4 neighbor search: N, S, E, W */
				if (area_grd[i - 1][j] == 0)
					edge = TRUE;
				if (area_grd[i + 1][j] == 0)
					edge = TRUE;
				if (area_grd[i][j - 1] == 0)
					edge = TRUE;
				if (area_grd[i][j + 1] == 0)
					edge = TRUE;


				/* this does an 8 neighbor search */
				/*
				for (y = i - 1; y <= i + 1; y++) {
					for (x = j - 1; x <= j + 1; x++) {
						if (area_grd[y][x] == 0) edge = TRUE;
					}
				}
				*/

				if (edge) {
					edges++;
				}
			}
		}
	}
	*area_p = area;
	*edges_p = edges;
	return(0);
}

/*******************************************************************
*******************************************************************/
calc_clusters(area_grd, clusters_p)
unsigned short area_grd[NROWS][NCOLS];
float	*clusters_p;
{
	/*
	 * Keith Clarke's highly Inefficient Cluster Algorithm
	 * 
	 */
	int             erode = 1;
	int             i, j, last;
	int             sum = 0, cross, pop;
	int             ii, jj;
	
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) ENTER calc_clusters()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	pop = 0;

	/* Make binary array */
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			if (area_grd[i][j] != 0) {
				cluster_arr[i][j] = 1;
				pop++;
			} else
				cluster_arr[i][j] = 0;
			/* Zero the edges, necessary for later on */
			if (i == 0)
				cluster_arr[i][j] = 0;
			if (j == 0)
				cluster_arr[i][j] = 0;
			if (i == nrows - 1)
				cluster_arr[i][j] = 0;
			if (j == ncols - 1)
				cluster_arr[i][j] = 0;
		}
	}
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) fill holes\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* this step fills in any holes, which otherwise become circles */
	for (i = 1; i < nrows - 1; i++) {
		for (j = 1; j < ncols - 1; j++) {
			sum = 0;
			for (ii = i - 1; ii <= i + 1; ii++) {
				for (jj = j - 1; jj <= j + 1; jj++) {
					if (!((i == ii) && (j == jj)))
						sum += cluster_arr[ii][jj];
				}
			}
			if (sum == 8)
				cluster_arr[i][j] = 1;
		}
	}

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) modify grid\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* Iterate while the operator modifies the grid */
	while (erode) {
		erode = 0;
		for (i = 1; i < nrows - 1; i++) {
			for (j = 1; j < ncols - 1; j++) {
				/* Only do this for white pixels */
				if (cluster_arr[i][j]) {
					/* save the lower left bit status */
					last = cluster_arr[i - 1][j - 1];
					cross = 0;
					if (cluster_arr[i - 1][j] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i - 1][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i + 1][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i + 1][j] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i + 1][j - 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i][j - 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i - 1][j - 1] != last) {
						cross++;
						last = !last;
					}
					/* Test for pixel deletion */
					if (cross == 2) {
						cluster_arr[i][j] = 0;
						erode = 1;
					}
				}
			}
		}
	}
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) count clusters\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* All done, now count how many clusters are left */
	cross = 0;
	for (i = 0; i < nrows; i++)
		for (j = 0; j < ncols; j++)
			if (cluster_arr[i][j])
				cross++;
	*clusters_p = cross;
	return (0);
}
