/*************************************************************************/
/* cluster.c  -  PREDICTION VERSION                                     */
/* computes the number of clusters in the distribution                   */
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <stdio.h>
#include <math.h>
#include "./defines.h"
#include "./typedefs.h"
#include "localdefs.h"

#ifdef DEBUG
extern FILE *DBG_FILE;
#endif

unsigned short  cluster_arr[NROWS][NCOLS];

STATS
cluster(Stat)
	STATS           Stat;
{
	/*
	 * Keith Clarke's highly Inefficient Cluster Algorithm
	 * 
	 */
	int             erode = 1;
	int             i, j, last;
	int             sum = 0, cross;
	int             ii, jj;
	
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) ENTER cluster()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	Stat.pop = 0;

	/* Make binary array */
	for (i = 0; i < nrows; i++) {
		for (j = 0; j < ncols; j++) {
			if (z[i][j] != 0) {
				cluster_arr[i][j] = 1;
				Stat.pop++;
			} else
				cluster_arr[i][j] = 0;
			/* Zero the edges, necessary for later on */
			if (i == 0)
				cluster_arr[i][j] = 0;
			if (j == 0)
				cluster_arr[i][j] = 0;
			if (i == nrows - 1)
				cluster_arr[i][j] = 0;
			if (j == ncols - 1)
				cluster_arr[i][j] = 0;
		}
	}
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) fill holes\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* this step fills in any holes, which otherwise become circles */
	for (i = 1; i < nrows - 1; i++) {
		for (j = 1; j < ncols - 1; j++) {
			sum = 0;
			for (ii = i - 1; ii <= i + 1; ii++) {
				for (jj = j - 1; jj <= j + 1; jj++) {
					if (!((i == ii) && (j == jj)))
						sum += cluster_arr[ii][jj];
				}
			}
			if (sum == 8)
				cluster_arr[i][j] = 1;
		}
	}

#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) modify grid\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* Iterate while the operator modifies the grid */
	while (erode) {
		erode = 0;
		for (i = 1; i < nrows - 1; i++) {
			for (j = 1; j < ncols - 1; j++) {
				/* Only do this for white pixels */
				if (cluster_arr[i][j]) {
					/* save the lower left bit status */
					last = cluster_arr[i - 1][j - 1];
					cross = 0;
					if (cluster_arr[i - 1][j] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i - 1][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i + 1][j + 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i + 1][j] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i + 1][j - 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i][j - 1] != last) {
						cross++;
						last = !last;
					}
					if (cluster_arr[i - 1][j - 1] != last) {
						cross++;
						last = !last;
					}
					/* Test for pixel deletion */
					if (cross == 2) {
						cluster_arr[i][j] = 0;
						erode = 1;
					}
				}
			}
		}
	}
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) count clusters\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	/* All done, now count how many clusters are left */
	cross = 0;
	for (i = 0; i < nrows; i++)
		for (j = 0; j < ncols; j++)
			if (cluster_arr[i][j])
				cross++;
	Stat.clusters = cross;
#ifdef xDEBUG
(void)fprintf(DBG_FILE, "(%s %d) exit clusters()\n", __FILE__, __LINE__);
(void)fflush(DBG_FILE);
#endif
	return (Stat);
}
