/*************************************************************************/
/* f_to_str.c - VISUAL.CALIB VERSION                                     */
/* converts a float to a string, used in Topograf menu routines          */
/* function call is in the form of: f_to_str(x, fstr)                    */ 
/* where x is a float or double and fstr is an externally kept char array */
/* of size determined elsewhere                                          */
/* written by Keith Clarke, 5/11/87                                      */
/* last revised: 7/14/93                                                 */	
/* Cellular Automaton for Urban Growth Simulation, Keith Clarke May 1990 */
/* kclarke@everest.hunter.cuny.edu                                       */
/*************************************************************************/

#include <math.h>

void f_to_str (value, places, fnum)
double value;
int places;
char    fnum[];
{
    double  divisor,
            fract;
    int     i,
            start,
            power,
            times,
            stop;
    long int    val;
    char    c,one_i();
/* first, deal with the negative sign */
    if (value < 0.0) {
	value *= -1.0;
	fnum[0] = '-';
	start = 1;
    }
    else
	start = 0;
/* remove the fractional part */
    fract = value - floor (value);
    value -= fract;
/* figure how many digits the value has */
    i = 1;
    power = 0;
    while (i <= (int) value) {
	i *= 10;
	power++;
    }
/* The value has power digits left of the dec. pt. */
    for (i = 0; i < power; i++) {
	divisor = pow (10.0, (double) (power - 1 - i));
	val = value / divisor;
	fnum[start + i] = one_i (val);
	value -= (val * pow (10.0, (double) (power - 1 - i)));
    }
/* Add the decimal point */
    fnum[start + power] = '.';
    power = start + power + 1;
    stop = 0;
    times = 0;
/* Now deal with the decimal part */
    while (!stop) {
	fract *= 10.0;
	c = one_i((int) floor (fract));
	fnum[power++] = c;
	    times++;
	fract -= (int) floor (fract);
	if (times >= places)
	    stop = 1;
    }
 /* Finally, add the null char */
    fnum[++power] = '\0';
    return;
}
